<?php
namespace App\Helpers;

use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Maatwebsite\Excel\Facades\Excel;

use App\AssignedSales;
use App\AssignedSaleDetails;
use App\Sale;
use Illuminate\Support\Facades\Log;

class CommonHelpers
{
  /*
  Guarda los reportes en el servidor y devuelve la ruta donde se almaceno (path), se puede pasar a url con
  storage_path('/app'.$path)
  Los archivos se guarda en storage/app/$basePath
  $basePath = ruta base de directorio donde se va a guardar el reporte "/public/reports"
  $directoryR = directorio donde se va a guardar el reporte, si no existe el metodo lo crea
  $data = data del reporte la primera fila son los titulos de la columnas
  $filename = Nombre del archivo sin extencion
  el archivo que almacena esta funcion es un xls
  NOTA: Si el excel tiene mas de 30000 lineas se dividira en paginas de 30000 cada una
   */
  public static function saveFile($basePath = false, $directoryR = false, $data = false, $filename = 'report', $limitFP = 30000, $ext = 'xls')
  {
    //Cargando directorios dentro de $basePath
    $directory  = Storage::disk('local')->directories($basePath);
    $pathReport = $basePath . '/' . $directoryR;

    //Si no existe el directorio de reporte para las altas se crea
    if (!in_array($pathReport, $directory)) {
      Storage::disk('local')->makeDirectory($pathReport);
    }

    //Guarda el excel en local
    \Excel::create($filename, function ($excel) use ($data, $limitFP, $ext) {
      if ($ext == 'xls') {
        if ((count($data) - 1) > $limitFP) {
          $pg = ceil((count($data) - 1) / $limitFP);

          $li = 0;

          for ($i = 1; $i <= $pg; $i++) {
            $part = array_slice($data, $li, $limitFP);

            if ($i > 1) {
              //$part = array_slice($data, $li, $limitFP);
              array_unshift($part, $data[0]);
              $li += $limitFP;
            } else {
              $li += $limitFP;
              $limitFP--;
            }

            $excel->sheet('Reporte ' . $i, function ($sheet) use ($part) {
              $sheet->fromArray($part, null, 'A1', false, false);
            });

          }
        } else {
          $excel->sheet('Reporte', function ($sheet) use ($data) {
            $sheet->fromArray($data, null, 'A1', false, false);
          });
        }
      } else {
        $excel->sheet('Reporte', function ($sheet) use ($data) {
          $sheet->fromArray($data, null, 'A1', false, false);
        });
      }
    })->store($ext, storage_path('app' . $pathReport)); //xls

    //Armando rutas
    $filename      = $filename . '.' . $ext; //xls
    $fullLocalPath = $pathReport . '/' . $filename;
    $s3FullPath    = 'reports/' . $directoryR . '/' . $filename;
    //return $fullLocalPath;
    //Subiendo el excel a s3
    Storage::disk('s3')->put($s3FullPath, file_get_contents(storage_path('app' . $fullLocalPath)), 'public');

    //Borrando el excel del local
    Storage::disk('local')->delete($fullLocalPath);

    //Retorna url de s3
    return Storage::disk('s3')->url($s3FullPath);
  }

  /*Verifica recaptcha de google*/
  public static function veifyCaptchaGoogle($data = false)
  {
    if ($data) {
      $curl = curl_init();

      curl_setopt_array($curl, array(
        CURLOPT_URL            => env('URL_VERIFY_CAPTCHA'),
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING       => "",
        CURLOPT_MAXREDIRS      => 10,
        CURLOPT_TIMEOUT        => 60,
        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST  => "POST",
        CURLOPT_POSTFIELDS     => $data,
        CURLOPT_HTTPHEADER     => array(
          "accept: */*",
          "Content-Type: application/x-www-form-urlencoded",
          "cache-control: no-cache",
        ),
      ));

      $response = curl_exec($curl);
      $err      = curl_error($curl);

      curl_close($curl);

      if ($err) {
        return ['success' => false, 'data' => $err];
      } else {
        return ['success' => true, 'data' => json_decode($response)];
      }

    }

    return ['success' => false, 'data' => 'Faltan datos.'];
  }

  public static function getMemoryUse()
  {
    $memory = memory_get_usage(true);

    if ($memory > 1024 * 1024) {
      $memory = round($memory / 1024 / 1024, 2) . ' MB';
    } elseif ($memory > 1024) {
      $memory = round($memory / 1024, 2) . ' KB';
    }

    return $memory;
  }
  /**
   *  Ejecuta un curl
   *
   *  @param String url endpoint
   *  @param String type tipo de ejecucion [GET, POST, DELETE, ..]
   *  @param Array header campo opcional, de ser enviado reemplaza la cabecera que se envia en el curl
   *  @return Array
   */
  public static function executeCurl($url = false, $type = false, $header = [], $data = [])
  {
    if ($url && $type) {
      $curl = curl_init();

      if (!count($header)) {
        $header = [
          "accept: */*",
          "Content-Type: application/json",
          "cache-control: no-cache",
          "accept-language: en-US,en;q=0.8",
        ];
      }

      $options = [
        CURLOPT_URL            => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING       => "",
        CURLOPT_MAXREDIRS      => 10,
        CURLOPT_TIMEOUT        => 30,
        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST  => $type,
        CURLOPT_HTTPHEADER     => $header,
      ];

      if (is_array($data) && count($data) && strtoupper($type) == 'POST') {
        $options[CURLOPT_POSTFIELDS] = json_encode($data);
      }

      curl_setopt_array($curl, $options);

      $response = curl_exec($curl);
      $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
      $err      = curl_error($curl);

      curl_close($curl);

      if ($err) {
        return [
          'success' => false,
          'data'    => $err,
          'code'    => !empty($httpcode) ? $httpcode : 0,
        ];
      } else {
        $dataJson = json_decode($response);

        if (!empty($dataJson)) {
          return [
            'success'  => true,
            'data'     => $dataJson,
            'original' => $response,
            'code'     => !empty($httpcode) ? $httpcode : 0,
          ];
        } else {
          return [
            'success'  => false,
            'data'     => 'No se pudo obtener json.',
            'original' => $response,
            'code'     => !empty($httpcode) ? $httpcode : 0,
          ];
        }

      }
    }

    return ['success' => false, 'data' => 'Faltan datos.'];
  }

  public static function getFloatValue($string)
  {
    $string = str_replace('$', '', $string);
    $string = str_replace(' ', '', $string);
    $string = str_replace(',', '', $string);
    //$string = str_replace(',', '.', $string);

    if (!empty($string)) {
      return floatval($string);
    }

    return 0;
  }

  public static function getCSVData($filepath, $bank, $delimiter = ',')
  {
    $file = fopen($filepath, "r");

    if ($file !== false) {
      ini_set('auto_detect_line_endings', true);

      $c   = 0;
      $res = [
        'OK'     => [],
        'NOT_OK' => [],
      ];

      while (($datos = fgetcsv($file, 0, $delimiter)) !== false) {
        if (count($datos) == $bank->column) {
          if ($bank->group == 'BV') {
            if (!empty($datos[5])) {
              $amount = self::getFloatValue($datos[5]);

              if ($amount > 0) {
                if (preg_match("/[a-z]{2}\d{4}/i", $datos[3], $matches)) {
                  $res['OK'][] = [
                    'hash'       => base64_encode($datos[3] . $datos[5] . $datos[6]),
                    'cod'        => strtoupper($matches[0]),
                    'concepto'   => $datos[2] . '-' . $datos[3],
                    'amount'     => $amount,
                    'amount_txt' => '$' . number_format($amount, 2, '.', ','),
                    'date_dep'   => Carbon::createFromFormat('d/m/Y', $datos[0])->format('Y-m-d'),
                    'line'       => implode(' + ', $datos),
                    'date_load'  => date('Y-m-d H:i:s'),
                    'bank'       => 'BBVA'];
                } else {
                  $res['NOT_OK'][] = [
                    'hash'       => base64_encode($datos[3] . $datos[5] . $datos[6]),
                    'concepto'   => $datos[2] . '-' . $datos[3],
                    'amount'     => $amount,
                    'amount_txt' => '$' . number_format($amount, 2, '.', ','),
                    'date_dep'   => Carbon::createFromFormat('d/m/Y', $datos[0])->format('Y-m-d'),
                    'reason'     => 'No se detecto el código',
                    'line'       => implode(' + ', $datos),
                    'date_load'  => date('Y-m-d H:i:s'),
                    'bank'       => 'BBVA'];
                }

                $c++;
              }
            }
          }

          if ($bank->group == 'AZ') {
            if (!empty($datos[4])) {
              $amount = self::getFloatValue($datos[4]);

              if ($amount > 0) {
                if (preg_match("/[.]\d{1,2}/i", $datos[4], $matches)) {
                  $cod = str_replace('.', '', $matches[0]);
                  $cod = strlen($cod) == 1 ? $cod . '0' : $cod;

                  $res['OK'][] = [
                    'hash'       => base64_encode($datos[1] . $datos[3] . $datos[4] . $datos[5] . $datos[6]),
                    'cod'        => $cod,
                    'concepto'   => $datos[3],
                    'amount'     => $amount,
                    'amount_txt' => '$' . number_format($amount, 2, '.', ','),
                    'date_dep'   => Carbon::createFromFormat('Y-m-d', $datos[1])->format('Y-m-d'),
                    'line'       => implode(' + ', $datos),
                    'date_load'  => date('Y-m-d H:i:s'),
                    'bank'       => 'Azteca'];
                } else {
                  $res['NOT_OK'][] = [
                    'hash'       => base64_encode($datos[1] . $datos[3] . $datos[4] . $datos[5] . $datos[6]),
                    'concepto'   => $datos[3],
                    'amount'     => $amount,
                    'amount_txt' => '$' . number_format($amount, 2, '.', ','),
                    'date_dep'   => Carbon::createFromFormat('Y-m-d', $datos[1])->format('Y-m-d'),
                    'reason'     => 'No se detecto el código',
                    'line'       => implode(' + ', $datos),
                    'date_load'  => date('Y-m-d H:i:s'),
                    'bank'       => 'Azteca'];
                }

                $c++;
              }
            }
          }
        }
      }

      ini_set('auto_detect_line_endings', false);
      fclose($file);

      if ($c > 0) {
        return [
          'success'       => true,
          'data'          => $res,
          'total_process' => $c];
      }

      return ['success' => false, 'msg' => 'No se procesaron los depósitos'];
    } else {
      return ['success' => false, 'msg' => 'No se pudo leer el archivo'];
    }
  }

  /* validacion de fechas de filtro de busqueda */
  public static function validateDate($filters)
  {
    //Validando que vengan los dos rangos de fechas y formateando fecha
    if (empty($filters['dateStar']) && empty($filters['dateEnd'])) {
      $filters['dateStar'] = Carbon::now()
        ->format('Y-m-d H:i:s');

      $filters['dateEnd'] = Carbon::now()
        ->addMonth()
        ->format('Y-m-d H:i:s');
    } elseif (empty($filters['dateStar']) && !empty($filters['dateEnd'])) {
      $filters['dateEnd'] = Carbon::createFromFormat('d-m-Y', $filters['dateEnd'])
        ->endOfDay()
        ->toDateTimeString();

      $filters['dateStar'] = Carbon::createFromFormat('Y-m-d H:i:s', $filters['dateEnd'])
        ->subMonth()
        ->startOfDay()
        ->toDateTimeString();
    } elseif (empty($filters['dateEnd']) && !empty($filters['dateStar'])) {
      $filters['dateStar'] = Carbon::createFromFormat('d-m-Y', $filters['dateStar'])
        ->startOfDay()
        ->toDateTimeString();

      $filters['dateEnd'] = Carbon::createFromFormat('Y-m-d H:i:s', $filters['dateStar'])
        ->endOfDay()
        ->addMonth()
        ->toDateTimeString();
    } else {
      $filters['dateStar'] = Carbon::createFromFormat('d-m-Y', $filters['dateStar'])
        ->startOfDay()
        ->toDateTimeString();

      $filters['dateEnd'] = Carbon::createFromFormat('d-m-Y', $filters['dateEnd'])
        ->endOfDay()
        ->toDateTimeString();
    }

    $filters['dateStar'] = (date("Ymd000000", strtotime($filters['dateStar'])));
    $filters['dateEnd']  = (date("Ymd235959", strtotime($filters['dateEnd'])));

    return $filters;
  }

/**
 * [getOptionColumn Devuelve los enum de una tabla]
 * @param  [type] $table [nombre de la tabla a la cual extraer informacion]
 * @param  [type] $field [nombre de la columna a revisar]
 * @return [type]        [description]
 */
  public static function getOptionColumn($table, $field)
  {
    //$table = 'islim_portability';
    //$field = 'status';

    $test = DB::select(DB::raw("show columns from {$table} where field = '{$field}'"));

    preg_match('/^enum\((.*)\)$/', $test[0]->Type, $matches);
    foreach (explode(',', $matches[1]) as $value) {
      $enum[] = trim($value, "'");
    }
    asort($enum);
    return $enum;
  }

  public static function customerSMS($sms, $dni = false, $name = false, $msisdn = false, $service = false, $expireService = false, $dateInstall = false) 
  {

    if ($dni || $name) {
      //busco el nombre por medio del dni
      if ($dni) {
        $client = DB::table('islim_clients')
          ->select('name')
          ->where('dni', $dni)
          ->first();
        if (!empty($client)) {
          $name = $client->name;
        } else {
          $name = '';
        }
      }
      if (!empty($name)) {
        $firstName = explode(" ", $name);
        $name = $firstName[0];
      }
      $sms = str_replace('#name#', $name, $sms);
    }
    if ($msisdn) {
      $sms = str_replace('#msisdn#', $msisdn, $sms);
    }
    if ($service) {
      //Con el id del servicio se sabe servicio y vencimiento
      $serviceDat = DB::table('islim_services')
        ->select('description', 'periodicity_id')
        ->where('id', $service)
        ->first();

      if (!empty($serviceDat)) {
        $serviceDescrip = $serviceDat->description;

        if (!$expireService) {
          $periodoDat = DB::table('islim_periodicities')
            ->select('days')
            ->where('id', $serviceDat->periodicity_id)
            ->first();

          if (!empty($periodoDat)) {
            $hoy = date('d-m-Y');
            $sum = $periodoDat->days + 1;
            $mod_date = strtotime($hoy . "+ " . $sum . " days");
            $expireService = date('d-m-Y', $mod_date);

          } else {
            $expireService = '';
          }
        }
        $sms = str_replace('#expiration#', $expireService, $sms);
      } else {
        $serviceDescrip = '';
      }
      $sms = str_replace('#service#', $serviceDescrip, $sms);
    }
    if ($dateInstall) {
      $sms = str_replace('#scheduling#', $dateInstall, $sms);
    }
    return $sms;
  }


   /*
  Retorna n digitos aleatorios
  @param Integer $n número de digitos a generar
  @param Integer $b rango inferior para la generacion de números aleatorios
  @param Integer $e rango superior para la generacion de números aleatorios
  @return Integer
   */
  public static function getRandDig($n = 0, $b = 1, $e = 9)
  {
    if ($n > 0) {
      mt_srand(time());
      $digits = '';
      for ($i = 0; $i < $n; $i++) {
        $digits .= mt_rand($b, $e);
      }
      return $digits;
    }
    return 0;
  }

  /**
  * [getRandText Genera una cadena de texto aleatoria]
  * @param  integer $limit [tamano de la cadena a generar]
  * @param  string  $type  [tipo de letras a usar]
  * @return [type]         [description]
  */
  public static function getRandText($limit = 1, $type = "upper")
  {
    if ($type == "upper") {
      $caracteres_permitidos = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    } elseif ($type == "lower") {
      $caracteres_permitidos = "abcdefghijklmnopqrstuvwxyz";
    } else {
      //mixto
      $caracteres_permitidos = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    }
    $longitud = $limit;
    $prefix = substr(str_shuffle($caracteres_permitidos), 0, $longitud);

    return $prefix;
  }

  /**
  * [getTokenConctact retorna una cadena con el formato: ABC123]
  * @param  integer $tam_L  [cantidad de letras]
  * @param  integer $tam_N  [cantidad de numeros]
  * @param  string  $type_L [Tipo de letras]
  * @return [type]          [description]
  */
  public static function getTokenVerify($tam_L = 0, $type_L = 'upper', $tam_N = 4)
  {
    $num = self::getRandDig($tam_N);
    $let = "";
    if ($tam_L > 0) {
      $let = self::getRandText($tam_L, $type_L);
    }
    return $let . $num;
  }

  public static function generateUID ($prefix, $full = false) {

    $baseId = null;

    if (!empty($prefix) && $full)
      $baseId = uniqid($prefix, true) . (bin2hex(openssl_random_pseudo_bytes(7)));
    else if (!empty($prefix) && !$full)
      $baseId = uniqid($prefix) . (bin2hex(openssl_random_pseudo_bytes(7)));
    else
      $baseId = uniqid() . (bin2hex(openssl_random_pseudo_bytes(7)));

    $sale = DB::table('islim_sales')
      ->select('unique_transaction')
      ->where('unique_transaction', $baseId)
      ->first();

      while (isset($sale) && is_object($sale) && isset($sale->unique_transaction) && (strpos($sale->unique_transaction, $baseId) !== false)) {

        if (!empty($prefix) && $full)
          $baseId = uniqid($prefix, true) . (bin2hex(openssl_random_pseudo_bytes(7)));
        else if (!empty($prefix) && !$full)
          $baseId = uniqid($prefix) . (bin2hex(openssl_random_pseudo_bytes(7)));
        else
          $baseId = uniqid() . (bin2hex(openssl_random_pseudo_bytes(7)));

        $sale = DB::table('islim_sales')
          ->select('unique_transaction')
          ->where('unique_transaction', $baseId)
          ->first();

      }

    return $baseId;

  }


  /**
  * [getDebtsSubordinate retorna el monto que debe un usuario subordinado (Vendedor o Instalador)]
  * @param  string $user  [email del usuario a consultar]
  * @param  date $date  [fecha hasta, fecha maxima para el calculo]
  * @return [float]     [monto de deuda]
  */

  public static function getDebtsSubordinate($user, $date){
    unset($filters);
    $filters['users_email'] = $user;
    $filters['status'] = ['V','I'];
    $filters['date_reg'] = $date;

    $cash_no_received=AssignedSales::getTotalCashMove($filters);
    $no_received_amount = $cash_no_received->sum('amount');

    $no_received = $cash_no_received->get();

    if($no_received){
        $assignedDetails = AssignedSaleDetails::getConnect('R')
          ->whereIn('asigned_sale_id',$no_received->pluck('id'))
          ->get();
    }

    //Ventas no conciliadas a la fecha (En espera de deposito) y que no se ha entregado el efectivo
    $sales = Sale::getConnect('R')
              ->whereIn('type',['V','P'])
              ->where('amount','>',0)
              ->where('users_email',$user)
              ->where('date_reg','<=',$date)
              ->where('status','=','E');

    if($assignedDetails){
      $sales = $sales->whereNotIn('unique_transaction',$assignedDetails->pluck('unique_transaction'));
    }

    $sales_amount = $sales->sum('amount');

    return $no_received_amount+$sales_amount;
  }

  public static function getNotificationSalesStatus($user){
    unset($filters);
    $filters['users_email'] = $user;

   // Obtener el estado de las ventas asignadas al usuario
   $sales_status = AssignedSales::getConnect('R')
   ->where('parent_email', $user)
   ->where('status', 'V')
   ->exists();

    return $sales_status;
  }

}
