<?php

namespace App\Http\Controllers;

//Otras clases
use App\Utilities\prova;
use App\Utilities\Altan;
use App\Utilities\Common;
use App\Utilities\Google;
use App\Utilities\voywey;
//use App\Utilities\CustomLogs;
use Illuminate\Http\Request;
use App\Utilities\NinetyNineMin;

//Modelos
use App\Models\Pack;
use App\Models\Client;
use App\Models\ApiKey;
use App\Models\TempCar;
use App\Models\LogApiV2;
use App\Models\Broadband;
use App\Models\PackPrices;
use App\Models\ArticlePack;
use App\Models\TempCarDetail;
use App\Models\TokenApiSeller;
use App\Models\ServiciabilityBuy;
use App\Models\ShipingMethodsConcentrator;
use App\Models\NinetyNineMinutes;
use App\Models\ProvaDelivery;
use App\Models\VoyweyDelivery;

use Log;

class BuyController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Consulta la servicialidad de una dirección y retorna los módem
     * y servicios con la informacion para la venta, primer paso para una compra HBB.
     *
     * @param Illuminate\Http\Request $request
     *
     * @return Laravel\Lumen\Http\ResponseFactory
     **/
    public function serviciability(Request $request)
    {
        //Iniciando log
        $log = LogApiV2::initLog($request);

        $response = [
            'success' => false,
            'data' => [
                'cod_err' => 'ERR_PARAM',
                'msg' => 'Faltan parámetros.',
            ],
        ];

        $inputs = $request->input();

        if (!empty($inputs['address']) || (!empty($inputs['lat']) && !empty($inputs['lng']))) {
            //Priorizando latitud y longitud enviada a dirección
            if (!empty($inputs['lat']) && !empty($inputs['lng'])) {
                $dataAddress['lat'] = $request->lat;
                $dataAddress['lng'] = $request->lng;
                $dataAddress['success'] = true;
            } else {
                //Consultando detalle de la direccion
                $dataAddress = Google::getAddressInfo($inputs['address'], $request, $log);
                //Obteniendo latitud y longitud de la direccion
                $dataAddress = Altan::getLatLngFromAddress($dataAddress, $log);
            }

            if ($dataAddress['success']) {
                //Obteniendo info del integrador
                $int = TokenApiSeller::getInfToken($request->bearerToken());

                //Obteniendo token de altan
                $tokeAlt = ApiKey::getTokenConc($int->concentrator_id);

                if (!empty($tokeAlt)) {
                    $typePack = empty($request->type) ? 'H' : $request->type;

                    //Consultando servicialidad
                    $serv = Altan::getServiceability($dataAddress['lat'], $dataAddress['lng'], $tokeAlt->api_key, $request, $log, $typePack);

                    if ($serv) {
                        //Consultando los artículos que se puede vender en la web
                        if ($int->packs == 'S') {
                            $packs = Pack::getArcticWeb($typePack, false, $int->coordination);
                        } else {
                            $packs = Pack::getArctic($typePack, false, $int->coordination);
                        }

                        //Consultando broadbands validos para el cliente
                        if ($typePack == 'H') {
                            $broads = Broadband::getBroadbands($serv);
                        }

                        if ($packs->count() && ($typePack == 'MH' || (!empty($broads) && count($broads)))) {
                            //Agrupando modems
                            $gm = $packs->groupBy('idArtic');
                            //Filtrando modems repetidos
                            $packs = $packs->unique('idArtic');

                            if ($typePack == 'H') {
                                $barr = $broads->pluck('broadband')->toArray();
                            }

                            //Consultando métodos de envío en caso de que venga el código postal
                            if (!empty($inputs['pc'])) {
                                $methodship = ShipingMethodsConcentrator::getMethods($int->concentrator_id);
                                
                                if ($methodship->count()) {
                                    //Validando inventario
                                    $nminM = $methodship->first(function ($value, $key) {
                                        return $value->code_name == '99_min_2';
                                    });

                                    if (!empty($nminM)) {
                                        $packs = NinetyNineMin::validateListInventory(
                                            $packs,
                                            $inputs['pc']
                                        );
                                    }

                                    $voywM = $methodship->first(function ($value, $key) {
                                        return $value->code_name == 'voywey';
                                    });

                                    if (!empty($voywM)) {
                                        $packs = voywey::validateListInventory(
                                            $inputs['pc'],
                                            $packs
                                        );
                                    }

                                    $packs = $packs->filter(function ($value) {
                                        return !empty($value->hasInventory) && $value->hasInventory;
                                    });
                                }
                            }

                            $is_concat = in_array($int->concentrator_id, explode(',', env('CONC_CONCAT', "")));

                            foreach ($packs as $pack) {
                                //Consultando servicios relacionados a los packs

                                if (!empty($pack->sku)) {

                                    $services = PackPrices::getServices(
                                        $gm[$pack->idArtic]->pluck('id')->toArray(),
                                        !empty($barr) ? $barr : []
                                    );

                                    if (count($services)) {
                                        //Armando array de servicios
                                        $servicesA = [];
                                        foreach ($services as $service) {
                                            $servicesA[] = [
                                                'id' => Common::getCleanBase64($service->packpriceId . '-' . $service->serviceId),
                                                'type' => 'normal',
                                                'name' => $service->title,
                                                'price' => $service->total_price,
                                                'description' => $service->description,
                                            ];
                                        }

                                        //No borrar esto, por ahora no se muestra la promo pero se puede volver a activar
                                        //Buscando servicios de la oferta
                                        /*$servicesOffert = PackPrices::getServicesOffer(
                                        [env('PROM_PACK_'.$pack->idArtic, 0)],
                                        $barr
                                        );

                                        foreach ($servicesOffert as $service) {
                                        $servicesA []= [
                                        'id' => Common::getCleanBase64($service->packpriceId.'-'.$service->serviceId),
                                        'type' => 'offert',
                                        'name' => $service->title,
                                        'price' => $service->total_price,
                                        'description' => $service->description
                                        ];
                                        }*/

                                        //Array de modems
                                        if ($is_concat === true) {
                                            $devices[] = [
                                                'id' => Common::getCleanBase64($pack->id . '-' . $pack->idArtic),
                                                'name' => $pack->title,
                                                'description' => $pack->description,
                                                'description_web' => substr($pack->desc_web, 0, strpos($pack->desc_web, ':')),
                                                'sku' => !empty($pack->sku) ? $pack->sku : '',
                                                'deliveries' => $pack->shipingMethods,
                                                'services' => $servicesA
                                            ];
                                        } else {
                                            $devices[] = [
                                                'id' => Common::getCleanBase64($pack->id . '-' . $pack->idArtic),
                                                'name' => $pack->title,
                                                'description' => $pack->description,
                                                'description_web' => substr($pack->desc_web, 0, strpos($pack->desc_web, ':')),
                                                'sku' => !empty($pack->sku) ? $pack->sku : '',
                                                'services' => $servicesA
                                            ];
                                        }
                                    }
                                }
                            }

                            if (!empty($devices) && count($devices)) {
                                $uniq = uniqid('api-store') . time();
                                $idserv = uniqid() . time();

                                //consultando direccion por latitud y longitud
                                if (empty($inputs['address'])) {
                                    $addressg = Google::getAddreddFromLatLng(
                                        $dataAddress['lat'],
                                        $dataAddress['lng'],
                                        $request,
                                        $log
                                    );

                                    if ($addressg['success']) {
                                        $addressg = $addressg['data'];
                                    } else {
                                        $addressg = 'S/D';
                                    }
                                }

                                //Creando session de compra
                                TempCar::insert([
                                    'transaction' => $uniq,
                                    'address_send' => !empty($inputs['address']) ? $inputs['address'] : $addressg,
                                    'key' => $request->bearerToken(),
                                    'status' => 'A',
                                    'date_reg' => date('Y-m-d H:i:s'),
                                    'date_update' => date('Y-m-d H:i:s'),
                                ]);

                                ServiciabilityBuy::insert([
                                    'transaction' => $uniq,
                                    'serviciability' => $serv,
                                    'lat' => $dataAddress['lat'],
                                    'lng' => $dataAddress['lng'],
                                    'id_serviciability' => $idserv,
                                    'address' => !empty($inputs['address']) ? $inputs['address'] : $addressg,
                                    'status' => 'A',
                                    'date_reg' => date('Y-m-d H:i:s'),
                                ]);

                                $response = [
                                    'success' => true,
                                    'data' => [
                                        'id' => $idserv,
                                        'transaction' => $uniq,
                                        'ttl_transaction' => env('TTL_TRANSACTION', 120),
                                        'date_transaction' => date('Y-m-d H:i:s'),
                                        'servicialidad' => ($typePack == 'H') ? $serv : 'N/A',
                                        'limit_device' => $int->max_artic,
                                        'devices' => $devices,
                                    ],
                                ];
                            } else {
                                $response['data']['cod_err'] = 'WT_SERV';
                                $response['data']['msg'] = 'Sin inventrio para los paquetes disponibles o no se encontraron servicios.';
                            }
                        } else {
                            $response['data']['cod_err'] = 'WT_PACK_BROAD';
                            $response['data']['msg'] = 'No se encontraron packs o broadbands.';
                        }
                    } else {
                        $response['data']['cod_err'] = 'WT_SERVICIABILITY';
                        $response['data']['msg'] = 'No hay cobertura para la dirección enviada.';
                    }
                } else {
                    $response['data']['cod_err'] = 'ERR_INT';
                    $response['data']['msg'] = 'Ocurrio un error al consultar servicialidad.';
                }
            } else {
                $response['data']['cod_err'] = 'ERR_ADDRESS';
                $response['data']['msg'] = 'No se pudo obtener latitud y longitud.';
            }
        }

        //Finalizando log
        LogApiV2::finishLog(
            $log,
            $response,
            0,
            $response['success'] ? 'D' : 'E',
            !$response['success'] ? $response['data']['msg'] : ''
        );

        return response()->json($response);
    }

    /**
     * Retorna los artículos de telefonía o mifi que hay para la venta y sus servicios
     * asociados, primer paso para una compra MBB o MIFI.
     *
     * @param Illuminate\Http\Request $request
     *
     * @return Laravel\Lumen\Http\ResponseFactory
     **/
    public function getArticles(Request $request)
    {
        //Iniciando log
        $log = LogApiV2::initLog($request);

        $response = [
            'success' => false,
            'data' => [
                'cod_err' => 'ERR_PARAM',
                'msg' => 'Faltan parámetros.'
            ],
        ];

        $inputs = $request->input();

        if (!empty($inputs['type']) && ($inputs['type'] == 'T' || $inputs['type'] == 'M')) {
            //Obteniendo info del integrador
            $int = TokenApiSeller::getInfToken($request->bearerToken());

            //Consultando los articulos que se puede vender en la web
            if ($int->packs == 'S') {
                $packs = Pack::getArcticWeb(
                    $inputs['type'],
                    //Quitar esta restriccion si se quire vender simcard
                    $inputs['type'] == 'T' ? env('SMARTCATID') : false,
                    $int->coordination
                );
            } else {
                $packs = Pack::getArctic(
                    $inputs['type'],
                    //Quitar esta restriccion si se quire vender simcard
                    $inputs['type'] == 'T' ? env('SMARTCATID') : false,
                    $int->coordination
                );
            }

            if ($packs->count()) {
                //Agrupando artículos
                $gm = $packs->groupBy('idArtic');
                //Filtrando artículos repetidos
                $packs = $packs->unique('idArtic');

                //Consultando métodos de envío en caso de que venga el código postal
                if (!empty($inputs['pc'])) {
                    $methodship = ShipingMethodsConcentrator::getMethods($int->concentrator_id);
                }

                $is_concat = in_array($int->concentrator_id, explode(',', env('CONC_CONCAT', "")));

                foreach ($packs as $pack) {
                    //Validando inventario
                    if (!empty($pack->sku)) {
                        if (!empty($inputs['pc']) && $methodship->count()) {
                            $invOk = false;

                            $voywM = $methodship->first(function ($value, $key) {
                                return $value->code_name == 'voywey';
                            });

                            if (!empty($voywM)) {
                                if (!empty($pack->sku)) {
                                    $vinvVW = voywey::validateInventory(
                                        $inputs['pc'],
                                        [$pack->sku],
                                        $log
                                    );

                                    if ($vinvVW) {
                                        $invOk = true;
                                    }
                                }
                            }

                            if (!$invOk) {
                                $nminM = $methodship->first(function ($value, $key) {
                                    return $value->code_name == '99_min_2';
                                });

                                if (!empty($nminM)) {
                                    $vinv99 = NinetyNineMin::validateInventory(
                                        [$pack->idArtic],
                                        $inputs['pc'],
                                        false,
                                        $log
                                    );

                                    if ($vinv99) {
                                        $invOk = true;
                                    }
                                }
                            }

                            if (!$invOk) {
                                $provaM = $methodship->first(function ($value, $key) {
                                    return $value->code_name == 'prova';
                                });

                                if (!empty($provaM)) {
                                    $prova = prova::getPrice($inputs['pc'], [$pack->sku]);

                                    if ($prova['success'] && !empty($prova['data']->prices)) {
                                        $vinvPro = prova::validateInventory(
                                            $prova['data']->warehouse,
                                            [$pack->sku],
                                            $log
                                        );

                                        if ($vinvPro) {
                                            $invOk = true;
                                        }
                                    }
                                }
                            }

                            if (!$invOk) {
                                continue;
                            }
                        }

                        //Consultando servicios relacionados a los packs
                        $services = PackPrices::getServices(
                            $gm[$pack->idArtic]->pluck('id')->toArray()
                        );

                        if (count($services)) {
                            //Armando array de servicios
                            $servicesA = [];
                            foreach ($services as $service) {
                                $servicesA[] = [
                                    'id' => Common::getCleanBase64($service->packpriceId . '-' . $service->serviceId),
                                    'type' => 'normal',
                                    'name' => $service->title,
                                    'price' => $service->total_price,
                                    'description' => $service->description
                                ];
                            }

                            //Array de artículos
                            if ($is_concat === true) {
                                $devices[] = [
                                    'id' => Common::getCleanBase64($pack->id . '-' . $pack->idArtic),
                                    'name' => $pack->title,
                                    'description' => $pack->description,
                                    'description_web' => substr($pack->desc_web, 0, strpos($pack->desc_web, ':')),
                                    'sku' => !empty($pack->sku) ? $pack->sku : '',
                                    'deliveries' => $pack->shipingMethods,
                                    'services' => $servicesA
                                ];
                            } else {
                                $devices[] = [
                                    'id' => Common::getCleanBase64($pack->id . '-' . $pack->idArtic),
                                    'name' => $pack->title,
                                    'description' => $pack->description,
                                    'description_web' => substr($pack->desc_web, 0, strpos($pack->desc_web, ':')),
                                    'sku' => !empty($pack->sku) ? $pack->sku : '',
                                    'services' => $servicesA
                                ];
                            }
                        }
                    }
                }

                if (!empty($devices) && count($devices)) {
                    $uniq = uniqid('api-store') . time();

                    //Creando session de compra
                    TempCar::insert([
                        'transaction' => $uniq,
                        'key' => $request->bearerToken(),
                        'status' => 'A',
                        'date_reg' => date('Y-m-d H:i:s'),
                        'date_update' => date('Y-m-d H:i:s')
                    ]);

                    $response = [
                        'success' => true,
                        'data' => [
                            'transaction' => $uniq,
                            'ttl_transaction' => env('TTL_TRANSACTION', 120),
                            'date_transaction' => date('Y-m-d H:i:s'),
                            'limit_device' => $int->max_artic,
                            'devices' => $devices
                        ],
                    ];
                } else {
                    $response['data']['cod_err'] = 'WT_SERV';
                    $response['data']['msg'] = 'Sin inventrio para los paquetes disponibles o no se encontraron servicios.';
                }
            } else {
                $response['data']['cod_err'] = 'WT_PACK';
                $response['data']['msg'] = 'No se encontraron packs.';
            }
        }

        //Finalizando log
        LogApiV2::finishLog(
            $log,
            $response,
            0,
            $response['success'] ? 'D' : 'E',
            !$response['success'] ? $response['data']['msg'] : ''
        );

        return response()->json($response);
    }

    /*
  Registra el dispositivo seleccionado por el cliente y retorna los metodos de envío.

  @param Illuminate\Http\Request $request
  @return Object
   */
    public function selectDevice(Request $request)
    {
        //Iniciando log
        $log = LogApiV2::initLog($request);

        $response = [
            'success' => false,
            'data' => [
                'cod_err' => 'ERR_PARAM',
                'msg' => 'Faltan parámetros.',
            ]
        ];

        $inputs = $request->input();

        if (
            !empty($inputs['devices_selected']) &&
            is_array($inputs['devices_selected']) &&
            count($inputs['devices_selected']) &&
            !empty($inputs['transaction'])
        ) {

            //Obteniendo info del integrador
            $int = TokenApiSeller::getInfToken($request->bearerToken());

            //Validando limite de compras
            if (count($inputs['devices_selected']) <= $int->max_artic) {
                //Verificando si la transaccion es válida
                $transaction = TempCar::getTransaction(
                    $inputs['transaction'],
                    $request->bearerToken()
                );

                if (!empty($transaction)) {
                    //Marcando como eliminados todos los elementos relacionados a la transaccion
                    TempCarDetail::deleteItemTransaction($inputs['transaction']);

                    foreach ($inputs['devices_selected'] as $deviceSel) {
                        if (!empty($deviceSel['id_device']) && !empty($deviceSel['id_service'])) {
                            //Validando articulo la posicion 0 tiene el id del pack, la posicion 1 tiene el id del articulo
                            $deviceA = explode('-', base64_decode($deviceSel['id_device']));

                            if (count($deviceA) == 2) {
                                $device = ArticlePack::getDevice($deviceA[0], $deviceA[1]);

                                if (!empty($device)) {
                                    //Si es HBB se hacen validaciones extras
                                    if ($device->pack_type == 'H' || $device->pack_type == 'MH') {
                                        //Validando que venga el id de servicialidad
                                        if (empty($deviceSel['serviciability_id'])) {
                                            $response['data']['cod_err'] = 'ERR_SERV';
                                            $response['data']['msg'] = 'Servicialidad no enviada.';
                                            break;
                                        }
                                        //Consultadno servicialidad
                                        $servReg = ServiciabilityBuy::getAddressServicibility(
                                            $inputs['transaction'],
                                            $deviceSel['serviciability_id']
                                        );

                                        if (empty($servReg)) {
                                            $response['data']['cod_err'] = 'ADDR_NOVALID';
                                            $response['data']['msg'] = 'Servicialidad no válida.';
                                            break;
                                        }
                                    }

                                    //Validando servicio la posicion 0 tiene el id del price_pack, la posicion 1 tiene el id del servicio
                                    $serviceA = explode('-', base64_decode($deviceSel['id_service']));

                                    if (count($serviceA) == 2) {
                                        $service = PackPrices::getService($serviceA[0], $serviceA[1]);

                                        if (!empty($service)) {
                                            //Verificando si el servicio es del pack de oferta (No borrar)
                                            //$type = (!empty(env('PROM_PACK_' . $deviceA[1])) && env('PROM_PACK_' . $deviceA[1]) == $serviceA[0]) ? 'offer' : 'normal';

                                            $devicesSelected[] = [
                                                'name' => $device->title,
                                                'description' => $device->description,
                                                'description_web' => !empty($device->desc_web) ? $device->desc_web : '',
                                                'service' => $service->title,
                                                'gb' => $service->gb,
                                                'service_desc' => $service->description,
                                                'price' => $service->total_price,
                                                'type' => 'normal' //$type,
                                            ];

                                            $artics[$deviceA[1]] = empty($artics[$deviceA[1]]) ? 1 : $artics[$deviceA[1]] + 1;
                                            $skus[] = ['sku' => $device->sku];
                                            $countArt[$device->sku] = empty($countArt[$device->sku]) ? 1 : $countArt[$device->sku] + 1;

                                            //Guardando en detalle temporal
                                            TempCarDetail::insert([
                                                'service_address' => !empty($servReg) ? $servReg->address : null,
                                                'serviciability_id' => !empty($deviceSel['serviciability_id']) ? $deviceSel['serviciability_id'] : null,
                                                'id_pack' => $deviceA[0],
                                                'id_artic' => $deviceA[1],
                                                'id_pack_price' => $serviceA[0],
                                                'id_service' => $serviceA[1],
                                                'transaction' => $inputs['transaction'],
                                                'price_pack' => $service->total_price,
                                                'date_reg' => date('Y-m-d H:i:s'),
                                                'date_update' => date('Y-m-d H:i:s'),
                                                'status' => 'A',
                                            ]);
                                        } else {
                                            $response['data']['cod_err'] = 'SE_NOTVALID';
                                            $response['data']['msg'] = 'Servicio no válido.';
                                            break;
                                        }
                                    } else {
                                        $response['data']['cod_err'] = 'SE_NOTVALID';
                                        $response['data']['msg'] = 'Servicio no válido.';
                                        break;
                                    }
                                } else {
                                    $response['data']['cod_err'] = 'DE_NOTVALID';
                                    $response['data']['msg'] = 'Artículo no válido.';
                                    break;
                                }
                            } else {
                                $response['data']['cod_err'] = 'DE_NOTVALID';
                                $response['data']['msg'] = 'Artículo no válido.';
                                break;
                            }
                        } else {
                            $response['data']['cod_err'] = 'ERR_PARAM';
                            $response['data']['msg'] = 'Faltan parámetros.';
                            break;
                        }
                    }

                    //Validando que se procesaran todos los artículos enviados
                    if (
                        !empty($devicesSelected) &&
                        count($devicesSelected) == count($inputs['devices_selected'])
                    ) {

                        $response = [
                            'success' => true,
                            'data' => [
                                'transaction' => $inputs['transaction'],
                                'date_transaction' => date('Y-m-d H:i:s'),
                                'selected_device' => $devicesSelected,
                            ],
                        ];
                    }
                } else {
                    $response['data']['cod_err'] = 'TR_NOTVALID';
                    $response['data']['msg'] = 'Transacción no válida.';
                }
            } else {
                $response['data']['cod_err'] = 'LIMIT_EXCEEDED';
                $response['data']['msg'] = 'Límite de artículos excedido.';
            }
        }

        //Finalizando log
        LogApiV2::finishLog(
            $log,
            $response,
            0,
            $response['success'] ? 'D' : 'E',
            !$response['success'] ? $response['data']['msg'] : ''
        );

        return response()->json($response);
    }

    /*
  Asigna un cliente a una compra
   */
    public function assignClient(Request $request)
    {
        //Iniciando log
        $log = LogApiV2::initLog($request);

        $response = [
            'success' => false,
            'data' => ['cod_err' => 'ERR_PARAM', 'msg' => 'Faltan parámetros.'],
        ];

        $inputs = $request->input();

        if (!empty($inputs['transaction']) && !empty($inputs['client_dni'])) {
            //Verificando si la transaccion es válida
            $transaction = TempCar::getTransaction(
                $inputs['transaction'],
                $request->bearerToken(),
                'W'
            );

            if (!empty($transaction)) {
                $client = Client::getClientByDni($inputs['client_dni']);

                if (!empty($client)) {
                    $transaction->client_dni = $client->dni;
                    $transaction->save();

                    $response = [
                        'success' => true,
                        'data' => [
                            'transaction' => $inputs['transaction'],
                            'dni' => $client->dni,
                            'client' => $client->name . ' ' . $client->last_name,
                            'phone' => !empty($client->phone_home) ? $client->phone_home : 'N/A',
                            'email' => !empty($client->email) ? $client->email : 'N/A',
                        ],
                    ];
                }
            } else {
                $response['data']['cod_err'] = 'TR_NOTVALID';
                $response['data']['msg'] = 'Transacción no válida.';
            }
        }

        //Finalizando log
        LogApiV2::finishLog(
            $log,
            $response,
            0,
            $response['success'] ? 'D' : 'E',
            !$response['success'] ? $response['data']['msg'] : ''
        );

        return response()->json($response);
    }

    /*
    Retorna status de una orden
  */
    public function getOrderStatus(Request $request)
    {
        //Iniciando log
        $log = LogApiV2::initLog($request);

        $response = [
            'success' => false,
            'data' => ['cod_err' => 'ERR_PARAM', 'msg' => 'Faltan parámetros.'],
        ];

        $inputs = $request->input();

        if (!empty($inputs['transaction'])) {
            $car = TempCar::getTransactionAP(
                $inputs['transaction'],
                $request->bearerToken()
            );

            if (!empty($car) && ($car->status == 'A' || $car->status == 'P')) {
                if (!empty($car->client_dni)) {
                    $client = Client::getClientByDni($car->client_dni);
                }

                $response = [
                    'success' => true,
                    'data' => [
                        'transaction' => $inputs['transaction'],
                        'client-data' => [
                            'client' => !empty($client) ? ($client->name . ' ' . $client->last_name) : 'S/I',
                            'phone' => (!empty($client) && !empty($client->phone_home)) ? $client->phone_home : 'S/I',
                            'email' => (!empty($client) && !empty($client->phone_home)) ? $client->phone_home : 'S/I'
                        ],
                        'date_reg' => $car->date_reg
                    ]
                ];

                if ($car->status == 'A') {
                    $response['data']['status'] = 'created';
                } else {
                    $response['data']['status'] = 'pay';

                    $nine = NinetyNineMinutes::getActOrSendDelivery($car->id);
                    $prova = ProvaDelivery::getActOrSendDelivery($car->id);
                    $voywey = VoyweyDelivery::getActOrSendDelivery($car->id);

                    if (!empty($nine)) {
                        $prov = '99min';
                        $folio = !empty($nine->order99) ? $nine->order99 : 'S/I';
                    }
                    if (!empty($prova)) {
                        $prov = 'prova';
                        if (!empty($prova->courier_g)) {
                            $prov = $prova->courier_g;
                        }
                        $folio = !empty($prova->folio) ? $prova->folio : 'S/I';
                    }
                    if (!empty($voywey)) {
                        $prov = 'voywey';
                        if (!empty($voywey->courier_g)) {
                            $prov = $voywey->courier_g;
                        }
                        $folio = !empty($voywey->folio) ? $voywey->folio : 'S/I';
                    }

                    $response['data']['data-shipping'] = [
                        'currier' => !empty($prov) ? $prov : 'S/I',
                        'folio' => !empty($folio) ? $folio : 'S/I'
                    ];
                }
            } else {
                $response['data']['cod_err'] = 'TR_NOTVALID';
                $response['data']['msg'] = 'Transacción no válida.';
            }
        }

        //Finalizando log
        LogApiV2::finishLog(
            $log,
            $response,
            0,
            $response['success'] ? 'D' : 'E',
            !$response['success'] ? $response['data']['msg'] : ''
        );

        return response()->json($response);
    }
}
