<?php

namespace App\Http\Controllers\Fiber;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Mail\Mail_tycfQr;
use App\Mail\Mail_formCircle;
use App\Mail\mailInstall;
use App\Mail\mailContract;

use App\Models\HistoryMailFiber;
use App\Models\FiberZone;
use App\Models\User;
use App\Models\Installations;
use App\Models\FiberHoliday;
use App\Models\Pack;
use App\Models\Bundle;
use App\Models\ArticlePack;
use App\Models\PackPrices;
use App\Models\Client;
use App\Models\ClientNetwey;
use App\Utilities\Common;
use App\Models\CircleCreditRequest;
use App\Models\CircleCredit;
use App\Models\Sms_notification_token;
use App\Models\Verify_contact_client;
use App\Models\FiberQuestionResult;
use App\Models\FiberPaymentForce;
use App\Models\InstallationsBundle;
use App\Models\FiberQuestions;
use App\Models\FiberControl;
use App\Models\HistorySmsFiber;
use App\Utilities\FiberTrait;
use App\Models\Sms_notification;
use App\Models\Client_document;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Mail;
use Log;
use Carbon\Carbon;
use Exception;
use DB;



class ScheduleController extends Controller
{
    use FiberTrait;

    public function getCalendar(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'id_olt' => 'required',
            'id_city' => 'required',
            'operation' => 'required',
        ]);

        if ($validator->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validator->errors()
            ];
            return Common::baseResponse(false, "Faltan datos para procesar la lista de planes", $data, 400);
        }


        $response = ['sucess' => false, 'msg' => "Faltan datos para procesar la solicitud"];

        $idZone = $request->id_olt;
        $operation = $request->operation;
        //dd($idZone);
        // $city = $request->city; //Momentaneamente no es requerido

        //reviso que la zona este disponible
        $infoZone = FiberZone::getInfoZone($idZone);
        if (!empty($infoZone)) {

            //Reviso cuantos instaladores posee la zona activos
            $isInstall = User::getInstallerZone($idZone);
            $cantInstaller = count($isInstall);

            if ($cantInstaller > 0) {
                //Reviso la capacidad de instalacion de la zona
                $configuration = json_decode(json_encode($infoZone->configuration));

                if (!isset($configuration->capacity_installer)) {
                    //Si no esta definido en el json por defecto es 1
                    $configuration->capacity_installer = 1;
                    //Capacidad por instalador por Turno
                }
                //Capacidad turno
                //$configuration->capacity_installer *= 2;
                $capacityTurno = $configuration->capacity_installer;

                //Capacidad de instalacion en la zona
                //$installZone = $cantInstaller * $configuration->capacity_installer;

                //Reviso cuantas citas de instalacion hay confirmadas en la zona
                $listPending = Installations::getInstallerPending($idZone);
                //dd($listPending);
                $BlockDay = [];

                $BlockDay = FiberHoliday::getDayFeriado($BlockDay, 'full');

                $BlockMediaDay = [];
                $BlockMediaDay = FiberHoliday::getDayFeriado($BlockMediaDay, 'media');

                foreach ($listPending as $agendaObject) {
                    $agenda = json_decode(json_encode($agendaObject));
                    if (!empty($BlockMediaDay)) {
                        foreach ($BlockMediaDay as $itemMedia) {
                            if (strtotime($itemMedia->fecha) == strtotime($agenda->date_instalation)) {
                                //Capacidad de medio dia
                                $installZone = $capacityTurno * $cantInstaller;
                                break;
                            } else {
                                //capacidad dia completo
                                $installZone = $capacityTurno * $cantInstaller * 2;
                            }
                        }
                    } else {
                        //capacidad dia completo
                        $installZone = $capacityTurno * $cantInstaller * 2;
                    }

                    if ($agenda->cant_installer >= $installZone) {
                        //$fecha = explode(" ", $agenda->date_instalation);
                        $festive = new \stdClass;
                        /*Se debe quitar el cero a la izquierda de la fecha para que el frontend lo tome*/
                        $DayBlock = strtotime($agenda->date_instalation);
                        $anio = date("Y", $DayBlock);
                        $mes = date("m", $DayBlock);
                        $dia = date("d", $DayBlock);
                        $DayBlock = intval($dia) . '-' . intval($mes) . '-' . intval($anio);

                        $festive->fecha = $DayBlock;
                        $festive->type = 'full';
                        array_push($BlockDay, $festive);
                    }
                }

                //Se toma agenda para el dia siguiente
                $starDate = date("d-m-Y", strtotime("+ 0 days", time()));

                $endDate = date("d-m-Y", strtotime("+ 8 month", time()));

                $respal = $BlockDay;
                $BlockDay = [];
                if (count($respal)) {
                    foreach ($respal as $item) {
                        array_push($BlockDay, $item->fecha);
                    }
                }
                $respal = null;

                $data = [
                    "calendar" => [
                        'block_day' => $BlockDay,
                        'star_date' => $starDate,
                        'end_date' => $endDate,
                        'operation' => $operation
                    ]
                ];

                return Common::baseResponse(true, "Calendario disponible", $data, 200);
            } else {
                $data = [
                    "error_code" => "no_installer",
                    "errors" => [
                        "no_installer" => ["No hay personal minimo en este momento que procese la solicitud"]
                    ]
                ];

                return Common::baseResponse(false, "No hay personal minimo en este momento que procese la solicitud", $data, 400);
            }
        } else {
            $data = [
                "error_code" => "zone_not_found",
                "errors" => [
                    "zone_not_found" => ["La zona no esta disponible para mostrar calendario"]
                ]
            ];

            return Common::baseResponse(false, "La zona no esta disponible para mostrar calendario", $data, 400);
        }
    }


    public function getClock(Request $request, $interno = false)
    {
        $validator = Validator::make($request->all(), [
            'date' => 'required|date_format:Y-m-d',
            'id_olt' => 'required',
            'id_city' => 'required',
            'operation' => 'required',
        ]);

        $response = ['sucess' => false, 'msg' => "Faltan datos para procesar la solicitud"];

        if ($validator->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validator->errors()
            ];
            return Common::baseResponse(false, "Faltan datos para procesar la lista de planes", $data, 400);
        }


        $dateSearch = $request->date;
        $idZone = $request->id_olt;
        $operation = $request->operation;
        // $city = $request->city; //Momentaneamente no es requerido
        //
        $infoZone = FiberZone::getInfoZone($idZone);
        if (!empty($infoZone)) {

            $fecha_actual = strtotime(date("Y-m-d"));
            $fecha_entrada = strtotime($dateSearch);

            if ($fecha_actual <= $fecha_entrada) {
                $listPendingDay = Installations::getInstallerPendingDay($idZone, $dateSearch);

                //Obtengo capacidad de instalacion de la zona
                $configuration = json_decode(json_encode($infoZone->configuration));

                if (!isset($configuration->capacity_installer)) {
                    //Si no esta definido en el json por defecto es 1
                    $configuration->capacity_installer = 1;
                }

                //Obtengo cuantos instaladores posee la zona activos
                $isInstall = User::getInstallerZone($idZone);
                $cantInstaller = count($isInstall);

                //Capacidad de instalacion en la zona por turno
                $installZoneTurno = $cantInstaller * $configuration->capacity_installer;

                //Lista de turnos
                $listTurno = array(
                    array("hour" => "09:00 - 13:00", "habilite" => 'Y'),
                    array("hour" => "14:00 - 18:00", "habilite" => 'Y'),
                );

                $BlockMediaDay = [];
                $BlockMediaDay = FiberHoliday::getDayFeriado($BlockMediaDay, 'media');

                if (!empty($BlockMediaDay)) {
                    foreach ($BlockMediaDay as $itemMedia) {
                        if (strtotime($itemMedia->fecha) == strtotime($request->date)) {
                            //Capacidad de la tarde no disponible por feriado
                            $listTurno[1]['habilite'] = 'N';
                            break;
                        }
                    }
                }
                $cantHours = count($listTurno);
                $cantHourFull = 0;

                foreach ($listTurno as &$turno) {
                    foreach ($listPendingDay as $PendingDayObjet) {
                        $PendingDay = json_decode(json_encode($PendingDayObjet));
                        if ($turno['hour'] == $PendingDay->schedule) {
                            if ($PendingDay->cant_installer >= $installZoneTurno) {
                                $turno['habilite'] = 'N';
                                $cantHourFull++;
                            }
                        }
                    }
                }
                $availableTime = true;
                if ($cantHours == $cantHourFull) {
                    $availableTime = false;
                }

                if ($dateSearch == date("Y-m-d") && $availableTime) {
                    //Se esta capturando la cita para el dia en curso
                    $hora = intval(date("H"));
                    if ($hora >= 12) {
                        //Cita pedida en la tarde queda para agendar manana
                        $availableTime = false;
                    } else {
                        //Cita pedida en la manana se puede maximo en la tarde
                        $listTurno[0]['habilite'] = 'N';
                    }
                }
                if (!$interno) {

                    $data = [
                        "clock" => [
                            "list_turn" => $listTurno,
                            "available_time" => $availableTime,
                            "operation" => $operation
                        ]
                    ];

                    return Common::baseResponse(true, "Horario disponible", $data, 200);
                } else {
                    //Se usa para validar el dia y el turno antes de registrar en BD
                    $data = [
                        "clock" => [
                            "list_turn" => $listTurno,
                            "available_time" => $availableTime,
                            "operation" => $operation
                        ]
                    ];

                    return Common::baseResponse(true, "Horario disponible", $data, 200);
                }
            } else {

                $data = [
                    "error_code" => "date_error",
                    "errors" => [
                        "date_error" => ["La fecha a verificar debe ser a futuro, por tanto no es valida para consultar"]
                    ]
                ];

                return Common::baseResponse(false, "La fecha a verificar debe ser a futuro, por tanto no es valida para consultar", $data, 400);
            }
        } else {

            $data = [
                "error_code" => "zone_not_found",
                "errors" => [
                    "zone_not_found" => ["La zona no esta disponible para mostrar horario"]
                ]
            ];

            return Common::baseResponse(false, "La zona no esta disponible para mostrar horario", $data, 400);
        }
    }







    public function upPhotoEvidency(Request $request)
    {
        $iserror = false;
        if (isset($request->type) && !empty($request->type)) {
            switch ($request->type) {
                case 'I':
                    $path = 'fiber-install/photos/';
                    $label = 'instalacion';
                    break;
                case 'S':
                    $path = 'fiber-support/photos/';
                    $label = 'soporte';
                    break;
                case 'C':
                    $path = 'fiber-contract/photos/';
                    $label = 'contrato';
                    break;
                default:
                    $iserror = true;
            }
        } else {
            $iserror = true;
        }
        if ($iserror == true) {
            $response = ['success' => false, 'title' => "Faltan datos", 'msg' => "type no existe o es invalido", 'icon' => "error", 'code' => 'ERR_DAT'];

            return Common::baseResponse(false, $response['msg'], [
                "error_code" => "ERR_DAT",
                "errors" => [
                    "ERR_DAT" => [$response['msg']]
                ]
            ], 400);
        }

        $response = ['success' => false, 'title' => "No se pudo registrar en S3 la fotografia de " . $label . " de fibra", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        $messages = [
            'id.required' => 'identificador de ' . $label . ' es requerido (GLS6575)',
            'name.required' => 'nombre de archivo es requerido (GLS6576)',
            'file.required' => 'archivo es requerido (GLS6577)',
            'type.required' => 'Tipo es requerido (GLS67185)',
        ];
        $requisitos = [
            'id' => array(
                'required',
            ),
            'name' => array(
                'required',
            ),
            'file' => array(
                'required',
            ),
            'type' => array(
                'required',
            ),
        ];
        $validate = Validator::make($request->all(), $requisitos, $messages);
        $errors = $validate->errors();

        if ($errors->any()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        } else {
            $id = $request->id;
            $nameEvidency = $request->name;
            try {
                //$path = 'fiber-install/photos/';
                $fileInput = $request->file('file');
                $filePath = $path . $id . "/" . $nameEvidency . "_" . uniqid() . time() . '.' . $fileInput->getClientOriginalExtension();

                Storage::disk('s3-fiber-contract')->put(
                    $filePath,
                    file_get_contents($fileInput->getPathname()),
                    'public'
                );

                $url = Storage::disk('s3-fiber-contract')->url($filePath);

                $response = ['success' => true, 'title' => "Evidencia cargada al S3 correctamente", 'url' => $url];

                return Common::baseResponse(true, $response['title'], ["url" => $url], 200);
            } catch (Exception $e) {
                $txmsg = "No se pudo registrar en S3 la evidencia fotografica de " . $nameEvidency . " adjuntada para " . $label . " de fibra: " . $id . " - " . (string) json_encode($e->getMessage());
                Log::error($txmsg);
                $response['msg'] = $txmsg;
            }
        }


        Log::error($response['title'] . " - " . $response['msg']);

        return Common::baseResponse(false, $response['title'], [
            "error_code" => "ERR_S3",
            "errors" => [
                "ERR_S3" => [$response['msg']]
            ]
        ], 400);
    }


    public function getQrForce(Request $request)
    {


        $response = ['success' => false, 'html' => '', 'htmlShare' => '', 'msg' => 'Hubo un error en procesar la generación de contrato de fibra', 'icon' => 'warning', 'code' => 'FAIL'];

        $messages = [
            'dni.required' => 'El identificador del cliente es requerida (IVF3640)',
            'pack.required' => 'El identificador del paquete es requerida (IVF3641)',
            'isBundle.required' => 'El identificador del tipo de paquete es requerida (IVF3642)',
            'date_instalation.required' => 'La fecha de instalación es requerida (IVF3643)',
            'schedule.required' => 'El turno de instalación es requerida (IVF3644)',
            'identity.required' => 'El numero o codigo de identidad del cliente es requerida (IVF3645)',
            'typeIdentity.required' => 'El tipo de identificación del cliente es requerida (IVF3646)',
            'photo_identiF.required' => 'La fotografia frontal del documento de identidad del cliente es requerida (IVF3647)',
            'photo_identiP.required' => 'La fotografia posterior del documento de  identidad del cliente es requerida (IVF3648)'
        ];

        $requisitos = [
            'dni' => array(
                'required',
            ),
            'pack' => array(
                'required',
            ),
            'isBundle' => array(
                'required',
            ),
            'date_instalation' => array(
                'required',
            ),
            'schedule' => array(
                'required',
            ),
            'identity' => array(
                'required',
            ),
            'typeIdentity' => array(
                'required',
            ),
            'photo_identiF' => array(
                'required',
            ),
            'photo_identiP' => array(
                'required',
            )
        ];

        $validate = Validator::make($request->all(), $requisitos, $messages);
        $errors = $validate->errors();

        if ($errors->any()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        }

        /* $this->validate($request, [
        'dni' => 'required',
        'pack' => 'required',
        'isBundle' => 'required',
        'date_instalation' => 'required',
        'schedule' => 'required',
        'identity' => 'required',
        'typeIdentity' => 'required',
        'photo_identiF' => 'required|image',
        'photo_identiP' => 'required|image',
        ]);*/

        $info_client = Client::getClientByDNI($request->dni);

        if (empty($info_client)) {

            return Common::baseResponse(false, "No se pudo encontrar el registro del prospecto", [
                "error_code" => "EMP_CLI",
                "errors" => [
                    "EMP_CLI" => ["No se pudo encontrar el registro del prospecto"]
                ]
            ], 400);
        }

        $request->pack = intval(Common::decodificarBase64($request->pack));
        $request->isBundle = Common::decodificarBase64($request->isBundle);

        $request->pack_bundle = null;
        if ($request->isBundle == 'Y') {
            //Buscamos el id del pack de fibra
            $confBundle = Bundle::getDetailBundleAlta($request->pack);
            $request->pack_bundle = $request->pack;
            if ($confBundle['success']) {
                $confBundle = json_decode(json_encode($confBundle['data']));
                if (isset($confBundle->info_F->id)) {
                    $request->pack = $confBundle->info_F->id;
                } else {


                    return Common::baseResponse(false, "Se esta tratando de generar un contrato para un pack de tipo bundle pero no se pudo obtener el id del servicio de fibra (CF3785)", [
                        "error_code" => "EMP_PAK",
                        "errors" => [
                            "EMP_PAK" => ["Se esta tratando de generar un contrato para un pack de tipo bundle pero no se pudo obtener el id del servicio de fibra (CF3785)"]
                        ]
                    ], 400);
                }
            } else {


                return Common::baseResponse(false, "Se esta tratando de generar un contrato para un pack de tipo bundle pero no se pudo obtener detalles del servicio de fibra (CF3788)", [
                    "error_code" => "EMP_BUN",
                    "errors" => [
                        "EMP_BUN" => ["Se esta tratando de generar un contrato para un pack de tipo bundle pero no se pudo obtener detalles del servicio de fibra (CF3788)"]
                    ]
                ], 400);
            }
        }
        $request->schedule = Common::decodificarBase64($request->schedule);
        /*
        $listPhoto = array('photo_identiF', 'photo_identiP');
        $path = 'installations/identity/';
        $urlPhotoF = '';
        $urlPhotoP = '';

        foreach ($listPhoto as $photo) {

        $photoIdent = $request->file($photo);
        $photoPath = $path . uniqid() . time() . '.' . $photoIdent->getClientOriginalExtension();

        Storage::disk('s3')->put(
        $photoPath,
        file_get_contents($photoIdent->getPathname()),
        'public'
        );

        if ($photo == "photo_identiF") {
        $urlPhotoF = (String) Storage::disk('s3')->url($photoPath);
        } elseif ($photo == "photo_identiP") {
        $urlPhotoP = (String) Storage::disk('s3')->url($photoPath);
        }
        }
        */
        $urlPhotoF = $request->photo_identiF;
        $urlPhotoP = $request->photo_identiP;

        if (!empty($urlPhotoF) && !empty($urlPhotoP)) {
            //Se registran los documentos del cliente
            $creDoc = Client_document::createRegister($request->dni, $request->identity, $urlPhotoF, $urlPhotoP, $request->typeIdentity);
            if (!$creDoc['success']) {



                return Common::baseResponse(false, "Los documentos de identificacion no se pudieron procesar", [
                    "error_code" => "EMP_PHO",
                    "errors" => [
                        "EMP_PHO" => ["Los documentos de identificacion no se pudieron procesar"]
                    ]
                ], 400);
            }
        } else {

            return Common::baseResponse(false, "Los documentos de identificacion no se pudieron procesar", [
                "error_code" => "EMP_PHO",
                "errors" => [
                    "EMP_PHO" => ["Los documentos de identificacion no se pudieron procesar"]
                ]
            ], 400);
        }

        /* $contract_url = FiberPaymentForce::generateContract($request);
        if (!$contract_url['success']) {
        return response()->json(['success' => false, 'html' => '', 'htmlShare' => '', 'msg' => $contract_url['msg'], 'icon' => 'warning', 'code' => 'FAIL_URL']);
        } else {
        $contract_url = $contract_url['url'];
        }*/

        $infoQr = FiberPaymentForce::newUrlQr($request->dni, $request->pack, $request->date_instalation, $request->schedule, 'START', $request, $request->pack_bundle);

        if (!$infoQr['success']) {

            return Common::baseResponse(false, $infoQr['msg'], [
                "error_code" => "EMP_QR",
                "errors" => [
                    "EMP_QR" => [$infoQr['msg']]
                ]
            ], 400);
        }

        $urlQr = env('SITE_WEB_NETWEY') . 'tycf/' . $infoQr['url'];

        //Segun el tipo de verificacion que tenga el celular de contacto, o se envia el SMS o se renderiza en pantalla
        //
        $codePhoneContact = "NOT_VERIFIED";
        $canalDecision = "";

        if (!empty($info_client->verify_phone_id)) {
            $infoVerify = Verify_contact_client::getRegisterVerify($info_client->verify_phone_id);
            if (!empty($infoVerify)) {
                $canalDecision = $infoVerify->platform;
                $codePhoneContact = $infoVerify->status;
                if ($canalDecision == "MAIL" && !empty($info_client->email)) {
                    $canalDecision = "MAIL";
                } else {
                    $canalDecision = "SMS";
                }
            } else {
                $response['msg'] = "El registro de verificación " . $infoClient->verify_phone_id . " no existe";
            }
        }
        if ($codePhoneContact == "VERIFIED") {
            //Envio el mensaje con el url de contrato
            //NOTA: el sms_type debe ser 'G' y el mensaje debe ir en sms_attribute y en sms(En ambos el mismo contenido)
            //
            if (empty($info_client->phone_home)) {

                return Common::baseResponse(false, "El cliente no posee telefono de contacto registrado", [
                    "error_code" => "EMP_PHO",
                    "errors" => [
                        "EMP_PHO" => ["El cliente no posee telefono de contacto registrado"]
                    ]
                ], 400);
            }

            //Obtengo el nombre para personalizar el mensaje
            $userText = "";
            if (!empty($info_client->name)) {
                $nombre = explode(" ", $info_client->name)[0];
                $userText = "Sr(a) " . $nombre . ", ";
            }
            $msgContract = $userText . "Su contrato de adhesion Netwey: " . $urlQr;

            //Solo llegara SMS o MAIL cuando esta verificado el cliente

            if ($canalDecision == "SMS") {

                try {
                    $smsSend = Sms_notification_token::Send_sms(
                        $info_client->phone_home,
                        $info_client->phone_home,
                        'G',
                        '1',
                        "Contrato de adhesion",
                        $msgContract,
                        $msgContract
                    );

                    if ($smsSend) {
                        $dataH = [
                            'sms_nip_id' => $smsSend,
                            'dni_client' => $request->dni,
                            'process' => "CONTRACT",
                            'phone_sms' => $info_client->phone_home
                        ];
                        HistorySmsFiber::newHistorySMSFiber($dataH);
                    }
                } catch (Exception $e) {
                    Log::error('No se pudo enviar el msj con el contrato de adhesión al telefono del cliente: ' . $info_client->phone_home . ' Error: ' . (string) json_encode($e->getMessage()));
                } 

                $icono = Sms_notification_token::getIcon('SEND_SMS');

                return Common::baseResponse(true, "Se envio un SMS con la información del contrato de adhesión al telefono: " . $info_client->phone_home . " del cliente el cual fue verificado, al recibir el SMS debe leer y aprobar para proseguir con el proceso de venta de fibra", [
                    "tyc" => $infoQr['id'],
                    "url" => $urlQr,
                    "share" => false
                ], 200);
            } else {
                //EMAIL con el contrato de adhesion
                //
                //Entrada: dni,type(En base64)
                $request->type = Common::codificarBase64("START");

                $sendMailTyc = self::sendMailQr($request);
                $sendMailTyc = json_decode(json_encode($sendMailTyc));
                if ($sendMailTyc->original->status) {
                    $icono = Sms_notification_token::getIcon('SEND_EMAIL');

                    return Common::baseResponse(true, "Se envio un email con la información del contrato de adhesión al correo: " . $info_client->email . " del cliente el cual fue utilizado en la verificación celular, al recibir el email debe leer y aprobar para proseguir con el proceso de venta de fibra", [
                        "tyc" => $infoQr['id'],
                        "url" => $urlQr,
                        "share" => false
                    ], 200);
                } else {

                    return Common::baseResponse(false, "No se pudo enviar el email", [
                        "error_code" => "ERR_MAIL",
                        "errors" => [
                            "ERR_MAIL" => [$sendMailTyc->original->msg]
                        ]
                    ], 400);
                }
            }
        } else {

            // ($request->codePhoneContact == "AUTHORIZED" || $request->codePhoneContact == "NO_REQUIRED_VERIFY")
            return Common::baseResponse(true,  "Comparte el siguiente enlace con el cliente para que pueda revisar y aceptar los términos y condiciones del contrato de adhesión de fibra Netwey", [
                "tyc" => $infoQr['id'],
                "url" => $urlQr,
                "share" => true,
            ], 200);



            /* $Qr_svg = ('' . \QrCode::format('svg')->size(310)->generate($urlQr));

            $phone_error = "";
            if (empty($info_client->phone_home)) {
                $phone_error = "El cliente no posee teléfono principal de contacto. Este dato es importante para contactar al cliente";
            }
            $dni = $request->dni;
            $name_client = $info_client->name;
            $phone_client = $info_client->phone_home;
            $type = "START";
            $tyc = $infoQr['id'];
            $msgWhatsapp = "Entendemos%20que%20puede%20haber%20ciertos%20t%C3%A9rminos%20y%20condiciones%20que%20usted%20querr%C3%A1%20revisar%20antes%20de%20proceder%20con%20este%20agendamiento%20del%20servicio%20de%20Fibra%20Netwey.%20T%C3%B3mese%20unos%20minutos%20para%20leerlas%20antes%20de%20aceptarlas%20para%20continuar%20con%20el%20proceso.";
            $msgText = "Adjunto contrato de adhesión del servicio de Fibra.";
            $htmlShare = view('fiber.share_tycf', compact('urlQr', 'phone_client', 'dni', 'name_client', 'type', 'tyc', 'msgWhatsapp', 'msgText', 'phone_error'))->render();


            return response()->json(['success' => true, 'html' => $Qr_svg, 'htmlShare' => $htmlShare, 'tyc' => $infoQr['id'], 'msg' => 'OK', 'icon' => 'success', 'code' => 'AUTHORIZED']); */
        }
    }

    public function getTypesDocuments(Request $request)
    {

        $types = Common::getOptionColumn('islim_client_document', 'type');

        //remove key index from types
        $types = array_values($types);

        return Common::baseResponse(true, "Tipos de documentos", [
            "types" => $types
        ], 200);
    }


    public function sendMailQr(Request $request)
    {
        $info_client = Client::getClientByDNI($request->dni);
        $type = Common::decodificarBase64($request->type);

        $notmail = true;
        if (!empty($info_client)) {
            if (!empty($info_client->email)) {

                $infoQr = FiberPaymentForce::getUrlQr($request->dni, $type, ['C', 'A']);
                if (!empty($infoQr)) {
                    $urlQr = env('SITE_WEB_NETWEY') . 'tycf/' . $infoQr->code_url;

                    if ($type == "START") {

                        $infodata = [
                            'asunto' => "Contrato de adhesión para agendamiento de cita de fibra Netwey",
                            'client_name' => $info_client->name,
                            'urlqr' => $urlQr,
                            'process' => "agendamiento de cita",
                            'bodytext' => "Entendemos que puede haber ciertos términos y condiciones que usted querrá revisar antes de proceder con este agendamiento del servicio de Fibra Netwey. Tómese unos minutos para leerlas antes de aceptarlas para continuar con el proceso.",
                            'nota' => "Una vez revisado y aceptado el contrato de adhesión puedes informar a tu asesor de ventas para proseguir con el proceso de agendamiento"
                        ];
                    } elseif ($type == "END") {

                        $infodata = [
                            'asunto' => "Contrato de adhesión post-instalación de servicio de fibra Netwey",
                            'client_name' => $info_client->name,
                            'urlqr' => $urlQr,
                            'process' => "post-instalación",
                            'bodytext' => "Gracias por preferir nuestro servicio de internet de fibra, nos alegra tenerte con nosotros, ya su servicio fue instalado y nos gustaria por favor conocer que estas conforme con nuestra atención. Tómese unos minutos para visitar el enlace adjunto  para concluir el proceso de instalación.",
                            'nota' => "Una vez revisado la informacion adjunta puedes informar al instalador"
                        ];
                    } else {

                        return Common::baseResponse(false, "El tipo de mensaje que se trata de enviar no esta configurado", [
                            "error_code" => "EMP_TYP",
                            "errors" => [
                                "EMP_TYP" => ["El tipo de mensaje que se trata de enviar no esta configurado"]
                            ]
                        ], 400);
                    }

                    try {
                        
                        Mail::to($info_client->email)->send(new Mail_tycfQr($infodata));

                        $dataM = [
                            'email_client' => $info_client->email,
                            'dni_client' => $request->dni,
                            'process' => "CONTRACT",
                            'data_send' => json_encode($infodata)
                        ];
                        HistoryMailFiber::newHistoryMailFiber($dataM);
 
                        return Common::baseResponse(true, "Correo enviado!", [
                            "msg" => "Por favor informar al cliente " . $info_client->name . " que revise su bandeja de correo electrónico: " . $info_client->email . ". Alli vera información de vital importancia para continuar el proceso que llevas a cabo."
                        ], 200);
                    } catch (\Exception $e) {
                        $tex0 = 'No se pudo enviar el correo de contrato de adhesión de la cita de fibra a: ' . $info_client->email;
                        Log::error($tex0 . ' Error: ' . (string) json_encode($e->getMessage()));

                        return Common::baseResponse(false, "No se pudo enviar el email", [
                            "error_code" => "ERR_MAIL",
                            "errors" => [
                                "ERR_MAIL" => [$tex0]
                            ]
                        ], 400);
                    }
                } else {
                    return Common::baseResponse(false, "No se pudo enviar el email", [
                        "error_code" => "EMP_URL",
                        "errors" => [
                            "EMP_URL" => ["No se pudo obtener el enlace de contrato de adhesión"]
                        ]
                    ], 400);
                }
            } else {
                return Common::baseResponse(false, "No se pudo enviar el email", [
                    "error_code" => "EMP_MAIL",
                    "errors" => [
                        "EMP_MAIL" => ["El cliente no posee correo"]
                    ]
                ], 400);
            }
        } else {

            return Common::baseResponse(false, "No se pudo enviar el email", [
                "error_code" => "EMP_CLI",
                "errors" => [
                    "EMP_CLI" => ["No se pudo encontrar el registro del prospecto"]
                ]
            ], 400);
        }
    }

    public function verifyQr(Request $request)
    {

        $response = ['success' => false, 'title' => "No se pudo verificar el QR", 'msg' => "Faltan datos para procesar la verificacion", 'code' => 'FAIL'];

        $messages = [
            'dni.required' => 'El identificador del cliente es requerida (IVF3225)',
            'tyc.required' => 'El identificador del contrato es requerida (IVF3226)',
        ];

        $requisitos = [
            'dni' => array(
                'required',
            ),
            'tyc' => array(
                'required',
            ),
        ];

        $validate = Validator::make($request->all(), $requisitos, $messages);
        $errors = $validate->errors();

        if ($errors->any()) {
            return Common::baseResponse(false, "Faltan datos", [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ], 400);
        }

        if (!empty($request->dni) && !empty($request->tyc)) {

            $infoQr = FiberPaymentForce::getUrlQr($request->dni, false, ['A'], Common::decodificarBase64($request->tyc));
            if (!empty($infoQr)) {
                #return response()->json(['success' => true]);
                return Common::baseResponse(true, "Contrato de adhesión aceptado", [], 200);
            }
            $info_client = Client::getClientByDNI($request->dni);
            $name = '';
            if (!empty($info_client)) {
                $name = $info_client->name;
            }

            $response = ['success' => false, 'title' => "No se puede continuar", 'msg' => 'El cliente ' . $name . ' "no ha aceptado" el contrato de adhesión para el servicio de fibra Netwey. Por favor recuerdale al cliente revisarlos para poder registrar la solicitud que se lleva a cabo', 'icon' => "warning"];
        }

        return Common::baseResponse(false, $response['title'], [
            "error_code" => "not_accept",
            "errors" => [
                "not_accept" => [$response['msg']]
            ]
        ], 400);
    }

    public function getCircleForm(Request $request)
    {

        $response = ['success' => false, 'title' => "No se pudo obtener el formulario de circulo de credito que se creo para al prospecto", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        ///New validate
        $messages = [
            'client_id.required' => 'El identificador del prospecto es requerido (IVF7335)',
        ];
        $requisitos = [
            'client_id' => array(
                'required',
            ),
        ];
        //Log::info('ENTRADA getCircleForm ' . (String) json_encode($request->all()));

        $validate = Validator::make($request->all(), $requisitos, $messages);
        $errors = $validate->errors();

        if ($errors->any()) {

            $data = [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        }

        $LastRegister = CircleCreditRequest::getLastRegisterByDni($request->client_id);

        if (!empty($LastRegister)) {

            $exist_form = CircleCredit::getRegisterByRequest($LastRegister->id, ['T']);

            $infoClient = Client::getClientByDNI($request->client_id);
            $nombre = "";
            if (!empty($infoClient->name)) {
                $nombre = explode(" ", $infoClient->name)[0];
            } else {
                $nombre = explode(" ", $infoClient->last_name)[0];
            }

            if (empty($exist_form)) {
                //O no existe o fue eliminado el formulario
                //Se busca el nombre que tiene en netwey

                $response = ['success' => true, 'title' => "Registro de enlace de formulario encontrado", 'msg' => "El prospecto '" . $nombre . "' posee un enlace creado y esta en espera de ser utilizado", 'icon' => "success", 'code' => "FOR_EMPTY", 'request_circle_id' => $LastRegister->id, 'circle_id' => null, 'prospecto' => $nombre, 'vigency' => true];
            } else {
                //ya lleno el formulario
                //Se debe evaluar si el formulario esta registrado como aprobado con tiempo de vigencia

                //Se revisa si esta en la vigencia

                $textstatus = "";
                $code = $exist_form->status;
                if (!empty($exist_form->date_caduce)) {
                    $FormCaduce = $exist_form->date_caduce;
                } else {
                    $FormCaduce = "";
                }

                $nombreProspect = (!empty($nombre)) ? $nombre : $exist_form->first_name;

                if ($code == "PC") {
                    $textstatus = "Estatus: Se comenzo a llenar el formulario y aun no ha terminado de completarlo";
                } elseif ($code == "C") {
                    $textstatus = "Estatus: Se lleno la información requerida pero esta EN ESPERA de revisión. Puedes utilizar el formulario que ya se completo";
                } elseif ($code == "A") {
                    $textstatus = "Estatus: La verificación fue APROBADA";
                } elseif ($code == "R") {
                    $textstatus = "Estatus: La verificación fue RECHAZADA. Se debe esperar luego del " . $FormCaduce . " para solicitar de nuevo una verificación";
                }

                if ($code == "A" || $code == "R") {
                    //Se mira si esta dentro del rango de tiempo
                    if (!empty($FormCaduce)) {
                        $isVigenteForm = CircleCredit::vigencyFromCircle($FormCaduce, 'finish');
                    } else {
                        //Todos los registros previos que no tengan fecha de caducidad se concideran vencidos
                        $isVigenteForm = false;
                    }
                } elseif ($code == "PC") {
                    $isVigenteForm = CircleCredit::vigencyFromCircle($exist_form->date_reg);
                } else {
                    if ($code != "C") {
                        //posible figure como eliminado
                        $code = 'FOR_EMPTY';
                    }
                    $isVigenteForm = true;
                }

                if ($isVigenteForm) {
                    if ($code != 'FOR_EMPTY') {

                        $response = [
                            'success' => true,
                            'title' => "Formulario existente",
                            'msg' => "El prospecto '" . $nombreProspect . "' cuenta con un formulario vigente. " . $textstatus . " (CF7423)",
                            'icon' => "success"
                        ];
                    } else {
                        $response = [
                            'success' => false,
                            'title' => "Sin formulario",
                            'msg' => "El prospecto '" . $nombreProspect . "' no tiene formulario registrado, debe solicitar uno",
                            'icon' => "warning"
                        ];
                    }

                    $response['code'] = $code;
                    //El campo circle_id es nulo xq no es un formulario heredado de uno vencido
                    $response['circle_id'] = null;
                } else {
                    $response = [
                        'success' => false,
                        'title' => "Formulario existente!",
                        'msg' => "El prospecto '" . $nombreProspect . "' lleno el formulario crediticio pero esta vencido y debe solicitar uno nuevo (CF7438)",
                        'icon' => "warning",
                        'code' => "FOR_OLD",
                        'circle_id' => $exist_form->id
                    ];
                }

                $response['request_circle_id'] = $LastRegister->id;
                $response['result'] = $exist_form->status;
                $response['prospecto'] = $nombreProspect;
                $response['vigency'] = $isVigenteForm;
            }
        } else {
            $response = ['success' => false, 'title' => "No se obtuvo el formulario correspondiente para el prospecto", 'msg' => "El prospecto no posee un registro creado por el vendedor para ver un formulario, se debera crear uno (CF7450)", 'icon' => "success", 'code' => "EMP_URL", 'vigency' => true];
        }

        $status = 200;
        if ($response['success'] == false) {
            $status = 400;
        }


        return Common::baseResponse($response['success'], $response['title'], $response, $status);
    }

    public function getQuestionCircle(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'form_request_id' => 'required'
        ]);

        if ($validator->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validator->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        }


        $response = ['success' => true, 'title' => "Preguntas de circulo de credito", 'html' => "", 'icon' => "OK", 'code' => 'OK'];

        $questions = FiberQuestions::getQuestionsByPlatform('START', 'CIRCLE');
        $name_form = "question_circle";
        $answers = [];
        //Se busca el id de formulario correspondiente con el enlace
        if (isset($request->form_request_id) && !empty($request->form_request_id)) {
            if ($request->form_request_id != 'X') {
                $formCC = CircleCredit::getRegisterByRequest($request->form_request_id, ['T', 'R', 'PC']);
                if (!empty($formCC)) {
                    $answers = FiberQuestionResult::getAnswersById($formCC->id, 'START', 'CIRCLE');
                }
            }
        }

        $response['questions'] = $questions;
        $response['answers'] = $answers;
        $response['name_form'] = $name_form;

        $data = [
            "questions" => $questions,
            "answers" => $answers,
            "name_form" => $name_form
        ];

        return Common::baseResponse($response['success'], $response['title'], $data, 200);
    }

    public function sendCircleForm(Request $request)
    {



        $response = ['success' => false, 'title' => "No se pudo enviar el enlace del formulario crediticio al prospecto", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        ///New validate
        $messages = [
            'client_id.required' => 'El identificador del prospecto es requerido (IVF7372)',
            'form_request_id.required' => 'El identificador del formulario es requerido (IVF7373)',
            'form_request_id.regex' => 'El identificador del formulario posee un valor no valido (IVF7374)',
            'send.required' => 'El tipo de envio del formulario es requerido (IVF7375)',
            'fromFather.required' => 'El registro padre debe ser informado (IVF7376)',
        ];
        $requisitos = [
            'client_id' => array(
                'required',
            ),
            'form_request_id' => array(
                'required',
                'regex:/(^[X]|([0-9]+)(\d+)?$)/u',
            ),
            'send' => array(
                'required',
            ),
            'fromFather' => array(
                'nullable',
            )
        ];
        //Log::info('ENTRADA sendCircleForm ' . (String) json_encode($request->all()));

        $validate = Validator::make($request->all(), $requisitos, $messages);
        $errors = $validate->errors();

        if ($errors->any()) {

            $data = [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        }

        //Se verifica que se encuentre verificado el celular
        $infoClient = Client::getClientByDNI($request->client_id);
        if (!empty($infoClient)) {

            $nombre = "";
            if (!empty($infoClient->name)) {
                $nombre = explode(" ", $infoClient->name)[0];
            } else {
                $nombre = explode(" ", $infoClient->last_name)[0];
            }
            if (!empty($infoClient->verify_phone_id)) {
                //Se evalua si esta verificado
                $is_verify = Verify_contact_client::getRegisterVerify($infoClient->verify_phone_id);
                if (!empty($is_verify)) {
                    if ($is_verify->status == "VERIFIED") {
                        //Generamos el registro de peticion de formulario
                        //
                        $canalDecision = $is_verify->platform;
                        if ($canalDecision == "MAIL" && !empty($infoClient->email)) {
                            $canalDecision = "MAIL";
                        } else {
                            $canalDecision = "SMS";
                        }

                        $ok_send = true;
                        if ($request->form_request_id == "X" || $request->send == "new") {
                            $prefix_title = "Se envio";
                            $DataForm = CircleCreditRequest::newRequestFormCircle($request->client_id, $infoClient->verify_phone_id, $request->fromFather);

                            //Log::info('Se pide crear nuevo formulario ' . (String) json_encode($DataForm));

                            if ($DataForm['success']) {
                                //Ya que esta verificado registramos el SMS para que se envie al prospecto
                                $urlForm = $DataForm['url'];
                                $response['request_circle_id'] = $DataForm['msg'];
                            } else {
                                $response = ['success' => false, 'title' => "Hubo un problema al momento de enviar el formulario", 'msg' => "El registro del formulario de circulo de credito no se pudo preparar correctamente (CF7424)", 'icon' => "success", 'code' => 'ERR_DAT'];
                                $ok_send = false;
                            }
                        } else {
                            $prefix_title = "Se re-envio";
                            //Solo envio de SMS Ya que fue generado previamente y esto se trata de un reenvio de sms,
                            //Buscamos cual fue el registro de formulario y reenviamos ese.
                            $registerForm = CircleCreditRequest::getRegisterById($request->form_request_id);
                            if (!empty($registerForm)) {
                                $urlForm = $registerForm->url;
                                $response['request_circle_id'] = $registerForm->id;
                            } else {
                                $response = ['success' => false, 'title' => "Hubo un problema al momento de enviar el formulario", 'msg' => "El registro del formulario que fue generado previamente no pudo ser encontrado, intenta nuevamente (CF7436)", 'icon' => "success", 'code' => 'EMP_VRF'];
                                $ok_send = false;
                            }
                        }

                        if ($ok_send) {
                            $userText = "";
                            $nombre = "";
                            if (!empty($infoClient->name)) {
                                $nombre = explode(" ", $infoClient->name)[0];
                            } else {
                                $nombre = explode(" ", $infoClient->last_name)[0];
                            }
                            $userText = "Sr(a) " . $nombre . ", ";

                            $msgNewCircle = $userText . "Netwey te invita a completar el siguiente formulario: ";

                            if ($canalDecision == "SMS") {
                                //NOTA: el sms_type debe ser 'G' y el mensaje debe ir en sms_attribute y en sms(En ambos el mismo contenido)
                                try {
                                    $smsSend = Sms_notification_token::Send_sms(
                                        $infoClient->phone_home,
                                        $infoClient->phone_home,
                                        'G',
                                        '1',
                                        "Form Circulo de Credito",
                                        $msgNewCircle . $urlForm,
                                        $msgNewCircle . $urlForm
                                    );

                                    if ($smsSend) {
                                        $dataH = [
                                            'sms_nip_id' => $smsSend,
                                            'dni_client' => $request->client_id,
                                            'process' => "CIRCLE",
                                            'phone_sms' => $infoClient->phone_home
                                        ];
                                        HistorySmsFiber::newHistorySMSFiber($dataH);
                                    }
                                } catch (Exception $e) {
                                    Log::error('No se pudo registrar el SMS del form de circulo de credito a telefono del prospecto: ' . $infoClient->phone_home . ' Error: ' . (string) json_encode($e->getMessage()));
                                } 

                                $icono = Sms_notification_token::getIcon('SEND_SMS');
                                $response['msg'] = "Se envio un SMS al telefono '" . $infoClient->phone_home . "' del prospecto '" . $nombre . "' con la información que se debe suministrar para la verificación crediticia requerida. Se recuerda el formulario es intransferible y debe ser llenado por el solicitante del agendamiento";
                            } else {
                                //Se envia el correo con el formulario
                                
                                $infodata = [
                                    'client_name' => $nombre,
                                    'urlForm' => $urlForm,
                                    'bodytext' => $msgNewCircle,
                                    'nota' => "Se recuerda que el presente correo electrónico posee un enlace hacia un formulario el cual es intransferible y debe ser llenado por el solicitante del agendamiento ya que de lo contrario el análisis ante Buró de Crédito será inválido"
                                ];

                                try {
                                    Mail::to($infoClient->email)->send(new Mail_formCircle($infodata));

                                    $dataM = [
                                        'email_client' => $infoClient->email,
                                        'dni_client' => $request->client_id,
                                        'process' => "CIRCLE",
                                        'data_send' => json_encode($infodata)
                                    ];
                                    HistoryMailFiber::newHistoryMailFiber($dataM);
                                } catch (Exception $e) {
                                    Log::error('No se pudo enviar el correo del formulario de Circulo de credito al correo: ' . $infoClient->email . ' Error: ' . (string) json_encode($e->getMessage()));
                                }
                                $icono = Sms_notification_token::getIcon('SEND_EMAIL');

                                $response['msg'] = "Se envio un correo electronico a: '" . $infoClient->email . "' del prospecto '" . $nombre . "' con la información que se debe suministrar para la verificación crediticia requerida. Se recuerda el formulario es intransferible y debe ser llenado por el solicitante del agendamiento";
                            }

                            $response['success'] = true;
                            $response['title'] = $prefix_title . " el formulario crediticio";
                            $response['icon'] = "success";
                            $response['iconSMS'] = $icono;
                            $response['code'] = 'OK';
                            $response['url'] = $urlForm;
                        }
                    } else {
                        $response = ['success' => false, 'title' => "Hubo un problema al momento de enviar el formulario", 'msg' => "El telefono celular " . $is_verify->phone_verify . " del prospecto '" . $infoClient->phone_home . "' no se encuentra verificado (CF7501)", 'icon' => "success", 'code' => 'NOT_VRF'];
                    }
                } else {
                    $response = ['success' => false, 'title' => "Hubo un problema al momento de enviar el formulario", 'msg' => "El registro de verificación celular del prospecto " . $nombre . " no se encuentra en el sistema (CF7504)", 'icon' => "success", 'code' => 'EMP_VRF'];
                }
            } else {
                $response = ['success' => false, 'title' => "Hubo un problema al momento de enviar el formulario", 'msg' => "El prospecto " . $nombre . " no cuenta registro de verificación celular (CF7507)", 'icon' => "success", 'code' => 'EMP_VRF'];
            }
        } else {
            $response = ['success' => false, 'title' => "Hubo un problema al momento de enviar el formulario", 'msg' => "El identificador del prospecto suministrado no se encuentra en el sistema (CF7510)", 'icon' => "success", 'code' => 'EMP_CLI'];
        }

        $status = 200;
        if ($response['success'] == false) {
            $status = 400;
        }
        $data = $response;
        unset($data['success']);

        return Common::baseResponse($response['success'], $data['title'], $data, $status);
    }


    public function getConfirmationQuestions(Request $request)
    {
        $questions = DB::table('islim_fiber_questions')
            ->where('destiny', 'START')
            ->where('platform', 'ADMIN')
            ->where('status', 'A')
            ->orderBy('order', 'asc')
            ->get();

        foreach ($questions as $value) {

            $value->options = DB::table('islim_fiber_response')
                ->select(
                    'islim_fiber_question_response.id',
                    'islim_fiber_response.description'
                )
                ->join('islim_fiber_question_response', 'islim_fiber_question_response.response_id', 'islim_fiber_response.id')
                ->where('islim_fiber_question_response.status', 'A')
                ->where('islim_fiber_response.status', 'A')
                ->where('islim_fiber_question_response.question_id', $value->id)
                ->orderBy('islim_fiber_question_response.order', 'asc')
                ->get();
        }

        return Common::baseResponse(true, "Preguntas de confirmación", [
            "questions" => $questions
        ], 200);
    }




    public function registerScheduling(Request $request)
    {

        $response = [
            'success' => false,
            'msg' => 'Por favor verifique los datos.',
            'code' => 'FAIL'
        ];

        $messages = [
            'client.required' => 'El id del prospecto es requerido (IVF737)',
            'city_id.required' => 'El id de Ciudad es requerido (IVF738)',
            'city_id.regex' => 'El id de Ciudad no es un valor valido (IVF739)',
            'state_id.required' => 'El id de Estado es requerido (IVF740)',
            'state_id.regex' => 'El id de Estado no es un valor valido (IVF740)',
            'reference.required' => 'El referencia de dirección de instalación de fibra es requerido (IVF742)',
            'pack.required' => 'El id del paquete de fibra es requerido (IVF743)',
            'date.required' => 'El fecha de instalación de fibra es requerido (IVF744)',
            'hour.required' => 'El turno de instalación de fibra es requerido (IVF745)',
            'pay.required' => 'El indicativo de pago de instalación de fibra es requerido (IVF746)',
            'pay.regex' => 'El indicativo de pago de instalación de fibra es invalido (IVF747)',
            'zone_id.required' => 'El fecha de instalación de fibra es requerido (IVF748)',
            'zone_id.regex' => 'El id de zona de instalación de fibra es requerido (IVF749)',
            'isbundle.required' => 'El indicativo  de instalación de fibra bundle es requerido (IVF750)',
            'isbundle.regex' => 'El indicativo de instalación de fibra bundle no es valido (IVF751)'
        ];

        $requisitos = [
            'client' => array(
                'required'
            ),

            'city_id' => array(
                'required',
                'regex:/(^([0-9]+)(\d+)?$)/u'
            ),

            'state_id' => array(
                'required',
                'regex:/(^([0-9]+)(\d+)?$)/u'
            ),

            'reference' => array(
                'required'
            ),

            'pack' => array(
                'required'
            ),

            'date' => array(
                'required'
            ),

            'hour' => array(
                'required'
            ),

            'pay' => array(
                'required',
                'regex:/(^([N]|[Y])$)/u'
            ),

            'zone_id' => array(
                'required',
                'regex:/(^([0-9]+)(\d+)?$)/u'
            ),

            'isbundle' => array(
                'required',
                'regex:/(^([N]|[Y])$)/u'
            )
        ];

        if (isset($request->isReferred) && !empty($request->isReferred)) {
            $messages['isReferred.regex'] = 'El dn de referencia no posee un valor valido (IVF793)';
            $requisitos['isReferred'] = array(
                'regex:/(^[0-9]{10}$)/u'
            );
        }

        if (isset($request->imeiPhone) && !empty($request->imeiPhone)) {
            $messages['imeiPhone.regex'] = 'El IMEI de celular no posee un valor valido (IVF800)';
            $requisitos['imeiPhone'] = array(
                'regex:/(^[0-9]{15}$)/u'
            );
            $messages['isBandTE.regex'] = 'El indicativo de la banda del celular no posee un valor valido (IVF804)';
            $requisitos['isBandTE'] = array(
                'regex:/(^([N]|[Y])$)/u'
            );
        }



        $validate = Validator::make($request->all(), $requisitos, $messages);

        if ($validate->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ];

            return Common::baseResponse(false, "Faltan datos para procesar la solicitud", $data, 400);
        }



        if (isset($request->isReferred) && !empty($request->isReferred)) {
            //Se debe verificar que el dn a referenciar exista
            $clientRef = ClientNetwey::getClientByDN($request->isReferred);
            if (empty($clientRef)) {

                return Common::baseResponse(false, "El missdn " . $request->isReferred . " el cual desea referenciar no es encuentra en sistema", null, 400);
            }
        }


        [$respuesta, $message, $data, $status] = $this->chekingCoverageFiber($request);
        $respCoverage = $respuesta;

        if ($respCoverage['success']) {
            $request->pack = intval(Common::decodificarBase64($request->pack));
            $request->hour = Common::decodificarBase64($request->hour);

            if (!empty($request->tyc_start)) {
                $request->tyc_start = intval(Common::decodificarBase64($request->tyc_start));
            } else {
                $request->tyc_start = null;
            }

            //Valido que la fecha y el turno sea valido y no este bloqueado por feriado
            //
            $BlockDay = [];
            $BlockDay = FiberHoliday::getDayFeriado($BlockDay, false);

            if (!empty($BlockDay)) {
                foreach ($BlockDay as $dayFestive) {
                    if (strtotime($dayFestive->fecha) == strtotime($request->date)) {
                        if ($dayFestive->type == 'media' && $request->hour == "14:00 - 18:00") {
                            return Common::baseResponse(false, "Debes ingresar otro turno de instalación ya que el día " . $dayFestive->fecha . " por la tarde es informado como feriado! (CVF934)", null, 400);
                        } elseif ($dayFestive->type == 'full') {

                            return Common::baseResponse(false, "Debes ingresar otro día de instalación ya que el día " . $dayFestive->fecha . " es informado como feriado! (CVF936)", null, 400);
                        }
                    }
                }
            }
            //Valido que la fecha y el turno sea valido y no reservado por otra cita confirmada
            //merge to request id_olt
            $request->merge(['id_city' => $request->city_id]);
            $request->merge(['id_olt' => $request->zone_id]);
            $request->merge(['operation' => "install"]);

            $validateDate = self::getClock($request, true);
            /* se transforma la respuesta a un array para poder acceder a los datos */
            $validateDate = json_decode(json_encode($validateDate->getData()), true);

            $validateDate = $validateDate["data"]["clock"];

            if (!($validateDate['available_time'] ?? null)) {
                return Common::baseResponse(false, "Debes ingresar otro turno de instalación ya que la disponibilidad del día " . $request->date . " fue agotada! (CVF944)", null, 400);
            } else {
                foreach ($validateDate['list_turn'] as $turno) {
                    if ($turno['habilite'] == 'N' && $turno['hour'] == $request->hour) {

                        return Common::baseResponse(false, "Debes ingresar otro turno de instalación ya que la disponibilidad del día " . $request->date . " fue agotada! (CVF948)", null, 400);
                    }
                }
            }
            //End validacion de fecha

            $client = Client::getClientByDNI($request->client);

            if (!empty($client)) {
                //Consultando pack para validar si esta activo
                if ($request->isbundle == 'N') {
                    $dataPack = Pack::getActivePackById($request->pack);
                } else {
                    //Agendamiento de bundle
                    $infPack = Bundle::getDetailBundleAlta($request->pack);
                    $infoAllBundle = json_decode(json_encode($infPack['data']));
                    if ($infPack['success'] && $infoAllBundle->general->containt_F == 'Y' && isset($infoAllBundle->info_F->id)) {
                        $request->bundle_id = $request->pack;
                        $request->pack = $infoAllBundle->info_F->id;
                        $dataPack = Pack::getActivePackById($request->pack);
                    } else {

                        return Common::baseResponse(false, "El bundle seleccionado no se encuentra disponible para agendar la cita", null, 400);
                    }
                }
                
                if (!empty($dataPack) && ((hasPermit('MIG-FIB') && $dataPack->is_migration == 'Y') || $dataPack->is_migration == 'N')){
                    //consultando servicio
                    $packprice = PackPrices::getPackPriceByPackId($request->pack);
                    if (!empty($packprice)) {
                        //consultando artículo
                        $articpack = ArticlePack::getArticPackByPackId($request->pack);
                        if (!empty($articpack)) {
                            $seller = User::getUserByEmail(session('user'));
                            if (!empty($seller)) {
                                $simbol = array(" ,", "'", '"');
                                $request->reference = str_replace($simbol, "", $request->reference);

                                $addressFormated = $request->state . ', ' . $request->city . ', ' . $request->muni . ', ' . $request->colony . ', ' . $request->route . ', ' . $request->numberhouse . '. Referencia: ' . $request->reference;

                                $addressFormated = str_replace($simbol, "", $addressFormated);

                                //Actualizando dirección del cliente si no tiene una registrada
                                if (empty($client->address)) {
                                    $updateddress = Client::UpdateAddressClient($request->client, $addressFormated);

                                    if (!$updateddress['success']) {

                                        return Common::baseResponse(false, $updateddress['msg'], [
                                            "error_code" => "ERR_DB",
                                            "errors" => [
                                                "ERR_DB" => [$updateddress['msg']]
                                            ]
                                        ], 400);
                                    }
                                }

                                $urlPhotoRecibo = $request->photo_recibo;
                                $urlPhoto = $request->photo_reference;
                                /*
                                $listPhoto = [];
                                if (!empty($request->photo)) {
                                //verificando si viene imagen de referencia
                                array_push($listPhoto, 'photo');
                                }
                                if (!empty($request->photo_recibo)) {
                                //verificando si viene imagen de recibo
                                array_push($listPhoto, 'photo_recibo');
                                }

                                $path = 'installations/reference-photo/';
                                $urlPhoto = '';
                                $urlPhotoRecibo = '';

                                foreach ($listPhoto as $photo) {

                                $photoIdent = $request->file($photo);
                                $photoPath = $path . uniqid() . time() . '.' . $photoIdent->getClientOriginalExtension();

                                Storage::disk('s3')->put(
                                $photoPath,
                                file_get_contents($photoIdent->getPathname()),
                                'public'
                                );

                                if ($photo == "photo") {
                                $urlPhoto = (String) Storage::disk('s3')->url($photoPath);
                                } elseif ($photo == "photo_recibo") {
                                $urlPhotoRecibo = (String) Storage::disk('s3')->url($photoPath);
                                }
                                }*/

                                // Log::info('DATA AGENDAMIENTO PACK ' . (String) json_encode($dataPack));

                                $formCicle_id = null;
                                if ($dataPack->is_circle_credit == "Y") {
                                    //El paquete que estamos registrando amerito envio de datos circulo de credito
                                    //Se buscara el formulario que registro el cliente con el vendedor
                                    $requestCircle = CircleCreditRequest::getLastRegisterByDni($request->client);

                                    $userTextName = "";
                                    if (!empty($client->name)) {
                                        $nombre = explode(" ", $client->name)[0];
                                        $userTextName = $nombre;
                                    }

                                    if (!empty($requestCircle)) {
                                        $infoCircle = CircleCredit::getRegisterByRequest($requestCircle->id);
                                        if (!empty($infoCircle)) {
                                            $formCicle_id = $infoCircle->id;

                                            //Guardamos en la tabla de chequeo la foto del recibo publico adjuntada que fue requerida para este tipo de plan
                                            //
                                            //TEMPORALMENTE el guardado en la tabla cheking del recibo se quita hasta que se vuelva usar el api de circulo
                                            /*$upPhoto = CircleCreditCheking::UpdateChekingPhotoRecibo($formCicle_id, $urlPhotoRecibo);

                        if (!$upPhoto['success']) {
                        return response()->json($upPhoto);
                        */
                                            //END DE QUITADO TEMPORAL
                                            //
                                            //Se Confirma que tenga el celular el cliente verificado
                                            if (!empty($client->verify_phone_id)) {
                                                $phoneVerify = Verify_contact_client::getRegisterVerify($client->verify_phone_id);
                                                if (!empty($phoneVerify)) {
                                                    if ($phoneVerify->status != "VERIFIED") {

                                                        return Common::baseResponse(false, "El celular registrado por el prospecto " . $userTextName . " no esta verificado para poder continuar. (CF1159)", [
                                                            "error_code" => "NOT_VFP",
                                                            "errors" => [
                                                                "NOT_VFP" => ["El celular registrado por el prospecto " . $userTextName . " no esta verificado para poder continuar. (CF1159)"]
                                                            ]
                                                        ], 400);
                                                    }
                                                } else {

                                                    return Common::baseResponse(false, "El registro de verificación celular del prospecto " . $userTextName . " no se encontro en el sistema. (CF1162)", [
                                                        "error_code" => "NOT_EVR",
                                                        "errors" => [
                                                            "NOT_EVR" => ["El registro de verificación celular del prospecto " . $userTextName . " no se encontro en el sistema. (CF1162)"]
                                                        ]
                                                    ], 400);
                                                }
                                            } else {

                                                return Common::baseResponse(false, "El plan que estas agendando requiere que el numero celular de contacto del prospecto " . $userTextName . " este verificado para poder continuar. (CF1164)", [
                                                    "error_code" => "NOT_VRP",
                                                    "errors" => [
                                                        "NOT_VRP" => ["El plan que estas agendando requiere que el numero celular de contacto del prospecto " . $userTextName . " este verificado para poder continuar. (CF1164)"]
                                                    ]
                                                ], 400);
                                            }

                                            //Verificamos que las preguntas economicas esten guardadas y asociadas al registro del formulario
                                            //
                                            $answers = FiberQuestionResult::getAnswersById($formCicle_id, 'START', 'CIRCLE');

                                            if (count($answers) == 0) {

                                                //Log::info('Se registraran las preguntas economicas');
                                                $conAn = FiberQuestionResult::insertAnswers($formCicle_id, $request->all(), 'CIRCLE', 'START');
                                            } else {
                                                //Log::info('Respuestas ya registradas');
                                                $conAn['success'] = true;
                                            }

                                            if (!$conAn['success']) {

                                                return Common::baseResponse(false, "No se pudieron registrar las respuestas del formulario de circulo de credito del prospecto " . $userTextName . ". (CF1171)", [
                                                    "error_code" => "NOT_QRCC",
                                                    "errors" => [
                                                        "NOT_QRCC" => ["No se pudieron registrar las respuestas del formulario de circulo de credito del prospecto " . $userTextName . ". (CF1171)"]
                                                    ]
                                                ], 400);
                                            }
                                        } else {

                                            return Common::baseResponse(false, "El formulario de circulo de credito no fue llenado por el prospecto " . $userTextName . ". (CF1061)", [
                                                "error_code" => "NOT_CCE",
                                                "errors" => [
                                                    "NOT_CCE" => ["El formulario de circulo de credito no fue llenado por el prospecto " . $userTextName . ". (CF1061)"]
                                                ]
                                            ], 400);
                                        }
                                    } else {

                                        return Common::baseResponse(false, "El formulario de circulo de credito no fue enviado al prospecto " . $userTextName . ", para agendar la cita con el plan seleccionado se requiere que el cliente llene el formulario. (CF1064)", [
                                            "error_code" => "NOT_CCS",
                                            "errors" => [
                                                "NOT_CCS" => ["El formulario de circulo de credito no fue enviado al prospecto " . $userTextName . ", para agendar la cita con el plan seleccionado se requiere que el cliente llene el formulario. (CF1064)"]
                                            ]
                                        ], 400);
                                    }
                                }

                                $dateIns = Carbon::createFromFormat('Y-m-d', $request->date)->startOfDay();

                                try {
                                    $install = new Installations;
                                    $install->origin_scheduling = 'api';
                                    $install->clients_dni = $request->client;
                                    $install->address_instalation = $addressFormated;
                                    $install->pack_id = $request->pack;
                                    $install->service_id = $packprice->service_id;
                                    $install->inv_article_id = $articpack->inv_article_id;
                                    if ($request->isbundle == 'N') {
                                        $install->price = $packprice->total_price;
                                    } else {
                                        if (isset($infoAllBundle)) {
                                            $mount = 0;
                                            //Log::info((String) json_encode($infoAllBundle));
                                            if ($infoAllBundle->general->containt_H == 'Y') {
                                                if (isset($infoAllBundle->info_H->total_price)) {
                                                    $mount += $infoAllBundle->info_H->total_price;
                                                } else {

                                                    return Common::baseResponse(false, "No se pudo obtener informacion del bundle (CF1124)", [
                                                        "error_code" => "EMP_BUN",
                                                        "errors" => [
                                                            "EMP_BUN" => ["No se pudo obtener informacion del bundle (CF1124)"]
                                                        ]
                                                    ], 400);
                                                }
                                            }

                                            if ($infoAllBundle->general->containt_M == 'Y') {
                                                if (isset($infoAllBundle->info_M->total_price)) {
                                                    $mount += $infoAllBundle->info_M->total_price;
                                                } else {

                                                    return Common::baseResponse(false, "No se cuenta con el precio del bundle de Mifi (CF1095)", [
                                                        "error_code" => "EMP_PAY",
                                                        "errors" => [
                                                            "EMP_PAY" => ["No se cuenta con el precio del bundle de Mifi (CF1095)"]
                                                        ]
                                                    ], 400);
                                                }
                                            }

                                            if ($infoAllBundle->general->containt_MH == 'Y') {
                                                if (isset($infoAllBundle->info_MH->total_price)) {
                                                    $mount += $infoAllBundle->info_MH->total_price;
                                                } else {

                                                    return Common::baseResponse(false, "No se cuenta con el precio del bundle de Mifi Huella (CF1103)", [
                                                        "error_code" => "EMP_PAY",
                                                        "errors" => [
                                                            "EMP_PAY" => ["No se cuenta con el precio del bundle de Mifi Huella (CF1103)"]
                                                        ]
                                                    ], 400);
                                                }
                                            }

                                            if ($infoAllBundle->general->containt_T == 'Y') {
                                                if (isset($infoAllBundle->info_T->total_price)) {
                                                    $mount += $infoAllBundle->info_T->total_price;
                                                } else {

                                                    return Common::baseResponse(false, "No se cuenta con el precio del bundle de Telefonia (CF1111)", [
                                                        "error_code" => "EMP_PAY",
                                                        "errors" => [
                                                            "EMP_PAY" => ["No se cuenta con el precio del bundle de Telefonia (CF1111)"]
                                                        ]
                                                    ], 400);
                                                }
                                            }

                                            if ($infoAllBundle->general->containt_F == 'Y') {
                                                if (isset($infoAllBundle->info_F->total_price)) {
                                                    $mount += $infoAllBundle->info_F->total_price;
                                                } else {

                                                    return Common::baseResponse(false, "No se cuenta con el precio del bundle de Fibra (CF1119)", [
                                                        "error_code" => "EMP_PAY",
                                                        "errors" => [
                                                            "EMP_PAY" => ["No se cuenta con el precio del bundle de Fibra (CF1119)"]
                                                        ]
                                                    ], 400);
                                                }
                                            }
                                            $install->price = $mount;
                                        } else {

                                            return Common::baseResponse(false, "No se pudo obtener informacion del bundle (CF1124)", [
                                                "error_code" => "EMP_BUN",
                                                "errors" => [
                                                    "EMP_BUN" => ["No se pudo obtener informacion del bundle (CF1124)"]
                                                ]
                                            ], 400);
                                        }
                                    }

                                    $install->seller = session('user');


                                    //$install->installer = $request->installer;
                                    $install->date_instalation = $dateIns->format('Y-m-d');
                                    $install->paid = !empty($request->pay) ? $request->pay : 'N';
                                    $install->photo = !empty($urlPhoto) ? $urlPhoto : null;
                                    $install->photo_document = !empty($urlPhotoRecibo) ? $urlPhotoRecibo : null;
                                    $install->is_migration = $dataPack->is_migration;
                                    if ($dataPack->is_migration == 'Y') {
                                        $install->migration_origin = !empty($request->migratableDn) ? $request->migratableDn : null;
                                    } else {
                                        $install->migration_origin = null;
                                    }
                                    $install->status = 'A';
                                    $install->date_reg = date('Y-m-d H:i:s');

                                    $install->user_mod = session('user');
                                    $install->date_mod = date('Y-m-d H:i:s');
                                    $install->date_paid = (!empty($request->pay) && $request->pay == 'Y') ? date('Y-m-d H:i:s') : null;

                                    if (!empty($request->route)) {
                                        $install->route = $request->route;
                                    }

                                    if (!empty($request->numberhouse)) {
                                        $install->house_number = $request->numberhouse;
                                    }

                                    if (!empty($request->colony)) {
                                        $install->colony = $request->colony;
                                    }

                                    if (!empty($request->muni)) {
                                        $install->municipality = $request->muni;
                                    }

                                    if (!empty($request->reference)) {
                                        $install->reference = $request->reference;
                                    }

                                    if (!empty($request->lat)) {
                                        $install->lat = $request->lat;
                                    }

                                    if (!empty($request->lng)) {
                                        $install->lng = $request->lng;
                                    }
                                    //
                                    $install->group_install = uniqid($request->client) . time();
                                    $install->schedule = $request->hour;
                                    $install->id_fiber_zone = $request->zone_id;
                                    /*$configConex = new \stdClass;
                                                            $configConex->nodo_de_red = $request->node_red;
                                                            $configConex->nodo_de_red_name = $request->node_red_name;
                                    */
                                    $install->id_state = $request->state_id;
                                    $install->id_fiber_city = $request->city_id;
                                    $install->payment_force_start = $request->tyc_start;
                                    $install->bundle_id = isset($request->bundle_id) ? $request->bundle_id : null;
                                    $install->referred_dn = (isset($request->isReferred) && !empty($request->isReferred)) ? $request->isReferred : null;
                                    $install->circle_credit_id = $formCicle_id;
                                    $install->save();
                                } catch (Exception $e) {
                                    $Txmsg = 'Error al insertar una nueva cita de instalación (CF1288). ' . (string) json_encode($e->getMessage());
                                    Log::error($Txmsg);

                                    return Common::baseResponse(false, $Txmsg, [
                                        "error_code" => "ERR_DB",
                                        "errors" => [
                                            "ERR_DB" => [$Txmsg]
                                        ]
                                    ], 400);
                                }

                                //Para temas de historial la tabla instalacion guarda fecha de registro
                                //
                                //Envio correo
                                $dataBody = [
                                    'name' => $client->name,
                                    'lastname' => $client->last_name,
                                    'seller' => $seller->name . ' ' . $seller->last_name,
                                    'sellerPhone' => $seller->phone,
                                    //'installer' => $installer->name . ' ' . $installer->last_name,
                                    'installer' => "Por establecer...",
                                    //'installerPhone' => $installer->phone,
                                    'installerPhone' => "Por establecer...",
                                    'address' => $addressFormated,
                                    'date' => $dateIns->format('d-m-Y') . ' / ' . $request->hour
                                ];
                                if (!empty($client->email)) {
                                    try {
                                        Mail::to($client->email)->send(new mailInstall($dataBody));

                                        $dataM = [
                                            'email_client' => $client->email,
                                            'dni_client' => $request->client,
                                            'process' => "AGENDA",
                                            'installation_id' => $install->id,
                                            'data_send' => json_encode($dataBody)];
                                        HistoryMailFiber::newHistoryMailFiber($dataM);
                                    } catch (\Exception $e) {
                                        Log::error('No se pudo enviar el correo de solicitud de instalación a: ' . $client->email . ' Error: ' . (String) json_encode($e->getMessage()) . '(CF1345)');
                                    }
                                }

                                if (!empty($request->tyc_start)) {
                                    if (!empty($client->email)) {

                                        $dataPaymentForce = FiberPaymentForce::find($request->tyc_start);

                                        if (!empty($dataPaymentForce)) {
                                            $url_contract = $dataPaymentForce->url_contract;

                                            $data_contract = [
                                                'url_contract' => $url_contract,
                                                'full_date' => $dateIns->format('d-m-Y') . ' / ' . $request->hour,
                                                'pack_title' => $dataPack->title,
                                                'pack_price' => $packprice->total_price,
                                                'phone' => $client->phone_home,
                                                'email' => $client->email
                                            ];

                                            try {
                                                Mail::to($client->email)->send(new mailContract($data_contract));

                                                $dataM = [
                                                    'email_client' => $client->email,
                                                    'dni_client' => $request->client,
                                                    'process' => "CONTRACT",
                                                    'installation_id' => $install->id,
                                                    'data_send' => json_encode($data_contract)];
                                                HistoryMailFiber::newHistoryMailFiber($dataM);

                                            } catch (\Exception $e) {
                                                Log::error('No se pudo enviar el correo de Contrato a: ' . $client->email . ' Error: ' . (String) json_encode($e->getMessage()) . ' (CF1353)');
                                            } 
                                        } else {
                                            Log::error('No se pudo enviar el correo de Contrato a: ' . $client->email . ' Error: PaymentForceStrat no existe');
                                        }
                                    }

                                    //registramos que se uso el registro de verificacion
                                    FiberPaymentForce::NotifyUseForcer($request->tyc_start);
                                }

                                if ($request->isbundle == 'Y') {
                                    //Es una cita con bundle, se registran los hijos de dicho bundle
                                    if (isset($infoAllBundle)) {

                                        $articlesBundle = [
                                            'H',
                                            'T',
                                            'M',
                                            'MH'
                                        ];

                                        foreach ($articlesBundle as $value) {
                                            $containt = "containt_" . $value;
                                            $info = "info_" . $value;

                                            if ($infoAllBundle->general->$containt == 'Y' && isset($infoAllBundle->$info)) {

                                                if ($value == "T") {
                                                    $objImei = new \stdClass;
                                                    $objImei->imei = $request->imeiPhone;
                                                    $objImei->brand = $request->imeiBrand;
                                                    $objImei->model = $request->imeiModel;
                                                } else {
                                                    $objImei = null;
                                                }
                                                $isBandTE = ($value == "T") ? $request->isBandTE : null;

                                                $data = [
                                                    'installations_id' => $install->id,
                                                    'dn_type' => $value,
                                                    'msisdn_parent' => null,
                                                    'pack_id' => $infoAllBundle->$info->id,
                                                    'service_id' => $infoAllBundle->$info->service_id,
                                                    'inv_article_id' => $infoAllBundle->$info->product_id,
                                                    'info_imei' => !empty($objImei) ? (string) json_encode($objImei) : $objImei,
                                                    'isBandTE' => $isBandTE,
                                                    'date_reg' => date('Y-m-d H:i:s'),
                                                ];
                                                InstallationsBundle::registerChildrenBundler($data);
                                            }
                                        }
                                    } else {
                                        $Txmsg = 'La cita de instalación fue registrada pero se presento un problema en obtener la información de los componentes del bundle. (CF1382)';
                                        Log::error($Txmsg);

                                        return Common::baseResponse(false, $Txmsg, [
                                            "error_code" => "EMP_BUN",
                                            "errors" => [
                                                "EMP_BUN" => [$Txmsg]
                                            ]
                                        ], 400);
                                    }
                                }
                                //Envio SMS informativo por zona
                                //
                                if (!empty($client->phone_home)) {
                                    $Infomsj = FiberZone::getInfoZone($request->zone_id);
                                    $configuration = json_decode(json_encode($Infomsj->configuration));
                                    $smsCita = '';
                                    if (isset($configuration->smsCita)) {
                                        if (!empty($configuration->smsCita)) {
                                            $smsCita = $configuration->smsCita;
                                        }
                                    }

                                    /*Altan::sendSms([
                                    "msisdn" => $client->phone_home,
                                    "phone_sms" => $client->phone_home,
                                    "service" => "Cita de instalacion",
                                    "concentrator" => 1,
                                    "type_sms" => "O",
                                    "sms_attrib" => "SMSCITAFIBRA",
                                    "smsCita" => $smsCita]);*/
                                    
                                    if (!empty($smsCita)) {

                                        $smsCita = customerSMS(
                                            $smsCita,
                                            false,
                                            $client->name,
                                            false,
                                            $packprice->service_id,
                                            false,
                                            $dateIns->format('d-m-Y') . ' en horario ' . $request->hour
                                        );
                                        try {
                                            Sms_notification::Send_sms(
                                                $client->phone_home,
                                                $client->phone_home,
                                                'F',
                                                '1',
                                                "Cita de instalacion",
                                                "SMSCITAFIBRA",
                                                $smsCita);
                                        

                                        //No es un mensaje critico por ello se envia en el proceso de sms original
                                        //
                                        /*$smsSend = Sms_notification_token::Send_sms(
                                            $infoClient->phone_home,
                                            $infoClient->phone_home,
                                            'G',
                                            '1',
                                            "Cita de instalacion",
                                            $msgNewToken,
                                            */

                                        /*if ($smsSend) {
                                                                        $dataH = ['sms_nip_id' => $smsSend,
                                                                        'dni_client' => $request->client,
                                                                        'installation_id' => $install->id,
                                                                        'process' => "AGENDA",
                                                                        'phone_sms' => $client->phone_home];
                                                                        HistorySmsFiber::newHistorySMSFiber($dataH);
                                            */

                                        } catch (Exception $e) {
                                            Log::error('No se pudo enviar el msj de guardado de cita al telefono del cliente: ' . $client->phone_home . ' Error: ' . (String) json_encode($e->getMessage()));
                                        } 
                                    } else {
                                        Log::error("No se envio sms de notificacion de cita al cliente (" . $request->client . "), la zona (" . $request->zone_id . ") no lo tiene configurado.");
                                    }
                                } else {
                                    Log::error("El cliente (" . $request->client . ") no tiene configurado un telefono de contacto para el envio de sms.");
                                }

                                if ($dataPack->is_circle_credit == "Y") {
                                    $msgModal = "Recordatorio: Mesa de control verificara los datos suministrados y contactara al cliente, a su vez la cita amerita aprobación por parte de circulo de credito para ser confirmado el registro (" . $install->id . ")";
                                } else {
                                    $msgModal = "En la brevedad posible Mesa de Control Netwey procesará la información suministrada y contactará al cliente para confirmar el registro (" . $install->id . ")";
                                }

                                return Common::baseResponse(true, $msgModal, [
                                    "installation_id" => $install->id,
                                    "msg" => $msgModal
                                ], 200);
                            } else {
                                $response['msg'] = 'Vendedor o instalador no válido. (CF1335)';
                            }
                        } else {
                            $response['msg'] = 'Plan no válido, no se consiguio el artículo. (CF1338)';
                        }
                    } else {
                        $response['msg'] = 'Plan no válido, no se consiguio el servicio. (CF1341)';
                    }
                } else {
                    $response['msg'] = 'Plan no válido. (CF1344)';
                }
            } else {
                $response['msg'] = 'Cliente no registrado. (CF1347)';
            }
        } else {
            $textAux = 'Por favor verificar los datos';

            $response['msg'] = !empty($respCoverage['msg']) ? $respCoverage['msg'] . '. ' . $textAux : $textAux;
        }
        $response['code'] = 'ERR';

        return Common::baseResponse(false, $response['msg'], [
            "error_code" => "FAIL",
            "errors" => [
                "FAIL" => [$response['msg']]
            ]
        ], 400);
    }


    public function updateCircleCredit(Request $request)
    {
        $response = ['success' => false, 'title' => "No se pudo registrar la apreciación crediticia de este cliente", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        ///New validate
        $messages = [
            'client_id.required' => 'El identificador del cliente es requerido (IVF6069)',
            'id_install.required' => 'El identificador del agendamiento es requerido (IVF6069)',
            'status.required' => 'El resultado de la verificación crediticia del cliente es requerido (IVF6070)',
            'status.regex' => 'El resultado de la verificación crediticia del cliente posee un valor invalido (IVF6071)',
            'id.required' => 'El identificador de círculo de crédito del cliente es requerido (IVF6072)',
            'id.regex' => 'El identificador de círculo de crédito posee un valor no valido (IVF6073)',
            'evidenceCircle.required' => 'La evidencia pdf generada por circulo de credito es requerida (IVF6074)',
        ];
        $requisitos = [
            'client_id' => array(
                'required',
            ),
            'id_install' => array(
                'required',
            ),
            'status' => array(
                'required',
                'regex:/(^([N]|[Y])$)/u',
            ),
            'id' => array(
                'required',
                'regex:/(^([0-9]+)(\d+)?$)/u',
            ),
            'evidenceCircle' => array(
                'required',
            ),
        ];
        //Log::info('ENTRADA apreciacionCircle ' . (String) json_encode($request->all()));

        $validate = Validator::make($request->all(), $requisitos, $messages);
        $errors = $validate->errors();

        if ($errors->any()) {
            return Common::baseResponse(false, "Faltan datos para procesar la solicitud", [
                "error_code" => "validation_error",
                "errors" => $errors
            ], 400);
        }

        $listEvidence = [];
        if (!empty($request->evidenceCircle)) {
            //verificando si viene imagen de recibo
            array_push($listEvidence, 'evidenceCircle');
        }

        $save_file = true;
        try {
            $path = 'circle-credit/evidence/';
            $urlPdfEvidenceCircle = '';

            foreach ($listEvidence as $itemEvidence) {

                $fileInput = $request->file($itemEvidence);
                $FilePath = $path . uniqid() . time() . '.' . $fileInput->getClientOriginalExtension();

                Storage::disk('s3')->put(
                    $FilePath,
                    file_get_contents($fileInput->getPathname()),
                    'public'
                );

                if ($itemEvidence == "evidenceCircle") {
                    $urlPdfEvidenceCircle = (string) Storage::disk('s3')->url($FilePath);
                }
            }
        } catch (Exception $e) {
            $txmsg = "No se pudo registrar la evidencia adjuntada para círculo de crédito: " . $request->id . " - " . (string) json_encode($e->getMessage());
            Log::error($txmsg);
            $response['msg'] = $txmsg;
            $save_file = false;
        }

        if ($save_file) {
            if ($request->status == 'Y') {
                $msg = "Se registró la apreciación crediticia del cliente";
                $icon = "success";
                $resp = 'A';
                //Se marca la instalación como creada y sin confirmar
                $data = [
                    'status_control' => 'C',
                    'date_mod' => date('Y-m-d H:i:s'),
                    'user_mod' => session('user'),
                ];

                Installations::getConnect('W')
                    ->where('id', $request->id_install)->update($data);
            } else {
                $msg = "Se registró la apreciación crediticia del cliente";
                $icon = "info";
                $resp = 'R';
                //Se marca la instalación como Cancelada y se le da una tipificación
                $data = [
                    'typification_id' => env('TYPIFICATION_CIRCLE_CREDIT_REJECT'),
                    'status_control' => 'CN',
                    'date_mod' => date('Y-m-d H:i:s'),
                    'user_mod' => session('user'),
                ];

                Installations::getConnect('W')
                    ->where('id', $request->id_install)->update($data);

                //Registramos el historial del movimiento de la cita
                FiberControl::insertHistory(session('user'), 'I', $request->id_install, null, 'CN');

                //Se marca el formulario como T
                /*DB::connection('netwey-w')
                ->table('islim_circle_credit_request')
                ->where('id', $request->id)
                ->update([
                'status' => 'T',
                'email_modify' => session('user.email'),
                'date_update' => date('Y-m-d H:i:s'),
                ]);*/
            }
            $UpdtCircle = CircleCredit::updateFormCircle($request->id, $resp, $urlPdfEvidenceCircle, $request->obs_circle, $request->id_install);

            if ($UpdtCircle['success']) {

                $response = ['success' => true, 'title' => "Se registro la información crediticia", 'msg' => $msg, 'icon' => $icon];
            } else {
                $response = $UpdtCircle;
            }
        } else {
            $response = ['success' => false, 'title' => "No se puede registrar la apreciación crediticia", 'msg' => "El procesado del archivo de evidencia fallo (CF6175)", 'icon' => "warning", 'code' => "EMP_FOR"];
        }
        

        $status = $response['success'] ? 200 : 400;
        return Common::baseResponse($response['success'], $response['msg'], $response, $status);
        
    }


    public function confirmInstallation(Request $request)
    {

        $validate = Validator::make($request->all(), [
            'id' => 'required',
        ]);

        if ($validate->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validate->errors()
            ];

            return Common::baseResponse(false, "Faltan datos para procesar la solicitud", $data, 400);
        }

        /*todo: remove false, se agrego false debido a que el usuario de prueba Vendedor no tiene el permiso*/
        if (!User::hasPermission(session('user'), 'MDC-UAG') && false) {

            return Common::baseResponse(false, "Usted no posee permisos para realizar esta operación.", [
                "error_code" => "NOT_PER",
                "errors" => [
                    "NOT_PER" => ["Usted no posee permisos para realizar esta operación."]
                ]
            ], 400);
        }

        //Se verifica si la cita está sin confirmar (status C) o si la fecha y hora no ha cambiado

        $data = [
            //'installer_boss' => $request->install_boss,
            'status_control' => 'CC',
            'date_mod' => date('Y-m-d H:i:s'),
            'user_mod' => session('user'),
            'contract_conscious' => 'Y',
            'resend_contract' => 'N',
            'accept_contract_resend' => 'N'
        ];


        Installations::getConnect('W')
            ->where('id', $request->id)->update($data);

        //Insertamos al historial de mesa de control
        //FiberControl::insertHistory(session('user.email'), 'I', $request->id, $request->install_boss, 'CC');
        FiberControl::insertHistory(session('user'), 'I', $request->id, null, 'CC');



        Installations::sendNotificateConfirmation($request->id);

        $answers = $request->input();
        FiberQuestionResult::insertAnswers($request->id, $answers, 'I');


        return Common::baseResponse(true, "Cita de instalación confirmada correctamente", [], 200);
    }
}
