<?php

namespace App\Http\Controllers\Fiber;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Utilities\Common;
use App\Models\FiberCity;
use App\Models\User;
use App\Models\FiberCityZone;
use App\Models\CircleCredit;
use App\Models\PackFiber as Pack;
use App\Models\ClientNetwey;
use App\Models\Client;
use App\Models\CircleCreditRequest;
use App\Models\Verify_contact_client;
use App\Models\FiberQuestions;
use App\Models\FiberQuestionResult;
use App\Models\Sms_notification_token;
use App\Models\Installations;
use App\Models\HistorySmsFiber;
use App\Models\HistoryMailFiber;
use App\Utilities\FiberTrait;
use Exception;
use Log;

class ServiceabilityController extends Controller
{
    use FiberTrait;
    public function getStates(Request $request)
    {
        $states = FiberCity::getStates();

        $data = [
            "states" => $states
        ];

        return Common::baseResponse(true, "Estados obtenidos correctamente", $data, 200);
    }

    public function getCities(Request $request, $id_state)
    {

        $cities = FiberCity::getCitysWithActivePlans($id_state);

        $data = [
            "cities" => $cities
        ];

        return Common::baseResponse(true, "Ciudades obtenidas correctamente", $data, 200);
    }

    public function getOlts(Request $request, $id_city)
    {

        $olts = FiberCity::getOltsWithActivePlans($id_city);
        //reviso que existan instaladores en la zona
        foreach ($olts as $itemOlt) {
            $isInstall = User::getInstallerZone($itemOlt->zone_id);
            if (count($isInstall) > 0) {
                //zona con instaladores registrados
                $itemOlt->installer = 'Y';
            } else {
                //No hay instaladores registrados
                $itemOlt->installer = 'N';
            }
        }

        $data = [
            "olts" => $olts
        ];

        return Common::baseResponse(true, "OLTS obtenidos correctamente", $data, 200);
    }

    public function getMap(Request $request, $id_city, $id_olt)
    {

        $DataPoligono = null;
        $PoligonoCental = null;

        if (
            !empty($request->id_city) &&
            !empty($request->id_olt)
        ) {
            $DataPoligono = FiberCityZone::getCoordenada($request->id_city, $request->id_olt);
            if (!empty($DataPoligono)) {

                $DataPoligono = $DataPoligono['poligono'];

                $PoligonoCental = FiberCityZone::getCoordCenter($DataPoligono['poligono']);
            }
        }

        $DataPoligono['polygon'] = $DataPoligono['poligono'];
        unset($DataPoligono['poligono']);

        $data = [
            "point_center" => $PoligonoCental,
            "data_polygon" => $DataPoligono,
        ];

        return Common::baseResponse(true, "Mapa obtenido correctamente", $data, 200);
    }

    public function checkCoverage(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'lat' => 'required',
            'lng' => 'required',
            'id_city' => 'required',
            'id_olt' => 'required',
        ]);

        if ($validator->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validator->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        }

        $request->merge(['city_id' => $request->id_city]);
        $request->merge(['zone_id' => $request->id_olt]);

        /* here */
        [$respuesta, $message, $data, $status] = $this->chekingCoverageFiber($request);

        /* $respuesta = ['success' => false, 'title' => "No se pudo verificar servicialidad", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        $coor_valid = false;
        $status = 200;
        if (isset($request->lat) && isset($request->lng) &&
            !empty($request->lat) && !empty($request->lng)) {

            $coor_valid = true;

            //verifico que sea valida la lat y la lng
            //en caso de mexico es positiva
            $exp_lat = "/^(0(\.\d{1,20})?|([1-9](\d)?)(\.\d{1,20})?|1[0-7]\d{1}(\.\d{1,20})?|180\.0{1,20})$/";
            if (!preg_match($exp_lat, $request->lat)) {
                $respuesta['msg'] = "La latitud (" . $request->lat . ") no es valida para Mexico. Verifica de nuevo la servicialidad! (CF3269)";
                $respuesta['code'] = 'ERR_COORD';
                $status = 400;

                $coor_valid = false;
            }

            //en caso de mexico es negativa
            $exp_lng = "/^[\-](0(\.\d{1,20})?|([1-9](\d)?)(\.\d{1,20})?|1[0-7]\d{1}(\.\d{1,20})?|180\.0{1,20})$/";
            if (!preg_match($exp_lng, $request->lng)) {
                $respuesta['msg'] = "La longitud (" . $request->lng . ") no es valida para Mexico. Verifica de nuevo la servicialidad! (CF3276)";
                $respuesta['code'] = 'ERR_COORD';
                $coor_valid = false;
                $status = 400;
            }
        } else {
            $respuesta['msg'] = 'Se amerita una coordenada valida de mexico para verificar cobertura de fibra (CF3280)';
            $respuesta['code'] = 'EMP_COORD';
            $status = 400;
        }

        if ($coor_valid) {

            if (isset($request->city_id) && isset($request->zone_id) && !empty($request->city_id) && !empty($request->zone_id)) {

                $poligono = FiberCityZone::getCoordenada($request->city_id, $request->zone_id);
                $DataPoligono = $poligono['poligono']['poligono'];
                $resSrv = FiberCityZone::servicialidadFibra($request->lat, $request->lng, $DataPoligono);

                if ($resSrv == 'VERTICE' || $resSrv == 'BORDE' || $resSrv == 'DENTRO') {
                    $respuesta = array('success' => true, 'title' => "Con cobertura", 'msg' => "Cobertura en la ubicación exitosa", 'icon' => "success", 'code' => $resSrv);
                } else {
                    $respuesta['title'] = "Sin cobertura";
                    $respuesta['msg'] = 'No se cuenta con cobertura en la ubicación especificada.';
                    $respuesta['code'] = 'NOT_COVERAGE';
                    $status = 400;
                }
            } else {
                $respuesta['msg'] = 'Faltan datos de la ciudad y zona para verificar cobertura de fibra';
                $respuesta['code'] = 'ERR_DAT';
                $status = 400;
            }
        }

        if($status == 400){
            $data = [
                "error_code" => $respuesta['code'],
                "errors" => [
                    "serviceability" => $respuesta['msg']
                ]
            ];
            $message = "No se pudo verificar servicialidad";
        }else{
            $data = [
                "serviceability" => $respuesta['msg']
            ];
            $message = "Servicialidad verificada correctamente";
        }
 */
        return Common::baseResponse($respuesta['success'], $message, $data, $status);
    }

    public function checkFastCoverage(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'lat' => 'required',
            'lng' => 'required',
        ]);

        if ($validator->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validator->errors()
            ];

            return Common::baseResponse(false, "Faltan datos", $data, 400);
        }

        $respuesta = ['success' => false, 'title' => "No se pudo verificar servicialidad", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        $coor_valid = false;
        $status = 200;
        if (
            isset($request->lat) && isset($request->lng) &&
            !empty($request->lat) && !empty($request->lng)
        ) {

            $coor_valid = true;

            //verifico que sea valida la lat y la lng
            //en caso de mexico es positiva
            $exp_lat = "/^(0(\.\d{1,20})?|([1-9](\d)?)(\.\d{1,20})?|1[0-7]\d{1}(\.\d{1,20})?|180\.0{1,20})$/";
            if (!preg_match($exp_lat, $request->lat)) {
                $respuesta['msg'] = "La latitud (" . $request->lat . ") no es valida para Mexico. Verifica de nuevo la servicialidad! (CF3269)";
                $respuesta['code'] = 'ERR_COORD';
                $status = 400;

                $coor_valid = false;
            }

            //en caso de mexico es negativa
            $exp_lng = "/^[\-](0(\.\d{1,20})?|([1-9](\d)?)(\.\d{1,20})?|1[0-7]\d{1}(\.\d{1,20})?|180\.0{1,20})$/";
            if (!preg_match($exp_lng, $request->lng)) {
                $respuesta['msg'] = "La longitud (" . $request->lng . ") no es valida para Mexico. Verifica de nuevo la servicialidad! (CF3276)";
                $respuesta['code'] = 'ERR_COORD';
                $coor_valid = false;
                $status = 400;
            }
        } else {
            $respuesta['msg'] = 'Se amerita una coordenada valida de mexico para verificar cobertura de fibra (CF3280)';
            $respuesta['code'] = 'EMP_COORD';
            $status = 400;
        }

        if ($coor_valid) {


            $poligonos = FiberCityZone::getCoordenadas();

            $respuesta['title'] = "Sin cobertura";
            $respuesta['msg'] = 'No se cuenta con cobertura en la ubicación especificada.';
            $respuesta['code'] = 'NOT_COVERAGE';
            $status = 400;

            foreach ($poligonos as $poligono) {
                $DataPoligono = $poligono['poligono']['poligono'];
                $resSrv = FiberCityZone::servicialidadFibra($request->lat, $request->lng, $DataPoligono);

                if ($resSrv == 'VERTICE' || $resSrv == 'BORDE' || $resSrv == 'DENTRO') {
                    $respuesta = array('success' => true, 'title' => "Con cobertura", 'msg' => "Cobertura en la ubicación exitosa", 'icon' => "success", 'code' => $resSrv);
                    $city = FiberCity::getCityById($poligono['fiber_city_id']);
                    $state = FiberCity::getStateById($city['localy_state_id']);
                    $olt = FiberCity::getOltById($poligono['fiber_zone_id']);
                    $respuesta['id_state'] = $city['localy_state_id'];
                    $respuesta['state'] = $state['location_state'];
                    $respuesta['id_olt'] = $poligono['fiber_zone_id'];
                    $respuesta['id_city'] = $poligono['fiber_city_id'];
                    $respuesta['city'] = $city['fiber_city'];
                    $respuesta['data_polygon'] = $poligono["poligono"];
                    $respuesta['require_phone_verification'] = $olt["is_verify_phone"];
                    $respuesta['olt'] = $olt["name_zone"];
                    $status = 200;
                    break;
                }
            }
        }

        if ($status == 400) {
            $data = [
                "error_code" => $respuesta['code'],
                "errors" => [
                    "serviceability" => $respuesta['msg']
                ]
            ];
            $message = "No se pudo verificar servicialidad";
        } else {
            $respuesta["data_polygon"]["polygon"] = $respuesta['data_polygon']["poligono"];
            unset($respuesta['data_polygon']["poligono"]);
            $data = [
                "point_center" => [
                    "lat" => $request->lat,
                    "lng" => $request->lng
                ],
                "id_state" => $respuesta['id_state'],
                "state" => $respuesta['state'],
                "id_city" => $respuesta['id_city'],
                "city" => $respuesta['city'],
                "id_olt" => $respuesta['id_olt'],
                "olt" => $respuesta['olt'],
                "require_phone_verification" => $respuesta['require_phone_verification'],
                "data_polygon" => $respuesta['data_polygon'],
            ];
            $message = "Servicialidad verificada correctamente";
        }

        return Common::baseResponse($respuesta['success'], $message, $data, $status);
    }



    /* todo get component address */

    public function getPlans(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id_olt' => 'required',
            'id_city' => 'required',
            'dni' => 'required',
            'type' => 'required',
        ]);

        if ($validator->fails()) {
            $data = [
                "error_code" => "validation_error",
                "errors" => $validator->errors()
            ];

            return Common::baseResponse(false, "Faltan datos para procesar la lista de planes", $data, 400);
        }


        $request->merge(['oltid' => $request->id_olt]);
        $request->merge(['city' => $request->id_city]);

        $init = ($request->type === "init") ? true : false;
        //NOTA: INIT se usa para explorar todos los planes: contrato, susbcripcion y sin ninguno de los anteriores para generar las opciones de seleccion de tipo de planes en el pre-renderizado para que luego en la segunda vuelta que es donde el usuario selecciona el plan de su conveniencia



        //Consultando si el cliente tiene un dn hbb a asociado
        $isMig = false;
        $migratablesDns = false;




         if (hasPermit('MIG-FIB')){
            $migratablesDns = ClientNetwey::getClientsByDniAndTypes($request->dni, ['H', 'M', 'MH'])->pluck('msisdn');
            $isMig = (count($migratablesDns) > 0) ? true : false;
            //$isMig = ClientNetwey::isClientByTypes($request->dni, ['H', 'M', 'MH']);
         }


        //Filtrar consulta de planes de migracion
        if (!empty($request->packmigra)) {
            $PacksMigra = $request->packmigra;
        } else {
            //FILTRAR POR MIGRACION DESDE LA VISTA ESTA AUN EN DESARROLLO
            $PacksMigra = $isMig;
        }

        //Filtrar de planes tradicionales / forzados o contrato
        if (!empty($request->packforcer)) {
            $Packforcer = $request->packforcer;
        } else {
            $Packforcer = 'N';
        }

        //Filtrar consulta de planes sin suscripcion / con sucripcion
        if (!empty($request->packSuscrip)) {
            $PackSuscrip = $request->packSuscrip;
        } else {
            $PackSuscrip = 'N';
        }

        //Filtrar consulta de planes sin bundle / con bundle (Fibra+simcard)
        if (!empty($request->packBundler)) {
            $PackBundler = $request->packBundler;
        } else {
            $PackBundler = 'N';
        }

        $PackCircle = ['N'];

        $CityTokenized = FiberCityZone::isCityZoneTokenizado($request->oltid, $request->city);
        $circleLoad = null;
        $PackCircleMsg = "";
        $viewPackForcer = true;

        if ($CityTokenized) {
            //Ciudad con tokenizado activo

            //Reviso si el cliente tiene un celular verificado para ver planes de circulo de credito junto a los normales sin verificar o solo debe ver los que no requieren circulo de credito
            $PackCircle = self::isListPackCircle($request->dni);

            //Reviso la ultima peticion que se tenga de la persona tiene peticiones de circulo de credito rechazadas para impedirle ver planes de circulo de credito
            $circleLoad = CircleCredit::getLastRegisterBySend($request->dni);

            if ($circleLoad['vigency'] && $circleLoad['code'] == 'R') {
                //Formulario rechazado, no deberia dejarlo ver planes de circulo de credito
                $PackCircle = ['N'];
                $PackCircleMsg = "El prospecto no tiene permitido planes con contrato por temas ante Buró de Crédito";
                $viewPackForcer = false;
            }
        }


        if ($init) {
            //Consultando packs
            $packs = Pack::getFiberPacks($request->oltid, $isMig);
            //Log::info('Pack int ' . (String) json_encode($packs));

        } else {
            $packs = Pack::getFiberPacks($request->oltid, $isMig, $Packforcer, $PackSuscrip, $PackBundler, null, $PackCircle);
            //Log::info('Pack not int ' . (String) json_encode($packs));
        }

        //Si es migración pero no se consigue ningun pack para migración se buscan los packs normales
        if ($isMig && $packs->count() == 0) {
            if ($init) {
                // Log::info('Pack mig int ' . (String) json_encode($packs));

                $packs = Pack::getFiberPacks($request->oltid, false);
            } else {
                $packs = Pack::getFiberPacks($request->oltid, false, $Packforcer, $PackSuscrip, $PackBundler, null, $PackCircle);
                //Log::info('Pack mig not int ' . (String) json_encode($packs));
            }
        }

        $packs = $packs->filter(function ($val, $key) {
            if (!empty($val->date_ini) && !empty($val->date_end)) {
                if (strtotime($val->date_ini) <= time() && strtotime($val->date_end) >= time()) {
                    return true;
                }
            } elseif (!empty($val->date_ini)) {
                if (strtotime($val->date_ini) <= time()) {
                    return true;
                }
            } elseif (!empty($val->date_end)) {
                if (strtotime($val->date_end) >= time()) {
                    return true;
                }
            } elseif (empty($val->date_ini) && empty($val->date_end)) {
                return true;
            }
        });

        $htmlDocument = '';

        if ($init) {
            //Indica que botones deberia mostrar para los tipos de planes disponibles
            $recient_agend = Installations::getInstallBy_dni_city_time($request->dni, $request->city);
            $blockAgend = false;
            // Log::info("HISTORY " . (String) json_encode($recient_agend));
            if (!empty($recient_agend)) {
                //Hay una cita recien, no se debe agendar una nueva
                $blockAgend = $recient_agend;

                $message = "El prospecto ya posee una cita recientemente agendada bajo el registro # " . $blockAgend->id . " desde el " . $blockAgend->date_reg . ". Debes esperar para volver a registrar una nueva solicitud.";

                return Common::baseResponse(false, $message, [], 400);
            }
            //Planes normales
            $packsN = $packs->filter(function ($val, $key) {
                if ($val->is_payment_forcer == 'N') {
                    return true;
                }
            });

            //Planes forzados
            $packsF = $packs->filter(function ($val, $key) {
                if ($val->is_payment_forcer == 'Y') {
                    return true;
                }
            });

            //Planes sin suscripcion
            $packsSS = $packs->filter(function ($val, $key) {
                if ($val->for_subscription == 'N') {
                    return true;
                }
            });

            //Planes con suscripcion
            $packsCS = $packs->filter(function ($val, $key) {
                if ($val->for_subscription == 'Y') {
                    return true;
                }
            });

            //Planes con bundle
            $packsBun = $packs->filter(function ($val, $key) {
                if ($val->is_bundle == 'Y') {
                    return true;
                }
            });

            //Puede ofrecer planes de migracion
            $packsMig = $packs->filter(function ($val, $key) {
                if ($val->is_migration == 'Y') {
                    return true;
                }
            });

            $data = [
                "packsN" => $packsN,
                "packsF" => $packsF,
                "packsSS" => $packsSS,
                "packsCS" => $packsCS,
                "packsBun" => $packsBun,
                "packsMig" => $packsMig,
                "PackCircleMsg" => $PackCircleMsg
            ];

            return Common::baseResponse(true, "Tipos Planes obtenidos correctamente", $data, 200);
        } else {

            $data = [
                "packs" => $packs,
                "migratablesDns" => $migratablesDns
            ];
            return Common::baseResponse(true, "Planes obtenidos correctamente", $data, 200);
        }
    }

    private function isListPackCircle($client_dni)
    {
        /*
        Si el cliente posee un celular verificado puede ver los planes de verificacion de circulo de credito y los que no lo requieren, si no tiene el celular verificado solo vera los que no requieren revision de circulo de credito
     */
        $PackCircle = ['N'];
        $datClient = Client::getClientByDNI($client_dni);
        if (!empty($datClient)) {
            if (!empty($datClient->verify_phone_id)) {
                $datVerify = Verify_contact_client::getRegisterVerify($datClient->verify_phone_id);

                if (!empty($datVerify)) {
                    if ($datVerify->status == "VERIFIED") {
                        //Listo los planes de que sean de circulo y no
                        $PackCircle = ['Y', 'N'];
                    }
                }
            }
        }
        return $PackCircle;
    }

}
