<?php

namespace App\Http\Controllers;

//Otras clases
use Illuminate\Http\Request;

use App\Utilities\NinetyNineMin;
use App\Utilities\CustomLogs;
use App\Utilities\Google;
use App\Utilities\Common;
use App\Utilities\voywey;
use App\Utilities\prova;

//Modelos
use App\Models\ShipingMethodsConcentrator;
use App\Models\NinetyNineMinutes;
use App\Models\TokenApiSeller;
use App\Models\VoyweyDelivery;
use App\Models\ProvaDelivery;
use App\Models\TempCarDetail;
use App\Models\LogApiV2;
use App\Models\PCvoywey;
use App\Models\Articles;
use App\Models\TempCar;
use App\Models\Client;

use Log;

class ShippingController extends Controller
{
	/**
	 * Create a new controller instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		//
	}

	/*
  Retorna los metodos de envio disponibles para una compra
  */
	public function getShippingMethods(Request $request)
	{
		//Iniciando log
		$log = LogApiV2::initLog($request);

		$response = [
			'success' => false,
			'data' => [
				'cod_err' => 'ERR_PARAM',
				'msg' => 'Faltan parámetros.',
			],
		];

		$inputs = $request->input();

		if (!empty($inputs['transaction']) && !empty($inputs['address'])) {
			//Verificando si la transaccion es válida
			$transaction = TempCar::getTransaction(
				$inputs['transaction'],
				$request->bearerToken()
			);

			$transactionD = TempCarDetail::getTransactionDetail(
				$inputs['transaction'],
				$request->bearerToken()
			);

			if (!empty($transaction) && count($transactionD)) {
				$conc = TokenApiSeller::getInfToken($request->bearerToken());

				if (!empty($conc)) {
					$methodship = ShipingMethodsConcentrator::getMethods($conc->concentrator_id);
				}
				if (!empty($methodship)) {
					//Quitar este metodo cuando inconcert acutualice su integración
					$nminM2 = $methodship->first(function ($value, $key) {
						return $value->code_name == '99_min';
					});

					//Validando si el integrador tiene asignado 99min como metodo de envío
					if (!empty($nminM2)) {
						$articsB = $transactionD->pluck('id_artic');

						//Obteniendo array con información del metodo de envío
						$arrNn = NinetyNineMin::getShippingArray2(
							$articsB->toArray(),
							$inputs['address'],
							$nminM2,
							$request,
							$log
						);

						if (count($arrNn)) {
							$shippings[] = $arrNn;
						}
					}
					//Quitar hasta aqui cuando inconcert actualice su integración

					$nminM = $methodship->first(function ($value, $key) {
						return $value->code_name == '99_min_2';
					});

					//Validando si el integrador tiene asignado 99min como método de envío
					if (!empty($nminM)) {
						$articsB = $transactionD->pluck('id_artic');

						//Obteniendo array con información del metodo de envío
						$arrNn = NinetyNineMin::getShippingArray(
							$articsB->toArray(),
							$inputs['address'],
							$nminM,
							$request,
							$log
						);

						CustomLogs::addMessage(
							'Tiempo 99_min_2: ' . round((microtime(true) - $log->initTime), 2),
							$log
						);

						if (count($arrNn)) {
							$shippings[] = $arrNn;
						}
					}

					//Obteniendo SKUs
					$skus = $transactionD->pluck('sku');

					//Prova
					$provaM = $methodship->first(function ($value, $key) {
						return $value->code_name == 'prova';
					});

					//Validando si el integrador tiene asignado prova como metodo de envío
					if (!empty($provaM)) {
						//Obteniendo array con información del metodo de envío
						$arrPro = prova::getShippingArray(
							$inputs['address'],
							$skus->toArray(),
							$provaM,
							$request,
							$log
						);

						if (count($arrPro)) {
							$shippings[] = $arrPro;
						}
					}

					//Voywey
					$voywM = $methodship->first(function ($value, $key) {
						return $value->code_name == 'voywey';
					});

					//Validando si el integrador tiene asignado prova como metodo de envío
					if (!empty($voywM)) {
						//Consultando datos del cliente
						$clientD = Client::getClientByDni($transaction->client_dni);

						$datavoy = [
							'postal_code' => $inputs['address'],
							'clientName' => !empty($clientD) ? $clientD->name . ' ' . $clientD->last_name : '',
							'clientPhone' => !empty($clientD) ? $clientD->phone_home : '',
							'clientEmail' => !empty($clientD) ? $clientD->email : '',
							'route' => !empty($inputs['route']) ? $inputs['route'] : '',
							'street_number' => !empty($inputs['street_number']) ? $inputs['street_number'] : '',
							'notes' => !empty($inputs['notes']) ? $inputs['notes'] : '',
							'alias' => '',
							'weight' => env('WEIGHT', 1),
							'length' => env('LENGTH', 10),
							'height' => env('HEIGHT', 10),
							'width' => env('WIDTH', 10),
							'neighborhood' => !empty($inputs['neighborhood']) ? $inputs['neighborhood'] : '',
							'state' => !empty($inputs['state']) ? $inputs['state'] : '',
							'sub_locality' => !empty($inputs['sub_locality']) ? $inputs['sub_locality'] : '',
							'locality' => !empty($inputs['locality']) ? $inputs['locality'] : '',
							'internal_number' => !empty($inputs['internal_number']) ? $inputs['internal_number'] : ''

						];

						//Obteniendo array con información del metodo de envío
						$arrVoy = voywey::getShippingArray(
							$datavoy,
							!empty($skus->toArray()[0]) ? $skus->toArray() : [],
							$voywM,
							$request,
							$log
						);

						CustomLogs::addMessage(
							'Tiempo voywey: ' . round((microtime(true) - $log->initTime), 2),
							$log
						);

						if (count($arrVoy)) {
							//Creando registro de voywey, solo si el enviío es por voywey externo
							if (strtolower($arrVoy['currier']) != 'voywey') {
								NinetyNineMinutes::resetDelivery($transaction->id);

								ProvaDelivery::resetDelivery($transaction->id);

								VoyweyDelivery::resetDelivery($transaction->id);

								$delivery = VoyweyDelivery::getConnect('W');;
								$delivery->id_temp_car = $transaction->id;
								$delivery->postal_code = $inputs['address'];
								$delivery->street = $inputs['route'];
								//$delivery->street2 = $inputs['route2'];
								$delivery->ext_number = $inputs['street_number'];
								if (!empty($inputs['neighborhood'])) {
									$delivery->colony =  $inputs['neighborhood'];
								}
								$delivery->courier_g = strtolower($arrVoy['currier']);
								$delivery->shipment_id = $arrVoy['sending_data']['shipmentsId'];
								$delivery->prices = json_encode($arrVoy['prices']);
								$delivery->notes = $inputs['notes'];
								$delivery->status = 'A';
								$delivery->date_reg = date('Y-m-d H:i:s');

								$delivery->save();
							}

							$shippings[] = $arrVoy;
						}
					}

					//Colocar otros metodos de envío desde aqui

					if (!empty($shippings) && count($shippings)) {
						$response = [
							'success' => true,
							'data' => [
								'transaction' => $inputs['transaction'],
								'shipping_methods' => $shippings,
							],
						];
					} else {
						$response['data']['cod_err'] = 'WT_INVENTARY';
						$response['data']['msg'] = 'Artículo(s) sin inventario o sin cobertura de delivery.';
					}
				} else {
					$response['data']['cod_err'] = 'NOT_MTHOSHIP';
					$response['data']['msg'] = 'Sin metodos de envio registrados.';

					CustomLogs::registerLog(
						'error',
						'Sin metodos de envio registrados.',
						'F',
						$request,
						$response
					);
				}
			} else {
				$response['data']['cod_err'] = 'TR_NOTVALID';
				$response['data']['msg'] = 'Transacción no válida.';
			}
		}

		//Finalizando log
		LogApiV2::finishLog(
			$log,
			$response,
			0,
			$response['success'] ? 'D' : 'E',
			!$response['success'] ? $response['data']['msg'] : ''
		);

		return response()->json($response);
	}

	/*
  	Registra envio para una compra
  */
	public function registerShipping(Request $request)
	{
		//Iniciando log
		$log = LogApiV2::initLog($request);

		$response = [
			'success' => false,
			'data' => [
				'cod_err' => 'ERR_PARAM',
				'msg' => 'Faltan parámetros.',
			],
		];

		$inputs = $request->input();

		if (!empty($inputs['transaction']) && !empty($inputs['shipping']) && !empty($inputs['shipping']['id'])) {
			//Verificando si la transaccion es válida
			$transaction = TempCar::getTransaction(
				$inputs['transaction'],
				$request->bearerToken(),
				'W'
			);

			$transactionD = TempCarDetail::getTransactionDetail(
				$inputs['transaction'],
				$request->bearerToken()
			);

			if (!empty($transaction) && count($transactionD)) {
				$conc = TokenApiSeller::getInfToken($request->bearerToken());

				//Obteniendo SKUs
				$skus = $transactionD->pluck('sku');

				if (!empty($conc)) {
					$methodship = ShipingMethodsConcentrator::getMethods($conc->concentrator_id);
				}

				if (!empty($methodship)) {
					$shipM = $methodship->first(function ($value, $key) use ($inputs) {
						return $value->code_name == $inputs['shipping']['id'];
					});
				}

				if (!empty($shipM)) {
					//Bandera para validar si el envio fue correcto
					$isShippingValid = false;

					//Verificando si el envio es por 99 minutos borrar este fragmento de codigo cuando inconcert actualice su integración
					if ($shipM->code_name == '99_min') {
						if (
							!empty($inputs['shipping']['route']) &&
							!empty($inputs['shipping']['neighborhood']) &&
							!empty($inputs['shipping']['state']) &&
							!empty($inputs['shipping']['postal_code'])
						) {
							//Borrando registro de envios anteriores
							NinetyNineMinutes::resetDelivery($transaction->id);

							ProvaDelivery::resetDelivery($transaction->id);

							VoyweyDelivery::resetDelivery($transaction->id);

							//Obteniendo precio del envio por 99 min
							$price = ninetyNineMin::getPriceDelivery(
								$inputs['shipping']['postal_code'],
								[
									'weight' => 90,
									'width' => 90,
									'height' => 90,
									'depth' => 90,
								]
							);

							if ($price['success'] && !empty($price['data']->rates)) {
								$priceSelected = ninetyNineMin::getpriceDetail($price['data']->rates);

								if ($priceSelected['success']) {
									//Guardando datos para envío
									$nine = new NinetyNineMinutes;
									$nine->id_temp_car = $transaction->id;
									$nine->postal_code = $inputs['shipping']['postal_code'];
									$nine->route = $inputs['shipping']['route'];
									$nine->neighborhood = $inputs['shipping']['neighborhood'];
									$nine->state = $inputs['shipping']['state'];
									$nine->price = $priceSelected['data']->cost;
									$nine->delivery_type = $priceSelected['data']->type;
									$nine->vehicle_type = $priceSelected['data']->vehicle;
									$nine->date_reg = date('Y-m-d H:i:s');
									$nine->status = 'A';

									if (!empty($inputs['shipping']['street_number'])) {
										$nine->street_number = $inputs['shipping']['street_number'];
									}

									if (!empty($inputs['shipping']['internal_number'])) {
										$nine->internal_number = $inputs['shipping']['internal_number'];
									}

									if (!empty($inputs['shipping']['notes'])) {
										$nine->notes = $inputs['shipping']['notes'];
									}

									if (!empty($inputs['shipping']['sub_locality'])) {
										$nine->internal_number = $inputs['shipping']['sub_locality'];
									}

									$nine->save();

									$transaction->address_send = null;
									$transaction->cod_estafeta = null;
									$transaction->price_sending = $priceSelected['data']->cost;
									$transaction->save();

									$isShippingValid = true;
								} else {
									CustomLogs::addMessage(
										'No se pudo obtener el precio en getpriceDetail:' . json_encode($priceSelected),
										$log
									);

									$response['data']['cod_err'] = 'ERR_99min';
									$response['data']['msg'] = 'Error en conexión con 99 minutos.';
								}
							} else {
								CustomLogs::addMessage(
									'No se pudo obtener el precio: ' . json_encode($price),
									$log
								);

								$response['data']['cod_err'] = 'ERR_COVERAGE_99min';
								$response['data']['msg'] = 'Sin cobertura de 99 minutos.';
							}
						} else {
							$response['data']['cod_err'] = 'ERR_SHIPPING_DATA';
							$response['data']['msg'] = 'Error en datos de envío.';
						}
					}

					//Verificando si el envio es por 99 minutos
					if ($shipM->code_name == '99_min_2') {
						if (
							!empty($inputs['shipping']['route']) &&
							!empty($inputs['shipping']['neighborhood']) &&
							!empty($inputs['shipping']['state']) &&
							!empty($inputs['shipping']['postal_code']) &&
							!empty($inputs['shipping']['street_number']) &&
							!empty($inputs['shipping']['notes']) &&
							!empty($inputs['shipping']['sub_locality']) &&
							!empty($inputs['shipping']['locality'])
						) {
							//Borrando registro de envios anteriores
							NinetyNineMinutes::resetDelivery($transaction->id);
							ProvaDelivery::resetDelivery($transaction->id);
							VoyweyDelivery::resetDelivery($transaction->id);

							//Obteniendo precio del envio por 99 min
							$price = ninetyNineMin::getPriceDelivery(
								$inputs['shipping']['postal_code'],
								[
									'weight' => 90,
									'width' => 90,
									'height' => 90,
									'depth' => 90,
								]
							);

							if ($price['success'] && !empty($price['data']->rates)) {
								$priceSelected = ninetyNineMin::getpriceDetail($price['data']->rates);

								if ($priceSelected['success']) {
									//Guardando datos para envío
									$nine = new NinetyNineMinutes;
									$nine->id_temp_car = $transaction->id;
									$nine->postal_code = $inputs['shipping']['postal_code'];
									$nine->route = $inputs['shipping']['route'];
									$nine->neighborhood = $inputs['shipping']['neighborhood'];
									$nine->state = $inputs['shipping']['state'];
									$nine->locality = $inputs['shipping']['locality'];
									$nine->price = $priceSelected['data']->cost;
									$nine->delivery_type = $priceSelected['data']->type;
									$nine->vehicle_type = $priceSelected['data']->vehicle;
									$nine->street_number = $inputs['shipping']['street_number'];
									$nine->notes = $inputs['shipping']['notes'];
									$nine->municipality = $inputs['shipping']['sub_locality'];
									$nine->date_reg = date('Y-m-d H:i:s');
									$nine->status = 'A';

									if (!empty($inputs['shipping']['internal_number'])) {
										$nine->internal_number = $inputs['shipping']['internal_number'];
									}

									$nine->save();

									$transaction->address_send = null;
									$transaction->cod_estafeta = null;
									$transaction->price_sending = $priceSelected['data']->cost;
									$transaction->save();

									$isShippingValid = true;
								} else {
									CustomLogs::addMessage(
										'No se pudo obtener el precio en getpriceDetail:' . json_encode($priceSelected),
										$log
									);

									$response['data']['cod_err'] = 'ERR_99min';
									$response['data']['msg'] = 'Error en conexión con 99 minutos.';
								}
							} else {
								CustomLogs::addMessage(
									'No se pudo obtener el precio: ' . json_encode($price),
									$log
								);

								$response['data']['cod_err'] = 'ERR_COVERAGE_99min';
								$response['data']['msg'] = 'Sin cobertura de 99 minutos.';
							}
						} else {
							$response['data']['cod_err'] = 'ERR_SHIPPING_DATA';
							$response['data']['msg'] = 'Error en datos de envío.';
						}
					}

					if ($shipM->code_name == 'prova') {
						if (
							!empty($inputs['shipping']['route']) &&
							!empty($inputs['shipping']['neighborhood']) &&
							!empty($inputs['shipping']['state']) &&
							!empty($inputs['shipping']['sub_locality']) &&
							!empty($inputs['shipping']['street_number']) &&
							!empty($inputs['shipping']['postal_code']) &&
							!empty($inputs['shipping']['notes']) &&
							!empty($inputs['shipping']['locality'])
						) {
							//consultando precio en prova
							NinetyNineMinutes::resetDelivery($transaction->id);

							ProvaDelivery::resetDelivery($transaction->id);

							VoyweyDelivery::resetDelivery($transaction->id);

							//$skus = $transactionD->pluck('sku');
							$prova = prova::getPrice($inputs['shipping']['postal_code'], $skus);

							if ($prova['success'] && !empty($prova['data']->prices)) {
								$delivery = new ProvaDelivery;
								$delivery->id_temp_car = $transaction->id;
								$delivery->deposit = $prova['data']->warehouse;
								$delivery->postal_code = $inputs['shipping']['postal_code'];
								$delivery->state = $inputs['shipping']['state'];
								$delivery->city = $inputs['shipping']['locality'];
								$delivery->municipality = $inputs['shipping']['sub_locality'];
								$delivery->colony = $inputs['shipping']['neighborhood'];
								$delivery->street = $inputs['shipping']['route'];
								$delivery->ext_number = $inputs['shipping']['street_number'];
								$delivery->courier_g = $prova['data']->currier;
								$delivery->price = $prova['data']->prices[0]->price;
								$delivery->notes = $inputs['shipping']['notes'];
								$delivery->status = 'A';
								$delivery->date_reg = date('Y-m-d H:i:s');

								if (!empty($inputs['shipping']['internal_number'])) {
									$delivery->int_number = $inputs['shipping']['internal_number'];
								}

								$delivery->save();

								$transaction->address_send = null;
								$transaction->cod_estafeta = null;
								$transaction->price_sending = $prova['data']->prices[0]->price;
								$transaction->save();

								$isShippingValid = true;
							} else {
								CustomLogs::addMessage(
									'Error en getPriceDelivery prova: ' . json_encode($prova),
									$log
								);

								$response['data']['cod_err'] = 'ERR_COVERAGE_PROVA';
								$response['data']['msg'] = 'Sin cobertura de Prova.';
							}
						} else {
							$response['data']['cod_err'] = 'ERR_SHIPPING_DATA';
							$response['data']['msg'] = 'Error en datos de envío.';
						}
					}

					//Verificando si el envio es por voywey
					if ($shipM->code_name == 'voywey') {
						if (
							!empty($inputs['shipping']['route']) &&
							!empty($inputs['shipping']['neighborhood']) &&
							!empty($inputs['shipping']['state']) &&
							!empty($inputs['shipping']['sub_locality']) &&
							!empty($inputs['shipping']['street_number']) &&
							!empty($inputs['shipping']['postal_code']) &&
							!empty($inputs['shipping']['notes']) &&
							!empty($inputs['shipping']['locality']) &&
							!empty($inputs['shipping']['lat']) &&
							!empty($inputs['shipping']['lng'])
						) {
							//consultando precio en voywey
							NinetyNineMinutes::resetDelivery($transaction->id);

							ProvaDelivery::resetDelivery($transaction->id);

							if (!empty($inputs['shipping']['shipmentsId']) && !empty($inputs['shipping']['object_id'])) {
								$delivery = VoyweyDelivery::getConnect('W')->where('shipment_id', $inputs['shipping']['shipmentsId'])->first();

								if (!empty($delivery)) {
									$prices = json_decode($delivery->prices);

									foreach ($prices as $pri) {
										if ($pri->object_id == $inputs['shipping']['object_id']) {
											$delivery->price = $pri->cost;
											break;
										}
									}

									$delivery->shipment_type_id = (string) $inputs['shipping']['object_id'];
								}
							} else {
								VoyweyDelivery::resetDelivery($transaction->id);

								//Consultando datos del cliente
								$clientD = Client::getClientByDni($transaction->client_dni);

								$voywey = voywey::getPrice([
									'postal_code' => $inputs['shipping']['postal_code'],
									'clientName' => !empty($clientD) ? $clientD->name . ' ' . $clientD->last_name : '',
									'clientPhone' => !empty($clientD) ? $clientD->phone_home : '',
									'clientEmail' => !empty($clientD) ? $clientD->email : '',
									'route' => $inputs['shipping']['route'],
									'neighborhood' => $inputs['shipping']['neighborhood'],
									'street_number' => $inputs['shipping']['street_number'],
									'notes' => $inputs['shipping']['notes'],
									'alias' => '',
									'weight' => env('WEIGHT', 1),
									'length' => env('LENGTH', 10),
									'height' => env('HEIGHT', 10),
									'width' => env('WIDTH', 10)
								]);
							}

							if ((!empty($voywey) && $voywey['success'] && !empty($voywey['data']->prices)) || !empty($delivery)) {
								//Consultando inventario para saber desde que bodega puede salir el envío, 
								//esto se hace aqui porque voywey puede tener mas de una bodega asociada al mismo código postal
								//$wh = PCvoywey::getWareHouse($inputs['shipping']['postal_code']);
								$wh = voywey::validateInventory(
									$inputs['shipping']['postal_code'],
									!empty($skus->toArray()[0]) ? $skus->toArray() : [],
									null
								);

								if ($wh) {
									// $varaux='exist';
									if (empty($delivery)) {
										$delivery = VoyweyDelivery::getConnect('W');
										$delivery->price = $voywey['data']->prices[0]->cost;
										$delivery->courier_g = 'voywey';
										// $varaux='new';
									}
									$delivery->id_temp_car = $transaction->id;
									$delivery->deposit = $wh; //!empty($wh) ? $wh->warehouse : null;
									$delivery->postal_code = $inputs['shipping']['postal_code'];
									$delivery->state = $inputs['shipping']['state'];
									$delivery->city = $inputs['shipping']['locality'];
									$delivery->municipality = $inputs['shipping']['sub_locality'];
									$delivery->colony = $inputs['shipping']['neighborhood'];
									$delivery->street = $inputs['shipping']['route'];
									$delivery->ext_number = $inputs['shipping']['street_number'];
									$delivery->notes = $inputs['shipping']['notes'];
									$delivery->status = 'A';
									$delivery->date_reg = date('Y-m-d H:i:s');

									if (!empty($inputs['shipping']['internal_number'])) {
										$delivery->int_number = $inputs['shipping']['internal_number'];
									}

									// if (!empty($inputs['shipping']['lat'])) {
									$delivery->lat = (string) $inputs['shipping']['lat'];
									// }

									// if (!empty($inputs['shipping']['lng'])) {
									$delivery->lng = (string) $inputs['shipping']['lng'];
									// }

									if (!empty($inputs['shipping']['delivery_date'])) {
										$delivery->delivery_date = $inputs['shipping']['delivery_date'];
									}

									$delivery->save();

									// Log::alert('DataVWDelivery: ($id = '.$delivery->id.', state ='.$inputs['shipping']['state'].', shipmentid = '.$inputs['shipping']['shipmentsId'].' - '.$varaux.' )');

									$transaction->address_send = null;
									$transaction->cod_estafeta = null;
									$transaction->price_sending = $delivery->price;
									$transaction->save();

									$isShippingValid = true;
								} else {
									$response['data']['cod_err'] = 'WT_INVENTARY';
									$response['data']['msg'] = 'Artículo(s) sin inventario o sin cobertura de delivery.';
								}
							} else {
								if (!empty($voywey)) {
									CustomLogs::addMessage(
										'Error en getPriceDelivery voywey: ' . json_encode($voywey),
										$log
									);
								} else {
									CustomLogs::addMessage(
										'Error en registro de envío voywey-externo, no se consiguio la cotización',
										$log
									);
								}

								$response['data']['cod_err'] = 'ERR_COVERAGE_VOYWEY';
								$response['data']['msg'] = 'Sin cobertura de Voywey.';
							}
						} else {
							$response['data']['cod_err'] = 'ERR_SHIPPING_DATA';
							$response['data']['msg'] = 'Error en datos de envío.';
						}
					}

					//Agregar otros metodos de envío desde aquí

					if ($isShippingValid) {
						$response = [
							'success' => true,
							'data' => [
								'transaction' => $inputs['transaction'],
								'shipping_data' => $inputs['shipping'],
							],
						];
					}
				} else {
					$response['data']['cod_err'] = 'ERR_SHIPMTH';
					$response['data']['msg'] = 'Metodo de envio no registrado para el integrador.';
				}
			} else {
				$response['data']['cod_err'] = 'TR_NOTVALID';
				$response['data']['msg'] = 'Transacción no válida.';
			}
		}

		//Finalizando log
		LogApiV2::finishLog(
			$log,
			$response,
			0,
			$response['success'] ? 'D' : 'E',
			!$response['success'] ? $response['data']['msg'] : ''
		);

		return response()->json($response);
	}

	/*
  DEPRECATED
  Retorna las estafetas cercanas a un codigo postal dado

  @param Illuminate\Http\Request $request
  @return Object
   */
	/*public function getEstafetas(Request $request)
  {	
  	//Iniciando log
    $log = LogApiV2::initLog($request);

    $response = ['success' => false, 'data' => ['cod_err' => 'ERR_PARAM', 'msg' => 'Faltan parámetros.']];

    $inputs = $request->input();

    if (!empty($inputs['postal_code'])) {
      $sapo = Sapomex::distinct()
        ->select(
          'd_estado as estado',
          'D_mnpio as municipe',
          'd_asenta as asenta'
        )
        ->where('d_codigo', $inputs['postal_code'])
        ->first();

      $estafetas = Estafeta::getEstafetas([['cp', $inputs['postal_code']]]);

      if ($estafetas->count() == 0 && !empty($sapo)) {
        $estafetas = Estafeta::getEstafetasWSapo($sapo);

        if (count($estafetas)) {
          $estafetas = Common::getEstafetasArray($estafetas);

          if (count($estafetas)) {
            $response = [
              'success' => true,
              'data' => $estafetas,
            ];
          }
        }
      } elseif ($estafetas->count()) {
        $estafetasA = Common::getEstafetasArray($estafetas);

        if (count($estafetasA)) {
          $response = [
            'success' => true,
            'data' => $estafetasA,
          ];
        }
      }

      if (!$response['success']) {
        $response['data']['cod_err'] = 'ESTAFETA_NOTFOUND';
        $response['data']['msg'] = 'No se consiguieron estafetas.';
      }
    }

    //Finalizando log
    LogApiV2::finishLog(
        $log, 
        $response, 
        0,
        $response['success'] ? 'D' : 'E',
        !$response['success'] ? $response['data']['msg'] : ''
      );

    return response()->json($response);
  }*/
}
