<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use App\Models\TokenApiSeller;

class ApiKey extends Model
{
    use HasFactory;

    protected $table = 'islim_api_keys';

    protected $fillable = [
        'api_key',
        'concentrators_id',
        'type',
        'date_reg',
        'status',
    ];

    protected $primaryKey = 'api_key';

    protected $keyType = 'string';

    public $incrementing = false;

    public $timestamps = false;

    /**
     * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
     * @param String $typeCon
     *
     * @return App\models\ApiKey
     */
    public static function getConnect($typeCon = false)
    {
        if ($typeCon) {
            $obj = new ApiKey;
            $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

            return $obj;
        }
        return null;
    }

    public static function getToken($token = false)
    {
        if ($token) {
            $type = (env('APP_ENV') == 'local' || env('APP_ENV') == 'test') ? 'test' : 'prod';

            $data = TokenApiSeller::getInfToken($token);

            if (!empty($data)) {
                return self::getConnect('R')
                    ->select('api_key')
                    ->where([
                        ['concentrators_id', $data->concentrator_id],
                        ['type', $type],
                    ])
                    ->first();
            }
        }

        return null;
    }

    public static function getTokenConc($idConc = false)
    {
        if ($idConc) {
            $type = (env('APP_ENV') == 'local' || env('APP_ENV') == 'test') ? 'test' : 'prod';

            return self::getConnect('R')
                ->select('api_key')
                ->where([
                    ['concentrators_id', $idConc],
                    ['type', $type],
                ])
                ->first();
        }

        return null;
    }
}
