<?php

namespace App\Models;

use App\Models\Client;
use App\Utilities\Common;
use DateTime;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Log;
use Exception;

class CircleCredit extends Model
{
  use HasFactory;

  protected $table = 'islim_circle_credit';
  public $timestamps = false;

  protected $fillable = [
    'id',
    'first_name',
    'second_name',
    'surname',
    'second_surname',
    'RFC',
    'CURP',
    'date_birthday',
    'status_civil',
    'phone_verify',
    'estado',
    'ciudad',
    'municipio',
    'colonia',
    'name_of_street',
    'number_interno',
    'number_externo',
    'code_postal',
    'status',
    'email_mesa',
    'date_reg',
    'date_update',
    'email_modify',
    'url_pdf_circle',
    'circle_request_id',
    'email_seller',
    'father_circle',
    'num_revisions',
    'status_circle',
    'obs_circle',
    'date_caduce',
    'score',
    'band'
  ];

  /**
   * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
   * @param String $typeCon
   *
   * @return App\Models\CircleCredit
   */
  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new self;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

      return $obj;
    }
    return null;
  }

  /**
   * [getRegisterByRequest Busqueda del formulario si fue cargado por medio del id de solicitud de formulario]
   * @param  [type] $id [description]
   * @return [type] $status   [Indica si fue creado, aprobado o rechazado]
   */
  public static function getRegisterByRequest($id, $statusNot = ['T', 'R'])
  {
    $query = self::getConnect('R')
      ->where('circle_request_id', $id)
      ->whereNotIn('status', $statusNot)
      ->orderBy('id', 'DESC')
      ->first();

    return $query;
  }

  /**
   * [vigencyFromCircle Recibe la fecha en que caduce la vigencia del formulario e indica si aun esta dentro del tiempo estipulado de vigencia basado en su fecha de expiracion]
   * @param  [type] $dateEvalue [Fecha a estudiar]
   * @param  [type] $typeDate   [Tipo de fecha a evaluar, la fecha de creacion o la fecha de caducidad del formulario]
   * @return [type]             [description]
   */
  public static function vigencyFromCircle($dateEvalue, $typeDate = "create")
  {
    $hoy = date('Y-m-d');
    $dateStudy = $dateEvalue;

    $fechaActual = strtotime($hoy);
    $fechaEnvio = strtotime($dateStudy);

    if ($fechaActual <= $fechaEnvio) {
      $Menor = true;
    } else {
      $Menor = false;
    }

    $date = new DateTime($dateStudy);
    $fechaEnvio = $date->format('Y-m-d');

    $datetime1 = date_create($fechaEnvio);
    $datetime2 = date_create($hoy);
    $contador = date_diff($datetime1, $datetime2);
    $differenceFormat = '%a';
    $cantDay = $contador->format($differenceFormat);

    if ($typeDate == "create") {
      //Tiempo de creado el registro
      if ($cantDay <= env('VIGENCY_TEMP_CIRCLE') && !$Menor) {
        //Aun puede terminar de llenar el formulario
        return true;
      }
      return false;
    } else {
      //caducidad del formulario qua ya fue aprobado o rechazado
      //VIGENCY_CIRCLE = 30 (Tiempo inicial de validez)
      if ($Menor) {
        //Aun el formulario es valido de usarse
        return true;
      }
      return false;
    }
  }

  /**
   * [getLastRegisterByinWait Obtener el ultimo enlace con formulario que tenga el cliente y que este en espera de terminar de llenar]
   * @param  [type] $client_dni [description]
   * @return [type]             [description]
   */
  public static function getLastRegisterByinWait($client_dni)
  {
    $previus = self::getConnect('R')
      ->select(
        'islim_circle_credit_request.id as Circle_request_id',
        'islim_circle_credit.date_reg as Date_form',
        'islim_circle_credit.date_caduce'
      )
      ->join(
        'islim_circle_credit_request',
        'islim_circle_credit_request.id',
        'islim_circle_credit.circle_request_id'
      )
      ->where([
        ['islim_circle_credit_request.dni_client', $client_dni],
        ['islim_circle_credit_request.status', '!=', 'T'],
        ['islim_circle_credit.status', 'PC']
      ])
      ->orderBy('islim_circle_credit.id', 'DESC')
      ->first();

    if (!empty($previus)) {

      $isVigenteForm = self::vigencyFromCircle($previus->Date_form);

      if ($isVigenteForm) {
        return ['success' => true, 'msg' => "OK", 'data' => $previus, 'code' => "OK"];
      }
    }
    return ['success' => false, 'msg' => "No hay formulario pendiente por llenar", 'code' => "EMPHY"];
  }

  /**
   * [getLastRegisterBySend Se evalua si el formulario fue revisado y si sigue vigente]
   * @param  [type] $client_dni [description]
   * @return [type]             [description]
   */
  public static function getLastRegisterBySend($client_dni)
  {
    $cheking = self::getConnect('R')
      ->select(
        'islim_circle_credit_request.id as Circle_request_id',
        'islim_circle_credit.id as Circle_id',
        'islim_circle_credit.status as Status_form',
        'islim_circle_credit.date_reg as Date_form',
        'islim_circle_credit.date_caduce'
      )
      ->join(
        'islim_circle_credit_request',
        'islim_circle_credit_request.id',
        'islim_circle_credit.circle_request_id'
      )
      ->where([
        ['islim_circle_credit_request.dni_client', $client_dni],
        ['islim_circle_credit_request.status', '!=', 'T']
      ])
      ->whereIn('islim_circle_credit.status', ['A', 'R'])
      ->orderBy('islim_circle_credit.id', 'DESC')
      ->first();

    if (!empty($cheking)) {

      if (!empty($cheking->date_caduce)) {
        $isVigenteForm = self::vigencyFromCircle($cheking->date_caduce, 'finish');
      } else {
        //Todos los registros previos que no tengan fecha de caducidad se concideran vencidos
        $isVigenteForm = false;
      }

      if ($isVigenteForm) {
        return ['success' => true, 'msg' => "OK", 'data' => $cheking, 'code' => $cheking->Status_form, 'vigency' => true];
      } else {
        return ['success' => false, 'msg' => "La validez del formulario cargado ha caducado", 'code' => $cheking->Status_form, 'vigency' => false];
      }
    }
    return ['success' => false, 'msg' => "No hay formulario registrado", 'code' => "EMPHY", 'vigency' => true];
  }

  //FUNCION DEPRECADA EN SELLER
  /*public static function updateFormCircleBySeller(
  $register_id,
  $resp_status,
  $url_evidence) {

  try {
  $Dat = self::getConnect('W')
  ->where([
  ['id', $register_id],
  ['status', '!=', 'T'],
  ])
  ->update([
  'status' => $resp_status,
  'url_pdf_circle' => $url_evidence,
  'email_modify' => session('user'),
  'date_update' => date('Y-m-d H:i:s'),
  ]);
  $response = ['success' => true, 'title' => "Se actualizaron los datos del formulario del cliente", 'msg' => "Actualizados los datos correctamente", 'icon' => "success", 'code' => 'OK'];
  } catch (Exception $e) {
  $txmsg = "No se pudo registrar la apreciacion crediticia para el formulario: " . $register_id . " - (MCC94) " . (String) json_encode($e->getMessage());
  Log::error($txmsg);
  $response = ['success' => false, 'title' => "No se pudo actualizar el registro del formulario del cliente", 'msg' => $txmsg, 'icon' => "error", 'code' => 'ERR_DAT'];
  }
  return $response;
  }*/

  public static function comparedClientCircle($form_id, $client_dni)
  {
    $response = ['success' => false, 'title' => "La comparacion de datos requiere atencion", 'msg' => "No se pudo verificar los datos del formulario con los datos del cliente", 'icon' => "error", 'code' => 'ERR_DAT'];

    $CircleDat = self::getConnect('R')
      ->join(
        'islim_circle_credit_request',
        'islim_circle_credit_request.id',
        'islim_circle_credit.circle_request_id'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_circle_credit_request.dni_client'
      )
      ->where([
        ['islim_circle_credit_request.dni_client', $client_dni],
        ['islim_circle_credit.id', $form_id]
      ])
      ->first();

    if (!empty($CircleDat)) {
      $EqualName = false;
      $EqualLast_name = false;
      $infoClien = Client::getClientINEorDN($client_dni);
      if (!empty($infoClien)) {

        if (!empty($infoClien->name)) {

          if (!empty($CircleDat->first_name)) {

            if (str_contains(strtolower(Common::normaliza($infoClien->name)), strtolower(Common::normaliza($CircleDat->first_name)))) {
              $EqualName = true;
            }
          }

          if (!empty($CircleDat->second_name)) {

            if (str_contains(strtolower(Common::normaliza($infoClien->name)), strtolower(Common::normaliza($CircleDat->second_name)))) {
              $EqualName = true;
            }
          }
        }

        if (!empty($infoClien->last_name)) {

          if (!empty($CircleDat->surname)) {

            if (str_contains(strtolower(Common::normaliza($infoClien->last_name)), strtolower(Common::normaliza($CircleDat->surname)))) {
              $EqualLast_name = true;
            }
          }

          if (!empty($CircleDat->second_surname)) {
            if (str_contains(strtolower(Common::normaliza($infoClien->last_name)), strtolower(Common::normaliza($CircleDat->second_surname)))) {
              $EqualLast_name = true;
            }
          }
        }

        if ($EqualName && $EqualLast_name) {
          //Actualizamos los datos que tenemos del formulario hacia los datos del cliente en netwey para que compartan la misma data

          $name = "";
          if (!empty($CircleDat->first_name)) {
            $name = ucfirst(strtolower(Common::normaliza($CircleDat->first_name)));
          }

          if (!empty($CircleDat->second_name)) {
            $name .= " " . ucfirst(strtolower(Common::normaliza($CircleDat->second_name)));
          }

          $last_name = "";
          if (!empty($CircleDat->surname)) {
            $last_name = ucfirst(strtolower(Common::normaliza($CircleDat->surname)));
          }

          if (!empty($CircleDat->second_surname)) {
            $last_name .= " " . ucfirst(strtolower(Common::normaliza($CircleDat->second_surname)));
          }

          $updateClient = Client::updateDataClient($client_dni, $name, $last_name, $CircleDat->date_birthday);

          if ($updateClient['success']) {
            $response = ['success' => true, 'title' => "Comparacion de datos exitosa", 'msg' => "Los datos del formulario con los datos del cliente fueron sincronizados", 'icon' => "success", 'code' => 'OK'];
          } else {
            $response['msg'] = $updateClient['msg'];
          }
        } else {
          $response['msg'] = "Los datos ingresados por el cliente en el formulario '" . $form_id . "' que se desean procesar ante Buró de Crédito no coinciden con los datos que se encuentran en la plataforma de ventas. Se recuerda que el formulario debe ser llenado por la persona que esta solicitando el presente agendamiento";
          $response['code'] = "DIFF_CLIENT";
        }
      } else {
        $response['msg'] = "No se encontro el propecto para la verificacion de datos";
        $response['code'] = "EMP_CLIENT";
      }
    }
    return $response;
  }

  public static function updateFormCircle(
    $register_id,
    $resp_status,
    $url_evidence,
    $obs_circle,
    $id_install
  ) {

    try {
      $Dat = self::getConnect('W')
        ->where([
          ['id', $register_id],
          ['status', '!=', 'T']
        ])->first();

      if ($resp_status == 'R') {
        $new_Dat = $Dat->replicate();
        $new_Dat->status = 'C';
        $new_Dat->father_circle = $register_id;
        $new_Dat->num_revisions++;
        $new_Dat->date_reg = date('Y-m-d H:i:s');
        $new_Dat->date_update = null;
        $new_Dat->email_mesa = null;
        $new_Dat->email_modify = null;
        $new_Dat->url_pdf_circle = null;
        $new_Dat->obs_circle = null;
        $new_Dat->save();

        //En la tabla de instalaciones actualizamos el circle_credit_id con el id del nuevo registro
        $id_new_circle = $new_Dat->id;
        Installations::getConnect('W')
          ->where('id', $id_install)
          ->update(['circle_credit_id' => $id_new_circle]);
      }

      $Dat->status = $resp_status;
      $Dat->url_pdf_circle = $url_evidence;
      $Dat->email_modify = session('user');
      $Dat->date_update = date('Y-m-d H:i:s');
      $Dat->email_mesa = session('user');
      $Dat->obs_circle = $obs_circle;
      $Dat->save();

      $response = ['success' => true, 'title' => "Se actualizaron los datos del formulario del cliente", 'msg' => "Actualizados los datos correctamente", 'icon' => "success", 'code' => 'OK'];
    } catch (Exception $e) {
      $txmsg = "No se pudo registrar la apreciacion crediticia para el formulario: " . $register_id . " - (MCC94) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      $response = ['success' => false, 'title' => "No se pudo actualizar el registro del formulario del cliente", 'msg' => $txmsg, 'icon' => "error", 'code' => 'ERR_DAT'];
    }
    return $response;
  }
}
