<?php

namespace App\Models;

use App\Models\CircleCredit;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class CircleCreditRequest extends Model
{
  use HasFactory;

  protected $table = 'islim_circle_credit_request';

  protected $fillable = [
    'id',
    'url_token',
    'seller_email',
    'date_reg',
    'status',
    'date_process',
    'dni_client',
    'verify_id',
    'url',
    'date_intro',
    'email_modify',
    'date_update',
    'father_circle_id'];

  public $incrementing = false;

  public $timestamps = false;

  /**
   * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
   * @param String $typeCon
   *
   * @return App\Models\CircleCreditRequest
   */
  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new self;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

      return $obj;
    }
    return null;
  }

  private static function newRequestForm($clientDni, $verify_id, $fromFather = false)
  {
    try {
      $url = uniqid() . time();
      $urlFull = env('SITE_WEB_NETWEY') . 'circle/' . $url;

      if (!filter_var($fromFather, FILTER_VALIDATE_BOOLEAN)) {
        $fromFather = null;
      }

      $register = self::getConnect('W')
        ->insertGetId([
          'url_token' => $url,
          'seller_email' => session('user'),
          'date_reg' => date('Y-m-d H:i:s'),
          'dni_client' => $clientDni,
          'verify_id' => $verify_id,
          'url' => $urlFull,
          'father_circle_id' => $fromFather]);

      $response['success'] = true;
      $response['title'] = "Nueva solicitud de formulario de circulo de credito creada";
      $response['msg'] = $register;
      $response['url'] = $urlFull;
      $response['icon'] = "success";
      $response['code'] = "OK";
    } catch (Exception $e) {
      $txmsg = "No se pudo registrar el nuevo formulario de circulo de credito para el cliente: " . $clientDni . " - (MCCR70) " . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      $response = ['success' => false, 'title' => "No se pudo procesar la solicitud de registro", 'msg' => $txmsg, 'icon' => "error", 'code' => 'ERR_DAT'];
    }
    return $response;
  }

  public static function newRequestFormCircle($clientDni, $verify_id, $fromFather)
  {
    $response = ['success' => false, 'title' => "No se pudo registrar el nuevo formulario de circulo de credito", 'msg' => "Fallo el registro de datos", 'icon' => "error", 'code' => 'ERR_DAT'];

    //Si piden nuevos enlaces y que no ha sido procesado el formulario se eliminan los enlaces
    //

    try {
      $previus = self::getConnect('W')
        ->select('islim_circle_credit_request.*')
        ->leftJoin(
          'islim_circle_credit',
          'islim_circle_credit.circle_request_id',
          'islim_circle_credit_request.id'
        )
        ->where([
          ['islim_circle_credit_request.status', '!=', 'T'],
          ['islim_circle_credit_request.dni_client', $clientDni]])
        ->whereIn('islim_circle_credit.status', ['PC', 'C'])
        ->get();

      if (count($previus)) {
        //Son registros que ya fueron o aprobados o rechazados
        foreach ($previus as &$item) {
          //Se elimina la peticion de enlace
          $item->status = "T";
          $item->date_update = date('Y-m-d H:i:s');
          $item->save();

          $Dat = CircleCredit::getConnect('W')
            ->where('circle_request_id', $item->id)
            ->update([
              'status' => 'T',
              'email_modify' => session('user'),
              'date_update' => date('Y-m-d H:i:s'),
              'obs_circle' => "El prospecto tenia un formulario cargado pero sin revision y el vendedor envio un nuevo formulario al prospecto",
            ]);
        }

        //pido crear uno nuevo
        $response = self::newRequestForm($clientDni, $verify_id, $fromFather);

      } else {
        // posiblemente existan registros en status de creado
        $isLoad = self::getConnect('R')
          ->select('islim_circle_credit_request.*')
          ->leftJoin(
            'islim_circle_credit',
            'islim_circle_credit.circle_request_id',
            'islim_circle_credit_request.id'
          )
          ->where([
            ['islim_circle_credit_request.status', '!=', 'T'],
            ['islim_circle_credit_request.dni_client', $clientDni]])
          ->whereIn('islim_circle_credit.status', ['C', 'PC'])
          ->first();

        if (!empty($isLoad)) {
          $response['success'] = true;
          $response['title'] = "Reenvio de formulario de circulo de credito ya cargado. Se debe esperar que sea revisado";
          $response['msg'] = $isLoad;
          $response['url'] = $isLoad->url;
          $response['icon'] = "success";
          $response['code'] = "OK";
        } else {

          //No existe se crea uno desde cero
          $response = self::newRequestForm($clientDni, $verify_id);
        }
      }
    } catch (Exception $e) {
      $txmsg = "No se pudo obtener el registro de solicitud de formulario de circulo de credito para el cliente: " . $clientDni . " - (MCCR134) " . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      $response['msg'] = $txmsg;
    }

    return $response;
  }

  public static function getRegisterById($id)
  {
    return self::getConnect('R')
      ->where([
        ['id', $id],
        ['status', '!=', 'T']])
      ->first();
  }

  public static function getLastRegisterByDni($client_dni)
  {
    return self::getConnect('R')
      ->where([
        ['dni_client', $client_dni],
        ['status', '!=', 'T']])
      ->orderBy('id', 'DESC')
      ->first();
  }

}
