<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class FiberPaymentForce extends Model
{
  use HasFactory;

  protected $table = 'islim_fiber_payment_force';

  protected $fillable = [
    'id',
    'code_url',
    'status',
    'type',
    'date_reg',
    'date_acept',
    'dni_client',
    'pack_id',
    'date_instalation',
    'schedule',
    'url_contract',
    'acept_mail',
    'bundle_id',
    'in_use'];

  public $timestamps = false;

  /**
   * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
   * @param String $typeCon
   *
   * @return App\Models\Client_document
   */
  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new self;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

      return $obj;
    }
    return null;
  }

/**
 * [newUrlQr Genera un nuevo QR para un cliente especifico o retorna el ultimo pendiente por aceptar]
 * @param  [type] $dni              [dni del cliente]
 * @param  [type] $pack             [id del pack de fibra]
 * @param  [type] $date_instalation [fecha que solicito el cliente para instalar]
 * @param  [type] $schedule         [horario que indico el cliente]
 * @param  [type] $type             [contrato inicial o final]
 * @param  [type] $contract_url     [url de s3 del contrato]
 * @param  [type] $bundle_id        [id del bundle si corresponde]
 * @return [type]                   [description]
 */
  public static function newUrlQr($dni, $pack, $date_instalation, $schedule, $type = "START", $dataContract = [], $bundle_id)
  {
    try {
      $QrData = self::getConnect('R')
        ->where([
          ['dni_client', $dni],
          ['type', $type],
          ['pack_id', $pack],
          ['bundle_id', $bundle_id],
          ['date_instalation', $date_instalation],
          ['schedule', $schedule]])
        ->where(function ($query) {
          $query->orWhere(function ($qry1) {
            $qry1->where('status', 'C')
              ->whereNull('date_acept');
          })
            ->orWhere(function ($qry2) {
              $qry2->where('status', 'A')
                ->whereNotNull('date_acept');
            });
        });

      if ($type == "START") {
        $QrData = $QrData->where('in_use', 'N');
      }

      $QrData = $QrData->orderBy('id', 'DESC')
        ->first();

      if (!empty($QrData)) {
        //Remover los contratos previos que puedan existir
        $remover = self::deleteOldForcer($QrData->id, $dni, $type);

        //Existe un Qr pendiente por aceptar o por asociar
        return array('success' => true, 'url' => $QrData->code_url, 'id' => $QrData->id);
      }
    } catch (Exception $e) {
      $txmsg = "No se pudo actualizar el registro del contrato de adhesion del cliente: " . $dni . " - (MFPF97) " . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }

    if ($type == "START") {
      //Generamos el archivo pdf del contrato
      $contract_url = self::generateContract($dataContract);
      if (!$contract_url['success']) {
        return array('success' => false, 'html' => '', 'htmlShare' => '', 'msg' => $contract_url['msg'], 'icon' => 'warning', 'code' => 'FAIL_URL');
      } else {
        $contract_url = $contract_url['url'];
      }
    } else {
      $contract_url = null;
    }

    $url = uniqid() . time();
    $data = [
      'code_url' => $url,
      'type' => $type,
      'date_reg' => date('Y-m-d H:i:s'),
      'dni_client' => $dni,
      'pack_id' => $pack,
      'bundle_id' => $bundle_id,
      'date_instalation' => $date_instalation,
      'schedule' => $schedule,
      'url_contract' => $contract_url,
    ];
    try {
      $id = self::getConnect('W')->insertGetId($data);
      $remover = self::deleteOldForcer($id, $dni, $type);
      return array('success' => true, 'url' => $url, 'id' => $id);

    } catch (Exception $e) {
      $Txmsg = 'Error al insertar el url del contrato. ' . (String) json_encode($e->getMessage());
      Log::error($Txmsg);
      return array('success' => false, 'msg' => $Txmsg);
    }
  }

  /**
   * [getUrlQr Obtiene el url para ver y aceptar los terminos y condiciones del servicio de fibra]
   * @param  [type] $dni  [description]
   * @param  string $type [description]
   * @return [type]       [description]
   */
  public static function getUrlQr($dni, $type = false, $status = [], $idUrl = false)
  {
    $qr = self::getConnect('R')
      ->where([
        ['dni_client', $dni]]);

    if ($type) {
      $qr = $qr->where('type', $type);

      //Se deshabilita de momento por el tema de que se estabilice la funcion de generacion de contratos
      /*if ($type == "START") {
    if ($idUrl) {
    $qr = $qr->where('in_use', 'Y');

    } else {
    $qr = $qr->where('in_use', 'N');
    }
    }*/
    }
    if (count($status) > 0) {
      $qr = $qr->whereIn('status', $status);
    }
    if ($idUrl) {
      $qr = $qr->where('id', $idUrl);
    }
    $qr = $qr->orderBy('id', 'DESC');

    return $qr->first();
  }

  public static function generateContract($request)
  {
    if (!empty($request->dni) && !empty($request->date_instalation) && !empty($request->schedule) && !empty($request->city_id) && !empty($request->reference) && !empty($request->typeIdentity) && !empty($request->identity)) {

      if (!empty($request->pack_bundle)) {
        //Log::info('Paquete bundle para contrato');
        $pack = Pack::getActivePackById($request->pack);
        $InfoBundle = Bundle::getDetailBundleAlta($request->pack_bundle);

        $infoAllBundle = json_decode(json_encode($InfoBundle['data']));

        //Titulo del bundle
        /*$pack = new \stdClass;
        if (isset($infoAllBundle->general->title)) {
        $pack->title = $infoAllBundle->general->title;
        } else {
        return array('success' => false, 'msg' => "Error al generar contrato de adhesion ya que no se pudo determinar titulo del combo");
        }*/

        $packprice = new \stdClass;
        if (isset($infoAllBundle->general->total_payment)) {
          $packprice->total_price = $infoAllBundle->general->total_payment;
        } else {
          return array('success' => false, 'msg' => "Error al generar contrato de adhesion ya que no se pudo determinar el precio total del combo");
        }

      } else {
        //Log::info('Paquete fibra para contrato');
        $pack = Pack::getActivePackById($request->pack);
        $packprice = PackPrices::getPackPriceByPackId($request->pack);
      }

      $dateIns = Carbon::createFromFormat('Y-m-d', $request->date_instalation)->startOfDay();

      $addressFormated = $request->state . ', ' . $request->city . ', ' . $request->muni . ', ' . $request->colony . ', ' . $request->route . ', ' . $request->numberhouse . '. Referencia: ' . $request->reference;
      $addressFormated = str_replace(' ,', '', $addressFormated);

      $current_date = Carbon::now()->locale('es');

      $city = FiberCity::getCityNameById($request->city_id);
      $client = Client::getClientByDNI($request->dni);

      if (!empty($client) && !empty($pack) && !empty($packprice) && !empty($city)) {

        $data = [
          'client_name' => ucwords($client->name),
          'client_lname' => ucwords($client->last_name),
          'client_phonehome' => $client->phone_home,
          'client_phone' => $client->phone,
          'address' => $addressFormated,
          'date' => $dateIns->format('d-m-Y'),
          'schedule' => $request->schedule,
          'price' => $packprice->total_price,
          'pack' => $pack->title,
          'city' => $city->location,
          'now' => $current_date,
          'doc_type' => $request->typeIdentity,
          'doc_id' => $request->identity,
        ];

        try {

          $pdf = \PDF::loadView('docs.fiber_force_contract', compact('data'));

          $doc_path = 'contracts/' . $request->dni . date('YmdHis') . '.pdf';
          Storage::disk('s3-fiber-contract')->put(
            $doc_path,
            $pdf->download()->getOriginalContent(),
            'public'
          );
          $ruta = (String) Storage::disk('s3-fiber-contract')->url($doc_path);

          return array('success' => true, 'url' => $ruta);
        } catch (Exception $e) {
          $txmsg = 'Error al generar el contrato de adhesión del cliente. ' . (String) json_encode($e->getMessage());
          Log::error($txmsg);
          return array('success' => false, 'msg' => $txmsg);
        }

      } else {
        return array('success' => false, 'msg' => "Fallo la obtención de los datos de la creación del contrato de adhesión");
      }
    } else {
      return array('success' => false, 'msg' => "Faltan datos para la creación del contrato de adhesión (MFPF_252)");
    }
  }

/**
 * [deleteOldForcer Eliminamos los contratos previos que no se aceptaron]
 * @param  [type] $qrActual [description]
 * @param  [type] $dni      [description]
 * @param  [type] $type     [description]
 * @return [type]           [description]
 */
  public static function deleteOldForcer($qrActual, $dni, $type)
  {
    try {
      $dele = self::getConnect('W')
        ->where([
          ['type', $type],
          ['dni_client', $dni],
          ['status', 'C'],
          ['id', '!=', $qrActual]])
        ->update(['status' => 'T']);
      return array('success' => true, 'msg' => "Eliminacion realizada");

    } catch (Exception $e) {
      $txmsg = 'Error al descartar los enlaces de contratos previos que no fueron procesados por el cliente. ' . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }

  public static function NotifyUseForcer($register)
  {
    try {
      $updt = self::getConnect('W')
        ->where([
          ['id', $register]])
        ->update(['in_use' => 'Y']);
      return array('success' => true, 'msg' => "Notificacion de uso de contrato realizada");

    } catch (Exception $e) {
      $txmsg = 'Error al actualizar registro de uso de enlace del contrato de adhesion. ' . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }
}
