<?php

namespace App\Models;

use App\Models\FiberQuestionResponse;
use App\Models\FiberQuestions;
use App\Models\FiberResponse;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class FiberQuestionResult extends Model
{
  protected $table = 'islim_fiber_questions_result';

  protected $fillable = [
    'installation_id',
    'question_response_id',
    'question_result'];

  public $timestamps = false;

/**
 * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
 * @param String $typeCon
 *
 * @return App\FiberQuestionResult
 */
  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new self;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');
      return $obj;
    }
    return null;
  }

  public static function insertAnswers($install, $answers, $typeQuestion, $destiny = "END")
  {

    $questions = FiberQuestions::getAllQuestion($destiny, $typeQuestion);

    foreach ($questions as $value) {

      if ($value->type == 'MS') {

        $options = FiberQuestionResponse::getByQuestion($value->id);

        foreach ($options as $option) {

          if (!empty($answers['question-' . $value->id . '-' . $option->id])) {
            try {
              if ($typeQuestion == 'SUPPORT') {
                self::getConnect('W')
                  ->insert([
                    'support_id' => $install,
                    'question_response_id' => $option->id,
                    'question_result' => null,
                  ]);
              } elseif ($typeQuestion == 'INSTALL') {
                self::getConnect('W')
                  ->insert([
                    'installation_id' => $install,
                    'question_response_id' => $option->id,
                    'question_result' => null,
                  ]);
              } else {
                self::getConnect('W')
                  ->insert([
                    'from_circle_credit_id' => $install,
                    'question_response_id' => $option->id,
                    'question_result' => null,
                  ]);
              }
            } catch (Exception $e) {
              $txmsg = 'Error al insertar una respuesta a una pregunta. ' . (String) json_encode($e->getMessage());
              Log::error($txmsg);
              return array('success' => false, 'msg' => $txmsg);
            }
          }
        }
      } else {

        if (!empty($answers['question-' . $value->id])) {
          if ($value->type == 'TX') {

            $question_response_id = FiberQuestionResponse::getQuestionTX($value->id);

            if (!empty($question_response_id)) {

              try {
                if ($typeQuestion == 'SUPPORT') {
                  self::getConnect('W')
                    ->insert([
                      'support_id' => $install,
                      'question_response_id' => $question_response_id->id,
                      'question_result' => $answers['question-' . $value->id],
                    ]);
                } elseif ($typeQuestion == 'INSTALL') {

                  self::getConnect('W')
                    ->insert([
                      'installation_id' => $install,
                      'question_response_id' => $question_response_id->id,
                      'question_result' => $answers['question-' . $value->id],
                    ]);
                } else {
                  self::getConnect('W')
                    ->insert([
                      'from_circle_credit_id' => $install,
                      'question_response_id' => $question_response_id->id,
                      'question_result' => $answers['question-' . $value->id],
                    ]);
                }

              } catch (Exception $e) {
                $txmsg = 'Error al insertar una respuesta a una pregunta. ' . (String) json_encode($e->getMessage());
                Log::error($txmsg);
                return array('success' => false, 'msg' => $txmsg);
              }
            }
          } else {
            try {
              if ($typeQuestion == 'SUPPORT') {
                self::getConnect('W')
                  ->insert([
                    'support_id' => $install,
                    'question_response_id' => $answers['question-' . $value->id],
                    'question_result' => @$answers['question-' . $value->id . '-ot'],
                  ]);
              } elseif ($typeQuestion == 'INSTALL') {

                self::getConnect('W')
                  ->insert([
                    'installation_id' => $install,
                    'question_response_id' => $answers['question-' . $value->id],
                    'question_result' => @$answers['question-' . $value->id . '-ot'],
                  ]);
              } else {
                self::getConnect('W')
                  ->insert([
                    'from_circle_credit_id' => $install,
                    'question_response_id' => $answers['question-' . $value->id],
                    'question_result' => @$answers['question-' . $value->id . '-ot'],
                  ]);
              }

            } catch (Exception $e) {
              $txmsg = 'Error al insertar una respuesta a una pregunta. ' . (String) json_encode($e->getMessage());
              Log::error($txmsg);
              return array('success' => false, 'msg' => $txmsg);
            }
          }
        }
      }
    }
    return array('success' => true, 'msg' => 'OK');
  }

  public static function getAnswersById($id, $dest = false, $type)
  {
    if ($id) {
      $answers = FiberQuestions::getQuestionForInstall($id, $type);
      foreach ($answers as $question) {

        $question->answers = FiberResponse::getQuestionInstall($question->id, $id, $type);
      }
      if (count($answers) > 0 && $dest) {
        $answers = $answers->where('destiny', $dest);
      }

      return $answers;
    }

    return [];
  }
}
