<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class FiberSupport extends Model
{
  use HasFactory;
  protected $table = 'islim_fiber_support';

  protected $fillable = [
    'id',
    'ticket',
    'msisdn',
    'dni_client',
    'address',
    'lat',
    'lng',
    'tipification_support',
    'description_fail_support',
    'date_support',
    'schedule',
    'status',
    'support_father',
    'user_collector',
    'user_dispatcher',
    'date_dispatcher',
    'user_installer',
    'soluction_fail',
    'date_process',
    'typification_cancel',
    'origin_collector',
    'origin_dispatcher'

  ];

  public $timestamps = false;

  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new self;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');
      return $obj;
    }
    return null;
  }

  public static function getPendingDatesForInstaller($installer, $dateS = false, $dateE = false, $typeUser = "installer")
  {
    $dates = self::getConnect('R')
      ->select(
        'islim_fiber_support.id',
        'islim_fiber_support.msisdn',
        'islim_fiber_support.dni_client',
        'islim_fiber_support.address as address_instalation',
        'islim_fiber_support.date_reg',
        DB::raw("DATE_FORMAT( islim_fiber_support.date_support, '%d-%m-%Y' ) AS date_instalation"),
        'islim_fiber_support.schedule',
        'islim_fiber_support.user_installer as installer',
        //'islim_installations.price',
        'islim_clients.name',
        'islim_clients.last_name'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_fiber_support.dni_client'
      )
      ->where('islim_fiber_support.status', 'A')
      ->whereIn('islim_fiber_support.status_control', ['AB', 'AI']);

    if ($typeUser == 'installer') {
      //Asignadas a alguien para ser instalado el equipo
      $dates = $dates->where('islim_fiber_support.user_installer', $installer);
    } elseif ($typeUser == 'ListinstallerBoss') {
      //Lista de instalaciones pendiente de los instaladores de un jefe dado
      $dates = $dates->where('islim_fiber_support.installer_boss', $installer)
        ->whereNotNull('islim_fiber_support.user_installer')
        ->whereNotIn('islim_fiber_support.user_installer', [$installer]);
    } elseif ($typeUser == 'installerBoss') {
      //Lista de citas agendadas pero por asignar a un instalador
      $dates = $dates->where('islim_fiber_support.installer_boss', $installer)
        ->whereNull('islim_fiber_support.user_installer');
    }

    if ($dateS && $dateE) {

      $dates->where([
        ['islim_fiber_support.date_support', '>=', $dateS],
        ['islim_fiber_support.date_support', '<=', $dateE]]);
    } else {
      if ($dateS) {
        $dates->where('islim_fiber_support.date_support', '>=', $dateS);
      } elseif ($dateE) {
        $dates->where('islim_fiber_support.date_support', '<=', $dateE);
      }
    }

    return $dates->orderBy('islim_fiber_support.date_support', 'ASC')->get();
  }

  public static function getSupportCaduce($installer, $typeUser = "installer")
  {
    $hoy = date("Y-m-d");
    $LastDay = date("Y-m-d", strtotime($hoy . "- 1 day"));
    $data = self::getConnect('R')
      ->where([
        ['status', 'A'],
        ['date_support', $LastDay],
        ['status_control', 'C']]);

    if ($typeUser == "installer") {
      //citas que se tienen asignadas como instalador
      $data = $data->where('user_installer', $installer);
    } elseif ($typeUser == "ListinstallerBoss") {
      //citas que se tenian instalador asignado
      $data = $data->where([
        ['installer_boss', $installer],
        ['user_installer', '!=', $installer]]);

    } elseif ($typeUser == "installerBoss") {
      //Citas que se tenian en agenda pendiente de asignar
      $data = $data->whereNull('installer_boss')
        ->whereNull('user_installer');
    }
    return $data->count();
  }

  public static function getConfirmDatesForInstaller($installer,$idfiberzone,$idfibercity)
  {
    $dateE = date("Y-m-d");
    $dates = self::getConnect('R')
      ->select(
        'islim_fiber_support.id',
        'islim_fiber_support.dni_client',
        'islim_fiber_support.msisdn',
        'islim_fiber_support.address as address_instalation',
        'islim_fiber_support.date_reg',
        DB::raw("DATE_FORMAT( islim_fiber_support.date_support, '%d-%m-%Y' ) AS date_instalation"),
        'islim_fiber_support.schedule',
        'islim_fiber_support.user_installer',
        //'islim_installations.price',
        'islim_clients.name',
        'islim_clients.last_name'
      )
      ->join('islim_clients','islim_clients.dni','islim_fiber_support.dni_client')
      ->join('islim_installations','islim_installations.msisdn','islim_fiber_support.msisdn')
      ->where('islim_fiber_support.status', 'A')
      ->where('islim_fiber_support.status_control', 'CC')
      ->where('islim_installations.status','P')
      ->where('islim_installations.id_fiber_city',$idfibercity)
      ->where('islim_installations.id_fiber_zone',$idfiberzone);

    if ($dateE) {

         $dates = $dates->where('islim_fiber_support.date_support', '>=', $dateE);
      }


    return $dates->orderBy('islim_fiber_support.date_support', 'ASC')->get();
  }

  public static function getConfirmSupportCaduce($installer,$idfiberzone,$idfibercity){
    $hoy = date("Y-m-d");
    $LastDay = date("Y-m-d", strtotime($hoy . "- 1 day"));
    $data = self::getConnect('R')
      ->where([
        ['status', 'A'],
        ['date_support', $LastDay],
        ['status_control', 'C']]);

    if ($typeUser == "installer") {
      //citas que se tienen asignadas como instalador
      $data = $data->where('user_installer', $installer);
    } elseif ($typeUser == "ListinstallerBoss") {
      //citas que se tenian instalador asignado
      $data = $data->where([
        ['installer_boss', $installer],
        ['user_installer', '!=', $installer]]);

    } elseif ($typeUser == "installerBoss") {
      //Citas que se tenian en agenda pendiente de asignar
      $data = $data->whereNull('installer_boss')
        ->whereNull('user_installer');
    }
    return $data->count();
  }

  public static function getDateDetailByID($id)
  {
    //leftJoin por el listado de instalacion por definir instalador
    return self::getConnect('R')
      ->select(
        'islim_installations.clients_dni',
        'islim_installations.id as installations_id',
        'islim_clients.name',
        'islim_clients.last_name',
        'islim_clients.email',
        'islim_clients.phone_home',
        'islim_clients.phone',
        'islim_installations.address_instalation',
        'islim_installations.photo',
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'islim_installations.date_install',
        'islim_fiber_support.user_installer as installer',
        'islim_fiber_support.installer_boss',
        'islim_installations.seller',
        'islim_installations.paid',
        'islim_installations.status',
        'islim_installations.inv_article_id',
        'islim_installations.msisdn',
        'islim_installations.price',
        'islim_installations.route',
        'islim_installations.house_number',
        'islim_installations.colony',
        'islim_installations.municipality',
        'islim_installations.reference',
        'islim_installations.lat',
        'islim_installations.lng',
        'islim_installations.schedule',
        'islim_installations.id_fiber_zone',
        'islim_installations.config_conex',
        'islim_fiber_zone.name AS name_zone',
        'islim_fiber_support.id',
        'islim_fiber_support.status as status_support',
        'islim_fiber_support.date_support',
        'islim_fiber_support.schedule as schedule_support',
        'islim_fiber_support.description_fail_support as description_fail_support',
        'islim_users.name as name_inst',
        'islim_users.last_name as last_name_inst',
        'islim_services.title as service',
        'islim_services.is_payment_forcer',
        'islim_packs.title as pack',
        'islim_installations.pack_id',
        'islim_installations.service_id',
        'islim_inv_articles.title as article',
        'state_tbl.location as state',
        'city_tbl.location as city',
        'islim_client_document.identification as doc_id',
        'islim_client_document.photo_front',
        'islim_client_document.type as doc_type',
        'islim_installations.payment_force_end',
        'islim_installations.pack_price_id',
        'islim_installations.bundle_id_payment',
        'islim_installations.payer_email',
        'islim_installations.bundle_id',
        'islim_installations.referred_dn',
        'islim_installations.client_bundle_id',
        'islim_installations.unique_transaction',
        'islim_installations.payment_url_subscription',
        'islim_installations.type_payment',
        'islim_installations.obs_activate',
        'islim_fiber_typification_desc.description as typification_descripcion'

      )
      ->join(
        'islim_installations',
        'islim_installations.msisdn',
        'islim_fiber_support.msisdn'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_installations.clients_dni'
      )
      ->leftJoin(
        'islim_users',
        'islim_users.email',
        'islim_fiber_support.user_installer'
      )
      ->join(
        'islim_services',
        'islim_services.id',
        'islim_installations.service_id'
      )
      ->join(
        'islim_inv_articles',
        'islim_inv_articles.id',
        'islim_installations.inv_article_id'
      )
      ->join(
        'islim_packs',
        'islim_packs.id',
        'islim_installations.pack_id'
      )
      ->join(
        'islim_localy_mexico as state_tbl',
        'state_tbl.id',
        'islim_installations.id_state'
      )
      ->join(
        'islim_fiber_city',
        'islim_fiber_city.id',
        'islim_installations.id_fiber_city'
      )
      ->join(
        'islim_localy_mexico as city_tbl',
        'city_tbl.id',
        'islim_fiber_city.localy_id'
      )
      ->join(
        'islim_fiber_zone',
        'islim_fiber_zone.id',
        'islim_installations.id_fiber_zone'
      )
      ->leftJoin(
        'islim_client_document',
        'islim_client_document.dni',
        'islim_installations.clients_dni'
      )
      ->leftjoin(
        'islim_fiber_typification',
        'islim_fiber_support.tipification_support',
        'islim_fiber_typification.id'
      )
      ->leftjoin(
        'islim_fiber_typification_desc',
        'islim_fiber_typification_desc.id',
        'islim_fiber_typification.id_typification_desc'
      )

      ->where('islim_fiber_support.id', $id)
      ->first();
  }

  public static function getInstallToEdit($id)
  {
    try {
      return self::getConnect('W')
        ->where('id', $id)
        ->first();
    } catch (Exception $e) {
      $txmsg = "No se pudo leer el Soporte (MS306) " . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      return false;
    }
  }

  public static function getAddressSupport($id)
  {
    return self::getConnect('R')
      ->select(
        'islim_installations.address_instalation',
        'islim_fiber_support.date_reg',
        DB::raw("DATE_FORMAT( islim_fiber_support.date_support, '%d-%m-%Y' ) AS date_instalation"),
        'islim_fiber_support.schedule',
        'islim_installations.id_fiber_zone',
        'islim_fiber_support.installer_boss',
        'islim_clients.name',
        'islim_clients.last_name',
        'islim_clients.email',
        'islim_clients.phone_home',
        'islim_installations.bundle_id',
        'islim_fiber_support.msisdn',
        'islim_fiber_support.sms_id'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_fiber_support.dni_client'
      )
      ->join(
        'islim_installations',
        'islim_fiber_support.msisdn',
        'islim_installations.msisdn'
      )
      ->where('islim_fiber_support.id', $id)
      ->first();
  }

  public static function markCancelled($dataCanceller)
  {
    try {
      self::getConnect('W')
        ->where('id', $dataCanceller->cita)
        ->update([
          'status_control' => 'SR',
          'date_mod' => date('Y-m-d H:i:s'),
          'user_mod' => session('user'),
          'reason_delete' => !empty($dataCanceller->msgTypi) ? $dataCanceller->msgTypi : null,
          'typification_cancel' => $dataCanceller->typification]);
      return array('success' => true, 'msg' => 'OK');

    } catch (Exception $e) {
      $txmsg = "No se pudo modificar el status de control de la instalacion (543) " . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }
  public function numeroTicket(){
    $supportNumber = self::getConnect('R')
      ->select('islim_fiber_support.ticket')
      ->orderBy('islim_fiber_support.id', 'desc')
      ->first();
      if (empty($supportNumber)) {
        $number = 0;
      } else {
        $number = $supportNumber->ticket;
      }
      //$number++;
      $number = str_pad($number, 5, '0', STR_PAD_LEFT);
      return $number;
  }
  public static function reagendaCita($data){
    $instancia = new self();
    $citaPrevia = self::getConnect('W')->find($data->cita);
    $citaActual = $citaPrevia->replicate();
    $citaActual->ticket = $instancia->numeroTicket();
    $citaActual->support_father = $data->cita;
    $citaActual->installer_boss = null;
    $citaActual->user_installer = null;
    $citaActual->user_mesa = session('user');
    $citaActual->user_dispatcher = null;
    $citaActual->date_dispatcher = null;
    $citaActual->user_mod = session('user');
    $citaActual->typification_cancel = null;
    $citaActual->reason_delete = null;
    $citaActual->date_support = $data->dia;
    $citaActual->date_reg = date('Y-m-d H:m:i');
    $citaActual->date_mod = date('Y-m-d H:m:i');
    $citaActual->schedule = $data->turno;
    $citaActual->soluction_fail = null;
    $citaActual->status_control = 'CC';
    $citaActual->sms_id=null;
    $citaActual->num_rescheduling=$citaPrevia->num_rescheduling + 1;
    $citaActual->save();
    $citaPrevia->reason_delete = $data->msgTypi;
    $citaPrevia->typification_cancel = $data->typification;
    $citaPrevia->status_control = 'SR';
    $citaPrevia->status = 'R';
    $citaPrevia->save();
    return ['cita_actual'=>$citaActual->id,'cita_previa'=>$citaPrevia->id];
  }

  public static function getInstallerAsignedUser($fiberZone, $date, $installerBoss = false)
  {
      $data = self::getConnect('R')
          ->select(
              DB::raw('COUNT(islim_fiber_support.schedule) as cant_turno'),
              'islim_fiber_support.user_installer as installer',
              DB::raw("DATE_FORMAT(islim_fiber_support.date_support, '%d-%m-%Y') AS date_schedule"),
              'islim_fiber_support.schedule',
              'islim_installations.id_fiber_zone'
          )
          ->join('islim_installations', 'islim_installations.msisdn', 'islim_fiber_support.msisdn')
          ->where([
              ['islim_fiber_support.status', 'A'],
              [DB::raw("DATE_FORMAT(islim_fiber_support.date_support, '%d-%m-%Y')"), '=', $date],
              ['islim_installations.id_fiber_zone', $fiberZone]
          ])
          ->whereNotIn('islim_fiber_support.status_control', ['C', 'SR']);

      if (!$installerBoss) {
          // Instaladores donde asigno el jefe
          $data = $data->where('islim_fiber_support.installer_boss', session('user'))
              ->whereNotNull('islim_fiber_support.user_installer');
      } else {
          // Las instalaciones que el jefe se auto asignó
          $data = $data->where('islim_fiber_support.user_installer', session('user'));
      }

      return $data->groupBy('islim_fiber_support.user_installer', 'islim_fiber_support.schedule')
          ->get();
  }


  public static function getSupportActiveByDN($msisdn = null)
    {
      if($msisdn) {
        $support = FiberSupport::getConnect('R')->select(
          DB::raw("(CONCAT(islim_clients.name, CONCAT(' ', islim_clients.last_name))) as client_fullname"),
          'islim_clients.phone_home',
          'islim_clients.phone',
          'islim_clients.email as client_email',
          'islim_fiber_support.dni_client',
          'islim_fiber_support.id',
          'islim_fiber_support.ticket',
          'islim_fiber_support.msisdn',
          'islim_fiber_support.address',
          'islim_fiber_support.lat',
          'islim_fiber_support.lng',
          'islim_fiber_support.tipification_support',
          'islim_fiber_typification_desc.description as typification_desc',
          'islim_fiber_support.description_fail_support',
          'islim_fiber_support.date_support',
          'islim_fiber_support.schedule',
          'islim_fiber_support.status',
          'islim_fiber_support.status_control',
          'islim_fiber_support.support_father',
          'islim_fiber_support.user_collector',
          'islim_fiber_support.user_mesa',
          'islim_fiber_support.user_dispatcher',
          'islim_fiber_support.date_dispatcher',
          'islim_fiber_support.installer_boss',
          'islim_fiber_support.user_installer',
          'islim_fiber_support.soluction_fail',
          'islim_fiber_support.date_process',
          'islim_fiber_support.typification_cancel',
          'islim_fiber_support.origin_collector',
          'islim_fiber_support.date_reg',
          'islim_fiber_support.num_rescheduling',
          'islim_fiber_support.tipification_support_installer',
          'typification_desc.description as tipification_support_installer_desc',
          'islim_fiber_support.reason_delete',
          'islim_fiber_support.sms_id',
          'islim_fiber_zone.name as zone_name',
          'islim_installations.id_fiber_zone as zone_id',
          'islim_localy_mexico.location',
          'islim_fiber_city_zone.id as city_zone_id')
        ->join('islim_installations', 'islim_installations.msisdn', 'islim_fiber_support.msisdn')
        ->join('islim_clients', 'islim_clients.dni', 'islim_installations.clients_dni')
        ->join('islim_fiber_zone', 'islim_fiber_zone.id', 'islim_installations.id_fiber_zone')
        ->join('islim_fiber_city', 'islim_fiber_city.id', 'islim_installations.id_fiber_city')
        ->join('islim_fiber_city_zone', function ($join) {
            $join->on('islim_fiber_city_zone.fiber_zone_id', '=', 'islim_installations.id_fiber_zone')
            ->on('islim_fiber_city_zone.fiber_city_id', '=', 'islim_installations.id_fiber_city');
        })
        ->join('islim_localy_mexico', 'islim_localy_mexico.id', 'islim_fiber_city.localy_id')

        ->join('islim_fiber_typification', 'islim_fiber_typification.id', 'islim_fiber_support.tipification_support')
        ->leftjoin(
        'islim_fiber_typification_desc',
        'islim_fiber_typification_desc.id',
        'islim_fiber_typification.id_typification_desc'
        )

        ->leftjoin('islim_fiber_typification as fiber_typification_solution', 'fiber_typification_solution.id', 'islim_fiber_support.tipification_support_installer')
        ->leftjoin(
        'islim_fiber_typification_desc as typification_desc',
        'typification_desc.id',
        'fiber_typification_solution.id_typification_desc'
        )


        ->where([['islim_fiber_support.msisdn', $msisdn],
                ['islim_fiber_support.status', 'A']])
        ->whereIn('islim_fiber_support.status_control', ['C', 'CC', 'AI', 'AB'])
        ->latest('date_reg')->first();

        //$support = $support
        //->orderBy('islim_fiber_support.date_support', 'asc')->first();
        //dd($supports->toSql());
        return $support;
      }
      return null;
    }
}
