<?php

namespace App\Models;

use App\Models\Bundle;
use App\Models\InstallationsBundle;
use App\Models\Pack;
use App\Models\PackPrices;
use App\Models\Service;
use App\Models\SMSNotification;
use App\Helpers\CommonHelpers;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Mail\mailInstallV2;
use Illuminate\Support\Facades\Mail;
use Exception;
use Carbon\Carbon;

class Installations extends Model
{
  use HasFactory;

  protected $table = 'islim_installations';

  protected $fillable = [
    'id',
    'clients_dni',
    'address_instalation',
    'pack_id',
    'service_id',
    'inv_article_id',
    'price',
    'seller',
    'installer',
    'installer_boss',
    'date_instalation',
    'paid',
    'photo',
    'is_migration',
    'migration_origin',
    'status',
    'date_reg',
    'user_mod',
    'date_mod',
    'date_install',
    'date_paid',
    'msisdn',
    'inv_detail_fiber_id',
    'route',
    'house_number',
    'colony',
    'municipality',
    'reference',
    'lat',
    'lng',
    'father_install',
    'num_rescheduling',
    'group_install',
    'schedule',
    'reason_delete',
    'id_fiber_zone',
    'id_state',
    'id_fiber_city',
    'config_conex',
    'typification_id',
    'payment_force_start',
    'payment_force_end',
    'photo_document',
    'type_payment',
    'payment_url_subscription',
    'pack_price_id',
    'bundle_id_payment',
    'payer_email',
    'notify_changer_service',
    'bundle_id',
    'referred_dn',
    'token_activate',
    'client_bundle_id',
    'obs_activate',
    'unique_transaction',
    'cant_attempts',
    'dateProcess',
    'processing_time_cron',
    'is_provisioned',
    'date_provisioned',
    'user_provisioned',
    'dataProvisioned',
    'circle_credit_id',
    'end_pi',
    'runway'
  ];

  public $timestamps = false;
  protected $casts = [
    'config_conex' => 'array',
    'obs_activate' => 'array',
    'dateProcess' => 'array',
    'dataProvisioned' => 'array'
  ];

  /**
   * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
   * @param String $typeCon
   *
   * @return App\Models\Installations
   */
  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new Installations;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

      return $obj;
    }
    return null;
  }

  public static function getPendingDatesForInstaller($installer, $dateS = false, $dateE = false, $typeUser = "installer")
  {
    $dates = self::getConnect('R')
      ->select(
        'islim_installations.id',
        'islim_installations.clients_dni',
        'islim_installations.address_instalation',
        'islim_installations.date_reg',
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'islim_installations.schedule',
        'islim_installations.installer',
        'islim_installations.price',
        'islim_clients.name',
        'islim_clients.last_name'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_installations.clients_dni'
      )
      ->where('islim_installations.status', 'A')
      ->whereIn('islim_installations.status_control', ['AB', 'AI']);

    if ($typeUser == 'installer') {
      //Asignadas a alguien para ser instalado el equipo
      $dates = $dates->where('islim_installations.installer', $installer);
    } elseif ($typeUser == 'ListinstallerBoss') {
      //Lista de instalaciones pendiente de los instaladores de un jefe dado
      $dates = $dates->where('islim_installations.installer_boss', $installer)
        ->whereNotNull('islim_installations.installer')
        ->whereNotIn('islim_installations.installer', [$installer]);
    } elseif ($typeUser == 'installerBoss') {
      //Lista de citas agendadas pero por asignar a un instalador
      $dates = $dates->where('islim_installations.installer_boss', $installer)
        ->whereNull('islim_installations.installer');
    }

    if ($dateS && $dateE) {

      $dates->where([
        ['islim_installations.date_instalation', '>=', $dateS],
        ['islim_installations.date_instalation', '<=', $dateE]
      ]);
    } else {
      if ($dateS) {
        $dates->where('islim_installations.date_instalation', '>=', $dateS);
      } elseif ($dateE) {
        $dates->where('islim_installations.date_instalation', '<=', $dateE);
      }
    }

    return $dates->orderBy('islim_installations.date_instalation', 'ASC')->get();
  }

  /**
   * [getPendingDatesForInstaller Retorna la cantidad de citas que se enviaron el dia anterior a reprogramar por incumplimiento]
   * @param  [type] $installer [description]
   * @param  string $typeUser  [description]
   * @return [type]            [description]
   */
  public static function getInstallationCaduce($installer, $typeUser = "installer")
  {
    $hoy = date("Y-m-d");
    $LastDay = date("Y-m-d", strtotime($hoy . "- 1 day"));
    $data = self::getConnect('R')
      ->where([
        ['status', 'A'],
        ['date_instalation', $LastDay],
        ['status_control', 'SR']
      ]);

    if ($typeUser == "installer") {
      //citas que se tienen asignadas como instalador
      $data = $data->where('installer', $installer);
    } elseif ($typeUser == "ListinstallerBoss") {
      //citas que se tenian instalador asignado
      $data = $data->where([
        ['installer_boss', $installer],
        ['installer', '!=', $installer]
      ]);
    } elseif ($typeUser == "installerBoss") {
      //Citas que se tenian en agenda pendiente de asignar
      $data = $data->whereNull('installer_boss')
        ->whereNull('installer');
    }
    return $data->count();
  }

  public static function getDateDetailByID($id)
  {
    //leftJoin por el listado de instalacion por definir instalador
    return self::getConnect('R')
      ->select(
        'islim_installations.clients_dni',
        'islim_clients.name',
        'islim_clients.last_name',
        'islim_clients.email',
        'islim_clients.phone_home',
        'islim_clients.phone',
        'islim_installations.address_instalation',
        'islim_installations.photo',
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'islim_installations.date_install',
        'islim_installations.installer',
        'islim_installations.installer_boss',
        'islim_installations.seller',
        'islim_installations.id',
        'islim_installations.paid',
        'islim_installations.status',
        'islim_installations.inv_article_id',
        'islim_installations.msisdn',
        'islim_installations.price',
        'islim_installations.route',
        'islim_installations.house_number',
        'islim_installations.colony',
        'islim_installations.municipality',
        'islim_installations.reference',
        'islim_installations.lat',
        'islim_installations.lng',
        'islim_installations.schedule',
        'islim_installations.id_fiber_zone',
        'islim_installations.id_fiber_city',
        'islim_fiber_zone.name AS name_zone',
        'islim_installations.config_conex',
        'islim_users.name as name_inst',
        'islim_users.last_name as last_name_inst',
        'islim_services.title as service',
        'islim_services.is_payment_forcer',
        'islim_packs.title as pack',
        'islim_installations.pack_id',
        'islim_installations.service_id',
        'islim_inv_articles.title as article',
        'state_tbl.location as state',
        'city_tbl.location as city',
        'islim_client_document.identification as doc_id',
        'islim_client_document.photo_front',
        'islim_client_document.type as doc_type',
        'islim_installations.payment_force_end',
        'islim_installations.pack_price_id',
        'islim_installations.bundle_id_payment',
        'islim_installations.payer_email',
        'islim_installations.bundle_id',
        'islim_installations.referred_dn',
        'islim_installations.client_bundle_id',
        'islim_installations.unique_transaction',
        'islim_installations.payment_url_subscription',
        'islim_installations.type_payment',
        'islim_installations.obs_activate',
        'islim_installations.runway',
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_installations.clients_dni'
      )
      ->leftJoin(
        'islim_users',
        'islim_users.email',
        'islim_installations.installer'
      )
      ->join(
        'islim_services',
        'islim_services.id',
        'islim_installations.service_id'
      )
      ->join(
        'islim_inv_articles',
        'islim_inv_articles.id',
        'islim_installations.inv_article_id'
      )
      ->join(
        'islim_packs',
        'islim_packs.id',
        'islim_installations.pack_id'
      )
      ->join(
        'islim_localy_mexico as state_tbl',
        'state_tbl.id',
        'islim_installations.id_state'
      )
      ->join(
        'islim_fiber_city',
        'islim_fiber_city.id',
        'islim_installations.id_fiber_city'
      )
      ->join(
        'islim_localy_mexico as city_tbl',
        'city_tbl.id',
        'islim_fiber_city.localy_id'
      )
      ->join(
        'islim_fiber_zone',
        'islim_fiber_zone.id',
        'islim_installations.id_fiber_zone'
      )
      ->leftJoin(
        'islim_client_document',
        'islim_client_document.dni',
        'islim_installations.clients_dni'
      )
      ->where('islim_installations.id', $id)
      ->first();
  }

  public static function getInstallToEdit($id)
  {
    try {
      return self::getConnect('W')
        ->where('id', $id)
        ->first();
    } catch (Exception $e) {
      $txmsg = "No se pudo leer la instalacion (MI306) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return false;
    }
  }

  public static function getInstallById($id)
  {
    try {
      return self::getConnect('W')
        ->select(
          'id',
          'installer',
          'seller',
          DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
          'schedule',
          'address_instalation',
          'pack_id',
          'service_id',
          'status',
          'paid',
          'inv_article_id',
          'price',
          'is_migration',
          'clients_dni',
          'id_fiber_city',
          'id_fiber_zone',
          'config_conex',
          'payment_force_start',
          'type_payment',
          'bundle_id',
          'referred_dn',
          'inv_detail_fiber_id',
          'msisdn',
          'client_bundle_id',
          'unique_transaction',
          'dateProcess',
          'payment_url_subscription',
          'pack_price_id',
          'bundle_id_payment',
          'runway'
        )
        ->where('id', $id)
        ->first();
    } catch (Exception $e) {
      $txmsg = "No se pudo leer la instalacion (MI363) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return null;
    }
  }

  public static function markAsInstalled($id, $msisdn, $prov = 'N')
  {
    try {
      if ($prov == 'Y') {
        self::getConnect('W')
          ->where('id', $id)
          ->update([
            'status' => 'PA',
            'date_install' => date('Y-m-d H:i:s'),
            'msisdn' => $msisdn,
            'token_activate' => null
          ]);
      } else {
        self::getConnect('W')
          ->where('id', $id)
          ->update([
            'status' => 'P',
            'date_install' => date('Y-m-d H:i:s'),
            'msisdn' => $msisdn
          ]);
      }

      return array('success' => true, 'msg' => 'OK');
    } catch (Exception $e) {
      $txmsg = "No se pudo procesar detalles de la instalacion_id " . $id . " - " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }

  public static function getPendingPay($seller, $filters = [])
  {
    $data = self::getConnect('R')
      ->select(
        'islim_installations.id',
        'islim_installations.clients_dni',
        'islim_installations.address_instalation',
        'islim_installations.date_reg',
        'islim_installations.date_install',
        'islim_installations.status',
        'islim_clients.name',
        'islim_clients.last_name',
        'islim_clients.phone_home'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_installations.clients_dni'
      )
      ->where([
        ['islim_installations.status', '<>', 'T'],
        ['islim_installations.seller', $seller],
        ['islim_installations.paid', 'N']
      ]);

    if (!empty($filters['status'])) {
      $data->where('islim_installations.status', $filters['status']);
    }

    $data->orderBy('islim_installations.date_install', 'ASC');

    return $data;
  }

  public static function markAsPaid($id)
  {
    try {
      self::getConnect('W')
        ->where('id', $id)
        ->update([
          'paid' => 'Y',
          'date_paid' => date('Y-m-d H:i:s')
        ]);
      return array('success' => true, 'msg' => 'OK');
    } catch (Exception $e) {
      $txmsg = "No se pudo actualizar la instalacion (381) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }

  /**
   * [getAddressInstalation Obtiene la informacion que sera enviada en el email de bienvenida]
   * @param  [type] $id  [description]
   * @return [type]      [description]
   */
  public static function getAddressInstalation($id)
  {
    return self::getConnect('R')
      ->select(
        'islim_installations.address_instalation',
        'islim_installations.date_reg',
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'islim_installations.schedule',
        'islim_installations.id_fiber_zone',
        'islim_installations.installer_boss',
        'islim_clients.name',
        'islim_clients.last_name',
        'islim_clients.email',
        'islim_clients.phone_home',
        'islim_installations.bundle_id'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_installations.clients_dni'
      )
      ->where('islim_installations.id', $id)
      ->first();
  }

  public static function getPendingInstalations($installer, $dateS = false, $dateE = false)
  {
    $dates = self::getConnect('R')
      ->select(
        'islim_installations.id',
        'islim_installations.address_instalation',
        DB::raw("DATE_FORMAT( islim_installations.date_reg, '%d-%m-%Y' ) AS date_instalation"),
        'islim_installations.schedule',
        'islim_installations.municipality',
        'islim_installations.num_rescheduling',
        'islim_clients.name',
        'islim_clients.last_name',
        DB::raw('TIMESTAMPDIFF(DAY, islim_installations.date_reg, NOW()) as daysElapsed'),
        'islim_fiber_zone.name as zoneName'
      )
      ->join(
        'islim_clients',
        'islim_clients.dni',
        'islim_installations.clients_dni'
      )
      ->join(
        'islim_fiber_zone',
        'islim_fiber_zone.id',
        'islim_installations.id_fiber_zone'
      )
      ->where([
        ['islim_installations.status', 'A'],
        ['islim_installations.installer', $installer]
      ]);

    return $dates->orderBy('islim_installations.date_instalation', 'ASC')->get();
  }

  /**
   * [getInstallerPending retorna la cantidad de instalacion pendientes por realizar en la zona por dias]
   * @param  [type] $fiberZone [description]
   * @return [type]            [description]
   */
  /* public static function getInstallerPending($fiberZone)
  {
  //Me retona las citas que han sido confirmada por la mesa y estan en poder de algun jefe de zona y que no esta por reagendar
  return self::getConnect('R')
  ->select(
  DB::raw('COUNT(date_instalation) as cant_installer'),
  DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
  'id_fiber_zone')
  ->where([
  ['status', 'A'],
  ['id_fiber_zone', $fiberZone]])
  ->whereNotNull('installer_boss')
  ->whereNotIn('status_control', ['C', 'SR'])
  ->groupBy('date_instalation')
  ->orderBy('date_instalation', 'ASC')
  ->get();
  }*/

  public static function getInstallerPending($fiberZone)
  {
    //Me retona las citas que han sido confirmada por la mesa y estan en poder de algun jefe de zona y que no esta por reagendar
    $install = self::getConnect('R')
      ->select(
        DB::raw('COUNT(date_instalation) as cant_installer'),
        'date_instalation',
        'id_fiber_zone'
      )
      ->where([
        ['status', 'A'],
        ['id_fiber_zone', $fiberZone]
      ])
      ->whereNotNull('installer_boss')
      ->whereNotIn('status_control', ['C', 'SR'])
      ->groupBy('date_instalation')
      ->get()->toArray();

    $support = FiberSupport::getConnect('R')
      ->select(
        DB::raw('COUNT(islim_fiber_support.date_support) as cant_installer'),
        'islim_fiber_support.date_support as date_instalation',
        'islim_installations.id_fiber_zone'
      )
      ->join('islim_installations', 'islim_installations.msisdn', 'islim_fiber_support.msisdn')
      ->where([
        ['islim_fiber_support.status', 'A'],
        ['islim_installations.id_fiber_zone', $fiberZone]
      ])
      ->whereNotNull('islim_fiber_support.installer_boss')
      ->whereNotIn('islim_fiber_support.status_control', ['C', 'SR'])
      ->groupBy('islim_fiber_support.date_support')
      ->get()->toArray();

    // Combina los dos arrays
    $combinedArray = array_merge($install, $support);

    // Inicializa un nuevo array para almacenar los resultados agrupados por fecha
    $resultArray = [];

    // Procesa cada elemento del array combinado
    foreach ($combinedArray as $item) {
      $installDate = $item['date_instalation'];

      // Si la fecha ya está en el resultado, suma la cantidad
      if (array_key_exists($installDate, $resultArray)) {
        $resultArray[$installDate]['cant_installer'] += $item['cant_installer'];
      } else {
        // Si es una nueva fecha, agrega el elemento al resultado
        $resultArray[$installDate] = $item;
      }
    }

    // Convierte el array asociativo resultante de nuevo a un array indexado
    $resultArray = array_values($resultArray);
    //dd($resultArray);
    return $resultArray;
  }

  /**
   * [getInstallerPendingDay Conoce la lista de instalacion dada una zona y una fecha en particular]
   * @param  [type] $date      [description]
   * @param  [type] $fiberZone [description]
   * @return [type]            [description]
   */
  public static function getInstallerPendingDay($fiberZone, $date)
  {
    //Citas en cada turno que esta verificadas y que no se hallan enviado a reprogramar en el dia especifico
    $install = self::getConnect('R')
      ->select(
        DB::raw('COUNT(schedule) as cant_installer'),
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'schedule',
        'id_fiber_zone'
      )
      ->where([
        ['status', 'A'],
        [DB::raw("DATE_FORMAT(islim_installations.date_instalation, '%Y-%m-%d')"), '=', $date],
        ['id_fiber_zone', $fiberZone]
      ])
      ->whereNotNull('installer_boss')
      ->whereNotIn('status_control', ['C', 'SR'])
      ->groupBy('date_instalation', 'schedule', 'id_fiber_zone')
      ->get()->toArray();

    $support = FiberSupport::getConnect('R')
      ->select(
        DB::raw('COUNT(islim_fiber_support.schedule) as cant_installer'),
        DB::raw("DATE_FORMAT(islim_fiber_support.date_support, '%d-%m-%Y') AS date_instalation"),
        'islim_fiber_support.schedule',
        'islim_installations.id_fiber_zone'
      )
      ->join('islim_installations', 'islim_installations.msisdn', 'islim_fiber_support.msisdn')
      ->where([
        ['islim_fiber_support.status', 'A'],
        [DB::raw("DATE_FORMAT(islim_fiber_support.date_support, '%Y-%m-%d')"), '=', $date],
        ['islim_installations.id_fiber_zone', $fiberZone],
      ])
      ->whereNotNull('islim_fiber_support.installer_boss')
      ->whereNotIn('islim_fiber_support.status_control', ['C', 'SR'])
      ->groupBy('islim_fiber_support.date_support', 'islim_fiber_support.schedule', 'islim_installations.id_fiber_zone')
      ->get()
      ->toArray();

    // Combina los dos arrays
    $combinedArray = array_merge($install, $support);

    // Inicializa un nuevo array para almacenar los resultados agrupados por fecha
    $resultArray = [];

    foreach ($combinedArray as $item) {
      $installDate = $item['schedule'];

      // Si la fecha ya está en el resultado, suma la cantidad
      if (array_key_exists($installDate, $resultArray)) {
        $resultArray[$installDate]['cant_installer'] += $item['cant_installer'];
      } else {
        // Si es una nueva fecha, agrega el elemento al resultado
        $resultArray[$installDate] = $item;
      }
    }

    // Convierte el array asociativo resultante de nuevo a un array indexado
    $resultArray = array_values($resultArray);
    //dd($support);
    return $resultArray;
  }
  /**
   * [getInstallerAsignedUser Regresa la lista de instalaciones un dia con cada instalador dado un jefe especifico o las instalaciones que se autoasigna el jefe de instaladores]
   * @param  [type] $userInstall [description]
   * @param  [type] $date        [description]
   * @return [type]              [description]
   */
  public static function getInstallerAsignedUser($fiberZone, $date, $installerBoss = false)
  {
    $data = self::getConnect('R')
      ->select(
        DB::raw('COUNT(schedule) as cant_turno'),
        'installer',
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'schedule',
        'id_fiber_zone'
      )
      ->where([
        ['status', 'A'],
        ['id_fiber_zone', $fiberZone]
      ])
      ->where(DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' )"), '=', $date)
      ->whereNotIn('status_control', ['C', 'SR']);

    if (!$installerBoss) {
      //Instaladores donde asigno el jefe
      $data = $data->where('installer_boss', session('user'))
        ->whereNotNull('installer');
    } else {
      //Las instalacion que el jefe se auto asigno
      $data = $data->where('installer', session('user'));
    }

    return $data->groupBy('installer', 'schedule')
      ->get();
  }

  public static function markCancelled($dataCanceller)
  {
    try {
      self::getConnect('W')
        ->where('id', $dataCanceller->cita)
        ->update([
          'status_control' => 'SR',
          'date_mod' => date('Y-m-d H:i:s'),
          'user_mod' => session('user'),
          'reason_delete' => !empty($dataCanceller->msgTypi) ? $dataCanceller->msgTypi : null,
          'typification_id' => $dataCanceller->typification
        ]);
      return array('success' => true, 'msg' => 'OK');
    } catch (Exception $e) {
      $txmsg = "No se pudo modificar el status de control de la instalacion (543) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }
  /**
   * [markAceptTyc Registra el id del tyc asociado a la cita]
   * @param  [type] $install_id [description]
   * @param  [type] $tyc_id     [description]
   * @param  string $type_tyc   [description]
   * @return [type]             [description]
   */
  public static function markAceptTyc($install_id, $tyc_id, $type_tyc = "END")
  {
    try {
      $regis = self::getConnect('W')
        ->where('id', $install_id);

      if ($type_tyc == "END") {
        $regis = $regis->update(['payment_force_end' => $tyc_id]);
      } else {
        $regis = $regis->update(['payment_force_start' => $tyc_id]);
      }
      return array('success' => true, 'msg' => 'OK');
    } catch (Exception $e) {
      $txmsg = "No se pudo modificar el id del contrato de fibra de la cita (569) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }

  /**
   * [setChangerPackService Paquete cambiado por el instalador]
   * @param [type] $id      [id de la cita]
   * @param [type] $pack    [id del pack]
   * @param [type] $service [id del servicio]
   * @param [type] $total_price [monto pagado]
   * @param [type] $service [description]
   * @param [type] $delPayment [indica si se coloca los campos en null de la subscripcion]

   */
  public static function setChangerPackService($id, $pack, $service, $total_price, $delPayment = false, $bundle_id = false)
  {
    try {
      $install_update = self::getConnect('W')
        ->where('id', $id)
        ->first();

      if (!empty($install_update)) {
        $install_update->pack_id = $pack;
        $install_update->service_id = $service;
        $install_update->price = $total_price;
        $install_update->user_mod = session('user');
        $install_update->date_mod = date('Y-m-d H:i:s');

        if ($bundle_id) {
          $install_update->bundle_id = $bundle_id;
        }

        if ($delPayment) {
          $install_update->payment_url_subscription = null;
          $install_update->unique_transaction = null;
          $install_update->pack_price_id = null;
          $install_update->bundle_id_payment = null;
          $install_update->payer_email = null;
        }
        $install_update->save();
        sleep(2);
        return array('success' => true, 'msg' => 'OK', 'code' => 'OK');
      }
      return array('success' => false, 'msg' => "No se encontro la cita a ser actualizada el pack y el servicio (MI634)", 'code' => 'EMP_INS');
    } catch (Exception $e) {
      $txmsg = "No se pudo modificar el servicio y el paquete de la cita (MI636) " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg, 'code' => 'FAIL');
    }
  }

  /**
   * [setMethodPayment Actualiza la forma como se pago la cita]
   * @param [integer] $install_id [id de la cita de instalacion]
   * @param [boolean] $verify [true=indica que se debe revisar el servicio si debe cambiar a uno subscrito, false=ignora el servicio]

   * @param [type] $method     [description]
   */
  public static function setMethodPayment($install_id, $method, $verify = false, $unique = false)
  {
    $response = ['success' => false, 'msg' => "No se pudo actualizar la forma en que se pago, faltan datos", 'code' => 'FAIL'];

    if ($install_id && $method) {
      try {
        $install = self::getConnect('W')
          ->where('id', $install_id)
          ->first();

        if (!empty($install)) {
          $install->type_payment = $method;
          $install->paid = ($method == "CARD") ? 'Y' : 'N';
          if ($unique) {
            $install->unique_transaction = $unique;
          } else {
            $install->unique_transaction = null;
          }
          $install->save();
          sleep(2);

          if (
            $method == "CARD" && $verify &&
            (!empty($install->payment_url_subscription) &&
              (!empty($install->pack_price_id) ||
                !empty($install->bundle_id_payment)
              )
            )
          ) {
            //Se verifica el plan que tiene configurado la cita posee pago recurrente, como realizo el pago el cambio de plan se debe hacer

            if (!empty($install->pack_price_id)) {
              //Se debe cambiar al plan de pago recurrente de fibra sola
              $packprice = PackPrices::getPackPriceDetail($install->pack_price_id);

              if (!empty($packprice)) {

                $upd = self::setChangerPackService(
                  $install_id,
                  $packprice->pack_id,
                  $packprice->service_id,
                  $packprice->total_price,
                  false,
                  null
                );
                if (!$upd['success']) {
                  $response = ['success' => false, 'msg' => $upd['msg'], 'code' => $upd['code']];
                  return $response;
                }
              } else {
                $response = ['success' => false, 'msg' => 'El detalle del paquete a cambiar no pudo se encontrado (MI698)', 'code' => 'EMP_PAK'];
                return $response;
              }
            } elseif (!empty($install->bundle_id_payment)) {
              //Se debe cambiar al plan de pago recurrente de fibra bundle
              //Aca se deben actualizar dos lugares, la instalacion de fibra y la instalacion_bundle donde esta el hijo del bundle

              $infPack = Bundle::getDetailBundleAlta($install->bundle_id_payment);
              if ($infPack['success']) {
                $infoAllBundle = json_decode(json_encode($infPack['data']));
                $update_fail = "";

                if ($infoAllBundle->general->containt_F == 'Y' && isset($infoAllBundle->info_F->id)) {
                  $up_F = self::setChangerPackService(
                    $install_id,
                    $infoAllBundle->info_F->id,
                    $infoAllBundle->info_F->service_id,
                    $infoAllBundle->general->total_payment,
                    false,
                    $infoAllBundle->general->id
                  );
                  if (!$up_F['success']) {
                    $update_fail .= $up_F['msg'];
                  }
                }
                //Se actualizan los hijos del bundle
                $productList = [
                  'T' => "Telefonia",
                  'M' => "Mifi",
                  'MH' => "Mifi Huella",
                  'H' => "Hogar"
                ];

                foreach ($productList as $key => $value) {
                  $containt = "containt_" . $key;
                  $infoArt = "info_" . $key;

                  if (
                    $infoAllBundle->general->$containt == 'Y' &&
                    !empty($infoAllBundle->$infoArt)
                  ) {
                    $UP = InstallationsBundle::updateChildrenChangerBundle(
                      $install_id,
                      $key,
                      $infoAllBundle->$infoArt->id,
                      $infoAllBundle->$infoArt->service_id
                    );
                    if (!$UP['success']) {
                      $update_fail .= ' ** ' . $UP['msg'];
                    }
                  }
                }

                if (empty($update_fail)) {
                  return array('success' => true, 'msg' => "OK", 'code' => 'OK');
                } else {
                  return array('success' => false, 'msg' => $update_fail, 'code' => 'FAIL_BD');
                }
              }
            }
          }
          return array('success' => true, 'msg' => "OK", 'code' => 'OK');
        } else {
          $response = ['success' => false, 'msg' => "No se encuentra la cita de instalacion para actualizar", 'code' => 'EMP_INS'];
        }
      } catch (Exception $e) {
        $txmsg = "No se pudo modificar la forma en que se pago la cita (MI685) " . (string) json_encode($e->getMessage());
        Log::error($txmsg);
        return array('success' => false, 'msg' => $txmsg, 'code' => 'FAIL');
      }
    }
    return $response;
  }

  /**
   * [notifyChangerMP Registra cuando se informa el cambio de plan a la pasarela de pago para que programe el proximo dia que debe cobrar y que servicio recargara]
   * @param  [type] $install_id [description]
   * @param  [type] $status     [description]
   * @return [type]             [description]
   */
  public static function notifyChangerMP($install_id, $status)
  {
    $response = ['success' => false, 'msg' => "No se pudo actualizar que se notifico el cambio de plan, faltan datos"];

    if ($install_id && $status) {
      try {
        $install = self::getConnect('W')
          ->where('id', $install_id)
          ->first();

        if (!empty($install)) {
          $install->notify_changer_service = $status;
          $install->save();

          return array('success' => true, 'msg' => "OK");
        } else {
          $response = ['success' => false, 'msg' => "No se encuentra la cita de instalacion para actualizar"];
        }
      } catch (Exception $e) {
        $txmsg = "No se pudo actualizar que se notifico el cambio de plan de la cita (MI793) " . (string) json_encode($e->getMessage());
        Log::error($txmsg);
        return array('success' => false, 'msg' => $txmsg);
      }
    }
    return $response;
  }

  public static function totalProcess($install_id)
  {
    $master = self::getConnect('R')
      ->find($install_id);
    if ($master->status != 'P') {
      return 'N';
    }

    $childs = InstallationsBundle::getConnect('R')
      ->where('installations_id', $install_id)
      ->whereIn('status', ['EC', 'E', 'PA', 'EA'])
      ->get();

    if (count($childs) > 0) {
      return 'N';
    }
    return 'Y';
  }

  /**
   * [getComponent Devuelve un componente el bundle para conocer el status de la activacion]
   * @param  [type] $installer_id [Id de installation bundle]
   * @return [type]               [description]
   */
  public static function getComponent($id)
  {
    $component = self::getConnect('R')
      ->select(
        'islim_installations.id',
        'islim_installations.status',
        DB::raw('IFNULL(islim_installations.msisdn,islim_inv_arti_details.msisdn) as msisdn'),
        'islim_installations.inv_detail_fiber_id',
        DB::raw('CONCAT("F") AS dn_type'),
        DB::raw('CONCAT("master") AS config'),
        'islim_installations.obs_activate',
        'islim_installations.id as installations_id',
        'islim_installations.dataProvisioned',
        'islim_inv_arti_details.serial',
        'islim_installations.config_conex'
      )
      ->leftJoin(
        'islim_inv_arti_details',
        function ($join) {
          $join->on(
            'islim_inv_arti_details.id',
            'islim_installations.inv_detail_fiber_id'
          )
            ->orWhere('islim_inv_arti_details.msisdn', DB::raw("
              islim_installations.msisdn"));
        }
      )
      ->where([
        ['islim_installations.status', '!=', 'T'],
        ['islim_installations.id', $id]
      ])
      ->first();

    return $component;
  }

  /**
   * [asigneUniqueTransaction Asignacion de codigo de transaccion y otros datos mas consernientes a una subscripcion, la unique transaction sera usado para otras operaciones como lo es el registro de venta]
   * @param  [type] $id     [description]
   * @param  [type] $unique [description]
   * @return [type]         [description]
   */
  public static function asigneSubscriptionInfo(
    $id,
    $unique,
    $url,
    $packPrice_id,
    $payer_email,
    $bundle_id,
    $runway
  ) {
    try {
      self::getConnect('W')
        ->where('id', $id)
        ->update([
          'unique_transaction' => $unique,
          'payment_url_subscription' => $url,
          'pack_price_id' => $packPrice_id,
          'bundle_id_payment' => $bundle_id,
          'payer_email' => $payer_email,
          'runway' => $runway
        ]);
      sleep(2);
      return array('success' => true, 'msg' => 'OK');
    } catch (Exception $e) {
      $txmsg = "No se pudo asignar la informacion conserniente al pago por subscripcion de la instalacion_id " . $id . " - " . (string) json_encode($e->getMessage());
      Log::error($txmsg);
      return array('success' => false, 'msg' => $txmsg);
    }
  }

  public static function isProvisioned($install_id)
  {
    $inst = self::getConnect('R')
      ->where('id', $install_id)
      ->where('is_provisioned', 'Y')
      ->whereNotNull('date_provisioned')
      ->first();
    //
    if ($inst) {
      return 'Y';
    }
    return 'N';
  }

  /**
   * [getInstallerPendingForPostInst Busca si un usuario tiene alguna instalacion con carga pendiente de los datos de postinstalacion (preguntas,evidencias e insumos) y retorna el id de esa instalacion]
   * @param  [type] $installer [description]
   * @return [type]            [description]
   */
  public static function getInstallerPendingForPostInst($installer)
  {
    $inst = self::getConnect('R')
      ->select(
        'id'
      )
      ->whereIn('status', ['P', 'EA'])
      ->where([
        ['installer', $installer],
        ['end_pi', 'N'],
      ])
      ->orderBy('id', 'ASC')
      ->first();

    if ($inst) {
      return $inst->id;
    }
    return 0;
  }

  public static function getConfirmDatesForInstaller($installer, $idfiberzone, $idfibercity)
  {
    $dateE = date("Y-m-d");
    $dates = self::getConnect('R')
      ->select(
        'islim_installations.id',
        'islim_installations.clients_dni',
        'islim_installations.address_instalation',
        'islim_installations.date_reg',
        DB::raw("DATE_FORMAT( islim_installations.date_instalation, '%d-%m-%Y' ) AS date_instalation"),
        'islim_installations.schedule',
        'islim_installations.installer',
        'islim_installations.price',
        'islim_clients.name',
        'islim_clients.last_name'
      )
      ->join('islim_clients', 'islim_clients.dni', 'islim_installations.clients_dni')
      ->where('islim_installations.status', 'A')
      ->where('islim_installations.status_control', 'CC')
      ->where('islim_installations.id_fiber_city', $idfibercity)
      ->where('islim_installations.id_fiber_zone', $idfiberzone);

    if ($dateE) {

      $dates = $dates->where('islim_installations.date_instalation', '>=', $dateE);
    }

    return $dates->orderBy('islim_installations.date_instalation', 'ASC')->get();
  }

  public static function getStateNameById($id)
  {
    $result = self::getConnect('R')
      ->select('islim_localy_mexico.location')
      ->join('islim_localy_mexico', 'islim_localy_mexico.id', 'islim_installations.id_state')
      ->where([
        ['islim_installations.id', $id]
      ])
      ->first();

    return $result ? $result->location : null;
  }

  public static function getCityNameById($id)
  {
    $result = self::getConnect('R')
      ->select('islim_localy_mexico.location')
      ->join('islim_fiber_city', 'islim_fiber_city.id', 'islim_installations.id_fiber_city')
      ->join('islim_localy_mexico', 'islim_localy_mexico.id', 'islim_fiber_city.localy_id')
      ->where('islim_installations.id', $id)
      ->first();

    return $result ? $result->location : null;
  }

  public static function sendNotificateConfirmation($install_id)
  {

    $data = self::getConnect('R')
      ->select(
        'islim_clients.name as c_name',
        'islim_clients.last_name as c_lastname',
        'islim_clients.email as c_email',
        'islim_clients.phone_home as c_phonehome',
        'islim_installations.address_instalation',
        'islim_installations.schedule',
        'islim_installations.id_fiber_zone',
        'islim_installations.service_id',
        DB::raw("(DATE_FORMAT(islim_installations.date_instalation, '%d-%m-%Y')) as date_install")
      )
      ->join('islim_users as seller', 'seller.email', 'islim_installations.seller')
      ->join('islim_clients', 'islim_clients.dni', 'islim_installations.clients_dni')
      ->where('islim_installations.id', $install_id)
      ->first();

    if (!empty($data->c_email)) {

      $dataBody = [
        'name' => $data->c_name,
        'lastname' => $data->c_lastname,
        'phone' => $data->c_phonehome,
        'email' => $data->c_email,
        'address' => $data->address_instalation,
        'date' => $data->date_install . ' / ' . $data->schedule,
      ];

      try {
        Mail::to($data->c_email)->send(new mailInstallV2($dataBody));
      } catch (\Exception $e) {
        Log::error('No se pudo enviar el correo de solicitud de instalación a: ' . $data->c_email . ' Error: ' . $e->getMessage());
      }
    }

    if (!empty($data->c_phonehome)) {

      $Infomsj = FiberZone::getInfoZone($data->id_fiber_zone);
      $configuration = json_decode(json_encode($Infomsj->configuration));
      $smsCita = '';

      if (isset($configuration->smsCita)) {
        if (!empty($configuration->smsCita)) {
          $smsCita = $configuration->smsCita;
        }
      }

      if (!empty($smsCita)) {
        $smsCita = CommonHelpers::customerSMS(
          $smsCita,
          false,
          $data->c_name,
          false,
          $data->service_id,
          false,
          $data->date_install . ' en horario ' . $data->schedule
        );
        SMSNotification::Send_sms(
          $data->c_phonehome,
          $data->c_phonehome,
          'F',
          '1',
          "Cita de instalacion",
          "SMSCITAFIBRA",
          $smsCita
        ); 
      } else {
        Log::error("No se envio sms de notificacion de cita al cliente (" . $data->c_name . "), la zona (" . $data->id_fiber_zone . ") no lo tiene configurado.");
      }
    } else {
      Log::error("El cliente (" . $data->c_name . ") no tiene configurado un telefono de contacto para el envio de sms.");
    }
  }

  /**
     * Citas del cliente en la ciudad en la ultima hora
     * */
  public static function getInstallBy_dni_city_time($client_dni, $fiber_city) {
    $today = Carbon::now()->format('Y-m-d H:i:s');
    $NuevaFecha = date("Y-m-d H:i:s", strtotime($today . "- 1 hour"));

    $result = self::getConnect('R')
    ->where([['islim_installations.clients_dni', $client_dni],
        ['islim_installations.id_fiber_city', $fiber_city],
        ['islim_installations.status', 'A'],
        ['islim_installations.status_control', '!=', 'CN'],
        ['islim_installations.date_reg', '>=', $NuevaFecha]])
    ->first();
    return $result;
  }
}
