<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use App\Models\Inventory;
use Log;

class Pack extends Model
{
    use HasFactory;

    protected $table = 'islim_packs';

    protected $fillable = [
        'id',
        'title',
        'description',
        'price_arti',
        'date_ini',
        'date_end',
        'date_reg',
        'status',
        'view_web',
        'desc_web',
        'pack_type'
    ];

    public $timestamps = false;

    /**
     * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
     * @param String $typeCon
     * 
     * @return App\models\Pack
     */
    public static function getConnect($typeCon = false)
    {
        if ($typeCon) {
            $obj = new Pack;
            $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

            return $obj;
        }
        return null;
    }

    /**
     * Retorna los articulos que se pueden vender en la web dado un tipo de pack 
     * y una categoria de producto (Opcional)
     *
     * @param String $pack_type
     * @param Int $cat_id
     *
     * @return App\Models\Pack
     **/
    public static function getArcticWeb($pack_type = 'H', $cat_id = false, $coord = null)
    {
        $data = self::getConnect('R')
            ->select(
                'islim_packs.id',
                'islim_packs.desc_web',
                'islim_inv_articles.id as idArtic',
                'islim_inv_articles.title',
                'islim_inv_articles.description',
                'islim_inv_articles.sku'
            )
            ->join('islim_arti_packs', 'islim_arti_packs.pack_id', 'islim_packs.id')
            ->join('islim_inv_articles', 'islim_inv_articles.id', 'islim_arti_packs.inv_article_id');

        if (!empty($coord)) {
            $data = $data->join('islim_pack_esquema', 'islim_pack_esquema.id_pack', 'islim_packs.id');
        }

        $data = $data->where([
            ['islim_packs.status', 'A'],
            ['islim_packs.view_web', 'Y'],
            ['islim_arti_packs.status', 'A'],
            ['islim_inv_articles.status', 'A'],
            ['islim_packs.pack_type', $pack_type]
        ]);

        if (empty($coord)) {
            $data = $data->where(DB::raw('(select count(pe.id) from islim_pack_esquema as pe where pe.id_pack = islim_packs.id and pe.status = "A")'), 0);
        } else {
            $data = $data->where([
                ['islim_pack_esquema.status', 'A'],
                ['islim_pack_esquema.id_esquema', $coord]
            ]);
        }


        if ($cat_id) {
            $data = $data->where('islim_inv_articles.category_id', $cat_id);
        }

        return $data->get();
    }

    /**
     * Retorna los articulos que se pueden vender dado un tipo de pack 
     * y una categoria de producto (Opcional)
     *
     * @param String $pack_type
     * @param Int $cat_id
     *
     * @return App\Models\Pack
     **/
    public static function getArctic($pack_type = 'H', $cat_id = false, $coord = null)
    {
        $inv_art = Inventory::getInventoryAva();

        $data = self::getConnect('R')
            ->select(
                'islim_packs.id',
                'islim_packs.desc_web',
                'islim_inv_articles.id as idArtic',
                'islim_inv_articles.title',
                'islim_inv_articles.description',
                'islim_inv_articles.sku'
            )
            ->join('islim_arti_packs', 'islim_arti_packs.pack_id', 'islim_packs.id')
            ->join('islim_inv_articles', 'islim_inv_articles.id', 'islim_arti_packs.inv_article_id');

        if (!empty($coord)) {
            $data = $data->join('islim_pack_esquema', 'islim_pack_esquema.id_pack', 'islim_packs.id');
        }

        $data = $data->where([
            ['islim_packs.status', 'A'],
            ['islim_packs.sale_type', 'N'],
            ['islim_packs.pack_type', $pack_type],
            ['islim_arti_packs.status', 'A'],
            ['islim_inv_articles.status', 'A'],
            ['islim_packs.is_migration', 'N']
        ])
            ->whereIn('islim_inv_articles.id', $inv_art->pluck('inv_article_id'));;

        if (empty($coord)) {
            $data = $data->where(DB::raw('(select count(pe.id) from islim_pack_esquema as pe where pe.id_pack = islim_packs.id and pe.status = "A")'), 0);
        } else {
            $data = $data->where([
                ['islim_pack_esquema.status', 'A'],
                ['islim_pack_esquema.id_esquema', $coord]
            ]);
        }


        if ($cat_id) {
            $data->where('islim_inv_articles.category_id', $cat_id);
        }

        // $query = vsprintf(str_replace('?', '%s', $data->toSql()), collect($data->getBindings())->map(function ($binding) {
        //         return is_numeric($binding) ? $binding : "'{$binding}'";
        //     })->toArray());

        // Log::alert($query);

        return $data->get();
    }

    /**
   * Metodo para consultar datos de un pack dado su id
   * @param String $id
   *
   * @return App\Models\Pack
   */
    public static function getActivePackById($id = false, $statusList = ['A'])
    {
        if ($id) {
        return self::getConnect('R')
            ->select(
            'title',
            'description',
            'sale_type',
            'is_migration',
            'is_circle_credit'
            )
            ->where('id', $id)
            ->whereIn('status', $statusList)
            ->first();
        }
        return null;
    }
}
