<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class ProvaDelivery extends Model
{
    use HasFactory;

    protected $table = 'islim_prova_delivery';

    protected $fillable = [
        'id_car',
        'id_temp_car',
        'deposit',
        'postal_code',
        'state',
        'city',
        'colony',
        'municipality',
        'street',
        'ext_number',
        'int_number',
        'notes',
        'transaction',
        'folio',
        'traking_url',
        'courier_g',
        'price',
        'status',
        'date_reg',
    ];

    public $timestamps = false;

    /**
     * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
     * @param String $typeCon
     *
     * @return App\models\ProvaDelivery
     */
    public static function getConnect($typeCon = false)
    {
        if ($typeCon) {
            $obj = new ProvaDelivery;
            $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

            return $obj;
        }
        return null;
    }

    public static function inactiveDeliverys($idcar = false)
    {
        if ($idcar) {
            self::where([
                ['id_temp_car', $idcar],
                ['status', 'A'],
            ])
                ->update(['status' => 'I']);

            return true;
        }

        return false;
    }

    public static function getActOrSendDelivery($idCar = false)
    {
        if ($idCar) {
            return self::getConnect('R')
                ->select('id', 'courier_g', 'folio')
                ->where('id_temp_car', $idCar)
                ->whereIn('status', ['A', 'S'])
                ->first();
        }

        return null;
    }

    public static function getActiveDelivery($car_id = false)
    {
        if ($car_id) {
            return self::getConnect('R')
                ->select(
                    'id',
                    'postal_code',
                    'deposit',
                    'street',
                    'municipality',
                    'colony',
                    'state',
                    'notes',
                    'ext_number',
                    'price',
                    'transaction'
                )
                ->where([
                    ['id_temp_car', $car_id],
                    ['status', 'A'],
                ])
                ->first();
        }

        return null;
    }

    public static function setSend($id = false, $data = [])
    {
        if ($id) {
            self::where('id', $id)
                ->update([
                    'transaction' => $data['transaction'],
                    'status' => 'S',
                    'folio' => $data['folio'],
                    'traking_url' => $data['url'],
                ]);

            return true;
        }
        return false;
    }

    public static function resetDelivery($transaction)
    {
        return self::getConnect('W')
            ->where('id_temp_car', $transaction)
            ->update(['status' => 'I']);
    }
}
