<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class TempCar extends Model
{
    use HasFactory;

    protected $table = 'islim_temp_car';

    protected $fillable = [
        'transaction',
        'client_dni',
        'address_send',
        'key',
        'cod_estafeta',
        'price_sending',
        'date_reg',
        'date_update',
        'status'
    ];

    const CREATED_AT = 'date_reg';
    const UPDATED_AT = 'date_update';

    /**
     * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
     * @param String $typeCon
     *
     * @return App\models\TempCar
     */
    public static function getConnect($typeCon = false)
    {
        if ($typeCon) {
            $obj = new TempCar;
            $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

            return $obj;
        }
        return null;
    }

    /*
  Retorna carrito consultado por medio de la transacción en estatus creado

  @param String $trans
  @return Mix
   */
    public static function getTransaction($trans = false, $key = false, $tc = 'R')
    {
        if ($trans && $key) {
            return self::getConnect($tc)
                ->where([
                    ['status', 'A'],
                    ['key', $key],
                    ['transaction', $trans]
                ])
                ->first();
        }

        return null;
    }

    public static function getTransactionAP($trans = false, $key = false)
    {
        if ($trans && $key) {
            return self::getConnect('R')
                ->where([
                    ['key', $key],
                    ['transaction', $trans]
                ])
                ->whereIn('status', ['A', 'P'])
                ->first();
        }

        return null;
    }

    public static function getDataClient($id = false, $key = false)
    {
        if ($id && $key) {
            return self::getConnect('R')
                ->select(
                    'islim_temp_car.id',
                    'islim_clients.name',
                    'islim_clients.last_name',
                    'islim_clients.phone_home',
                    'islim_clients.email'
                )
                ->join(
                    'islim_clients',
                    'islim_clients.dni',
                    'islim_temp_car.client_dni'
                )
                ->where([
                    ['islim_temp_car.key', $key],
                    ['islim_temp_car.id', $id]
                ])
                ->first();
        }

        return null;
    }

    public static function getTempCarAP($transaction)
    {
        return self::getConnect('R')
            ->select('date_reg')
            ->where('transaction', $transaction)
            ->whereIn('status', ['A', 'P'])
            ->first();
    }

    public static function getTempCarByT($transaction)
    {
        return self::getConnect('R')
            ->select('date_reg')
            ->where([
                ['transaction', $transaction],
                ['status', 'A']
            ])
            ->first();
    }
}
