<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class User extends Model
{
    use HasFactory;

    protected $table = 'islim_users';

    protected $fillable = [
        'name',
        'last_name',
        'id_org',
        'email',
        'parent_email',
        'password',
        'dni',
        'platform',
        'phone',
        'phone_job',
        'profession',
        'position',
        'address',
        'status',
        'charger_com',
        'charger_balance',
        'residue_amount',
        'password_date',
        'url_latter_contract',
        'concentrator_id',
        'activation_im',
    ];

    protected $hidden = [
        'password', 'date_reg', 'date_mod',
    ];

    protected $primaryKey = 'email';

    protected $keyType = 'string';

    public $incrementing = false;

    const CREATED_AT = 'date_reg';

    const UPDATED_AT = 'date_mod';

    /**
     * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
     * @param String $typeCon
     *
     * @return App\models\User
     */
    public static function getConnect($typeCon = false)
    {
        if ($typeCon) {
            $obj = new User;
            $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

            return $obj;
        }
        return null;
    }

    /**
     * Consulta un usuario dado un id de concentrador.
     *
     * @param String $email
     * @return Mix
     */
    public static function getEmailByConc($idConc = false)
    {
        if ($idConc) {
            return self::getConnect('R')
                ->select('email')
                ->where([
                    ['status', 'A'],
                    ['concentrator_id', $idConc],
                ])
                ->first();
        }

        return null;
    }

    /**
     * Consulta un usuario activo dado su email.
     *
     * @param String $email
     * @return Mix
     */
    public static function getUser($email = false)
    {
        if ($email) {
            return self::getConnect('R')
                ->select('concentrator_id')
                ->where([
                    ['status', 'A'],
                    ['email', $email],
                ])
                ->first();
        }

        return null;
    }



    public static function getUserToFiber($bearer_token, $tc = 'R', $status = ['A'])
    {
    return self::getConnect($tc)
        ->select(
        'islim_users.email',
        'islim_users.dni',
        'islim_users.name',
        'islim_users.last_name',
        'islim_users.password',
        'islim_users.platform',
        'islim_users.id_org',
        'islim_users.status',
        'islim_users.reset_session',
        'islim_profiles.id as id_profile',
        'islim_profiles.name as name_profile',
        'islim_profiles.hierarchy',
        'islim_profiles.type'
        )
        ->join(
        'islim_profile_details',
        'islim_profile_details.user_email',
        'islim_users.email'
        )
        ->join(
        'islim_profiles',
        'islim_profiles.id',
        'islim_profile_details.id_profile'
        )
        ->where([
        ['islim_users.token_api_fiber', $bearer_token],
        ['islim_profile_details.status', 'A']])
        ->whereIn('islim_users.status', $status)
        ->first();
    }



    /**
     * [getInstallerZone Obtiene la lista de usuarios con perfil de instalador o jefe de instaladores con permiso de instalar fibra]
     * @param  [type] $zone_id [zona de fibra]
     * @return [type]          [description]
     */
    public static function getInstallerZone($zone_id)
    {
        return self::getConnect('R')
        ->join(
            'islim_fiber_city_zone',
            'islim_fiber_city_zone.id',
            'islim_users.fiber_city_zone_id'
        )
        ->join(
            'islim_user_roles',
            'islim_user_roles.user_email',
            'islim_users.email'
        )
        ->join(
            'islim_profile_details',
            'islim_profile_details.user_email',
            'islim_users.email'
        )
        ->where([
            ['islim_fiber_city_zone.status', 'A'],
            ['islim_fiber_city_zone.fiber_zone_id', $zone_id],
            ['islim_users.status', 'A'],
            ['islim_user_roles.policies_id', 247],
            ['islim_user_roles.value', 1],
            ['islim_user_roles.status', 'A'],
            ['islim_profile_details.status', 'A']])
        ->whereIn('islim_profile_details.id_profile', [24, 25])
        ->get();
    }

    public static function getUserByEmail($email)
    {
        return self::getConnect('R')
        ->select(
            'islim_users.email',
            'islim_users.parent_email',
            'islim_users.dni',
            'islim_users.name',
            'islim_users.last_name',
            'islim_users.phone',
            'islim_users.charger_balance',
            'islim_users.platform',
            'islim_users.date_reg',
            'islim_users.dateToken',
            'islim_users.tokenPassword',
            'islim_users.is_locked',
            'islim_users.status',
            'islim_profile_details.id_profile'
        )
        ->join('islim_profile_details','islim_profile_details.user_email','islim_users.email')
        ->where([
            ['islim_users.email', $email],
            ['islim_profile_details.status','A']
        ])
        ->first();
    }

    /**
     * [existPhoneUser Busca dentro de los usuarios quien tiene registrado el telefono]
     * @param   $phone [telefono a buscar]
     * @param   $seller [Correo del vendedor a buscar]
     * @return [type]         [description]
    */
    public static function existPhoneUser($phone, $seller = false)
    {
        $dataUser = self::getConnect('R')
        ->where([
            ['phone', $phone],
            ['status', '!=', 'T']]);

        if ($seller) {
        $dataUser = $dataUser
            ->where('email', '!=', $seller);
        }
        return $dataUser->first();
    }

    /**
     GESTIÓN DE PERMISOS
    */
    public static function hasPermission($email, $code)
    {
        $response = false;
        $values = UserRole::getConnect('R')->select('policies_id', 'roles_id')->where('user_email', $email)->where('status', 'A')->where('value', '>', 0)->get();
        foreach ($values as $value) {
        $policies = Policy::getConnect('R')->select('code')->where('status', 'A')->where('id', $value->policies_id)->where('roles_id', $value->roles_id)->get();
        foreach ($policies as $policy) {
            if ($policy->code == $code) {
            $response = true;
            }
        }
        }
        return $response;
    }
}
