<?php

namespace App\Models;

use App\Models\Client;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class Verify_contact_client extends Model {
	use HasFactory;

	protected $table = 'islim_verify_phone_contact';

	protected $fillable = [
		'id',
		'phone_verify',
		'dni_client',
		'token_verify',
		'email_seller',
		'email_mesa',
		'type_verify',
		'date_reg',
		'date_update',
		'status',
		'observacion',
		'origin',
		'tried',
		'platform'];

	public $timestamps = false;

/**
 * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
 * @param String $typeCon
 *
 * @return App\\Models\Verify_contact_client
 */
	public static function getConnect($typeCon = false) {
		if ($typeCon) {
			$obj = new self;
			$obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');

			return $obj;
		}
		return null;
	}

/**
 * [getRegisterVerifyInDay Obtiene la cantidad de veces que sale el dni y el telefono en el dia]
 * @param  [type] $dni         [dni del cliente]
 * @return [type]              [description]
 */
	public static function getRegisterVerifyInDay($phone, $dni) {
		$cantidad = self::getConnect('R')
			->where([
				['dni_client', $dni],
				['phone_verify', $phone],
				['type_verify', 'VERIFIED']])
			->whereBetween('date_reg', [date('Y-m-d 00:00:00'), date('Y-m-d 23:59:59')])
			->get();

		return (!empty($cantidad)) ? count($cantidad) : 0;
	}

/**
 * [getRegisterVerifyByPhone Busca el ultimo registro que se tiene del telefono con el dni del cliente]
 * @param  [type] $phone      [Telefono que se piensa verificar]
 * @param  [type] $clientDni  [DNI de la persona que tiene segun el telefono a verificar]
 * @param  [type] $sellerMail [Persona que esta haciendo la peticion de verificacion]
 * @return [type]             [description]
 */
	public static function getCodeVerifyByPhone($phone, $clientDni) {
		$code = "";
		$verify_id = "";
		$msg = "";
		//Se revisa si el telefono lo tiene tiene registrado otro cliente o vendedor

		$clienteVerify = Client::verifyPhones($clientDni, $phone, '');
		if (!$clienteVerify['success']) {
			//Significa que alguien mas tiene el telefono registrado
			$code = $clienteVerify['code'];
			$msg = $clienteVerify['msg'];
		}

		$UserNetwey = User::existPhoneUser($phone);
		if (!empty($UserNetwey)) {
			if ($UserNetwey->email == session('user')) {
				//Significa que el telefono lo tiene el vendedor de netwey que esta registrando la verificacion
				$code = "IS_PHONE_USER_SELLER";
			}
		}

		if (empty($code)) {
			$register = self::getConnect('R')
				->where([
					['phone_verify', $phone],
					['status', '!=', 'T']])
				->orderBy('id', 'DESC')
				->first();

			$code = "PHONE_EMPTY";
			//PHONE_OK Significa que no esta registrado en ningun lugar de netwey
			if (!empty($register)) {
				//significa que esta en algun proceso de verificacion de telefono
				//              //enum('CREATE','VERIFIED','REJECTED','AUTHORIZED','T')

				$code = $register->status;
				$verify_id = $register->id;
				switch ($code) {
				case 'CREATE':
					$msg = "Registro de verificacion celular esta creado";
					break;
				case 'VERIFIED':
					$msg = "Telefono verificado";
					break;
				case 'REJECTED':
					$msg = "Autorizacion rechazada. El telefono ingreso a una lista negra";
					break;
				case 'AUTHORIZED':
					$msg = "Registro autorizado para comprar pero sin posibilidad de ver planes de circulo de credito";
					break;
				default:
					$msg = "Estatus del registro no definido";
					break;
				}
			}
		}
		return array('code' => $code, 'verify_id' => $verify_id, 'msg' => $msg);
	}

	public static function getMsgVerify($code) {
		$msg = null;
		switch ($code) {
		case "IS_PHONE_CLIENT":
			$msg = "El telefono ingresado lo tiene otro cliente de Netwey";
			break;

		case "IS_PHONE_USER":
			$msg = "El telefono ingresado lo tiene un usuario de Netwey";
			break;

		case "IS_PHONE_USER_SELLER":
			$msg = "El telefono ingresado lo tiene un usuario de Netwey que realiza la venta";
			break;

		case "PHONE_EMPTY":
			$msg = "El telefono ingresado no posee registros en otro cliente de Netwey";
			break;

		case "CREATE":
			$msg = "El telefono ingresado posee registros de creado en espera de acciones segun el tipo de registro (Autorizacion de mesa o verificacion celular)";
			break;

		case "VERIFIED":
			$msg = "El telefono ingresado esta verificado";
			break;

		case "REJECTED":
			$msg = "El telefono ingresado esta en lista negra, se debe ingresar otro numero";
			break;

		case "AUTHORIZED":
			$msg = "El telefono ingresado esta autorizado para seguir sin verificación de celular, pero no podra ver planes de circulo de credito";
			break;

		default:
			$msg = "Mensaje no definido";
			break;
		}
		return $msg;
	}

/**
 * [registerNewToken Se guarda en BD un nuevo proceso de token para verificar un celular]
 * @param  [type] $phone     [description]
 * @param  [type] $clientDni [description]
 * @return [type]            [description]
 */
	public static function registerNewToken($phone, $clientDni, $newToken, $Obs = null, $platform = 'SMS', $mailSend = false) {
		$response = ['success' => false, 'title' => "No se pudo registrar el token de verificación para el celular", 'msg' => "Fallo el registro de datos", 'icon' => "error", 'code' => 'ERR_DAT'];

		try {
			//$newToken = Common::getTokenVerify();
			//Token permitido al formato ABC123
			//if (preg_match("/^[a-zA-Z]{3}[0-9]{3}$/", $newToken)) {
			if (preg_match("/^[0-9]{4}$/", $newToken)) {

				$register = self::getConnect('W')
					->insertGetId([
						'phone_verify' => $phone,
						'dni_client' => $clientDni,
						'token_verify' => $newToken,
						'email_seller' => session('user'),
						'type_verify' => 'VERIFIED',
						'date_reg' => date('Y-m-d H:i:s'),
						'status' => 'CREATE',
						'comment_register' => $Obs,
						'platform' => $platform,
						'email_verify' => ($mailSend) ? $mailSend : null]);

				$response['success'] = true;
				$response['title'] = "Nuevo token de verificación registrado";
				$response['msg'] = "OK";
				$response['icon'] = "success";
				$response['code'] = "OK";
				$response['verify_id'] = $register;

			} else {
				$response['msg'] = "El codigo token no cumple el patrón definido (MV201)";
			}
		} catch (Exception $e) {
			$txmsg = "No se pudo registrar la petición de generar token de verificación para el telefono " . $phone . " del cliente: " . $clientDni . " - " . (String) json_encode($e->getMessage());
			Log::error($txmsg);
			$response['msg'] = $txmsg;
		}
		return $response;
	}

	public static function deleteOldVerified($phone, $clientDni, $type = ["VERIFIED"], $Remove = ['CREATE'], $platform = ['SMS']) {
		try {
			self::getConnect('W')
				->where([
					['dni_client', $clientDni],
					['phone_verify', $phone]])
				->whereIn('type_verify', $type)
				->whereIn('status', $Remove)
				->whereIn('platform', $platform)
				->update([
					'status' => 'T',
					'date_update' => date('Y-m-d H:i:s'),
					'observacion' => "Eliminacion por re-envio o descarte de token"]);

			return array('success' => true, 'title' => "Eliminación de registro previos", 'msg' => "Eliminación satisfactoria", 'icon' => "success", 'code' => 'OK');

		} catch (Exception $e) {
			$txmsg = 'Error al eliminar el registro de verificación del cliente. ' . (String) json_encode($e->getMessage());
			Log::error($txmsg);
			return array('success' => false, 'title' => "Eliminación de registro previos no se pudieron remover", 'msg' => $txmsg . '  (MV229)', 'icon' => "error", 'code' => 'FAIL');
		}
	}

	public static function verifyToken($phone, $clientDni, $token) {
		try {

			$verify = self::getConnect('W')
				->where([
					['dni_client', $clientDni],
					['phone_verify', $phone],
					['type_verify', 'VERIFIED'],
					['token_verify', $token]])
				->first();

			if (!empty($verify)) {
				if ($verify->status == "CREATE") {
					//Verifico que sea halla creado menos de 24h
					$today = date('Y-m-d H:i:s');
					$today_time = strtotime($today);
					$expire_time = strtotime($verify->date_reg);
					$diff = $today_time - $expire_time;
					if ($diff < 86400) {

						//Si tiene registros previos de verificacion los colocamos en T
						$Dat = self::getConnect('W')
							->where([
								['dni_client', $clientDni],
								['phone_verify', $phone],
								['type_verify', 'VERIFIED'],
								['status', 'VERIFIED'],
							])
							->update([
								'status' => 'T',
								'date_update' => date('Y-m-d H:i:s'),
								'observacion' => "La verificacion fue reemplazada por un nuevo proceso",
							]);
						sleep(1);
						//////

						//Esta vigente entre las 24 horas
						//existe el token y se puede confirmar la verificacion
						$verify->status = "VERIFIED";
						$verify->date_update = date('Y-m-d H:i:s');
						$verify->observacion = "Verificado desde Seller";
						$verify->save();
						//Actualizo el telefono de contacto
						$updateC = Client::updatePhoneContact($clientDni, $phone, $verify->id);
						if ($updateC['success']) {

							return array('success' => true, 'title' => "Verificación de celular OK", 'msg' => "El telefono " . $phone . " fue verificado con exito!", 'icon' => "success", 'code' => 'OK');
						}
						return array('success' => false, 'title' => "Verificación de celular no se pudo completar", 'msg' => "No se pudo completar la actualizacion de datos posterior a la verificación del celular " . $phone . " (MV278)", 'icon' => "error", 'code' => 'FAIL_UP');
					} else {
						return array('success' => false, 'title' => "Verificación celular invalida", 'msg' => "El token suministrado tiene una antiguedad mayor a 24 horas, se debe solicitar uno nuevo (MV280)", 'icon' => "error", 'code' => 'TOKEN_CADUCE');
					}
				} else {
					switch ($verify->status) {
					case 'T':
						return array('success' => false, 'title' => "Verificación celular invalida", 'msg' => "El token suministrado no puede usarse, fue descartado por sistema debido a otro proceso de verificacion llevado a cabo por el prospecto. Te envitamos a solicitar un nuevo token (MV304)", 'icon' => "error", 'code' => 'TOKEN_DEL');
						break;
					case 'VERIFIED':
						return array('success' => false, 'title' => "Verificación celular invalida", 'msg' => "El token suministrado ya fue utilizado. Los token son de un solo uso de verificacion (MV307)", 'icon' => "error", 'code' => 'TOKEN_USE');
						break;
					default:
						return array('success' => false, 'title' => "Verificación celular invalida", 'msg' => "El token suministrado se encuentra en un status no definido (MV310)", 'icon' => "error", 'code' => 'TOKEN_ERROR');
						break;
					}
				}
			} else {
				return array('success' => false, 'title' => "Verificación de celular invalido", 'msg' => "No se encuentra registros del token " . strtoupper($token) . " para el celular " . $phone . " en sistema, verifique los datos e intente nuevamente (MV315)", 'icon' => "error", 'code' => 'FAIL_TOKEN');
			}
		} catch (Exception $e) {
			$txmsg = 'Error al realizar el proceso de revision del token de verificación del cliente ' . $clientDni . ' ' . (String) json_encode($e->getMessage());
			Log::error($txmsg);
			return array('success' => false, 'title' => "Verificacion de celular fallo", 'msg' => $txmsg, 'icon' => "error", 'code' => 'FAIL_BD');
		}
	}

	/**
	 * [getRegisterVerify Obtiene el registro de verificacion de celular]
	 * @param  [type] $dni         [dni del cliente]
	 * @return [type]              [description]
	 */
	public static function getRegisterVerify($id) {
		return self::getConnect('R')
			->where('id', $id)
			->first();
	}

	public static function registerNewAutorized($phone, $clientDni, $Obs = null) {
		$response = ['success' => false, 'title' => "No se pudo registrar la nueva autorizacion sin verificar el celular", 'msg' => "Fallo el registro de datos", 'icon' => "error", 'code' => 'ERR_DAT'];

		try {
			$register = self::getConnect('W')
				->insertGetId([
					'phone_verify' => $phone,
					'dni_client' => $clientDni,
					'email_seller' => session('user'),
					'type_verify' => 'AUTHORIZED',
					'date_reg' => date('Y-m-d H:i:s'),
					'status' => 'CREATE',
					'comment_register' => $Obs,
					'platform' => 'MESA']);

			$response['success'] = true;
			$response['title'] = "Nueva solicitud de autorización se registro";
			$response['msg'] = $register;
			$response['icon'] = "success";
			$response['code'] = "OK";
		} catch (Exception $e) {
			$txmsg = "No se pudo registrar la solicitud de autorizacion de continuar sin verificación del telefono " . $phone . " del cliente: " . $clientDni . " - " . (String) json_encode($e->getMessage());
			Log::error($txmsg);
			$response['msg'] = $txmsg;
		}
		return $response;
	}

	public static function getIconVerified($Typeicon, $heigh = '120', $width = '120') {
		switch ($Typeicon) {
		case "VERIFIED":
			$icon = '<svg version="1.0" xmlns="http://www.w3.org/2000/svg"
           width="' . $width . 'px" height="' . $heigh . 'px" viewBox="0 0 100.000000 100.000000"
           preserveAspectRatio="xMidYMid meet">
          <g transform="translate(0.000000,100.000000) scale(0.100000,-0.100000)"
          fill="#00c292" stroke="none">
          <path d="M264 915 l-25 -25 3 -396 3 -396 24 -19 c21 -17 41 -19 231 -19 190
          0 210 2 231 19 l24 19 0 402 0 402 -24 19 c-21 17 -41 19 -233 19 l-209 0 -25
          -25z m456 -45 l0 -30 -220 0 -220 0 0 30 0 30 220 0 220 0 0 -30z m0 -360 l0
          -290 -220 0 -220 0 0 290 0 290 220 0 220 0 0 -290z m0 -370 l0 -40 -220 0
          -220 0 0 40 0 40 220 0 220 0 0 -40z"/>
          <path d="M538 503 l-88 -87 -40 39 c-31 30 -43 36 -52 27 -9 -9 -1 -23 37 -62
          l49 -50 103 102 c90 89 110 118 85 118 -4 0 -46 -39 -94 -87z"/>
          <path d="M435 150 c-13 -21 12 -31 71 -28 44 2 59 7 59 18 0 19 -119 28 -130
          10z"/>
          </g>
          </svg></br>';
			break;
		case "NOT_VERIFIED":
			$icon = '<svg version="1.0" xmlns="http://www.w3.org/2000/svg"
           width="' . $width . 'px" height="' . $heigh . 'px" viewBox="0 0 100.000000 100.000000"
           preserveAspectRatio="xMidYMid meet">
          <g transform="translate(0.000000,100.000000) scale(0.100000,-0.100000)"
          fill="#e64942" stroke="none">
          <path d="M272 877 c-22 -23 -22 -28 -22 -379 l0 -357 26 -20 c25 -20 40 -21
          226 -21 198 0 199 0 223 25 l25 24 0 349 c0 338 -1 350 -21 376 l-20 26 -208
          0 c-204 0 -208 0 -229 -23z m440 -14 c10 -9 18 -27 18 -40 l0 -23 -230 0
          c-229 0 -230 0 -230 22 0 12 7 30 17 40 15 16 35 18 211 18 168 0 198 -2 214
          -17z m16 -350 l2 -263 -230 0 -230 0 0 258 c0 142 3 262 7 266 4 3 106 5 227
          4 l221 -3 3 -262z m0 -321 c2 -22 -3 -41 -14 -53 -16 -17 -33 -19 -212 -19
          -168 0 -198 2 -214 17 -18 16 -25 73 -11 87 4 3 106 5 227 4 l221 -3 3 -33z"/>
          <path d="M390 840 c0 -5 5 -10 10 -10 6 0 10 5 10 10 0 6 -4 10 -10 10 -5 0
          -10 -4 -10 -10z"/>
          <path d="M440 840 c0 -6 27 -10 60 -10 33 0 60 4 60 10 0 6 -27 10 -60 10 -33
          0 -60 -4 -60 -10z"/>
          <path d="M412 598 c-9 -9 -3 -21 30 -55 l42 -43 -42 -43 c-33 -34 -39 -46 -30
          -55 9 -9 21 -3 55 30 l43 42 45 -44 c35 -34 48 -41 57 -32 9 9 2 22 -32 57
          l-44 45 44 45 c34 35 41 48 32 57 -9 9 -22 2 -57 -32 l-45 -44 -43 42 c-34 33
          -46 39 -55 30z"/>
          <path d="M483 184 c-8 -21 13 -42 28 -27 13 13 5 43 -11 43 -6 0 -13 -7 -17
          -16z"/>
          </g>
          </svg></br>';
			break;
		case 'EDIT':
			$icon = '<svg version="1.0" xmlns="http://www.w3.org/2000/svg"
           width="' . $width . 'px" height="' . $heigh . 'px" viewBox="0 0 980.000000 980.000000"
           preserveAspectRatio="xMidYMid meet">

          <g transform="translate(0.000000,980.000000) scale(0.100000,-0.100000)"
          fill="#FF6A70" stroke="none">
          <path d="M7342 8569 c-131 -23 -320 -107 -420 -185 -26 -21 -84 -74 -130 -118
          l-82 -81 687 -687 c379 -379 693 -688 700 -688 19 0 179 177 230 253 62 96
          106 193 134 302 20 76 24 114 24 245 -1 131 -4 168 -24 240 -95 343 -340 594
          -677 695 -75 23 -112 27 -239 30 -82 2 -174 0 -203 -6z"/>
          <path d="M4883 6358 c-1127 -1128 -1587 -1594 -1598 -1620 -13 -32 -15 -120
          -15 -635 0 -559 2 -601 19 -638 22 -48 44 -69 94 -90 31 -13 118 -15 623 -15
          413 0 599 3 628 12 36 10 240 210 1631 1600 l1590 1588 -690 690 c-379 379
          -692 690 -695 690 -3 0 -717 -712 -1587 -1582z"/>
          <path d="M2450 7659 c-508 -47 -945 -406 -1089 -894 -54 -184 -52 -77 -48
          -2365 l2 -2115 23 -85 c69 -257 170 -438 342 -611 189 -189 434 -316 687 -354
          143 -22 3536 -23 3586 -1 19 8 398 380 909 892 749 751 878 885 888 919 8 29
          10 367 8 1221 l-3 1180 -25 37 c-38 56 -90 80 -159 75 -70 -5 -116 -34 -142
          -92 -18 -39 -19 -89 -19 -1118 l0 -1078 -782 0 c-742 0 -785 -1 -824 -19 -50
          -23 -80 -63 -94 -125 -7 -30 -10 -305 -10 -784 0 -405 0 -745 0 -755 0 -17
          -78 -17 -1647 -15 l-1648 3 -88 28 c-327 102 -571 366 -638 691 -18 87 -19
          175 -19 2141 0 1352 4 2072 10 2115 30 185 123 365 261 501 119 117 251 192
          427 242 77 22 79 22 1338 27 l1260 5 36 25 c52 35 72 75 73 141 0 44 -6 65
          -28 97 -55 84 59 77 -1297 78 -663 1 -1243 -2 -1290 -7z m4700 -4746 c0 -5
          -245 -253 -545 -553 l-545 -544 0 552 0 552 545 0 c300 0 545 -3 545 -7z"/>
          </g>
          </svg>';
			break;
		default:
			$icon = "<div class='text-center'> Estatus no definido para el proceso</div><br/>";
			break;
		}
		return $icon;
	}

	public static function getIconAutorized($Typeicon, $heigh = '120', $width = '120', $colorAutorized = "#00c292") {
		switch ($Typeicon) {
		case "REJECTED":
			$icon = '<svg version="1.0" xmlns="http://www.w3.org/2000/svg" width="' . $width . 'px" height="' . $heigh . 'px" viewBox="0 0 270.000000 270.000000" preserveAspectRatio="xMidYMid meet">
          <g transform="translate(0.000000,270.000000) scale(0.100000,-0.100000)"
fill="#e64942" stroke="none">
<path d="M1157 2499 c-32 -4 -98 -20 -147 -35 -77 -23 -87 -29 -73 -39 16 -12
16 -14 1 -29 -21 -20 -28 -20 -43 0 -13 19 -47 15 -79 -9 -17 -13 -17 -14 -1
-31 22 -21 14 -31 -13 -17 -31 17 -61 13 -94 -10 -23 -17 -27 -24 -16 -31 9
-7 7 -7 -9 -3 -16 4 -36 -6 -80 -43 -66 -56 -184 -178 -210 -219 -10 -16 -20
-30 -23 -33 -15 -12 -80 -131 -108 -195 -62 -143 -103 -337 -102 -478 1 -81
19 -217 29 -217 4 0 8 -21 8 -47 2 -105 171 -453 219 -453 8 0 13 -4 10 -9 -3
-5 1 -12 10 -15 8 -3 12 -2 9 3 -3 5 4 12 15 16 11 3 34 22 52 41 27 28 34 31
38 18 4 -9 21 -22 40 -28 39 -13 52 -36 20 -36 -16 0 -20 -4 -16 -15 3 -10 -1
-15 -13 -15 -10 0 -26 -10 -35 -23 l-17 -22 5 23 c2 13 1 21 -2 19 -4 -2 -22
-9 -40 -16 l-33 -13 22 -24 c12 -13 27 -24 33 -24 6 0 18 -12 27 -26 8 -14 51
-52 95 -85 192 -144 411 -223 651 -236 95 -5 126 -3 138 7 12 10 18 11 24 2 6
-10 51 -6 51 5 0 2 -7 15 -16 29 -9 13 -14 27 -10 31 5 5 34 -25 57 -59 6 -9
171 31 246 61 241 95 445 264 578 476 61 99 62 100 26 121 -17 10 -31 24 -31
31 0 7 -4 13 -10 13 -5 0 -7 -7 -3 -16 5 -13 2 -15 -18 -9 -13 4 -26 11 -29
14 -3 4 -19 8 -35 9 -35 2 -78 -35 -197 -170 -38 -42 -98 -99 -135 -126 -75
-56 -238 -146 -268 -148 -11 -1 -48 -9 -81 -18 -34 -9 -73 -16 -86 -16 -14 0
-27 -3 -31 -6 -10 -10 -166 -6 -252 6 -187 27 -369 117 -514 254 -130 124
-214 264 -268 451 -24 79 -27 108 -27 240 0 134 3 161 28 250 32 115 100 252
171 343 38 49 46 65 36 75 -8 8 -14 9 -19 1 -8 -13 -74 -15 -66 -2 3 4 13 8
23 8 9 0 26 7 37 15 10 8 27 15 37 15 10 0 15 5 12 10 -3 6 3 10 14 10 10 0
21 -5 23 -10 2 -6 34 13 71 43 36 29 100 71 142 93 78 40 92 62 53 79 -13 6
-24 18 -26 28 -3 13 3 17 26 17 31 0 42 -14 42 -52 0 -23 -1 -23 110 7 119 32
226 42 322 29 47 -7 95 -15 106 -18 27 -9 39 12 16 28 -11 8 -13 15 -6 20 14
8 65 -12 57 -23 -3 -5 7 -11 22 -13 30 -3 44 -27 16 -29 -10 -1 -3 -6 17 -13
223 -76 438 -264 546 -477 18 -37 40 -69 50 -73 12 -5 13 -10 6 -19 -8 -9 -6
-30 8 -75 28 -88 43 -233 36 -327 -8 -97 -26 -197 -37 -209 -5 -6 -9 -17 -9
-25 0 -9 -11 -40 -25 -70 -14 -30 -23 -56 -20 -59 2 -3 23 7 45 22 46 30 71
35 47 8 -25 -28 -20 -42 14 -42 18 0 47 -9 66 -21 26 -16 37 -18 48 -9 16 13
65 154 65 188 0 13 5 32 10 42 6 11 16 70 22 131 13 128 2 261 -32 399 -22 85
-35 114 -50 105 -15 -9 -51 29 -41 44 7 11 13 9 31 -9 l22 -22 -7 24 c-10 33
-34 68 -46 68 -6 0 -8 8 -5 18 6 19 -27 82 -44 82 -5 0 -7 6 -4 13 6 18 -81
136 -159 215 -116 119 -303 242 -365 242 -18 0 -32 4 -32 9 0 14 -65 41 -98
41 -16 0 -32 6 -35 14 -5 13 -45 23 -127 33 -76 9 -328 11 -383 2z m3 -40 c0
-21 -3 -22 -49 -13 -44 9 -38 29 9 31 33 1 40 -2 40 -18z m319 -8 c-3 -6 -10
-7 -18 -2 -7 5 -10 14 -6 21 9 14 29 -2 24 -19z m-204 -26 c0 -10 -6 -20 -12
-22 -8 -3 -13 5 -13 22 0 17 5 25 13 23 6 -3 12 -13 12 -23z m-75 5 c0 -5 -7
-10 -15 -10 -8 0 -15 5 -15 10 0 6 7 10 15 10 8 0 15 -4 15 -10z m-97 -66 c9
-4 17 -12 17 -19 0 -15 -51 -27 -63 -15 -5 4 -26 5 -48 3 -31 -5 -39 -2 -39
10 0 21 37 32 56 16 13 -11 18 -10 26 5 5 9 15 15 21 12 7 -3 20 -9 30 -12z
m336 -8 c56 -16 89 -36 58 -36 -9 0 -34 6 -54 14 -21 8 -50 17 -65 20 -16 4
-28 11 -28 17 0 6 8 9 18 6 9 -3 41 -12 71 -21z m426 -6 c-3 -6 6 -15 20 -20
14 -5 23 -14 20 -20 -7 -12 -65 -14 -65 -2 1 21 13 52 22 52 5 0 6 -5 3 -10z
m-235 -10 c0 -5 -4 -10 -9 -10 -6 0 -13 5 -16 10 -3 6 1 10 9 10 9 0 16 -4 16
-10z m-730 -25 c8 -9 8 -15 2 -15 -18 0 -42 12 -42 21 0 14 26 10 40 -6z m300
-11 c0 -8 -7 -14 -15 -14 -15 0 -21 21 -9 33 10 9 24 -2 24 -19z m40 10 c0
-17 -22 -14 -28 4 -2 7 3 12 12 12 9 0 16 -7 16 -16z m510 -14 c0 -5 -4 -10
-10 -10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m57 -32 c8 -18 6
-27 -6 -40 -15 -15 -16 -14 -17 23 -2 44 8 50 23 17z m-907 -7 c0 -10 -24 -21
-43 -21 -7 0 -3 7 7 15 22 17 36 19 36 6z m837 1 c7 -5 9 -13 3 -22 -7 -11
-13 -12 -36 -1 -22 10 -25 14 -13 22 17 11 30 11 46 1z m258 -21 c-3 -6 -11
-11 -17 -11 -6 0 -6 6 2 15 14 17 26 13 15 -4z m85 -12 c0 -5 -7 -9 -15 -9
-15 0 -20 12 -9 23 8 8 24 -1 24 -14z m-1290 -9 c0 -5 -2 -10 -4 -10 -3 0 -8
5 -11 10 -3 6 -1 10 4 10 6 0 11 -4 11 -10z m-48 -32 c-6 -6 -16 -7 -23 -3 -9
6 -7 12 7 22 21 15 34 -1 16 -19z m-212 -128 c0 -5 -4 -10 -9 -10 -6 0 -13 5
-16 10 -3 6 1 10 9 10 9 0 16 -4 16 -10z m1646 -35 c10 -8 14 -15 8 -15 -5 0
-19 7 -30 15 -10 8 -14 15 -8 15 5 0 19 -7 30 -15z m74 -35 c0 -5 -4 -10 -9
-10 -6 0 -13 5 -16 10 -3 6 1 10 9 10 9 0 16 -4 16 -10z m-1810 -41 c0 -11 -4
-17 -10 -14 -5 3 -10 13 -10 21 0 8 5 14 10 14 6 0 10 -9 10 -21z m1813 -51
c15 -6 27 -13 27 -17 0 -9 -42 -8 -57 2 -6 4 -18 4 -27 1 -10 -4 -16 -1 -16 8
0 17 35 20 73 6z m96 -47 c-15 -5 -22 6 -14 19 6 9 10 9 16 -1 5 -8 4 -15 -2
-18z m-2024 -151 c3 -5 1 -10 -4 -10 -6 0 -11 5 -11 10 0 6 2 10 4 10 3 0 8
-4 11 -10z m2125 -47 c0 -24 -25 -9 -28 17 -3 21 -2 22 12 11 9 -7 16 -20 16
-28z m-100 -3 c8 -5 12 -11 10 -14 -3 -2 -14 2 -24 10 -22 15 -10 19 14 4z
m70 -22 c0 -5 -9 -8 -20 -8 -23 0 -25 6 -7 24 12 13 26 4 27 -16z m60 -28 c0
-5 -4 -10 -9 -10 -6 0 -13 5 -16 10 -3 6 1 10 9 10 9 0 16 -4 16 -10z m-133
-14 c-3 -7 0 -19 6 -25 14 -14 3 -25 -18 -18 -21 9 -20 57 1 57 10 0 14 -6 11
-14z m59 -303 c-10 -10 -19 5 -10 18 6 11 8 11 12 0 2 -7 1 -15 -2 -18z m54
13 c0 -11 -19 -15 -25 -6 -3 5 1 10 9 10 9 0 16 -2 16 -4z m-110 -46 c0 -5 -5
-10 -11 -10 -5 0 -7 5 -4 10 3 6 8 10 11 10 2 0 4 -4 4 -10z m100 -36 c0 -8
-5 -12 -10 -9 -6 4 -8 11 -5 16 9 14 15 11 15 -7z m-2210 -39 c0 -21 -17 -18
-30 4 -15 28 -13 33 10 21 11 -6 20 -17 20 -25z m2130 -4 c12 -24 13 -31 1
-31 -15 0 -42 31 -36 41 10 15 24 10 35 -10z m-3 -76 c7 -20 0 -27 -20 -19 -9
3 -13 12 -10 20 7 19 22 18 30 -1z m83 -45 c0 -5 -5 -10 -11 -10 -5 0 -7 5 -4
10 3 6 8 10 11 10 2 0 4 -4 4 -10z m-2090 -150 c0 -11 -4 -20 -10 -20 -5 0
-10 9 -10 20 0 11 5 20 10 20 6 0 10 -9 10 -20z m18 -63 c2 -10 -3 -17 -12
-17 -10 0 -16 9 -16 21 0 24 23 21 28 -4z m107 13 c3 -5 -1 -10 -9 -10 -9 0
-16 5 -16 10 0 6 4 10 9 10 6 0 13 -4 16 -10z m-135 -80 c0 -5 -5 -10 -11 -10
-5 0 -7 5 -4 10 3 6 8 10 11 10 2 0 4 -4 4 -10z m1886 -177 c-10 -10 -19 5
-10 18 6 11 8 11 12 0 2 -7 1 -15 -2 -18z m-276 -159 c0 -9 -5 -14 -12 -12
-18 6 -21 28 -4 28 9 0 16 -7 16 -16z m-1105 -4 c3 -5 1 -10 -4 -10 -6 0 -11
5 -11 10 0 6 2 10 4 10 3 0 8 -4 11 -10z m895 0 c0 -5 -2 -10 -4 -10 -3 0 -8
5 -11 10 -3 6 -1 10 4 10 6 0 11 -4 11 -10z m-1120 -10 c0 -5 -2 -10 -4 -10
-3 0 -8 5 -11 10 -3 6 -1 10 4 10 6 0 11 -4 11 -10z m1091 -23 c-1 -12 -15 -9
-19 4 -3 6 1 10 8 8 6 -3 11 -8 11 -12z m-1011 -7 c8 -5 12 -11 10 -14 -3 -2
-14 2 -24 10 -22 15 -10 19 14 4z m204 1 c3 -5 -1 -12 -10 -15 -25 -10 -34 -7
-34 9 0 16 35 21 44 6z m-74 -73 c0 -13 -23 -5 -28 10 -2 7 2 10 12 6 9 -3 16
-11 16 -16z m881 -33 c-1 -15 -24 -12 -29 3 -3 9 2 13 12 10 10 -1 17 -7 17
-13z"/>
<path d="M1225 2181 c-280 -45 -523 -223 -646 -473 -27 -56 -31 -69 -20 -82 8
-9 20 -16 27 -16 8 0 14 -7 14 -15 0 -20 -24 -19 -40 0 -17 21 -25 11 -44 -53
l-15 -53 27 -11 c15 -5 39 -16 54 -24 14 -8 33 -14 41 -14 21 0 32 -19 19 -32
-8 -8 -15 -8 -24 -1 -7 6 -35 18 -61 28 -42 16 -50 16 -58 4 -12 -19 -11 -182
2 -255 12 -72 38 -154 47 -153 19 3 25 -2 13 -10 -13 -8 -12 -16 4 -52 11 -23
24 -47 30 -53 5 -6 18 -28 28 -48 11 -19 39 -58 64 -84 25 -27 40 -42 34 -32
-14 20 0 32 27 24 13 -4 18 -17 19 -42 1 -22 10 -45 24 -60 27 -28 141 -104
157 -104 19 0 2 29 -21 36 -23 7 -71 53 -64 61 3 2 11 -3 19 -11 11 -10 18
-12 26 -4 7 7 20 0 42 -26 18 -20 43 -39 56 -42 14 -4 29 -14 34 -24 20 -37
97 -70 183 -80 127 -15 204 -13 211 5 4 13 9 13 25 4 21 -14 104 -1 179 28 33
12 41 20 37 34 -8 24 11 34 24 12 9 -17 12 -17 48 2 125 64 244 171 320 289
42 64 46 81 20 81 -5 0 -12 6 -15 13 -3 9 3 11 20 7 15 -4 28 -1 35 7 16 21
53 129 48 138 -3 5 -1 11 4 15 6 3 13 27 16 53 3 26 8 59 10 73 4 27 -8 36
-57 43 -41 6 -29 18 15 15 l47 -4 0 40 c-1 142 -53 330 -102 376 -21 18 -26
28 -17 31 17 6 -38 88 -113 173 -43 48 -52 54 -66 43 -13 -11 -15 -10 -10 9
11 42 -179 151 -338 193 -88 23 -260 34 -339 21z m483 -171 c3 -20 0 -22 -20
-17 -12 4 -31 7 -40 7 -10 0 -18 8 -18 18 0 13 8 17 37 16 32 -1 39 -5 41 -24z
m-513 10 c-3 -5 -10 -10 -16 -10 -5 0 -9 5 -9 10 0 6 7 10 16 10 8 0 12 -4 9
-10z m618 -23 c7 3 9 3 5 -2 -4 -4 -3 -14 3 -21 7 -9 6 -14 -2 -20 -7 -4 -15
-2 -17 4 -2 7 -12 12 -22 12 -19 0 -43 23 -34 33 3 3 17 1 31 -4 14 -5 30 -6
36 -2z m-883 -57 c0 -11 -7 -20 -15 -20 -15 0 -21 21 -8 33 12 13 23 7 23 -13z
m710 12 c0 -13 -12 -22 -22 -16 -10 6 -1 24 13 24 5 0 9 -4 9 -8z m240 -1 c0
-10 -30 -24 -37 -17 -3 3 -2 9 2 15 8 13 35 15 35 2z m-870 -11 c0 -5 -9 -10
-20 -10 -11 0 -20 5 -20 10 0 6 9 10 20 10 11 0 20 -4 20 -10z m80 0 c0 -5 -4
-10 -10 -10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m670 0 c0 -5
-4 -10 -10 -10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m-1008
-58 c-27 -4 -37 2 -27 17 4 8 13 7 28 -1 l22 -13 -23 -3z m1233 19 c3 -5 2
-12 -2 -14 -13 -7 -73 4 -73 14 0 12 67 12 75 0z m-1147 -33 c-6 -18 -28 -21
-28 -4 0 9 7 16 16 16 9 0 14 -5 12 -12z m160 -50 c-6 -18 -28 -21 -28 -4 0 9
7 16 16 16 9 0 14 -5 12 -12z m320 -27 c3 -8 -1 -12 -9 -9 -7 2 -15 10 -17 17
-3 8 1 12 9 9 7 -2 15 -10 17 -17z m748 -20 c-1 -3 -7 -15 -11 -26 -7 -18 -9
-17 -15 10 -8 37 -2 50 16 34 7 -6 12 -14 10 -18z m-1198 -21 l23 -30 -24 0
c-28 0 -47 17 -47 42 0 27 24 22 48 -12z m317 20 c3 -5 -1 -10 -10 -10 -9 0
-13 5 -10 10 3 6 8 10 10 10 2 0 7 -4 10 -10z m825 -15 c0 -3 -9 -5 -20 -5
-11 0 -20 7 -20 15 0 13 4 14 20 5 11 -6 20 -13 20 -15z m-916 -17 c14 -6 77
-63 140 -127 l115 -116 123 121 c68 67 134 125 146 129 58 18 123 -20 141 -82
15 -53 -9 -94 -125 -213 -60 -61 -111 -115 -113 -121 -4 -13 218 -239 235
-239 6 0 21 -11 34 -25 16 -17 34 -25 57 -25 23 0 33 -5 33 -16 0 -8 -5 -12
-10 -9 -6 4 -33 4 -60 0 -38 -5 -50 -11 -50 -24 0 -26 -26 -57 -65 -75 -57
-27 -86 -11 -224 126 l-122 122 -117 -117 c-64 -64 -129 -126 -144 -137 l-27
-21 -41 21 c-22 11 -45 18 -49 15 -16 -10 -42 6 -32 19 7 8 7 24 -1 48 -16 54
-2 78 127 212 63 65 115 121 115 125 0 3 -52 58 -115 121 -119 118 -142 156
-130 209 6 28 47 74 75 84 26 9 58 8 84 -5z m-264 -48 c0 -5 -4 -10 -10 -10
-5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m1250 -10 c0 -5 -4 -10
-10 -10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m-1359 -173 c-13
-13 -26 -3 -16 12 3 6 11 8 17 5 6 -4 6 -10 -1 -17z m1424 -115 c-7 -8 -35 7
-35 18 0 6 7 6 20 -2 10 -7 17 -14 15 -16z m-124 -52 c5 0 9 -4 9 -10 0 -5
-11 -10 -25 -10 -16 0 -25 6 -25 16 0 11 5 14 16 10 9 -3 20 -6 25 -6z m-1453
-58 c2 -7 -2 -10 -12 -6 -9 3 -16 11 -16 16 0 13 23 5 28 -10z m91 -49 c28
-21 46 -40 40 -42 -6 -2 -24 6 -40 18 -16 12 -40 26 -54 31 -15 6 -23 14 -20
20 10 16 21 12 74 -27z m1451 -18 c7 -9 11 -17 8 -20 -2 -2 -12 4 -21 15 -9
11 -13 20 -8 20 5 0 14 -7 21 -15z m-1540 -5 c0 -5 -4 -10 -10 -10 -5 0 -10 5
-10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m1466 -15 c4 -8 3 -23 0 -32 -6 -15
-8 -15 -16 -3 -13 20 -12 50 0 50 6 0 13 -7 16 -15z m54 -56 c0 -14 -16 -11
-29 5 -10 12 -8 13 8 9 12 -3 21 -9 21 -14z m-1540 -19 c0 -5 -4 -10 -10 -10
-5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m66 -37 c16 -21 18 -53 4
-53 -5 0 -10 10 -10 23 0 13 -5 28 -12 35 -7 7 -8 12 -4 12 5 0 15 -8 22 -17z
m1334 -85 c0 -11 -22 -10 -39 1 -10 7 -7 9 12 8 15 -1 27 -5 27 -9z m77 -43
c-1 -14 -7 -25 -12 -25 -6 0 -10 5 -9 10 1 6 2 17 3 25 2 26 20 16 18 -10z
m-927 -75 c0 -5 -2 -10 -4 -10 -3 0 -8 5 -11 10 -3 6 -1 10 4 10 6 0 11 -4 11
-10z m-264 -109 c6 -12 19 -28 29 -35 10 -7 15 -18 11 -25 -5 -8 -12 -9 -19
-2 -7 5 -23 11 -37 13 -17 2 -26 10 -28 26 -8 53 22 68 44 23z m59 20 c3 -5 1
-12 -5 -16 -5 -3 -10 1 -10 9 0 18 6 21 15 7z m65 -102 c0 -5 -4 -9 -10 -9 -5
0 -10 7 -10 16 0 8 5 12 10 9 6 -3 10 -10 10 -16z m-120 -9 c0 -5 -4 -10 -10
-10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m200 -11 c0 -6 -4 -7
-10 -4 -5 3 -10 11 -10 16 0 6 5 7 10 4 6 -3 10 -11 10 -16z m420 -5 c0 -8 -4
-12 -10 -9 -5 3 -10 10 -10 16 0 5 5 9 10 9 6 0 10 -7 10 -16z m-332 -12 c2
-7 -3 -12 -12 -12 -9 0 -16 7 -16 16 0 17 22 14 28 -4z m382 8 c0 -5 -4 -10
-10 -10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m-350 -20 c0 -5
-2 -10 -4 -10 -3 0 -8 5 -11 10 -3 6 -1 10 4 10 6 0 11 -4 11 -10z m84 -9 c19
-11 37 -17 40 -14 3 4 6 2 6 -4 0 -15 -52 -13 -77 3 -23 15 -31 34 -13 34 5 0
25 -9 44 -19z m406 -47 c0 -17 -22 -14 -28 4 -2 7 3 12 12 12 9 0 16 -7 16
-16z m-451 -21 c23 -3 34 -20 18 -26 -9 -3 -25 6 -38 20 -13 15 -17 22 -9 17
8 -4 21 -9 29 -11z m131 -43 c0 -5 -4 -10 -10 -10 -5 0 -10 5 -10 10 0 6 5 10
10 10 6 0 10 -4 10 -10z"/>
</g></svg><br/>';
			break;
		case "AUTHORIZED":
			$icon = '<svg version="1.0" xmlns="http://www.w3.org/2000/svg"
 width="' . $width . 'px" height="' . $heigh . 'px" viewBox="0 0 270.000000 270.000000"
 preserveAspectRatio="xMidYMid meet">
<g transform="translate(0.000000,270.000000) scale(0.100000,-0.100000)"
fill="' . $colorAutorized . '" stroke="none">
<path d="M1185 2519 c-87 -12 -102 -17 -48 -18 32 -1 43 -5 48 -21 3 -11 11
-20 16 -20 5 0 9 -7 9 -15 0 -8 -7 -15 -15 -15 -8 0 -15 7 -15 16 0 8 -4 13
-9 9 -14 -8 -91 18 -91 32 0 8 -8 9 -27 2 -16 -5 -47 -14 -71 -19 -35 -9 -40
-13 -30 -24 7 -8 16 -12 21 -9 4 2 7 -1 7 -6 0 -6 5 -11 11 -11 6 0 20 -11 31
-26 19 -23 22 -24 38 -9 12 11 26 14 44 9 31 -7 38 -17 30 -37 -8 -21 -63 -28
-73 -9 -6 10 -10 11 -15 3 -4 -6 -21 -11 -37 -11 -34 0 -78 33 -59 45 7 4 15
3 19 -3 4 -7 12 -8 21 -2 18 11 6 24 -29 35 -15 5 -21 4 -17 -3 13 -22 -17
-24 -34 -2 -18 24 -44 22 -83 -5 -18 -12 -17 -14 4 -29 19 -13 21 -18 11 -28
-10 -10 -14 -10 -20 -1 -15 24 -65 26 -91 4 -12 -11 -20 -24 -17 -29 3 -6 -2
-16 -11 -23 -10 -8 -14 -9 -10 -2 4 6 3 14 -2 18 -17 10 -159 -113 -241 -210
-85 -101 -192 -292 -194 -346 -1 -16 -4 -29 -7 -29 -10 0 -37 -80 -51 -150
-17 -91 -17 -412 1 -434 17 -20 31 -21 31 -1 0 8 5 15 10 15 6 0 10 -11 10
-24 0 -17 -7 -26 -24 -30 -21 -5 -23 -9 -16 -38 19 -80 63 -197 81 -216 10
-12 19 -29 19 -38 0 -19 38 -88 79 -144 38 -51 81 -53 126 -6 37 39 52 45 57
20 2 -9 19 -22 38 -28 39 -13 54 -46 21 -46 -12 0 -21 -6 -21 -15 0 -8 -9 -15
-20 -15 -10 0 -24 -9 -30 -20 -13 -25 -33 -16 -25 11 6 18 5 18 -24 2 -28 -17
-29 -19 -14 -35 10 -11 24 -16 35 -12 14 4 18 0 18 -19 0 -13 9 -32 20 -42 11
-10 20 -13 20 -7 0 7 5 12 11 12 6 0 9 -7 5 -15 -13 -35 201 -166 368 -226
151 -53 456 -90 456 -54 0 19 27 20 32 1 2 -8 10 -11 20 -7 16 6 16 9 3 35
-19 36 -19 46 0 46 8 0 15 -3 15 -7 0 -5 11 -21 24 -36 l24 -28 76 16 c289 60
574 262 739 525 32 51 57 93 55 94 -44 28 -60 35 -72 30 -9 -3 -16 -2 -16 4 0
10 -27 21 -67 27 -14 2 -33 -7 -52 -26 -17 -16 -36 -29 -42 -29 -6 0 -8 -7 -5
-17 4 -10 4 -14 0 -11 -10 10 -47 -29 -39 -41 3 -7 3 -11 -2 -10 -4 1 -43 -33
-87 -75 -84 -84 -162 -139 -266 -188 -36 -17 -70 -38 -76 -46 -6 -8 -16 -12
-22 -8 -14 8 -191 -31 -211 -47 -9 -8 -15 -8 -18 -2 -2 7 -28 9 -66 7 -35 -2
-100 3 -145 11 -46 9 -89 14 -97 10 -8 -3 -15 -1 -15 5 0 5 -17 13 -37 17 -21
4 -49 13 -63 20 -14 7 -50 25 -80 39 -72 33 -142 78 -212 139 -32 27 -58 44
-58 38 0 -6 -7 -8 -15 -5 -16 7 -21 34 -6 34 6 0 -4 15 -20 32 -17 18 -42 52
-57 75 -18 28 -36 45 -55 49 -16 3 -40 12 -55 20 -23 12 -24 14 -8 20 11 4 31
0 49 -11 l31 -18 -26 44 c-14 24 -31 63 -38 85 -10 33 -16 40 -31 37 -13 -4
-19 0 -19 11 0 11 5 14 12 10 8 -5 9 0 5 17 -4 13 -12 57 -19 97 -9 52 -16 72
-27 72 -12 0 -12 3 -3 12 17 17 15 38 -3 38 -20 0 -19 26 0 34 10 4 14 13 10
29 -8 29 31 257 51 306 13 32 13 37 -1 47 -19 14 -19 24 0 24 8 0 15 -5 15
-11 0 -6 18 25 40 70 23 44 58 102 79 128 44 56 65 92 58 100 -3 3 -12 -4 -19
-16 -15 -24 -30 -28 -21 -5 4 11 0 14 -18 12 -32 -4 -37 14 -8 29 13 7 18 13
12 13 -7 0 -13 10 -13 23 l1 22 18 -22 c18 -22 38 -32 27 -14 -3 4 6 6 19 3
18 -3 25 0 25 12 0 10 9 16 25 16 14 0 25 -5 25 -11 0 -6 13 0 28 13 63 52
147 106 198 129 63 26 64 31 9 77 -24 21 -12 37 28 38 29 1 37 -3 37 -16 0
-10 10 -21 23 -25 l22 -8 -22 -4 c-23 -4 -33 -23 -13 -23 6 0 36 9 67 20 31
11 85 23 120 26 35 4 66 11 70 17 4 6 -14 7 -51 2 -52 -6 -57 -5 -54 11 1 9
16 22 31 27 22 7 32 5 48 -9 11 -10 19 -12 19 -6 0 20 17 13 23 -9 5 -17 11
-21 33 -16 15 2 74 -1 133 -8 65 -8 110 -10 116 -4 5 6 3 9 -8 9 -9 0 -17 7
-17 15 0 19 28 19 71 1 27 -11 30 -15 13 -19 -11 -3 -2 -6 21 -7 22 -2 47 -4
55 -5 8 -1 23 -3 33 -4 13 -1 16 -9 15 -32 -3 -28 2 -34 47 -60 28 -16 57 -32
65 -36 31 -14 127 -92 177 -144 86 -91 133 -156 179 -251 19 -39 34 -58 46
-58 22 0 24 -16 3 -27 -12 -7 -13 -12 -1 -35 8 -15 20 -53 27 -85 20 -91 16
-493 -5 -490 -5 1 -21 -30 -36 -70 -29 -77 -26 -81 32 -41 14 10 29 18 32 18
3 0 17 11 32 25 14 13 31 22 36 19 6 -4 -13 -27 -41 -52 -28 -25 -51 -49 -51
-54 0 -5 15 -8 33 -7 17 2 49 -6 70 -16 33 -17 39 -17 47 -5 15 23 60 167 60
190 0 12 4 29 9 39 5 9 13 56 17 103 6 75 5 86 -9 85 -9 -1 -16 3 -16 8 0 6 7
9 16 8 14 -1 15 12 10 110 -4 62 -16 148 -28 192 -12 44 -23 89 -26 101 -3 15
-11 20 -26 17 -15 -3 -26 3 -35 17 -15 25 -6 44 18 37 16 -4 16 -2 3 24 -8 16
-18 27 -23 24 -4 -3 -10 2 -13 9 -3 8 -1 16 5 18 16 6 -22 68 -40 68 -8 0 -12
6 -8 15 13 34 -139 216 -264 316 -110 89 -223 152 -256 143 -12 -3 -30 4 -47
20 -15 14 -27 22 -27 19 0 -3 -7 0 -15 7 -8 7 -22 10 -31 6 -8 -3 -25 0 -37 6
-37 21 -122 46 -125 37 -5 -13 -30 -11 -34 3 -5 15 -247 20 -343 7z m212 -5
c-3 -3 -12 -4 -19 -1 -8 3 -5 6 6 6 11 1 17 -2 13 -5z m40 0 c-3 -3 -12 -4
-19 -1 -8 3 -5 6 6 6 11 1 17 -2 13 -5z m61 -38 c3 -13 -1 -17 -14 -14 -11 2
-20 11 -22 22 -3 13 1 17 14 14 11 -2 20 -11 22 -22z m-248 -1 c0 -8 -4 -15
-9 -15 -13 0 -22 16 -14 24 11 11 23 6 23 -9z m40 -31 c0 -15 -6 -24 -15 -24
-15 0 -21 40 -9 53 12 11 24 -3 24 -29z m154 -20 c22 -9 20 -24 -3 -24 -21 0
-34 13 -24 24 8 7 8 7 27 0z m326 -13 c0 -4 -11 -13 -25 -19 -30 -14 -35 3 -8
24 18 12 33 10 33 -5z m-301 -35 c67 -26 83 -52 25 -42 -21 4 -47 14 -58 22
-11 8 -30 14 -42 14 -23 0 -48 22 -37 33 8 8 41 0 112 -27z m31 24 c0 -5 -7
-10 -15 -10 -8 0 -15 5 -15 10 0 6 7 10 15 10 8 0 15 -4 15 -10z m387 -29 c3
-11 14 -22 25 -24 32 -9 10 -27 -33 -27 -43 0 -45 3 -26 43 14 32 27 35 34 8z
m-241 -2 c11 -19 -13 -34 -31 -19 -13 11 -13 15 -3 21 18 12 26 11 34 -2z
m-732 -30 c15 -14 -1 -22 -29 -15 -14 4 -25 13 -25 22 0 11 6 14 23 9 12 -4
26 -11 31 -16z m846 -14 c0 -8 -7 -15 -15 -15 -8 0 -15 7 -15 15 0 8 7 15 15
15 8 0 15 -7 15 -15z m57 -35 c3 -11 12 -20 19 -20 8 0 14 -5 14 -11 0 -6 -8
-8 -19 -4 -13 4 -20 0 -24 -15 -10 -37 -25 -21 -24 25 2 47 24 63 34 25z
m-897 -4 c0 -16 -57 -39 -65 -27 -5 9 11 25 35 34 25 9 30 8 30 -7z m1100 -5
c0 -21 -13 -41 -26 -41 -17 0 -18 22 -2 38 14 14 28 16 28 3z m-1190 -25 c0
-13 -20 -26 -41 -26 -16 0 -9 28 9 33 29 8 32 7 32 -7z m1268 -9 c2 -10 -3
-17 -12 -17 -9 0 -16 2 -16 4 0 2 -3 12 -6 21 -5 13 -2 16 12 13 10 -2 20 -11
22 -21z m-1338 -27 c0 -13 -7 -20 -20 -20 -19 0 -27 20 -13 33 13 14 33 6 33
-13z m30 -21 c0 -5 -4 -9 -10 -9 -5 0 -10 7 -10 16 0 8 5 12 10 9 6 -3 10 -10
10 -16z m1208 -67 c2 -7 -3 -12 -12 -12 -9 0 -16 7 -16 16 0 17 22 14 28 -4z
m77 -33 c10 -15 -1 -23 -20 -15 -9 3 -13 10 -10 16 8 13 22 13 30 -1z m-1528
-14 c3 -9 3 -18 1 -21 -9 -9 -38 8 -38 22 0 19 29 18 37 -1z m1644 -35 c10 -6
19 -15 19 -20 0 -16 -26 -11 -51 10 -21 17 -22 19 -6 20 10 0 27 -5 38 -10z
m-1696 -30 c3 -5 12 -7 20 -3 8 3 15 0 15 -6 0 -6 -6 -11 -14 -11 -8 0 -17 -6
-20 -12 -5 -10 -7 -10 -12 0 -3 6 -12 12 -21 12 -11 0 -13 -7 -8 -31 8 -35
-10 -38 -29 -5 -9 13 -7 20 9 31 11 8 25 19 30 24 13 13 22 13 30 1z m1763 -4
c10 -6 11 -11 3 -19 -7 -7 -15 -6 -26 3 -29 24 -10 37 23 16z m60 -35 c3 -8
-1 -12 -9 -9 -7 2 -15 10 -17 17 -3 8 1 12 9 9 7 -2 15 -10 17 -17z m-83 -1
c3 -5 1 -10 -4 -10 -6 0 -11 5 -11 10 0 6 2 10 4 10 3 0 8 -4 11 -10z m-1695
-51 c0 -5 -4 -9 -10 -9 -5 0 -10 7 -10 16 0 8 5 12 10 9 6 -3 10 -10 10 -16z
m1661 1 c13 -7 19 -7 19 0 0 15 26 12 44 -5 8 -8 19 -15 25 -15 6 0 11 -6 11
-14 0 -19 -60 -18 -67 1 -4 10 -9 11 -18 3 -10 -8 -20 -7 -39 5 -39 26 -18 47
25 25z m-1834 -52 c-3 -8 -6 -5 -6 6 -1 11 2 17 5 13 3 -3 4 -12 1 -19z m2009
8 c24 -18 17 -26 -18 -19 -16 3 -28 12 -28 19 0 18 21 18 46 0z m-2028 -169
c2 -10 -3 -14 -15 -13 -10 2 -17 6 -15 9 1 4 2 12 2 18 0 16 25 5 28 -14z
m2095 3 c21 -25 22 -56 3 -76 -19 -19 -56 -18 -56 1 0 8 5 15 11 15 8 0 7 5
-1 15 -7 9 -10 18 -7 22 4 3 13 0 22 -7 12 -10 15 -10 15 3 0 8 -7 18 -16 21
-8 3 -12 10 -9 16 9 15 19 12 38 -10z m-2043 -21 c0 -6 -7 -17 -15 -25 -16
-16 -19 -7 -9 20 7 18 24 22 24 5z m1980 1 c0 -5 -4 -10 -10 -10 -5 0 -10 5
-10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m-50 -13 c12 3 20 0 20 -6 0 -7 5 -9
10 -6 6 3 10 -3 10 -14 0 -20 -1 -20 -35 -3 -19 9 -34 24 -34 32 0 9 3 10 5 4
3 -7 14 -10 24 -7z m-2070 -57 c0 -5 -7 -10 -15 -10 -8 0 -15 5 -15 10 0 6 7
10 15 10 8 0 15 -4 15 -10z m2238 -7 c2 -8 -5 -13 -17 -13 -12 0 -21 6 -21 16
0 18 31 15 38 -3z m-135 -30 c7 -42 4 -48 -18 -40 -9 4 -15 19 -15 42 0 48 25
47 33 -2z m163 -198 c-3 -9 -11 -13 -16 -10 -8 5 -7 11 1 21 14 18 24 11 15
-11z m-136 -11 c0 -8 -4 -12 -10 -9 -5 3 -10 10 -10 16 0 5 5 9 10 9 6 0 10
-7 10 -16z m-2068 -19 c0 -5 -5 -11 -11 -13 -6 -2 -11 4 -11 13 0 9 5 15 11
13 6 -2 11 -8 11 -13z m2108 -50 c0 -19 -20 -29 -32 -17 -8 8 -7 16 2 26 14
17 30 12 30 -9z m55 6 c3 -6 -3 -15 -14 -21 -21 -11 -37 2 -26 20 8 13 32 13
40 1z m-2215 -9 c0 -13 -12 -22 -22 -16 -10 6 -1 24 13 24 5 0 9 -4 9 -8z m56
-40 c-9 -9 -37 8 -31 18 5 8 11 7 21 -1 8 -6 12 -14 10 -17z m2074 -13 c0 -9
-57 -5 -67 4 -14 15 18 24 43 13 13 -6 24 -14 24 -17z m70 -24 c0 -8 -7 -15
-15 -15 -9 0 -12 -6 -9 -15 4 -9 0 -15 -10 -15 -19 0 -20 13 -4 40 15 24 38
26 38 5z m-2210 -46 c0 -11 -4 -19 -9 -19 -5 0 -16 -3 -24 -6 -8 -3 -17 2 -21
11 -3 9 -2 13 4 10 6 -3 10 1 10 9 0 8 -5 18 -10 21 -6 4 -8 11 -5 16 9 14 55
-21 55 -42z m2126 4 c18 -24 17 -33 -4 -33 -21 0 -42 18 -42 37 0 19 31 16 46
-4z m-14 -55 c25 -29 19 -54 -12 -50 -23 2 -40 17 -40 36 0 5 7 3 15 -4 9 -8
15 -9 15 -2 0 6 -5 14 -10 17 -10 6 -5 25 7 25 3 0 14 -10 25 -22z m-2042 -38
c0 -5 -4 -10 -9 -10 -6 0 -13 5 -16 10 -3 6 1 10 9 10 9 0 16 -4 16 -10z m40
0 c0 -5 -4 -10 -9 -10 -6 0 -13 5 -16 10 -3 6 1 10 9 10 9 0 16 -4 16 -10z
m2093 -47 c-7 -18 -22 -16 -30 3 -7 19 18 38 28 22 4 -6 5 -17 2 -25z m-2128
8 c7 -12 -12 -24 -25 -16 -11 7 -4 25 10 25 5 0 11 -4 15 -9z m65 -61 c0 -5
-7 -10 -15 -10 -8 0 -15 5 -15 10 0 6 7 10 15 10 8 0 15 -4 15 -10z m-113 -45
c8 -22 -6 -33 -17 -15 -13 20 -12 30 0 30 6 0 13 -7 17 -15z m92 -38 c-2 -12
2 -32 9 -43 9 -18 9 -24 -3 -34 -21 -17 -32 -5 -27 30 2 18 0 30 -7 30 -14 0
-14 23 1 38 16 16 30 5 27 -21z m-19 -132 c0 -8 -4 -15 -10 -15 -5 0 -10 7
-10 15 0 8 5 15 10 15 6 0 10 -7 10 -15z m190 0 c0 -8 -4 -15 -10 -15 -5 0
-10 7 -10 15 0 8 5 15 10 15 6 0 10 -7 10 -15z m-120 -50 c0 -8 -7 -15 -15
-15 -16 0 -20 12 -8 23 11 12 23 8 23 -8z m1880 -79 c0 -2 -7 -9 -15 -16 -12
-10 -15 -10 -15 4 0 9 7 16 15 16 8 0 15 -2 15 -4z m-50 -31 c0 -8 -9 -15 -20
-15 -11 0 -20 7 -20 15 0 8 9 15 20 15 11 0 20 -7 20 -15z m-1512 -37 c-6 -18
-28 -21 -28 -4 0 9 7 16 16 16 9 0 14 -5 12 -12z m-38 -18 c0 -5 -5 -10 -11
-10 -5 0 -7 5 -4 10 3 6 8 10 11 10 2 0 4 -4 4 -10z m-104 -57 c-10 -10 -19 5
-10 18 6 11 8 11 12 0 2 -7 1 -15 -2 -18z m34 -3 c0 -5 -4 -10 -10 -10 -5 0
-10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m-10 -30 c0 -5 -4 -10 -10 -10
-5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m140 -5 c0 -8 -7 -15 -15
-15 -8 0 -15 7 -15 15 0 8 7 15 15 15 8 0 15 -7 15 -15z m1209 -9 c9 -10 8
-16 -4 -26 -19 -16 -41 1 -32 24 8 20 21 20 36 2z m-1331 -24 c2 -7 -3 -12
-12 -12 -9 0 -16 7 -16 16 0 17 22 14 28 -4z m225 3 c1 -5 -6 -11 -15 -13 -11
-2 -18 3 -18 13 0 17 30 18 33 0z m897 -1 c0 -17 -22 -14 -28 4 -2 7 3 12 12
12 9 0 16 -7 16 -16z m115 -5 c10 -15 -1 -23 -20 -15 -9 3 -13 10 -10 16 8 13
22 13 30 -1z m-149 -14 c3 -8 3 -19 -1 -25 -10 -16 -35 7 -28 26 7 18 22 18
29 -1z m-1026 -5 c0 -5 7 -10 15 -10 8 0 15 -7 15 -16 0 -9 -5 -13 -12 -10 -7
3 -21 9 -30 12 -22 8 -24 34 -3 34 8 0 15 -4 15 -10z m217 -6 c6 -16 -12 -34
-35 -34 -19 0 -27 31 -11 41 21 13 39 11 46 -7z m-17 -59 c0 -8 -7 -15 -15
-15 -8 0 -15 7 -15 15 0 8 7 15 15 15 8 0 15 -7 15 -15z m-52 -17 c5 -25 -25
-23 -38 3 -10 18 -9 20 12 17 12 -2 24 -11 26 -20z m879 -24 c8 -21 -13 -42
-28 -27 -13 13 -5 43 11 43 6 0 13 -7 17 -16z m-437 -64 c0 -5 -5 -12 -11 -16
-14 -8 -33 11 -25 25 7 11 36 4 36 -9z m67 -2 c-3 -8 -6 -5 -6 6 -1 11 2 17 5
13 3 -3 4 -12 1 -19z m-27 -35 c0 -5 -5 -15 -10 -23 -8 -12 -10 -11 -10 8 0
12 5 22 10 22 6 0 10 -3 10 -7z"/>
<path d="M1667 2275 c-16 -12 -16 -13 9 -19 14 -4 29 -5 32 -1 3 3 0 5 -6 5
-6 0 -12 6 -14 14 -2 12 -6 13 -21 1z"/>
<path d="M1220 2190 c-163 -29 -315 -102 -444 -214 -34 -29 -38 -37 -27 -50 7
-8 19 -12 27 -9 8 3 14 -1 14 -11 0 -12 -10 -16 -35 -16 -26 0 -35 4 -35 17 0
23 -83 -96 -125 -177 -27 -54 -30 -67 -19 -77 7 -7 17 -13 23 -13 13 0 20 -25
11 -39 -6 -10 -12 -10 -29 0 -11 8 -21 18 -22 24 -1 14 -37 -86 -38 -106 -1
-11 10 -19 33 -23 19 -4 39 -13 46 -21 7 -8 23 -15 36 -15 18 0 24 -6 24 -21
0 -18 -4 -20 -28 -14 -15 3 -40 13 -56 21 -16 7 -32 11 -36 9 -5 -3 -10 1 -13
7 -8 22 -14 5 -21 -54 -5 -51 -4 -58 11 -58 16 -1 16 -2 2 -13 -13 -10 -14
-24 -8 -87 4 -41 8 -81 8 -88 1 -7 9 -12 18 -10 10 2 19 -5 21 -16 2 -11 0
-16 -7 -12 -12 8 -14 -7 -5 -40 4 -14 17 -24 36 -28 34 -7 34 -33 0 -28 -17 2
-20 -1 -16 -13 4 -10 13 -13 25 -10 13 4 19 1 19 -9 0 -9 -4 -16 -10 -16 -6 0
-6 -9 -1 -22 5 -13 16 -22 24 -20 21 4 31 -14 12 -22 -14 -5 -12 -11 7 -38 39
-56 71 -89 78 -78 4 6 20 8 36 4 27 -5 29 -9 28 -47 -1 -36 4 -46 34 -72 44
-37 133 -95 145 -95 15 0 -13 29 -34 35 -11 4 -31 17 -44 31 -13 13 -31 24
-38 24 -12 0 -27 20 -27 36 0 11 30 2 38 -12 7 -12 11 -13 17 -5 8 14 -23 41
-41 34 -13 -5 -44 23 -44 40 1 7 6 22 13 34 12 21 14 22 35 7 12 -8 20 -19 17
-24 -3 -5 3 -15 15 -22 11 -7 20 -20 20 -30 0 -9 5 -20 10 -23 6 -4 8 -11 5
-16 -4 -5 -1 -17 4 -26 10 -17 10 -17 11 0 0 26 18 20 35 -12 8 -16 32 -37 55
-48 22 -10 42 -25 45 -33 3 -8 27 -27 54 -42 54 -30 180 -51 277 -47 45 2 54
6 54 21 0 10 7 18 15 18 8 0 15 -7 15 -16 0 -14 7 -15 43 -10 23 4 71 17 107
29 57 19 65 25 62 45 -4 25 25 32 34 7 5 -12 12 -12 44 5 126 64 258 190 339
323 23 37 23 37 2 37 -12 0 -21 5 -21 10 0 17 -21 11 -39 -11 -14 -18 -19 -19
-29 -7 -9 11 -9 21 1 41 13 29 22 33 32 17 4 -6 11 -8 16 -5 5 4 9 3 9 -2 0
-4 16 -9 35 -10 21 -1 35 3 35 10 0 7 7 26 16 42 8 17 19 48 24 70 5 22 16 69
24 105 18 77 20 102 7 81 -6 -8 -14 -10 -22 -5 -8 5 -18 9 -24 9 -21 2 -45 18
-45 29 0 15 82 8 93 -8 15 -20 -2 156 -22 239 -27 109 -59 174 -88 181 -25 7
-32 34 -8 34 14 0 13 3 -1 25 -9 14 -21 25 -26 25 -7 0 -7 -4 0 -12 6 -7 8
-25 5 -41 -4 -22 -8 -25 -19 -16 -16 13 -19 75 -5 84 5 3 -2 23 -17 45 -18 29
-28 37 -33 27 -5 -8 -20 -12 -43 -9 -27 3 -36 9 -36 23 0 15 7 19 32 19 30 0
31 1 16 16 -22 22 -59 23 -82 2 -13 -12 -23 -15 -30 -8 -5 5 -18 10 -28 10
-11 0 -24 5 -31 10 -7 6 -21 17 -31 24 -13 9 -17 19 -13 31 7 16 9 16 32 1 23
-15 26 -15 31 -1 9 22 27 18 21 -5 -3 -11 -1 -20 4 -20 4 0 5 -8 2 -17 -5 -15
-4 -16 8 -7 8 6 27 14 43 18 l29 7 -59 40 c-71 48 -203 112 -262 129 -40 10
-127 23 -232 34 -25 3 -92 -4 -150 -14z m483 -129 c26 -10 43 -50 24 -57 -21
-7 -92 23 -96 41 -2 14 2 17 18 12 12 -3 21 -1 21 4 0 11 3 11 33 0z m-495
-18 c-5 -26 -38 -29 -38 -4 0 15 6 21 21 21 14 0 19 -5 17 -17z m402 -48 c0
-8 -7 -15 -15 -15 -8 0 -15 7 -15 15 0 8 7 15 15 15 8 0 15 -7 15 -15z m-662
-32 c3 -23 -27 -40 -39 -22 -9 15 12 50 26 46 5 -2 11 -13 13 -24z m708 16 c3
-6 -1 -18 -10 -27 -15 -15 -18 -15 -26 -2 -5 9 -5 19 2 27 13 16 25 17 34 2z
m-633 -14 c-1 -5 -8 -14 -17 -20 -18 -11 -42 7 -31 25 8 13 51 9 48 -5z m87
-5 c0 -11 -7 -20 -15 -20 -8 0 -15 9 -15 20 0 11 7 20 15 20 8 0 15 -9 15 -20z
m668 -3 c4 -19 -6 -21 -27 -8 -16 10 -5 34 12 28 7 -2 13 -11 15 -20z m-920
-70 c3 -12 -4 -17 -22 -17 -27 0 -37 24 -13 33 19 8 32 2 35 -16z m52 -14 c0
-10 -6 -13 -16 -9 -9 3 -18 6 -20 6 -2 0 -4 6 -4 14 0 9 7 12 20 9 11 -3 20
-12 20 -20z m100 -33 c0 -13 -7 -20 -20 -20 -19 0 -27 20 -13 33 13 14 33 6
33 -13z m40 -5 c0 -8 -4 -15 -10 -15 -5 0 -10 7 -10 15 0 8 5 15 10 15 6 0 10
-7 10 -15z m280 -10 c10 -13 11 -18 1 -28 -10 -9 -15 -8 -27 7 -8 11 -14 23
-14 28 0 14 27 9 40 -7z m-360 -36 c0 -10 -4 -19 -9 -19 -12 0 -23 29 -15 41
8 14 24 0 24 -22z m-91 -9 c38 -36 39 -50 6 -50 -40 0 -81 51 -58 73 11 12 19
8 52 -23z m266 20 c-3 -5 -10 -10 -16 -10 -5 0 -9 5 -9 10 0 6 7 10 16 10 8 0
12 -4 9 -10z m55 -10 c0 -11 -7 -20 -15 -20 -15 0 -21 21 -8 33 12 13 23 7 23
-13z m-490 0 c0 -5 -7 -10 -15 -10 -8 0 -15 5 -15 10 0 6 7 10 15 10 8 0 15
-4 15 -10z m581 -11 c7 -15 8 -24 0 -32 -7 -7 -13 -2 -21 17 -12 34 -12 36 0
36 5 0 14 -9 21 -21z m723 15 c9 -3 16 -13 16 -21 0 -10 -6 -13 -16 -9 -9 3
-22 6 -30 6 -8 0 -14 7 -14 15 0 16 17 19 44 9z m-204 -17 c0 -8 18 -19 39
-26 32 -9 43 -20 61 -56 42 -87 27 -109 -230 -325 -74 -63 -137 -117 -140
-120 -3 -3 -45 -40 -95 -82 -49 -42 -130 -110 -179 -152 -48 -41 -101 -78
-117 -81 -34 -8 -111 32 -129 65 -6 11 -20 20 -31 20 -12 0 -19 6 -17 14 2 8
-31 53 -72 102 -100 116 -110 133 -110 176 0 28 8 45 33 72 53 56 107 60 165
11 16 -13 33 -21 37 -17 5 4 5 2 2 -4 -4 -6 9 -29 27 -50 19 -22 45 -52 59
-69 22 -27 26 -28 45 -15 19 15 262 219 272 230 3 3 38 34 77 69 40 34 70 68
66 74 -3 6 -1 7 6 3 7 -5 31 9 59 34 26 23 59 53 73 66 15 13 39 28 53 34 17
6 24 14 20 25 -4 10 0 15 10 15 9 0 16 -6 16 -13z m-660 -52 c0 -18 -4 -23
-15 -19 -8 4 -15 15 -15 25 0 10 7 19 15 19 9 0 15 -9 15 -25z m-300 -21 c0
-8 -7 -14 -15 -14 -15 0 -21 21 -9 33 10 9 24 -2 24 -19z m110 -10 c0 -8 -5
-12 -10 -9 -6 4 -8 11 -5 16 9 14 15 11 15 -7z m360 0 c0 -8 -5 -12 -10 -9 -6
4 -8 11 -5 16 9 14 15 11 15 -7z m780 1 c0 -8 -6 -15 -14 -15 -8 0 -16 7 -20
15 -4 10 1 15 14 15 11 0 20 -7 20 -15z m-1330 -9 c0 -22 -30 -28 -48 -9 -16
16 -16 16 5 11 13 -3 26 -2 29 3 8 13 14 11 14 -5z m37 -33 c-3 -3 -11 -2 -17
2 -9 6 -9 10 1 16 14 8 27 -7 16 -18z m478 17 c3 -5 -3 -10 -15 -10 -12 0 -18
5 -15 10 3 6 10 10 15 10 5 0 12 -4 15 -10z m-594 -13 c-1 -12 -15 -9 -19 4
-3 6 1 10 8 8 6 -3 11 -8 11 -12z m1439 -12 c0 -8 -7 -15 -15 -15 -8 0 -15 7
-15 15 0 8 7 15 15 15 8 0 15 -7 15 -15z m-1382 -107 c-5 -15 -48 -24 -48 -10
0 14 12 22 33 22 10 0 17 -5 15 -12z m1302 -66 c0 -13 -12 -22 -22 -16 -10 6
-1 24 13 24 5 0 9 -4 9 -8z m103 -42 c14 -11 32 -20 41 -20 9 0 16 -7 16 -15
0 -17 -4 -18 -28 -6 -9 5 -29 14 -44 22 -16 7 -28 18 -28 26 0 18 13 16 43 -7z
m-1468 -40 c-3 -5 -10 -10 -16 -10 -5 0 -9 5 -9 10 0 6 7 10 16 10 8 0 12 -4
9 -10z m53 -21 c-2 -6 -8 -10 -13 -10 -5 0 -11 4 -13 10 -2 6 4 11 13 11 9 0
15 -5 13 -11z m1285 -1 c33 -16 35 -38 4 -38 -25 0 -47 17 -47 37 0 16 9 16
43 1z m162 2 c3 -5 1 -10 -4 -10 -6 0 -11 5 -11 10 0 6 2 10 4 10 3 0 8 -4 11
-10z m-1600 -64 c6 -6 20 -10 32 -8 14 2 23 -3 28 -17 3 -11 22 -26 41 -33 21
-7 34 -19 34 -30 0 -25 -10 -23 -58 12 -23 17 -49 30 -57 30 -8 0 -15 3 -15 8
0 4 -14 15 -30 24 -36 20 -41 43 -7 33 12 -4 27 -12 32 -19z m1480 4 c3 -6 -1
-13 -10 -16 -19 -8 -30 0 -20 15 8 14 22 14 30 1z m-1348 -22 c-3 -8 -6 -5 -6
6 -1 11 2 17 5 13 3 -3 4 -12 1 -19z m-32 -18 c3 -5 1 -10 -4 -10 -6 0 -11 5
-11 10 0 6 2 10 4 10 3 0 8 -4 11 -10z m1369 -45 c-2 -27 1 -43 7 -40 5 4 18
0 29 -8 17 -12 18 -16 7 -26 -11 -9 -17 -8 -27 4 -7 8 -21 15 -31 15 -15 0
-19 8 -19 40 0 22 5 40 11 40 5 0 7 5 4 10 -3 6 0 10 8 10 10 0 13 -12 11 -45z
m68 18 c22 -20 23 -33 3 -33 -8 0 -15 5 -15 10 0 6 -7 10 -15 10 -8 0 -15 7
-15 15 0 20 19 19 42 -2z m-1532 -7 c0 -14 -24 -29 -33 -19 -3 3 -3 11 0 19 7
18 33 18 33 0z m170 -41 c0 -8 -7 -15 -15 -15 -8 0 -15 7 -15 15 0 8 7 15 15
15 8 0 15 -7 15 -15z m-180 -5 c0 -5 -4 -10 -10 -10 -5 0 -10 5 -10 10 0 6 5
10 10 10 6 0 10 -4 10 -10z m45 -60 c4 -16 11 -30 16 -30 12 0 12 -47 -1 -55
-6 -3 -13 2 -16 12 -3 10 -9 27 -14 38 -15 37 -19 53 -13 59 13 13 22 5 28
-24z m225 14 c0 -9 -7 -14 -17 -12 -25 5 -28 28 -4 28 12 0 21 -6 21 -16z
m810 -1 c0 -11 9 -19 23 -20 12 -1 39 -4 59 -5 24 -2 48 -14 66 -31 18 -18 39
-27 60 -27 22 0 32 -5 32 -16 0 -11 -8 -15 -33 -13 -18 2 -37 0 -43 -3 -11 -7
-49 1 -124 26 -25 8 -62 17 -82 21 -38 6 -51 27 -25 39 10 5 10 7 0 12 -23 10
-14 25 12 18 15 -4 25 -2 25 5 0 6 7 11 15 11 8 0 15 -8 15 -17z m170 -14 c0
-5 -7 -9 -15 -9 -15 0 -20 12 -9 23 8 8 24 -1 24 -14z m340 6 c0 -8 -4 -15
-10 -15 -5 0 -10 7 -10 15 0 8 5 15 10 15 6 0 10 -7 10 -15z m-100 -4 c0 -14
-18 -23 -30 -16 -6 4 -8 11 -5 16 8 12 35 12 35 0z m-1338 -113 c-15 -15 -26
-4 -18 18 5 13 9 15 18 6 9 -9 9 -15 0 -24z m1216 16 c24 -7 31 -34 8 -34 -25
0 -46 12 -46 26 0 16 5 16 38 8z m-930 -52 c2 -7 -3 -12 -12 -12 -9 0 -16 7
-16 16 0 17 22 14 28 -4z m774 3 c0 -5 -5 -11 -11 -13 -6 -2 -11 4 -11 13 0 9
5 15 11 13 6 -2 11 -8 11 -13z m-808 -36 c-7 -13 -31 -11 -40 2 -9 16 18 30
33 17 7 -6 10 -14 7 -19z m650 -10 c-8 -14 -34 -11 -34 4 0 8 3 17 7 20 9 9
34 -13 27 -24z m-544 -14 c0 -8 -6 -15 -14 -15 -17 0 -28 14 -19 24 12 12 33
6 33 -9z m-50 -20 c0 -8 -6 -15 -14 -15 -17 0 -28 14 -19 24 12 12 33 6 33 -9z
m75 -15 c-5 -8 -11 -8 -17 -2 -6 6 -7 16 -3 22 5 8 11 8 17 2 6 -6 7 -16 3
-22z m45 16 c0 -2 -7 -6 -15 -10 -8 -3 -15 -1 -15 4 0 6 7 10 15 10 8 0 15 -2
15 -4z m-280 -16 c0 -5 -4 -10 -10 -10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0
10 -4 10 -10z m1050 -29 c0 -6 -4 -13 -10 -16 -5 -3 -10 1 -10 9 0 9 5 16 10
16 6 0 10 -4 10 -9z m-970 -26 c0 -8 -7 -15 -15 -15 -8 0 -15 7 -15 15 0 8 7
15 15 15 8 0 15 -7 15 -15z m-50 -31 c0 -16 -16 -19 -25 -4 -8 13 4 32 16 25
5 -4 9 -13 9 -21z m164 -9 c-4 -8 -8 -15 -10 -15 -2 0 -4 7 -4 15 0 8 4 15 10
15 5 0 7 -7 4 -15z m-24 -10 c0 -8 -7 -15 -16 -15 -14 0 -14 3 -4 15 7 8 14
15 16 15 2 0 4 -7 4 -15z m70 -10 c0 -8 -4 -15 -10 -15 -5 0 -10 7 -10 15 0 8
5 15 10 15 6 0 10 -7 10 -15z m-192 -43 c2 -7 -3 -12 -12 -12 -9 0 -16 7 -16
16 0 17 22 14 28 -4z m182 3 c0 -9 -9 -15 -21 -15 -12 0 -23 -8 -26 -20 -5
-20 -23 -26 -37 -12 -4 4 -13 7 -18 7 -18 0 -20 22 -3 28 8 4 15 2 15 -3 0
-18 39 -10 50 10 13 23 40 27 40 5z m-137 -31 c-3 -8 -9 -14 -13 -14 -14 0
-22 20 -16 37 5 12 9 13 21 3 7 -6 11 -18 8 -26z m505 -18 c3 -13 -1 -17 -14
-14 -11 2 -20 11 -22 22 -3 13 1 17 14 14 11 -2 20 -11 22 -22z m42 -6 c0 -11
-7 -20 -15 -20 -27 0 -18 30 13 39 1 1 2 -8 2 -19z m-430 0 c0 -5 -4 -10 -10
-10 -5 0 -10 5 -10 10 0 6 5 10 10 10 6 0 10 -4 10 -10z m50 -5 c0 -8 -9 -15
-20 -15 -20 0 -26 11 -13 23 12 13 33 7 33 -8z m38 -12 c-4 -22 -22 -20 -26 1
-2 10 3 16 13 16 10 0 15 -7 13 -17z m52 9 c0 -5 18 -15 40 -22 43 -14 59 -47
18 -36 -13 3 -30 6 -39 6 -20 0 -61 39 -54 51 8 11 35 12 35 1z m-196 -27 c-4
-8 -10 -15 -15 -15 -5 0 -9 7 -9 15 0 8 7 15 15 15 9 0 12 -6 9 -15z m76 -5
c0 -5 -4 -10 -9 -10 -6 0 -13 5 -16 10 -3 6 1 10 9 10 9 0 16 -4 16 -10z m550
-21 c0 -5 4 -8 9 -5 5 4 7 -3 4 -14 -6 -23 -23 -26 -41 -8 -18 18 -14 38 8 38
11 0 20 -5 20 -11z m-390 -18 c0 -5 -9 -7 -20 -4 -11 3 -20 7 -20 9 0 2 9 4
20 4 11 0 20 -4 20 -9z m-47 -11 c20 0 23 -23 4 -38 -10 -8 -20 -5 -40 14 -33
30 -34 38 -4 30 12 -3 30 -6 40 -6z m141 -56 c4 -10 -2 -14 -19 -14 -25 0 -40
14 -31 29 8 13 43 2 50 -15z m43 -11 c2 -6 -5 -13 -15 -15 -15 -4 -18 -1 -14
14 5 19 23 20 29 1z m-121 -6 c4 -4 -7 -7 -24 -7 -18 0 -32 5 -32 10 0 11 45
9 56 -3z"/></g></svg><br/>';
			break;
		default:
			$icon = "<div class='text-center' style='font-size: 40px;'><i class='fa fa-spin fa-spinner'></i></div><br/>";
			break;
		}
		return $icon;
	}

	public static function verifiedAutorized($phone, $clientDni) {
		$response = ['success' => false, 'title' => "No se pudo verificar el status de solicitud de autorizacion sin verificar el celular", 'msg' => "Fallo el registro de datos", 'icon' => "error", 'code' => 'ERR_DAT'];

		$consult = self::getConnect('R')
			->where([
				['phone_verify', $phone],
				['dni_client', $clientDni],
				['type_verify', 'AUTHORIZED'],
				['status', '!=', 'T']])
			->orderBy('id', 'DESC')
			->first();

		$msg = "No se encuentra registro de solicitud de autorización";
		$icon = "";
		$code = "EMPTY";

		if (!empty($consult)) {
			$response['success'] = true;
			$msg = "Aun en espera por revisión por parte de mesa de control";
			$icon = self::getIconAutorized("LOADING");

			switch ($consult->status) {
			case 'CREATE':
				$code = "CREATE";
				$msg = "Aun en espera por revisión por parte de mesa de control. Solicitud: #" . $consult->id;
				break;

			case 'REJECTED':
				$code = "REJECTED";
				$msg = "La solicitud de autorización para el telefono fue rechazado por mesa de control";
				$icon = self::getIconAutorized("REJECTED");
				break;

			case 'AUTHORIZED':
				$code = "AUTHORIZED";
				$msg = "La solicitud de autorización fue aprobado por parte de mesa de control";
				$icon = self::getIconAutorized("AUTHORIZED");
				break;

			default:
				$code = $consult->status;
				$msg = "La solicitud de autorización se encuentra en un status no definido para el proceso";
				$icon = "";
				break;
			}
		}
		$response['title'] = "Estatus de solicitud consultada";
		$response['msg'] = $msg;
		$response['icon'] = $icon;
		$response['code'] = $code;
		return $response;
	}

	public static function getLastRegisterVerify($phone, $dni) {
		$register = self::getConnect('R')
			->where('phone_verify', $phone)
			->whereNotIn('status', ['T', 'CREATE'])
			->orderBy('id', 'DESC')
			->first();

		if (!empty($register)) {
			//Evaluamos a que cliente esta asociado el registro
			$client = Client::getClientByDNI($register->dni_client);
			$nameClient = "";
			if (!empty($client)) {
				$nameClient = ". Registro del cliente: " . $client->name . ' ' . $client->last_name;
			}

			$nameSeller = "";
			$UserNetwey = User::getUserByEmail($register->email_seller);
			if (!empty($UserNetwey)) {
				$nameSeller = ". Solicitud cargada por el vendedor: " . $UserNetwey->name . ' ' . $UserNetwey->last_name;
			}

			switch ($register->status) {
			case 'REJECTED':
				$msg = "El telefono fue rechazado por parte de mesa de control";
				break;

			case 'AUTHORIZED':
				$msg = "El telefono tiene registros de aprobacion por parte de mesa de control";
				break;

			case 'VERIFIED':
				$msg = "El telefono posee un registro de verificacion exitosa";
				break;

			default:
				$msg = "El telefono se encuentra en un status no definido para el proceso";
				break;
			}
			$response['success'] = true;
			$response['title'] = "Hay registros del telefono";
			$response['msg'] = $msg . $nameClient . $nameSeller;
			$response['icon'] = "success";
			$response['code'] = 'OK';
		} else {
			$response = ['success' => false, 'title' => "No hay registros del telefono", 'msg' => "No hay historico de verificacion para el telefono consultado", 'icon' => "error", 'code' => 'DAT_EMP'];
		}
		return $response;
	}

/**
 * [getLastVerified Retorna el token de verificacion por SMS que tenga menos de 3 envios y que sean de las ultimas 24 horas]
 * @param  [type] $phone     [description]
 * @param  [type] $clientDni [description]
 * @param  array  $type      [description]
 * @param  array  $Remove    [description]
 * @param  string $platform  [description]
 * @return [type]            [description]
 */
	public static function getLastVerified($phone, $clientDni, $tried = 3) {
		// $today = date('Y-m-d H:i:s');
		//Restamos 24 horas
		//$NuevaFecha = strtotime('-24 hour', strtotime($today));

		$today = Carbon::now()->format('Y-m-d H:i:s');
		$NuevaFecha = date("Y-m-d H:i:s", strtotime($today . "- 1 day"));

		return self::getConnect('R')
			->where([
				['dni_client', $clientDni],
				['phone_verify', $phone],
				['type_verify', "VERIFIED"],
				['status', "CREATE"],
				['tried', '<=', $tried],
				['date_reg', '>=', $NuevaFecha]])
			->orderBy('id', 'DESC')
			->first();
	}

	public static function addSendVerify($verify_id) {
		$response = ['success' => false, 'title' => "No se pudo actualizar el intento de uso del token de verificación celular", 'msg' => "Fallo el registro de datos", 'icon' => "error", 'code' => 'ERR_DAT'];

		try {
			$register = self::getConnect('W')
				->where('id', $verify_id)
				->first();

			if (!empty($register)) {
				$cant = $register->tried;
				$register->tried = $cant + 1;
				$register->save();
				sleep(2);
				$response = ['success' => true, 'title' => "Insertando un intento de uso del token de verificación celular", 'msg' => "OK", 'icon' => "success", 'code' => 'OK'];
			} else {
				$response['msg'] = "No se encontro el registro de verificación";
				$response['code'] = "EMPTY_VERIFY";
			}
		} catch (Exception $e) {
			$txmsg = "No se pudo actualizar el intento de uso del token de verificación celular del registro " . $verify_id . " - " . (String) json_encode($e->getMessage());
			Log::error($txmsg);
			$response['msg'] = $txmsg;
		}
		return $response;
	}

/**
 * [isVerifyLasted Evalua si el telefono lo verifico o fue rechazado por otro cliente distinto al que va verificar actualmente]
 * @param  [type]  $phone [description]
 * @param  [type]  $dni   [description]
 * @return boolean        [description]
 */
	public static function isVerifyLasted($phone, $dni) {
		$register = self::getConnect('R')
			->where([
				['phone_verify', $phone],
				['dni_client', '!=', $dni]])
			->whereIn('status', ['VERIFIED', 'REJECTED'])
			->first();

		if (empty($register)) {
			return false;
		}
		return true;
	}

}
