<?php

namespace App\Utilities;

use Log;
use App\Models\LogApi;
use App\Models\LogApiV2;
use Illuminate\Support\Facades\Mail;

/*
	Clase que contiene metodos para conectarse con la API altan-netwey, se pueden utilizar desde cualqueir controlador
*/

class CustomLogs
{
	/*
   		Registra un log utilizando la libreria logger que implementa laravel
   		doc https://laravel.com/docs/5.5/errors
   		@param String $type
   		@param String $message
   		@param Array $data
   		@param String $register F para guardar en archivo, BD para guardar en base de datos, E para guardarlo donde indique el .env
   		@param Illuminate\Http\Request $request
   		@param Array $send reemplaza los input que puedan venir en $request
   		@return void 
   	*/
	public static function registerLog($type = 'info', $message = '', $register = 'E', $request = false, $send = false)
	{
		if ($request) {
			if ($register == 'E')
				$register = env('TYPE_LOG', 'F');

			$data = [
				'header' => $request->header(),
				'ip' => $request->ip(),
				'url' => $request->fullUrl(),
				'inputs' => (string)json_encode($request->input()),
				'outputs' => $send ? (string)json_encode($send) : []
			];

			if ($register == 'F') {
				if ($type == 'debug')
					Log::debug($message, $data);
				if ($type == 'info')
					Log::info($message, $data);
				if ($type == 'warning')
					Log::warning($message, $data);
				if ($type == 'notice')
					Log::notice($message, $data);
				if ($type == 'error')
					Log::error($message, $data);
				if ($type == 'critical')
					Log::critical($message, $data);
				if ($type == 'alert')
					Log::alert($message, $data);
				if ($type == 'emergency')
					Log::emergency($message, $data);
			}

			if ($register == 'DB') {
				$log = new LogApi;
				$log->ip = $request->ip();
				$log->user = $request->bearerToken();
				$log->url = '[' . $request->method() . ']' . $request->fullUrl();
				$log->header = (string)json_encode($request->header());
				$log->ip = $request->ip();
				$log->message = $message;
				$log->data = $send ? (string)json_encode($send) : (string)json_encode($request->input());
				$log->date_reg = date('Y-m-d H:i:s');

				if ($type == 'debug')
					$log->type_log = 'D';
				if ($type == 'info')
					$log->type_log = 'I';
				if ($type == 'warning')
					$log->type_log = 'W';
				if ($type == 'notice')
					$log->type_log = 'N';
				if ($type == 'error')
					$log->type_log = 'ER';
				if ($type == 'critical')
					$log->type_log = 'C';
				if ($type == 'alert')
					$log->type_log = 'A';
				if ($type == 'emergency')
					$log->type_log = 'EM';

				$log->save();
			}
		}

		return;
	}

	public static function registerLogWH($type = 'info', $message = '')
	{
		if ($type == 'debug') {
			Log::debug($message);
		}
		if ($type == 'info') {
			Log::info($message);
		}
		if ($type == 'warning') {
			Log::warning($message);
		}
		if ($type == 'notice') {
			Log::notice($message);
		}
		if ($type == 'error') {
			Log::error($message);
		}
		if ($type == 'critical') {
			Log::critical($message);
		}
		if ($type == 'alert') {
			Log::alert($message);
		}
		if ($type == 'emergency') {
			Log::emergency($message);
		}

		return;
	}

	public static function addMessage(String $message, LogApiV2 $log)
	{
		$log->message = !empty($log->message) ? ($log->message . ' ' . $message) : $message;
		$log->save();

		return $log;
	}

	public static function sendErrorDelMail($oper, $transaction)
	{
		try {
			Mail::send([], [], function ($message) use ($transaction, $oper) {
				$html = '<h1> Error creando delivery ' . $oper . ' desde api seller</h1>';
				$html .= '<p> Error creando el envio para el transaction-id: ' . $transaction;

				$message->to(explode(',', env('ERROR_ADDRESS')))
					->subject('Error en creación de delivery ' . $oper . ' - API seller.')
					->from(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'))
					->setBody($html, 'text/html');
			});
		} catch (\Exception $e) {
			Log::error('No se pudo enviar email de error al crear delivery: ' . $transaction . ' Operador: ' . $oper . ' Error: ' . $e->getMessage());
		}
	}

	public static function sendEmailSale($conc, $client, $phone, $email = '', $desc, $orden, $colony, $state, $paymentM = '', $currier, $folio = false)
	{
		$html = '<h1> Se ha realizado una compra desde el API de ventas (' . $conc . ') </h1>';
		$html .= '<p> El cliente: <b>' . $client . '</b>, ';
		$html .= 'tel&eacute;fono: <b>' . $phone . '</b>, ';
		$html .= 'email: <b>' . $email . '</b> realiz&oacute; la compra de: ';
		$html .= '<b>' . $desc . '</b></p>';
		$html .= '<p>N&uacute;mero de orden netwey: <b>' . $orden . '</b></p>';
		$html .= '<p>Colonia: <b>' . $colony . '</b></p>';
		$html .= '<p>Estado: <b>' . $state . '</b></p>';
		$html .= '<p>M&eacute;todo de pago: <b>' . $paymentM . '</b></p>';
		$html .= '<p>N&uacute;mero de orden o folio de ' . $currier . ': <b>' . ($folio ? $folio : 'Falló creaci&oacute;n de orden, se debe hacer de forma manual.') . '</b></p>';
		$html .= '<p>Fecha: <b>' . date('d-m-Y H:i:s') . '</b></p>';

		try {
			Mail::send([], [], function ($message) use ($html) {
				$message->to(explode(',', env('SALES_EMAIL')))
					->subject('Compra exitosa.')
					->from(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'))
					->setBody($html, 'text/html');
			});
		} catch (\Exception $e) {
			Log::error('No se envio correo de Compra exitosa. para la orden: ' . $orden . ' Error: ' . $e->getMessage());
		}
	}
}
