<?php

namespace App\Utilities;

use App\Models\FiberCityZone;
use Illuminate\Http\Request;


trait FiberTrait{

    public function chekingCoverageFiber(Request $request){
        $respuesta = ['success' => false, 'title' => "No se pudo verificar servicialidad", 'msg' => "Faltan datos", 'icon' => "error", 'code' => 'ERR_DAT'];

        $coor_valid = false;
        $status = 200;
        if (isset($request->lat) && isset($request->lng) &&
            !empty($request->lat) && !empty($request->lng)) {

            $coor_valid = true;

            //verifico que sea valida la lat y la lng
            //en caso de mexico es positiva
            $exp_lat = "/^(0(\.\d{1,20})?|([1-9](\d)?)(\.\d{1,20})?|1[0-7]\d{1}(\.\d{1,20})?|180\.0{1,20})$/";
            if (!preg_match($exp_lat, $request->lat)) {
                $respuesta['msg'] = "La latitud (" . $request->lat . ") no es valida para Mexico. Verifica de nuevo la servicialidad! (CF3269)";
                $respuesta['code'] = 'ERR_COORD';
                $status = 400;

                $coor_valid = false;
            }

            //en caso de mexico es negativa
            $exp_lng = "/^[\-](0(\.\d{1,20})?|([1-9](\d)?)(\.\d{1,20})?|1[0-7]\d{1}(\.\d{1,20})?|180\.0{1,20})$/";
            if (!preg_match($exp_lng, $request->lng)) {
                $respuesta['msg'] = "La longitud (" . $request->lng . ") no es valida para Mexico. Verifica de nuevo la servicialidad! (CF3276)";
                $respuesta['code'] = 'ERR_COORD';
                $coor_valid = false;
                $status = 400;
            }
        } else {
            $respuesta['msg'] = 'Se amerita una coordenada valida de mexico para verificar cobertura de fibra (CF3280)';
            $respuesta['code'] = 'EMP_COORD';
            $status = 400;
        }

        if ($coor_valid) {

            if (isset($request->city_id) && isset($request->zone_id) && !empty($request->city_id) && !empty($request->zone_id)) {

                $poligono = FiberCityZone::getCoordenada($request->city_id, $request->zone_id);
                $DataPoligono = $poligono['poligono']['poligono'];
                $resSrv = FiberCityZone::servicialidadFibra($request->lat, $request->lng, $DataPoligono);

                if ($resSrv == 'VERTICE' || $resSrv == 'BORDE' || $resSrv == 'DENTRO') {
                    $respuesta = array('success' => true, 'title' => "Con cobertura", 'msg' => "Cobertura en la ubicación exitosa", 'icon' => "success", 'code' => $resSrv);
                } else {
                    $respuesta['title'] = "Sin cobertura";
                    $respuesta['msg'] = 'No se cuenta con cobertura en la ubicación especificada.';
                    $respuesta['code'] = 'NOT_COVERAGE';
                    $status = 400;
                }
            } else {
                $respuesta['msg'] = 'Faltan datos de la ciudad y zona para verificar cobertura de fibra';
                $respuesta['code'] = 'ERR_DAT';
                $status = 400;
            }
        }

        if($status == 400){ 
            $data = [
                "error_code" => $respuesta['code'],
                "errors" => [
                    "serviceability" => $respuesta['msg']
                ]
            ];
            $message = "No se pudo verificar servicialidad";
        }else{
            $data = [
                "serviceability" => $respuesta['msg']
            ];
            $message = "Servicialidad verificada correctamente";
        }


        return [$respuesta, $message, $data, $status ];

    }

}
