<?php

namespace App\Utilities;

use Illuminate\Support\Arr;

use MercadoPago\Client\Common\RequestOptions;
use MercadoPago\Client\Payment\PaymentClient;
use MercadoPago\Client\PaymentMethod\PaymentMethodClient;
use MercadoPago\Exceptions\MPApiException;
use MercadoPago\MercadoPagoConfig;
use MercadoPago\Net\MPSearchRequest;
use MercadoPago\Net\MPResponse;

//IMPORTANTE: Validar que funcionen correctamente

class MercadoPagoCus
{
	//Devuelve metodos de pagos de mercado libre
	public static function getPaymentCashMethods()
	{

		MercadoPagoConfig::setAccessToken(env('KEYMP'));
		$client = new PaymentMethodClient();
		$paymentMethods = $client->list()->getResponse() ?? false;
		$paymentMethods = $paymentMethods != false ? $paymentMethods->getContent() : false;

		if (!empty($paymentMethods) && $paymentMethods != false) {
			$payment = Arr::where($paymentMethods, function ($value, $key) {
				return ($value['payment_type_id'] == 'ticket' || $value['payment_type_id'] == 'atm') && $value['status'] == 'active';
			});

			if (count($payment)) {
				foreach ($payment as $method) {
					$pay[] = Arr::only($method, ['id', 'name', 'payment_type_id', 'secure_thumbnail', 'status']);
				}
				return $pay;
			}
		}
		return false;
	}

	//Procesa un pago
	public static function doPay($data)
	{

		MercadoPagoConfig::setAccessToken(env('KEYMP'));
		$client = new PaymentClient();

		try {
			$request = [
				"transaction_amount" => $data['transaction_amount'],
				"description" => $data['description'],
				"payment_method_id" => $data['payment_method_id'],
				"external_reference" => $data['external_reference'],
				"binary_mode" => $data['binary_mode'],
				"notification_url" => $data['notification_url'],
				"payer_email" => [
					"email" => $data['payer_email']
				],
				"additional_info" => $data['additional_info'],
			];

			if (!empty($data['token'])) {
				$request['token'] = $data['token'];
				$request['installments'] = 1;
			}

			$request_options = new RequestOptions();

			$payment = $client->create($request, $request_options);

			return $payment;
		} catch (MPApiException $e) {
			return false;
		} catch (\Exception $e) {
			return false;
		}

		return false;
	}

	public static function cancelledPay($payment_id = false)
	{
		if ($payment_id) {
			MercadoPagoConfig::setAccessToken("ACCESS_TOKEN");
			$client = new PaymentClient();
			try {
				$payment = $client->cancel($payment_id);
				if ($payment) {
					return $payment;
				}
			} catch (MPApiException $e) {
				return false;
			} catch (\Exception $e) {
				return false;
			}
		}

		return false;
	}

	public static function getPayment($payment_id = false)
	{

		$payment = false;
		if ($payment_id) {
			try {
				MercadoPagoConfig::setAccessToken(env('MP_ACCESS_TOKEN'));
				$client  = new PaymentClient();
				$payment = $client->get($payment_id);
				return $payment;
			} catch (MPApiException $e) {
				return false;
			} catch (\Exception $e) {
				return false;
			}
		}
		return false;
	}
}
