<?php

namespace App\Utilities;

use Log;

use Illuminate\Support\Arr;

use App\Utilities\Common;
use App\Utilities\CustomLogs;

//Modelos
use App\Models\PC99;
use App\Models\TempCar;
use App\Models\LogApiV2;
use App\Models\TempCarDetail;
use App\Models\NinetyNineMinutes;


class NinetyNineMin
{
	/*
		Recibe el array de address_components que devuelve la api place detail de google

		@param Array $addressComp
	*/
	public static function validateWgoogle($addressComp = false)
	{
		if ($addressComp && is_array($addressComp)) {
			$destination = [
				'route' => false,
				'neighborhood' => false,
				'state' => false,
				'postal_code' => false,
				'locality' => false,
				'country' => 'MX'
			];

			$isValid = false;

			foreach ($addressComp as $ele) {
				if (in_array('route', $ele->types))
					$destination['route'] = $ele->long_name;

				if (in_array('sublocality_level_1', $ele->types) && in_array('sublocality', $ele->types))
					$destination['neighborhood'] = $ele->long_name;

				if (in_array('locality', $ele->types))
					$destination['locality'] = $ele->long_name;

				if (in_array('postal_code', $ele->types))
					$destination['postal_code'] = $ele->long_name;

				if (in_array('street_number', $ele->types))
					$destination['street_number'] = $ele->long_name;

				if (in_array('administrative_area_level_1', $ele->types))
					$destination['state'] = $ele->long_name;
			}

			if (!empty($destination['street_number']))
				$destination['route'] .= ' ' . $destination['street_number'];

			if (
				$destination['route'] &&
				$destination['neighborhood'] &&
				$destination['state'] &&
				$destination['postal_code']
			)
				$isValid = true;

			return ['success' => true, 'data' => $destination, 'isValid' => $isValid];
		}

		return ['success' => false, 'data' => 'Parametro erroneo'];
	}

	/*
		Solicita cotizacion a 99 min
		@param array $destination, datos del destino ref api99
		Ejm
		[
			"route" => "",
		    "neighborhood" => "",
		    "postalCode" => "56577", Requerido
		    "locality" => "",
		    "subLocality" => "",
		    "state" => "",
		    "country" => "Mexico" Requerido
	    ]

		@param array $package, medidas del envio
		ejm
		[
			"weight" => 90,
			"width" => 90,
			"height" => 90,
			"depth" => 90,
		]
	*/
	public static function getPriceDelivery($cp = false, $package = false, $carid = false)
	{
		if ($cp && $package) {
			$destination = [
				'route' => '',
				'neighborhood' => '',
				'state' => '',
				'postalCode' => $cp,
				'country' => 'MX'
			];

			$addrw = PC99::getAddressWh($cp);

			if (!empty($addrw) && !empty($addrw->route)) {
				$origin = [
					'route' => $addrw->route,
					'neighborhood' => !empty($addrw->neighborhood) ? $addrw->neighborhood : '',
					'locality' => !empty($addrw->locality) ? $addrw->locality : '',
					'subLocality' => !empty($addrw->subLocality) ? $addrw->subLocality : '',
					'state' => !empty($addrw->state) ? $addrw->state : '',
					'postalCode' => !empty($addrw->cp) ? $addrw->cp : '',
					'country' => 'MX'
				];
			}

			$data = [
				'userId' => env('USER_99'),
				'apiKey' => env('KEY_99'),
				'weight' => $package['weight'],
				'width' => $package['width'],
				'height' => $package['height'],
				'depth' => $package['depth'],
				'origin' => !empty($origin) ? $origin : $destination,
				'destination' => $destination
			];

			$res = Common::executeCurl(env('URL_99') . 'shipping_rates', 'POST', [], $data);

			if ($res['success']) {
				if (!empty($res['data']) && $res['data']->status == 'OK') {
					return ['success' => true, 'data' => $res['data']];
				} else {
					return ['success' => false, 'data' => 'Fuera de cobertura.', 'original' => $res];
				}
			}

			return ['success' => false, 'data' => 'Error consultando contización.', 'original' => $res];
		}

		return ['success' => false, 'data' => 'Parametro erroneo.'];
	}

	public static function getpriceDetail($rates = false)
	{
		if ($rates) {
			//Buscando precio del envío
			$min = 999999999;
			$min2 = 999999999;
			foreach ($rates as $rate) {
				//Esta condicion es para darle preferencia al menor costo pero con el titulo 'Entrega dia siguiente'
				if ($rate->cost <= $min2 && $rate->title == 'Entrega dia siguiente') {
					$min2 = $rate->cost;
					$selectedP = $rate;
					$selectedP->cost = ($selectedP->cost < 100) ? 100 : $selectedP->cost;
				}
				//Guarda el menor costo de envío
				if ($rate->cost <= $min) {
					$min = $rate->cost;
					$selected = $rate;
					$selected->cost = ($selected->cost < 100) ? 100 : $selected->cost;
				}
			}

			if (!empty($selectedP)) {
				return ['success' => true, 'data' => $selectedP];
			}

			if (!empty($selected)) {
				return ['success' => true, 'data' => $selected];
			}
		}
		return ['success' => false, 'data' => 'no se consiguio precio.'];
	}

	public static function getInventary($postalCode = false, $request = false)
	{
		if ($postalCode) {
			$url = env('URL_99_NETWEY') . 'inventary-99/' . $postalCode;

			$res = Common::executeCurl($url, 'GET');

			if ($res['success'] && $res['data']->success) {
				return ['success' => true, 'data' => $res['data']->data];
			}

			return [
				'success' => false,
				'data' => 'fallo api 99min - netwey.',
				'original' => $res
			];
		}

		return ['success' => false, 'data' => 'Sin codigo postal.'];
	}

	public static function generateDelivery($carId = false, $transactionId = false, $token = false)
	{
		if ($carId && $transactionId && $token) {
			$sendData = NinetyNineMinutes::getActiveDelivery($carId);

			if (!empty($sendData)) {
				$transaction = TempCar::getTransaction($transactionId, $token);
				$transactionD = TempCarDetail::getTransactionDetail($transactionId, $token);

				$client = TempCar::getDataClient($transaction->id, $token);

				if (!empty($transaction) && !empty($transactionD) && !empty($client)) {
					$modems = TempCarDetail::getProductsDetail($transactionId, $token);

					$note = '';
					$modems = $modems->groupBy('id')->toArray();

					foreach ($modems as $modem => $title) {
						$note .= 'SKU: ' . $modems[$modem][0]['title'] . ' | Cantidad: ' . count($title) . ' | ';
					}

					$addrw = NinetyNineMinutes::getAddressWh($sendData->postal_code);

					if (!empty($addrw) && !empty($addrw->route)) {
						$origin = [
							'sender' => 'Netwey',
							'route' => $addrw->route,
							'street_number' => !empty($addrw->street_number) ? $addrw->street_number : '',
							'internalNumber' => '',
							'neighborhood' => !empty($addrw->neighborhood) ? $addrw->neighborhood : '',
							'locality' => !empty($addrw->locality) ? $addrw->locality : '',
							'subLocality' => !empty($addrw->subLocality) ? $addrw->subLocality : '',
							'state' => !empty($addrw->state) ? $addrw->state : '',
							'postal_code' => !empty($addrw->cp) ? $addrw->cp : '',
							'country' => 'MX'
						];
					} else {
						$origin = [
							'sender' => 'Netwey',
							'route' => $sendData->route,
							'street_number' => !empty($sendData->street_number) ? $sendData->street_number : '',
							'internalNumber' => !empty($sendData->internal_number) ? $sendData->internal_number : '',
							'neighborhood' => $sendData->neighborhood,
							'locality' => '',
							'subLocality' => $sendData->sub_locality,
							'state' => $sendData->state,
							'postal_code' => $sendData->postal_code,
							'country' => 'MX'
						];
					}

					$data = [
						'userId' => env('USER_99'),
						'apiKey' => env('KEY_99'),
						'deliveryType' => $sendData->delivery_type,
						'vehicleType' => $sendData->vehicle_type,
						'notes' => $note . (!empty($sendData->notes) ? (' | ' . $sendData->notes . ' |') : ''),
						'cashOnDelivery' => false,
						'amount' => 0,
						'receivedOrderId' => $transactionId,
						'origin' => $origin,
						'destination' => [
							'receiver' => $client->name . ' ' . $client->last_name,
							'phoneNumber' => $client->phone_home,
							'email' => $client->email,
							'route' => $sendData->route,
							'street_number' => !empty($sendData->street_number) ? $sendData->street_number : '',
							'internalNumber' => !empty($sendData->internal_number) ? $sendData->internal_number : '',
							'neighborhood' =>  $sendData->neighborhood,
							'locality' => $sendData->locality,
							'subLocality' => $sendData->sub_locality,
							'state' => $sendData->state,
							'postal_code' => $sendData->postal_code,
							'country' => 'MX'
						]
					];

					/**************************************
					 * se creo este parche para solucionar problema de metodo de envio que falla en 99min
					 **************************************/
					$deliveryTypes = array("sameDay", "nextDay", "99minutos");
					$valorAct = $sendData->delivery_type;

					if (($clave = array_search($valorAct, $deliveryTypes)) !== false) {
						unset($deliveryTypes[$clave]);
					}

					//coloco el valor actual al principio del array para que sea el primer metodo en probar
					array_unshift($deliveryTypes, $valorAct);

					$exito = 0;
					// $band = 0;
					foreach ($deliveryTypes as $deltyp) {
						if ($exito == 0) {
							$data['deliveryType'] = $deltyp;
							$res = Common::executeCurl(env('URL_99') . 'request', 'POST', [], $data);

							if ($res['success'] && !empty($res['data']->status) && $res['data']->status == 'OK') {
								$exito = 1;
								//if($valorAct != $deltyp){
								//$band = 1;
								//Log::alert("Orden de 99min creada con el parche implementado con deliveryType = ".$deltyp." en api de ventas, response: ".(String)json_encode($res));
								//}
							}
						}
					}

					/**************************************
					 * Fin del parche
					 **************************************/

					//$res = Common::executeCurl(env('URL_99').'request', 'POST', [], $data);

					if ($res['success'] &&  $res['data']->status == 'OK') {
						// if($band == 0){
						// 	Log::alert("Orden de 99min creada POR FUERA del parche implementado con deliveryType = ".$valorAct." en api de ventas, response: ".(String)json_encode($res));
						// }

						NinetyNineMinutes::where('id', $sendData->id)
							->update([
								'received_order' => $transactionId,
								'status' => 'S',
								'order99' => $res['data']->order,
								'url_pdf' => $res['data']->Guia_PDF
							]);

						return ['success' => true, 'data' => $res['data']];
					}

					return [
						'success' => false,
						'data' => 'Ocurrio un error al crear el delivery.',
						'99min' => $res
					];
				}
			}
		}

		return ['success' => false, 'data' => 'Ocurrio un error al crear el delivery.'];
	}

	public static function validateInventory($artics = [], $pc = false, $request = false, LogApiV2 $log = null)
	{
		if (!empty($artics) && count($artics) && $pc) {
			$inv99 = self::getInventary($pc, false);

			if ($inv99['success']) {
				foreach ($artics as $artic) {
					$articsc[$artic] = empty($artics[$artic]) ? 1 : ($artics[$artic] + 1);
				}

				$obj = Arr::where($inv99['data'], function ($value, $key) use ($articsc) {
					return $value->type == 'temp' && !empty($articsc[$value->id]) && $value->quantity >= $articsc[$value->id];
				});

				if (count($obj) == count($articsc)) return true;
			}

			/*if(!empty($log)){
				CustomLogs::addMessage(
					'respuesta de inventario: '. json_encode($inv99),
					$log
				);
			}*/
		}

		return false;
	}

	public static function validateListInventory($packs, $pc)
	{
		$inv99 = self::getInventary($pc, false);

		if ($inv99['success']) {
			foreach ($packs as $pack) {
				$sm = explode(" - ", $pack->shipingMethods);

				if (empty($pack->hasInventory) || !$pack->hasInventory  || (in_array("99minutos", $sm) == false)) {
					$obj = Arr::where($inv99['data'], function ($value, $key) use ($pack) {
						return $value->type == 'temp' && $value->id == $pack->idArtic && $value->quantity > 0;
					});

					if (count($obj)) {
						$pack->hasInventory = true;
						empty($pack->shipingMethods) ? $pack->shipingMethods = "99minutos"  : $pack->shipingMethods .= " - 99minutos";
					}
				}
			}
		}

		return $packs;
	}

	public static function getShippingArray($artics = [], $postal_code = false, $nnMth = false, $request = false, LogApiV2 $log = null)
	{
		$msgError = 'Error en metodo (NinetyNineMin::getShippingArray)';

		if ($postal_code) {
			//Validando inventario de 99 min
			$hasInv = self::validateInventory($artics, $postal_code, $request);

			if ($hasInv) {
				$price = self::getPriceDelivery($postal_code, [
					"weight" => 90,
					"width" => 90,
					"height" => 90,
					"depth" => 90
				]);

				if ($price['success'] && !empty($price['data']->rates)) {
					$priceSelected = self::getpriceDetail($price['data']->rates);

					if ($priceSelected['success']) {
						return [
							'id' => $nnMth->code_name,
							'currier' => '99min',
							'name' => $nnMth->Name,
							'price' => $priceSelected['data']->cost,
							'sub_request' => false,
							'aprox_time_hrs' => env('TIME_99'),
							'sending_data' => [
								'route' => '',
								'street_number' => '',
								'neighborhood' => '',
								'sub_locality' => '',
								'locality' => '',
								'state' => '',
								'notes' => '',
								'postal_code' => $postal_code
							]
						];
					} else {
						$msgError = 'Error obteniendo precio de 99min (NinetyNineMin::getShippingArray)';

						if ($request) {
							CustomLogs::registerLog(
								'error',
								$msgError,
								'F',
								$request,
								$priceSelected
							);
						}

						$msgError = $msgError . ' ' . json_encode($priceSelected);
					}
				} else {
					$msgError = 'Error consultando precio en 99min (NinetyNineMin::getShippingArray)';
					//$msgError = $msgError.' '.json_encode($price);

					if ($request) {
						CustomLogs::registerLog(
							'error',
							$msgError,
							'F',
							$request,
							$price
						);
					}
				}
			} else {
				$msgError = 'articulo sin inventario en 99min (NinetyNineMin::getShippingArray) ' . json_encode($artics);
			}
		}

		if (!empty($log)) {
			CustomLogs::addMessage(
				$msgError,
				$log
			);
		}

		return [];
	}

	//Este metodo solo se va a dejar mientras inconcert actualiza su integracion
	public static function getShippingArray2($artics = [], $postal_code = false, $nnMth = false, $request = false, LogApiV2 $log = null)
	{
		$msgError = 'Error en metodo (NinetyNineMin::getShippingArray2)';

		if ($postal_code) {
			//Validando inventario de 99 min
			$hasInv = self::validateInventory($artics, $postal_code, $request);

			if ($hasInv) {
				$price = self::getPriceDelivery($postal_code, [
					"weight" => 90,
					"width" => 90,
					"height" => 90,
					"depth" => 90
				]);

				if ($price['success'] && !empty($price['data']->rates)) {
					$priceSelected = self::getpriceDetail($price['data']->rates);

					if ($priceSelected['success']) {
						return [
							'id' => $nnMth->code_name,
							'currier' => $nnMth->Name,
							'name' => $nnMth->Name,
							'price' => $priceSelected['data']->cost,
							'sub_request' => false,
							'aprox_time_hrs' => env('TIME_99'),
							'sending_data' => [
								'route' => '',
								'neighborhood' => '',
								'state' => '',
								'postal_code' => $postal_code
							]
						];
					} else {
						$msgError = 'Error obteniendo precio de 99min (NinetyNineMin::getShippingArray2)';

						if ($request) {
							CustomLogs::registerLog(
								'error',
								$msgError,
								'F',
								$request,
								$priceSelected
							);
						}

						$msgError = $msgError . ' ' . json_encode($priceSelected);
					}
				} else {
					$msgError = 'Error consultando precio en 99min (NinetyNineMin::getShippingArray2)';

					if ($request) {
						CustomLogs::registerLog(
							'error',
							$msgError,
							'F',
							$request,
							$price
						);
					}

					//$msgError = $msgError.' '.json_encode($price);
				}
			} else {
				$msgError = 'artículo(s) sin inventario en 99min (NinetyNineMin::getShippingArray2) ' . json_encode($artics);
			}
		}

		if (!empty($log)) {
			CustomLogs::addMessage(
				$msgError,
				$log
			);
		}

		return [];
	}
}

//Direccion fija del origin
/*
	"origin": {
	    "sender": "Netwey",
	    "route": "Mina 116",
	    "street_number": "116",
	    "internalNumber": "",
	    "neighborhood": "guerrero",
	    "locality": "centro historico",
	    "subLocality": "Ciudad de Mexico",
	    "state": "Mexico",
	    "postal_code": "06700",
	    "country": "MX"
	},
*/

/*
	"destination": {
	    "receiver": "Paola Sasso",
	    "phoneNumber": "5530587040",
	    "email": "ejemplo@gmail.com",
	    "route": "Calle Concepcion", importante
	    "street_number": "", importante, si esta
	    "internalNumber": "",
	    "neighborhood": "Del Valle Norte", importante
	    "locality": "Benito Juarez",
	    "subLocality": "Ciudad de Mexico",
	    "state": "Mexico", importante
	    "postal_code": "03103", importante
	    "country": "MX" importante
  	}
*/

//Estatus de la orden
/*
	delivery_statuses = {
		"1": "Creado",
		"2": "Recolectado",
		"3": "Asignado",
		"4": "En la Estacion",
		"5": "Ya llegó",
		"6": "Entregado",  //Nos envian DN
		"7": "Cancelado",
		"8": "Nadie Recibe",
		"9": "Segundo Intento",
		"10": "Asignado Segundo Intento",
		"11": "Segundo Intento no Exitoso",
		"12": "Entregado Segundo Intento", //Nos envian DN
		"13": "Cancelado Sin Segundo Intento",
		"14": "Devolucion",  # Solo backend
		"15": "Robo de Carga y/o extravio de carga",  # Solo backend
		"16": "Envío dañado",
		"17": "Dirección incorrecta",
		"18": "Rechazó en la recepción del envío",
		"19": "En proceso de devolución", # automatico
		"20": "Envio Demorado",
		"21": "En la estación por segunda vez"
	}
*/

//Estatus orden
/*{
    "resultData": [
        {
            "DN": [
                "34242123123",
                ""
            ],
            "status": "6",
            "nombre_status": "Entregado",
            "Tipo_envio": "Programado",
            "name_veloz": "Alfredo Hernandez",
            "Tipo_vehiculo": "bike",
            "direccion_client": "Benito Juarez Mexico",
            "Folio99": 638741
        }
    ],
    "resultStatus": [
        {
            "Status": "1",
            "Descripcion": "Creado",
            "DateStatus": "2019-04-02 11:20"
        },
        {
            "Status": "3",
            "Descripcion": "Asignado",
            "DateStatus": "2019-07-29 12:32"
        },
        {
            "Status": "2",
            "Descripcion": "Recolectado",
            "DateStatus": "2019-07-29 12:32"
        }
    ]
}*/

/*
	//envio fuera de rango
	{
	    "status": "Error",
	    "message": "Out of range coverage"
	}
*/