<?php

namespace App\Utilities;

use App\Utilities\Common;

use Illuminate\Support\Arr;

//Modelos
use App\Models\PCvoywey;
use App\Models\LogApiV2;
use Log;

class voywey
{
	public static function getPrice($data = [])
	{
		if (count($data)) {
			$res = Common::executeCurl(
				env('URL_VOYWEY') . 'price',
				'POST',
				[
					"Content-Type: application/json",
					"Authorization: Bearer " . env('TOKEN_VOYWEY'),
					"Content-Type: application/json"
				],
				$data
			);

			if ($res['success'] && !empty($res['data']) && $res['data']->success) {
				return ['success' => true, 'data' => $res['data']->data];
			}

			return [
				'success' => false,
				'data' => 'Error consultando precio.',
				'original' => $res
			];
		}

		return ['success' => false, 'data' => 'Error consultando precio, faltan parametros.'];
	}

	public static function getInventary($wh = false, $skus = [])
	{
		if ($wh && count($skus) && !empty($skus[0])) {
			$res = Common::executeCurl(
				env('URL_VOYWEY') . 'inventory',
				'POST',
				[
					"Content-Type: application/json",
					"Authorization: Bearer " . env('TOKEN_VOYWEY'),
					"Content-Type: application/json"
				],
				[
					'warehouse' => $wh,
					'skus' => $skus
				]
			);

			if ($res['success'] && !empty($res['data']) && $res['data']->success) {
				return ['success' => true, 'data' => $res['data']->data[0]];
			}
		}

		return ['success' => false];
	}

	public static function getAllInventary()
	{
		$res = Common::executeCurl(
			env('URL_VOYWEY') . 'inventory',
			'POST',
			[
				"Content-Type: application/json",
				"Authorization: Bearer " . env('TOKEN_VOYWEY'),
				"Content-Type: application/json"
			]
		);

		if ($res['success'] && !empty($res['data']) && $res['data']->success) {
			return ['success' => true, 'data' => $res['data']->data];
		}

		return ['success' => false];
	}

	public static function makeDelivery($car_id = false, $transaction = false, $typePayment = false, $coupon = false, $paymentFolio = false)
	{
		if ($car_id && $transaction) {
			$data = [
				'car_id' => $car_id,
				'transaction' => $transaction,
				'origin' => 'API-SELLER',
				'type_payment' => $typePayment
			];

			if ($coupon) {
				$data['coupon'] = $coupon;
			}

			if ($paymentFolio) {
				$data['payment_folio'] = $paymentFolio;
			}

			// Log::alert('MakeDeliveryData: ($car_id = '.$car_id.', transaction ='.$transaction.', origin = API-SELLER'.', type_payment ='.$typePayment.')');

			$res = Common::executeCurl(
				env('URL_VOYWEY') . 'make-delivery',
				'POST',
				[
					"Content-Type: application/json",
					"Authorization: Bearer " . env('TOKEN_VOYWEY'),
					"Content-Type: application/json"
				],
				$data
			);

			if ($res['success'] && !empty($res['data']) && $res['data']->success) {
				return ['success' => true, 'data' => $res['data']->data];
			}

			return ['success' => false, 'data' => 'No se pudo crear el delivery', 'original' => $res];
		}

		return ['success' => false, 'data' => 'No se pudo crear el delivery faltan parametros.'];
	}

	public static function validateInventory($postal_code, $skus = [], LogApiV2 $log = null)
	{
		$whs = PCvoywey::getWareHouses($postal_code);

		foreach ($whs as $wh) {
			if (!empty($wh) && count($skus) && !empty($skus[0])) {
				$invVoy = self::getInventary($wh->warehouse, $skus);

				if ($invVoy['success']) {
					$obj = Arr::where(
						$invVoy['data']->inventary,
						function ($value, $key) use ($skus) {
							$qtySKU = array_count_values($skus);

							return in_array($value->sku, $skus) && $value->qty_available >= $qtySKU[$value->sku];
						}
					);

					if (count($obj) == count($skus)) {
						return $wh->warehouse;
					}
				}
			}

			/*if(!empty($log)){
				CustomLogs::addMessage(
					'respuesta de inventario: '. (!empty($invVoy) ? json_encode($invVoy) : 'No hay sku'),
					$log
				);
			}*/
		}

		return false;
	}

	public static function validateListInventory($postal_code, $packs)
	{
		$whs = PCvoywey::getWareHouses($postal_code);

		$invVoy = self::getAllInventary();

		if ($invVoy['success']) {
			foreach ($whs as $wh) {
				$invWH = Arr::where(
					$invVoy['data'],
					function ($value, $key) use ($wh) {
						return $value->warehouse_id == $wh->warehouse;
					}
				);
				foreach ($packs as $pack) {
					$sm = explode(" - ", $pack->shipingMethods);
					if ((empty($pack->hasInventory) || !$pack->hasInventory || (in_array("Voywey", $sm) == false)) && (!empty($pack->sku) && count($invWH))) {
						$obj = Arr::where(
							reset($invWH)->inventary,
							function ($value, $key) use ($pack) {
								return $value->sku == $pack->sku && $value->qty_available > 0;
							}
						);

						if (count($obj)) {
							$pack->hasInventory = true;
							empty($pack->shipingMethods) ? $pack->shipingMethods = "Voywey"  : $pack->shipingMethods .= " - Voywey";
						}
					}
				}
			}
		}

		return $packs;
	}

	public static function getShippingArray($data = [], $skus = [], $voyMth = false, $request = false, LogApiV2 $log = null)
	{
		$msgError = 'Error en metodo (voywey::getShippingArray)';

		if (count($data) && count($skus) && !empty($skus[0]) && $voyMth) {
			//Consultando precio prova
			$voywey = self::getPrice($data);

			if ($voywey['success'] && !empty($voywey['data']->prices)) {
				//validando inventario y disponibilidad de prova
				$hasInv = self::validateInventory($data['postal_code'], $skus, $log);

				if ($hasInv) {
					return [
						'id' => $voyMth->code_name,
						'currier' => $voywey['data']->type,
						'name' => $voyMth->name,
						'prices' => $voywey['data']->prices,
						'sub_request' => false,
						//'aprox_time_hrs' => env('TIME_PROV'),
						'sending_data' => [
							'shipmentsId' => $voywey['data']->shipmentsId,
							'object_id' => '',
							'route' => !empty($data['route']) ? $data['route'] : '',
							'street_number' => !empty($data['street_number']) ? $data['street_number'] : '',
							'neighborhood' => !empty($data['neighborhood']) ? $data['neighborhood'] : '',
							'sub_locality' => !empty($data['sub_locality']) ? $data['sub_locality'] : '',
							'locality' => !empty($data['locality']) ? $data['locality'] : '',
							'state' => !empty($data['state']) ? $data['state'] : '',
							'internal_number' => !empty($data['internal_number']) ? $data['internal_number'] : '',
							'notes' => !empty($data['notes']) ? $data['notes'] : '',
							'postal_code' => $data['postal_code'],
							'lat' => '',
							'lng' => '',
							'delivery_date' => ''
						]
					];
				} else {
					$msgError = 'artículo(s) sin inventario en voywey (voywey::getShippingArray) ' . json_encode($skus);
				}
			} else {
				$msgError = 'Error consultando precio (voywey::getShippingArray)';

				if ($request) {
					CustomLogs::registerLog(
						'error',
						$msgError,
						'F',
						$request,
						$voywey
					);
				}

				//$msgError = $msgError.' '.json_encode($voywey);
			}
		}

		if (!empty($log)) {
			CustomLogs::addMessage(
				$msgError,
				$log
			);
		}

		return [];
	}
}
