<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class Sms_notification_token extends Model
{
  use HasFactory;
  protected $table = 'islim_sms_nip_tokens';

  protected $fillable = [
    'id',
    'bash',
    'msisdn',
    'phone_sms',
    'sms_type',
    'concentrator_id',
    'service',
    'sms_attribute',
    'sms',
    'response',
    'status',
    'date_reg',
    'date_process'];

  public $timestamps = false;

  /**
   * Metodo para seleccionar conexion a la bd, escritura-lectura o solo escritura
   * @param String $typeCon
   *
   * @return App\Models\Sms_notification
   */
  public static function getConnect($typeCon = false)
  {
    if ($typeCon) {
      $obj = new self;
      $obj->setConnection($typeCon == 'W' ? 'mysql::write' : 'mysql::read');
      return $obj;
    }
    return null;
  }

  public static function Send_sms($DN_netwey, $phone_client, $sms_type, $concentrator_id, $service, $sms_attribute, $sms = '')
  {
    try {
      return self::getConnect('W')
        ->insertGetId([
          'msisdn' => $DN_netwey,
          'phone_sms' => $phone_client,
          'sms_type' => $sms_type,
          'concentrator_id' => $concentrator_id,
          'service' => $service,
          'sms_attribute' => $sms_attribute,
          'sms' => $sms,
          'date_reg' => date('Y-m-d H:i:s')]);
    } catch (Exception $e) {
      $txmsg = "No se pudo crear el registro de envio de sms (MNT61)" . (String) json_encode($e->getMessage());
      Log::error($txmsg);
      return false;
    }
  }

  public static function resend_sms($phoneVal, $service)
  {
    $registro = self::getConnect('R')
      ->where([
        ['msisdn', $phoneVal],
        ['phone_sms', $phoneVal],
        ['service', $service]])
      ->orderBy('id', 'DESC')
      ->first();

    //Log::info('gphone ' . (String) $phoneVal);
    //Log::info('contrato ' . (String) json_encode($registro));

    if (!empty($registro)) {
      // Clonar el registro
      $nuevo_registro = $registro->replicate();

      $nuevo_registro->status = 'E';
      $nuevo_registro->date_reg = date('Y-m-d H:i:s');
      $nuevo_registro->date_process = null;
      $nuevo_registro->response = null;
      $nuevo_registro->save();

      // Devolver el id del nuevo registro
      return $nuevo_registro->id;
    }
    return null;
  }

  public static function getIcon($icon)
  {
    switch ($icon) {
      case 'SEND_SMS':
        $icono = '<svg version="1.1" id="sendSMS" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="120px" height="150px" viewBox="0 0 512 512" enable-background="new 0 0 512 512" xml:space="preserve">  <image id="image0" width="512" height="512" x="0" y="0"
    href="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAgAAAAIACAYAAAD0eNT6AAAABGdBTUEAALGPC/xhBQAAACBjSFJN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" />
</svg>
';
        break;

      case 'SEND_EMAIL':
        $icono = '<svg version="1.1" viewBox="0 0 1600 1600" width="150px" height="150px" xmlns="http://www.w3.org/2000/svg">
<path transform="translate(0)" d="m0 0h1600v1600h-1600v-1600z" fill="#FCFDFF"/>
<path transform="translate(544 393)" d="m0 0c5.21 0.00259 10.4-7.42e-4 15.6-0.00324 9.1-0.00358 18.2-0.00352 27.3-9.53e-4 13.5 0.00381 27.1 0.00183 40.6-0.00121 22.8-0.00494 45.6-0.00515 68.3-0.00242 22.6 0.0027 45.1 0.00375 67.7 0.00248 1.42-7.75e-5 2.84-1.55e-4 4.27-2.31e-4 5.72-3.11e-4 11.4-6.29e-4 17.1-9.6e-4 16-9.17e-4 31.9-0.00149 47.9-0.00179 0.727-1.36e-5 1.45-2.73e-5 2.2-4.14e-5 23.1-4.09e-4 46.3 7.15e-4 69.4 0.00289 0.75 7.02e-5 1.5 1.4e-4 2.27 2.13e-4 12.2 0.00114 24.3 0.00236 36.5 0.00359 48.2 0.00485 96.5 0.00463 145-2.31e-4 12.2-0.00122 24.3-0.00243 36.5-0.00357 0.749-7.04e-5 1.5-1.41e-4 2.27-2.13e-4 23.1-0.00216 46.2-0.00307 69.3-0.00264 1.09 2.04e-5 1.09 2.04e-5 2.2 4.12e-5 16 3.13e-4 31.9 9.47e-4 47.9 0.00187 5.71 3.29e-4 11.4 6.42e-4 17.1 9.51e-4 1.42 7.74e-5 2.84 1.56e-4 4.26 2.36e-4 22.5 0.00127 45.1-1.46e-4 67.6-0.00285 23.4-0.00277 46.7-0.00171 70.1 0.00353 13.4 0.00292 26.9 0.00316 40.3-6.46e-4 9.01-0.00227 18-8.02e-4 27 0.0035 5.11 0.00234 10.2 0.00327 15.3-4.06e-4 11.8-0.0076 23.5 0.0186 35.3 0.109 1.93 0.0147 3.87 0.0281 5.8 0.0406 6.19 0.0409 12.4 0.105 18.6 0.198 1.96 0.0268 3.92 0.0473 5.88 0.0609 23.8 0.179 23.8 0.179 32.8 8.28 3.67 4 6.47 8.45 9.21 13.1 1.49 2.56 1.49 2.56 4.54 4.32 0.374 2.26 0.374 2.26 0.373 5.14 0.00674 1.1 0.0135 2.2 0.0204 3.33-0.00788 1.22-0.0158 2.44-0.0239 3.69 0.00346 1.29 0.00692 2.58 0.0105 3.91 0.00926 3.61-3.59e-4 7.22-0.0128 10.8-0.0103 3.92-0.0032 7.83 0.00173 11.8 0.00624 6.87 0.00126 13.7-0.0109 20.6-0.0181 10.2-0.019 20.4-0.0164 30.7 0.00395 17.2-0.0061 34.4-0.0241 51.6-0.0177 17-0.0293 34.1-0.0326 51.1-2.06e-4 1.06-4.12e-4 2.12-6.25e-4 3.2-0.00101 5.37-0.00192 10.7-0.00278 16.1-0.00631 38.1-0.0265 76.1-0.0549 114-0.0275 37-0.0488 74-0.0615 111-3.95e-4 1.14-7.89e-4 2.28-0.0012 3.45-0.00396 11.4-0.00778 22.9-0.0115 34.3-0.00761 23.3-0.0161 46.6-0.0251 69.9-4.12e-4 1.07-8.24e-4 2.14-0.00125 3.25-0.0276 71.6-0.0752 143-0.126 215h-3c-0.371 1.05-0.371 1.05-0.75 2.12-3.78 8.68-9.85 14-18.2 18.1-5.33 1.94-10.9 1.91-16.5 1.92-0.833 0.00485-1.67 0.0097-2.52 0.0147-2.8 0.0151-5.6 0.0232-8.4 0.0313-2.03 0.0094-4.06 0.0192-6.09 0.0294-5.6 0.0267-11.2 0.0472-16.8 0.0665-4.81 0.0173-9.62 0.0381-14.4 0.0588-13.9 0.0599-27.8 0.111-41.7 0.16-1.04 0.00362-1.04 0.00362-2.1 0.00731-1.04 0.00363-1.04 0.00363-2.11 0.00733-7.08 0.0247-14.2 0.0512-21.2 0.0778-75.1 0.281-150 0.453-393 0.519-22.9-0.00436-45.8-0.00394-68.7 4.56e-4 -76.2 0.0146-152 0.0152-388-0.464-10.4-0.0356-20.7-0.0664-31.1-0.0945-11.6-0.0316-23.1-0.0706-34.7-0.119-5.85-0.0245-11.7-0.046-17.5-0.0593-5.34-0.0123-10.7-0.0337-16-0.0625-1.92-0.00881-3.84-0.0143-5.77-0.0162-32.6-0.0382-32.6-0.0382-44.8-11.3-8.01-9.04-9.14-17.9-9.16-29.6-0.00494-0.832-0.00988-1.66-0.015-2.52-0.0153-2.77-0.0237-5.53-0.0321-8.3-0.00959-1.99-0.0196-3.98-0.03-5.97-0.027-5.39-0.048-10.8-0.068-16.2-0.0219-5.64-0.0493-11.3-0.0761-16.9-0.0499-10.7-0.0948-21.3-0.138-32-0.0494-12.2-0.104-24.3-0.16-36.5-0.114-25-0.221-50-0.323-75-0.918-0.00331-1.84-0.00662-2.78-0.01-22.4-0.0813-44.7-0.173-67.1-0.277-10.8-0.0501-21.6-0.0965-32.4-0.134-9.42-0.033-18.8-0.0729-28.3-0.121-4.99-0.0252-9.98-0.047-15-0.0606-4.7-0.0129-9.4-0.035-14.1-0.064-1.72-0.00902-3.44-0.0148-5.16-0.0168-14-0.0199-26-0.121-37.2-9.32-7.02-7.14-10.1-13.9-10.4-23.8 0.248-9.68 3.03-17.6 9.24-25.1 8.98-8.49 17.8-10.2 29.6-10.6 1.19-0.0422 2.37-0.0844 3.59-0.128 15.6-0.498 31.2-0.444 46.8-0.417 4.38 0.00637 8.77 0.00685 13.2 0.00803 8.25 0.00307 16.5 0.0112 24.8 0.0213 9.42 0.0113 18.8 0.0167 28.3 0.0217 19.3 0.0104 38.7 0.0279 58 0.0503-0.0157-1.83-0.0157-1.83-0.0317-3.7-0.0365-4.57-0.0593-9.13-0.0781-13.7-0.01-1.97-0.0236-3.94-0.041-5.91-0.0245-2.84-0.0357-5.69-0.0444-8.53-0.0103-0.867-0.0206-1.73-0.0313-2.63-0.00221-12.3 2.98-24.1 11.4-33.5 7.08-6.48 16.5-9.42 25.9-9.26 9.63 0.661 15.4 5.27 21.9 12.2 7.29 8.71 9.31 16.5 9.27 27.6 0.00763 1.4 0.00763 1.4 0.0154 2.84 0.0146 3.12 0.0152 6.24 0.0158 9.36 0.00767 2.24 0.0162 4.49 0.0256 6.73 0.0182 4.82 0.0303 9.64 0.0375 14.5 0.0115 7.62 0.037 15.2 0.0643 22.9 0.0369 10.7 0.0676 21.3 0.0928 32 0.0016 0.677 0.00319 1.35 0.00484 2.05 0.0016 0.677 0.0032 1.35 0.00485 2.05 0.0227 9.62 0.0473 19.2 0.0743 28.8 0.0019 0.677 0.00379 1.35 0.00575 2.05 0.0318 11.3 0.0675 22.6 0.105 33.9 0.025 7.58 0.0444 15.2 0.0594 22.7 0.0102 4.7 0.0243 9.41 0.0395 14.1 0.00617 2.19 0.0106 4.37 0.0133 6.56 0.00379 2.98 0.0136 5.95 0.0249 8.93-3.86e-4 0.873-7.72e-4 1.75-0.00117 2.65 0.0179 3.36 0.0776 6.11 1.15 9.32 5.17-3.1 8.87-6.83 12.9-11.3 3.41-3.79 6.78-7.47 10.7-10.8 5.31-4.6 10.2-9.57 15.2-14.5 1.07-1.07 2.14-2.14 3.21-3.21 2.92-2.92 5.84-5.84 8.76-8.77 3.16-3.16 6.32-6.31 9.47-9.47 6.18-6.18 12.4-12.4 18.5-18.5 5.02-5.02 10-10 15.1-15.1 0.716-0.716 1.43-1.43 2.17-2.17 1.45-1.45 2.91-2.91 4.36-4.36 13.6-13.6 27.2-27.2 40.9-40.9 11.7-11.7 23.4-23.4 35.1-35.1 13.6-13.6 27.2-27.1 40.7-40.7 1.45-1.45 2.9-2.9 4.35-4.35 0.713-0.713 1.43-1.43 2.16-2.16 5.01-5.02 10-10 15-15 6.12-6.11 12.2-12.2 18.3-18.3 3.12-3.12 6.23-6.24 9.35-9.35 2.86-2.86 5.71-5.71 8.57-8.57 1.03-1.03 2.06-2.06 3.09-3.09 5.09-5.08 10.1-10.2 14.7-15.7 1.71-1.96 3.57-3.67 5.57-5.33 0.998-0.851 0.998-0.851 2.02-1.72 0.593-0.505 1.19-1.01 1.8-1.53 3.27-3 3.27-3 5-7h-2c-0.536-0.681-1.07-1.36-1.62-2.06-2.46-3.05-5.08-5.39-8.05-7.93-2.6-2.24-5.12-4.56-7.64-6.89-4.47-4.08-9.06-7.93-13.8-11.7-10.4-8.48-20.3-17.4-30-26.7-4.56-4.33-9.21-8.55-13.9-12.7-0.623-0.552-1.25-1.1-1.89-1.67-4.24-3.73-8.52-7.4-12.8-11.1-0.906-0.777-0.906-0.777-1.83-1.57-2.9-2.49-5.81-4.96-8.75-7.41-7.93-6.61-15.5-13.6-23-20.6-8.91-8.35-18.1-16.4-27.4-24.3-0.902-0.773-0.902-0.773-1.82-1.56-3.01-2.58-6.02-5.14-9.04-7.7-7.55-6.39-14.8-13-22-19.8-9.11-8.57-18.5-16.8-28-24.9-2.48-2.12-4.96-4.25-7.44-6.38-0.624-0.535-1.25-1.07-1.89-1.62-7.84-6.73-15.5-13.6-23.1-20.6-6.21-5.78-12.5-11.4-18.9-17-2.31-2.03-4.59-4.11-6.86-6.18-3.44-3.11-6.95-6.09-10.6-9-5.17-4.18-10-8.61-14.9-13.1-4.68-4.81-4.68-4.81-10.7-6.48-0.00331 0.708-0.00662 1.42-0.01 2.15-0.0813 17.3-0.173 34.5-0.277 51.8-0.0501 8.34-0.0965 16.7-0.134 25-0.033 7.27-0.0729 14.5-0.121 21.8-0.0252 3.85-0.047 7.7-0.0606 11.5-0.0129 3.63-0.035 7.26-0.064 10.9-0.0133 1.96-0.017 3.91-0.0203 5.87-0.0832 8.81-0.512 16.8-5.31 24.5l-1.56 2.57c-0.474 0.617-0.949 1.23-1.44 1.87h-2c-0.33 0.99-0.66 1.98-1 3-2.36 1.57-2.36 1.57-5.31 3.12-0.966 0.518-1.93 1.04-2.93 1.57-7.57 3.58-17.1 3.75-25.1 1.28-9.05-4.25-17.8-10.6-22.1-19.9-1.75-5.97-1.77-12-1.75-18.2-0.00227-0.893-0.00453-1.79-0.00687-2.7-0.00631-2.98-0.00551-5.96-0.00469-8.94-0.00299-2.14-0.0064-4.28-0.0102-6.42-0.00883-5.81-0.0112-11.6-0.0118-17.4-7.11e-4 -3.63-0.00285-7.26-0.0055-10.9-0.00925-12.7-0.0133-25.4-0.0125-38 6.01e-4 -11.8-0.00995-23.6-0.0257-35.4-0.0131-10.1-0.0184-20.3-0.0178-30.4 2.55e-4 -6.05-0.00257-12.1-0.0132-18.2-0.0097-5.7-0.00974-11.4-0.00248-17.1 0.00104-2.09-0.0013-4.17-0.00736-6.26-0.082-30.2-0.082-30.2 10.1-41.2 7.99-7.4 15.6-9.97 26.4-10.3 1.08-0.0307 1.08-0.0307 2.18-0.0621 2.39-0.0657 4.77-0.124 7.16-0.181 0.836-0.021 1.67-0.042 2.53-0.0636 19.6-0.486 39.3-0.524 58.9-0.51z"/>
<path transform="translate(565,462)" d="m0 0c88.4-0.0274 177-0.0418 594 0.339 0.881 0.00136 1.76 0.00271 2.67 0.00411 2.67 0.00412 5.34 0.00824 8.02 0.0124 1.32 0.00204 1.32 0.00204 2.67 0.00412 0.883 0.00136 1.77 0.00272 2.68 0.00412 0.883 0.00136 1.77 0.00272 2.68 0.00413 1.78 0.00275 3.57 0.00551 5.35 0.00826 28.5 0.044 57 0.0838 85.4 0.121 32.8 0.0427 65.6 0.0888 98.3 0.14 3.47 0.00537 6.94 0.0107 10.4 0.0161 0.853 0.00132 1.71 0.00263 2.58 0.00399 13.6 0.0209 27.2 0.0386 40.8 0.0552 13.5 0.0165 26.9 0.0366 40.4 0.0605 7.97 0.0141 15.9 0.0254 23.9 0.0327 5.92 0.00576 11.8 0.0167 17.8 0.0297 2.4 0.00446 4.8 0.0072 7.21 0.00813 3.24 0.00149 6.48 0.00901 9.72 0.0182 0.95-0.00102 1.9-0.00205 2.88-0.0031 6.4 0.0276 6.4 0.0276 7.52 1.14 0.0968 1.36 0.124 2.73 0.124 4.1 0.00225 0.891 0.00449 1.78 0.00681 2.7-0.00394 1.49-0.00394 1.49-0.00796 3.01 0.00115 1.05 0.00231 2.1 0.0035 3.18 0.00268 3.56-0.00186 7.12-0.00629 10.7 4.36e-4 2.56 0.00131 5.12 0.00259 7.67 0.00208 5.59 4.2e-4 11.2-0.00364 16.8-0.00602 8.31-0.00634 16.6-0.00546 24.9 0.00132 14-0.00204 27.9-0.00802 41.9-0.00591 13.8-0.00976 27.7-0.0109 41.5-6.87e-5 0.859-1.37e-4 1.72-2.08e-4 2.6-3.38e-4 4.36-6.4e-4 8.73-9.27e-4 13.1-0.0021 30.9-0.00884 61.9-0.0183 92.8-0.00918 30.1-0.0163 60.1-0.0205 90.2-1.97e-4 1.39-1.97e-4 1.39-3.99e-4 2.81-0.00132 9.3-0.00259 18.6-0.00384 27.9-0.00254 18.9-0.00536 37.9-0.00835 56.8-1.37e-4 0.871-2.75e-4 1.74-4.16e-4 2.64-0.00921 58.2-0.0251 116-0.0419 175-6.41-2.48-10.6-6.25-15-11.5-4.89-5.63-10.2-10.8-15.5-16.1-1.16-1.16-2.32-2.32-3.48-3.49-3.18-3.18-6.36-6.35-9.54-9.53-3.43-3.43-6.87-6.86-10.3-10.3-5.94-5.94-11.9-11.9-17.8-17.8-8.6-8.59-17.2-17.2-25.8-25.8-14-14-27.9-27.9-41.9-41.9-13.5-13.5-27.1-27.1-40.6-40.6-0.835-0.834-1.67-1.67-2.53-2.53-4.19-4.19-8.38-8.37-12.6-12.6-28.9-28.8-57.7-57.7-86.6-86.5-0.615-0.615-1.23-1.23-1.86-1.86-2.85-2.85-5.7-5.69-8.55-8.54-0.506-0.506-1.01-1.01-1.53-1.53-1.34-1.34-2.68-2.68-4.02-4.02-2.47-2.47-4.94-4.94-7.41-7.41 3.65-3.74 7.45-7.19 11.4-10.6 1.25-1.07 2.51-2.13 3.76-3.2 0.952-0.809 0.952-0.809 1.92-1.63 1.78-1.51 3.55-3.04 5.31-4.56 6.29-5.43 12.6-10.8 19.1-16 9.58-7.86 19.1-15.8 28.2-24.2 5.27-4.83 10.7-9.34 16.3-13.8 10.3-8.32 20.4-16.8 30.2-25.7 4.18-3.79 8.51-7.38 12.9-10.9 3.74-3.05 7.41-6.17 11.1-9.3 3.43-2.91 6.9-5.76 10.4-8.59 3.94-3.21 7.72-6.54 11.4-10 5.48-5.08 11.2-9.82 17-14.5 3.36-2.73 6.72-5.46 10.1-8.19 3.04-2.47 6.09-4.94 9.15-7.4 4.56-3.68 8.85-7.39 12.9-11.6 2.32-2.24 4.79-4.23 7.31-6.25 8.81-7.15 16.9-14.5 20.6-25.6 0.861-12.6-0.437-21.1-8.43-31.2-5.64-6.43-11.8-9.6-20.2-10.7-16.7-0.977-29.1 9.49-40.9 19.8-2.26 2.01-4.5 4.04-6.72 6.09-0.505 0.465-1.01 0.93-1.53 1.41-1.37 1.27-2.73 2.55-4.09 3.83-2.11 1.77-2.11 1.77-4.11 1.77-0.33 0.99-0.66 1.98-1 3-1.79 1.62-1.79 1.62-4.19 3.44-4.36 3.39-8.58 6.89-12.8 10.5-5.1 4.41-10.3 8.71-15.5 12.9-3.36 2.79-6.57 5.71-9.77 8.68-1.78 1.43-1.78 1.43-3.78 1.43-0.33 0.99-0.66 1.98-1 3-1.72 1.59-1.72 1.59-4.06 3.38-0.898 0.7-1.8 1.4-2.72 2.12-0.507 0.393-1.01 0.786-1.54 1.19-3.26 2.55-6.47 5.15-9.68 7.75-0.694 0.562-1.39 1.12-2.1 1.7-4.64 3.78-9.13 7.7-13.5 11.7-2.56 2.29-5.19 4.45-7.86 6.61-3.59 2.91-7.08 5.89-10.5 9-4.24 3.86-8.62 7.5-13.1 11.1-7.36 6.01-14.6 12.2-21.7 18.5-7.65 6.7-15.3 13.3-23.4 19.6-5.29 4.09-10.3 8.42-15.2 13-3.54 3.21-7.25 6.19-11 9.19-2.63 2.17-5.16 4.42-7.68 6.71-4.22 3.84-8.59 7.47-13 11.1-3.93 3.23-7.79 6.54-11.6 9.86-4.46 3.83-8.94 7.64-13.4 11.4-3.44 2.91-6.87 5.84-10.3 8.78-4.46 3.84-8.93 7.66-13.6 11.3-8.73 6.91-17.1 14.3-25.3 21.8-4.19 3.79-8.51 7.38-12.9 10.9-3.99 3.24-7.94 6.55-11.9 9.85-2.8 2.33-5.63 4.62-8.47 6.9-3.9 3.18-7.64 6.5-11.3 9.94-4.76 4.44-9.72 8.52-14.9 12.5-3.05 2.35-6.01 4.74-8.87 7.32-0.577 0.515-1.15 1.03-1.75 1.56-1.1 0.987-2.18 1.99-3.25 3-2.63 2.36-4 3.26-7.58 3.28-3.1-1.1-4.39-2.07-6.54-4.52-3-3.2-6.07-6.02-9.5-8.75-5.51-4.42-10.7-9.16-15.8-14-2.29-2.14-4.62-4.23-6.99-6.27-2.94-2.54-5.79-5.15-8.62-7.8-5.37-5.01-10.8-9.76-16.6-14.4-3.8-3.16-7.38-6.53-11-9.91-3.28-3.06-6.63-5.95-10.1-8.75-5.54-4.44-10.6-9.29-15.8-14.2-4.55-4.3-9.24-8.31-14.1-12.2-4.01-3.33-7.77-6.92-11.6-10.5-4.45-4.17-9.07-8.02-13.8-11.8-2.69-2.23-5.08-4.65-7.5-7.16-4.03-3.9-8.51-7.31-12.9-10.8-2.87-2.32-5.66-4.66-8.27-7.27-2.95-2.94-5.98-5.78-9.04-8.6l-1.75-1.61c-5.29-4.86-10.7-9.59-16.1-14.3-6.54-5.58-13-11.2-19.3-17-2.77-2.53-5.6-5-8.43-7.46-4.89-4.28-9.7-8.65-14.5-13-4.17-3.79-8.38-7.47-12.8-11-5.07-4.1-9.8-8.47-14.5-12.9-4.45-4.17-9.06-8.01-13.8-11.8-2.76-2.29-5.24-4.74-7.73-7.31-2.39-2.4-4.94-4.59-7.51-6.8-2.36-2.05-4.66-4.15-6.97-6.25-3.42-3.09-6.91-6.05-10.5-8.94-4.82-3.89-9.34-8.02-13.9-12.3-3.26-3.04-6.59-5.91-10.1-8.68-5-4-9.62-8.33-14.2-12.8-5.47-5.21-11-10.2-16.9-14.9-4.1-3.36-7.95-6.98-11.8-10.6-2.88-2.66-5.82-5.25-8.79-7.82-3.22-2.82-6.39-5.7-9.57-8.57-2.04-1.83-4.1-3.63-6.17-5.42-3.15-2.74-6.2-5.56-9.24-8.42-5.59-5.24-11.2-10.3-17.2-15.1-3.21-2.59-6.34-5.26-9.48-7.94-4.64-3.96-9.37-7.73-14.3-11.3 0.33-1.32 0.66-2.64 1-4z" fill="#FCFDFF"/>
<path transform="translate(866,824)" d="m0 0c7.43 5.2 13.8 11.3 20.4 17.5 5.05 4.71 10.4 9.13 15.6 13.6 11.4 9.61 22.6 19.4 33.5 29.5 6.56 6.09 13.3 12 20.1 17.8 3.91 3.34 7.74 6.77 11.5 10.3 4.59 4.27 9.35 8.27 14.2 12.2 2.41 1.98 4.67 4.05 6.95 6.18 7.51 6.71 16.4 13 26.6 13.9 1.92 0.186 1.92 0.186 3.88 0.375 13.6-0.456 25-10 34.4-18.8 3.49-3.23 7.15-6.2 10.9-9.18 3.21-2.62 6.36-5.29 9.52-7.98 3.62-3.09 7.27-6.11 11.1-9 5.02-3.85 9.65-8.1 14.3-12.4 6.29-5.75 12.7-11.3 19.3-16.6 19.3-15.6 19.3-15.6 37.9-32.1 3.52-3.25 7.05-6.46 10.8-9.5 0.681-0.562 1.36-1.12 2.06-1.7 1.94-1.05 1.94-1.05 3.84-1.08 3.21 1.59 5.28 4.19 7.59 6.85 1.07 1.19 2.13 2.37 3.2 3.56 0.545 0.604 1.09 1.21 1.65 1.83 5.53 6.04 11.4 11.9 17.6 17.2 3.86 3.29 7.46 6.81 11.1 10.4 0.884 0.864 0.884 0.864 1.78 1.75 1.44 1.44 2.84 2.94 4.22 4.44v2h2v2c0.589 0.264 1.18 0.529 1.78 0.801 2.3 1.24 3.82 2.55 5.67 4.39 0.928 0.924 0.928 0.924 1.88 1.87 0.634 0.642 1.27 1.28 1.92 1.95 0.65 0.642 1.3 1.28 1.97 1.95 0.614 0.616 1.23 1.23 1.86 1.87 0.561 0.563 1.12 1.13 1.7 1.71 1.22 1.48 1.22 1.48 1.22 3.48h2v2c0.551 0.227 1.1 0.453 1.67 0.687 3.03 1.71 5.13 3.86 7.59 6.32 0.518 0.515 1.04 1.03 1.57 1.56 1.73 1.73 3.46 3.46 5.19 5.19 1.24 1.24 2.48 2.48 3.73 3.72 3.37 3.36 6.74 6.73 10.1 10.1 3.52 3.52 7.04 7.04 10.6 10.6 5.91 5.91 11.8 11.8 17.7 17.7 6.84 6.85 13.7 13.7 20.5 20.5 5.87 5.86 11.7 11.7 17.6 17.6 3.51 3.51 7.01 7.02 10.5 10.5 3.3 3.29 6.59 6.59 9.88 9.89 1.21 1.21 2.42 2.43 3.64 3.64 1.65 1.65 3.3 3.3 4.94 4.95 0.924 0.925 1.85 1.85 2.8 2.8 1.96 2.21 1.96 2.21 1.96 4.21h2v2h2v2h2v2c0.536 0.206 1.07 0.412 1.62 0.625 3.29 1.91 5.97 4.42 8.38 7.38v2c0.866 0.371 0.866 0.371 1.75 0.75 2.58 1.43 4.31 3.03 6.25 5.25v2h2v2c0.536 0.206 1.07 0.412 1.62 0.625 3.29 1.91 5.97 4.42 8.38 7.38v2c0.536 0.206 1.07 0.412 1.62 0.625 3.29 1.91 5.97 4.42 8.38 7.38v2c0.536 0.206 1.07 0.412 1.62 0.625 3.29 1.91 5.97 4.42 8.38 7.38v2c0.804 0.309 0.804 0.309 1.62 0.625 3.29 1.91 5.97 4.42 8.38 7.38v2c0.589 0.264 1.18 0.529 1.78 0.801 2.3 1.24 3.82 2.55 5.67 4.39 0.619 0.616 1.24 1.23 1.88 1.87 0.634 0.642 1.27 1.28 1.92 1.95 0.65 0.642 1.3 1.28 1.97 1.95 0.614 0.616 1.23 1.23 1.86 1.87 0.842 0.845 0.842 0.845 1.7 1.71 1.22 1.48 1.22 1.48 1.22 3.48h2v2c0.835 0.34 0.835 0.34 1.69 0.688 2.96 1.68 5.03 3.84 7.31 6.31 0.742 0.784 1.49 1.57 2.25 2.38 0.804 0.866 1.61 1.73 2.44 2.62 0.767 0.825 1.53 1.65 2.32 2.5 1.99 2.5 1.99 2.5 3.99 7.5h-920c-0.33-0.66-0.66-1.32-1-2 2.62-3.33 5.17-6.24 8.44-8.94 2.43-2.01 4.49-4.06 6.5-6.5 2.16-2.61 4.41-4.81 7-7 3.49-2.96 6.39-6.17 9.35-9.66 2.14-2.37 4.51-4.32 6.97-6.34 1.91-1.71 3.53-3.54 5.18-5.5 2.36-2.78 4.78-5.21 7.56-7.56 3.44-2.92 6.37-6.05 9.26-9.51 2.14-2.45 4.42-4.71 6.74-6.99l2.72-2.72c60-60 120-120 182-182 1.45-1.45 2.89-2.9 4.32-4.36 2.24-2.21 2.24-2.21 5.12-4.52 2.3-1.88 4.21-3.86 6.12-6.12 5.11-6 10.8-11.4 16.4-17 1.46-1.45 2.91-2.91 4.35-4.37 2.25-2.22 2.25-2.22 5.12-4.53 3.19-2.59 5.72-5.63 8.38-8.75 2.37-2.63 4.95-4.76 7.75-6.94z" fill="#FCFDFF"/>
<path transform="translate(45.1 719)" d="m0 0c0.99-0.00175 1.98-0.00351 3-0.00531 3.32-0.00465 6.64-0.0022 9.96 2.34e-4 2.4-0.00187 4.8-0.00417 7.21-0.00687 5.22-0.00484 10.4-0.00657 15.7-0.00554 7.77 0.00153 15.5-0.00264 23.3-0.00787 14.7-0.00967 29.4-0.0125 44.1-0.0131 11.3-4.6e-4 22.7-0.00252 34-0.0057 3.27-8.98e-4 6.53-0.00179 9.8-0.00267 0.812-2.21e-4 1.62-4.42e-4 2.46-6.7e-4 22.8-0.00615 45.7-0.0101 68.5-0.00966 0.861 1.56e-5 1.72 3.12e-5 2.61 4.72e-5 4.36 8.31e-5 8.73 1.82e-4 13.1 2.85e-4 0.866 1.99e-5 1.73 3.99e-5 2.63 6.04e-5 1.75 4.04e-5 3.5 8.22e-5 5.26 1.25e-4 27.3 6.13e-4 54.5-0.00689 81.8-0.0206 30.6-0.0154 61.3-0.024 91.9-0.023 3.27 7.46e-5 6.53 1.23e-4 9.8 1.54e-4 0.804 1.17e-5 1.61 2.33e-5 2.44 3.53e-5 12.9 3.08e-5 25.9-0.00635 38.8-0.015 13-0.00848 26-0.00946 39-0.00289 7.72 0.00369 15.4 0.00187 23.2-0.00794 5.77-0.00672 11.5-0.00294 17.3 0.00487 2.33 0.00154 4.66-2.9e-4 6.99-0.00586 33.6-0.0749 33.6-0.0749 43.8 8.38 7.84 9.25 13.2 17.6 12.8 30-1.1 9-5.1 17.4-12.1 23.3-11.5 7.89-25 8.7-38.5 8.67-0.984 0.00175-1.97 0.00351-2.98 0.00531-3.29 0.00464-6.58 0.0022-9.87-2.34e-4 -2.39 0.00187-4.77 0.00417-7.16 0.00687-5.18 0.00484-10.4 0.00657-15.5 0.00554-7.71-0.00153-15.4 0.00264-23.1 0.00787-14.6 0.00966-29.1 0.0125-43.7 0.0131-11.2 4.6e-4 -22.5 0.00252-33.7 0.0057-3.24 8.98e-4 -6.48 0.00179-9.71 0.00267-0.805 2.21e-4 -1.61 4.42e-4 -2.44 6.7e-4 -22.6 0.00615-45.3 0.0101-67.9 0.00966-0.853-1.56e-5 -1.71-3.12e-5 -2.59-4.72e-5 -4.32-8.31e-5 -8.65-1.82e-4 -13-2.85e-4 -0.859-1.99e-5 -1.72-3.99e-5 -2.6-6.04e-5 -1.74-4.04e-5 -3.47-8.22e-5 -5.21-1.25e-4 -27-6.13e-4 -54.1 0.0069-81.1 0.0206-30.4 0.0154-60.7 0.024-91.1 0.023-3.24-7.46e-5 -6.48-1.23e-4 -9.71-1.54e-4 -0.797-1.17e-5 -1.59-2.33e-5 -2.42-3.53e-5 -12.8-3.08e-5 -25.7 0.00635-38.5 0.015-12.9 0.00847-25.8 0.00947-38.7 0.00289-7.66-0.0037-15.3-0.00186-23 0.00794-5.72 0.0067-11.4 0.00295-17.2-0.00487-2.31-0.00154-4.63 3.03e-4 -6.94 0.00586-3.14 0.00705-6.29 0.00177-9.43-0.00656-0.898 0.00497-1.8 0.00994-2.72 0.0151-14.2-0.0796-25.8-4.44-36.5-13.9-6.13-6.99-7.77-12-7.35-21.3 0.0226-0.966 0.0451-1.93 0.0684-2.93 0.0584-2.36 0.14-4.72 0.244-7.07 0.66-0.33 1.32-0.66 2-1 0.754-1.5 1.45-3.02 2.12-4.56 4.5-9.14 10.4-13.8 19.8-17.5 6.96-2.03 14-2.06 21.2-2.04z"/>
<path transform="translate(104 528)" d="m0 0c1.08-0.00297 2.16-0.00594 3.28-9e-3 3.62-0.00866 7.24-0.0102 10.9-0.0117 2.6-0.00463 5.2-0.00965 7.8-0.015 7.06-0.0136 14.1-0.0196 21.2-0.024 4.41-0.00277 8.83-0.00701 13.2-0.0115 12.2-0.0122 24.5-0.0225 36.7-0.0259 0.782-2.2e-4 1.56-4.39e-4 2.37-6.65e-4 0.784-2.18e-4 1.57-4.37e-4 2.38-6.61e-4 1.59-4.43e-4 3.18-8.9e-4 4.77-0.00134 1.18-3.32e-4 1.18-3.32e-4 2.39-6.71e-4 12.8-0.00394 25.5-0.0214 38.3-0.0447 13.1-0.0238 26.2-0.0362 39.3-0.0373 7.36-9.14e-4 14.7-0.0066 22.1-0.0248 6.92-0.0171 13.8-0.0186 20.8-0.00986 2.53 7.68e-4 5.07-0.00355 7.6-0.0135 33.5-0.123 33.5-0.123 42.5 7.64 7.76 8.03 12.4 16.2 12.3 27.5-0.00838 0.877-0.0168 1.75-0.0254 2.66-0.334 9.64-3.4 16.9-10.5 23.6-6.98 5.47-13.6 8.06-22.5 8.2-1.08 0.0193-2.16 0.0387-3.28 0.0586-1.19 0.0161-2.37 0.0322-3.59 0.0489-1.91 0.0305-1.91 0.0305-3.85 0.0616-3.55 0.0556-7.1 0.104-10.7 0.15-2.28 0.0298-4.57 0.0611-6.85 0.0926-42.3 0.576-84.6 0.574-127 0.571-8.85 3.54e-4 -17.7 0.0121-26.6 0.0259-8.68 0.0131-17.4 0.0184-26.1 0.0178-5.12-2.52e-4 -10.2 0.00245-15.4 0.0132-4.77 0.00974-9.54 0.00972-14.3 0.00248-1.73-0.00104-3.46 0.00125-5.2 0.00736-31.6 0.104-31.6 0.104-43-9.25-0.677-0.538-1.35-1.08-2.05-1.63-6.47-5.75-10.6-14.3-11.1-23 0.469-8.42 3.12-15.7 8.13-22.4 0.619-0.887 1.24-1.77 1.88-2.69 9.33-10.2 20.8-11.5 34-11.5z"/>
<path transform="translate(35.7 1047)" d="m0 0c1.37-0.00576 1.37-0.00576 2.77-0.0116 0.994 8.66e-4 1.99 0.00173 3.01 0.00263 1.06-0.00297 2.13-0.00594 3.22-9e-3 3.57-0.00868 7.14-0.0102 10.7-0.0117 2.56-0.00463 5.11-0.00965 7.67-0.015 6.95-0.0131 13.9-0.0196 20.8-0.024 4.34-0.00277 8.68-7e-3 13-0.0115 12-0.0122 24.1-0.0225 36.1-0.0259 0.769-2.2e-4 1.54-4.39e-4 2.33-6.65e-4 1.16-3.27e-4 1.16-3.27e-4 2.34-6.61e-4 1.56-4.43e-4 3.13-8.9e-4 4.69-0.00134 0.775-2.21e-4 1.55-4.43e-4 2.35-6.71e-4 12.6-0.00395 25.1-0.0214 37.7-0.0447 12.9-0.0237 25.8-0.0362 38.7-0.0373 7.24-9.15e-4 14.5-0.00664 21.7-0.0248 6.81-0.017 13.6-0.0186 20.4-0.00986 2.5 7.7e-4 4.99-0.00357 7.49-0.0135 3.41-0.0127 6.83-0.00684 10.2 0.00358 1.47-0.0121 1.47-0.0121 2.98-0.0244 9.47 0.0674 18.7 2.17 26.1 8.39 7.78 8.13 11.6 15.6 11.4 26.9-0.0083 0.789-0.0166 1.58-0.0251 2.39-0.266 9.2-2.92 16.2-9.29 22.9-9.15 7.87-18.5 8.93-30.1 9.13-1.53 0.0305-1.53 0.0305-3.08 0.0616-3.49 0.0662-6.97 0.124-10.5 0.179-1.25 0.0208-2.49 0.0416-3.78 0.0631-41.4 0.669-82.7 0.574-124 0.571-9.21 3.53e-4 -18.4 0.0121-27.6 0.0259-9 0.0131-18 0.0184-27 0.0178-5.33-2.53e-4 -10.7 0.00252-16 0.0132-4.96 0.00968-9.93 0.00976-14.9 0.00248-1.81-0.00104-3.62 0.0013-5.44 0.00736-18.1 0.0567-29.5-2.25-42.7-15.3-0.723-0.598-1.45-1.2-2.19-1.81-3.99-3.87-4.98-7.18-5.23-12.6-0.0812-5.52 0.0995-11 0.426-16.5 0.66-0.33 1.32-0.66 2-1 0.536-1.18 1.07-2.35 1.62-3.56 4.67-9.53 12.7-14.6 22.4-18.4 3.27-0.771 6.31-1.13 9.66-1.13z"/>
</svg>';
        break;

      default:
        $icono = "Icono no definido";
        break;
    }
    return $icono;
  }

}
